package HSPC::Application;

use strict;

use SOAP::Lite;
use DBI;
use HSPC::Logger;
use HSPC::Exc;
use HSPC::MT::GM::Constants qw(SW_GM_DEF_TIMEZONE);

sub new {
	my ($self, $class, %h) = ({ }, @_);
	if ($ENV{system_obj} && ! $h{reinit_ok}) {
		require HSPC::ProblemReporter;
		my $cs = HSPC::ProblemReporter->_get_stack_trace;
		warn 'Double initialization of ' . __PACKAGE__ . ' package.' .
			" Call stack:\n$cs";
	}
	bless $self, $class;

	## Prepare logging engine
	HSPC::Logger->cleanup() unless $ENV{system_obj};
	###### STORE IN ENV
	$ENV{system_obj} = $ENV{HSPC_SYSTEM} = $self;
}

## Returns stored/newly created SOAP::Lite client object,
## to which ->call method is applicable.
sub get_soap_client {
	SOAP::Lite->proxy('http://127.0.0.1:8080/hspc/xml-api');
}

## DEPRECATED
sub DEPRECATED {
	my $level = (shift || 0) + 1;
	my $sub = shift || (caller $level)[3];
	my $message = $sub . ' is DEPRECATED. ';
	$message .= 'The function is called from ';
	$message .= (caller $level + 1)[3];
	warn $message . "\n";
}

sub trace { DEPRECATED() }

BEGIN {
	no strict 'refs';
	## HSPc code was cleanup using following grep:
	## grep -rP '(\>|\:\:)(trace|log(|_try|_error|_ok|_special|_debug|_info|_warn))([^a-z_]|$)' 
	## To be removed for sp2
	## Create aliases for logging subroutines
	my %aliases = (
		'log'		=> 'info',
		'log_try'	=> 'info',
		'log_error'	=> 'error',
		'log_ok'	=> 'info',
		'log_special'	=> 'info',
		'log_debug'	=> 'debug',
		'log_info'	=> 'info',
		'log_warn'	=> 'warn',
	);

	foreach my $sub (keys %aliases) {
		my $level  = $aliases{$sub};
		my $symbol = __PACKAGE__ . '::' . $sub;
		*{$symbol} = sub {
			DEPRECATED(0, $symbol);
			HSPC::Logger->$level($_[1]);
		}
	}
}

## >> self
## >> message
sub system_die {
	my $self = shift;
	my $message = shift;
	my $t = HSPC::Logger->no_trace;
	$self->generate_bugreport($message);
	throw HSPC::Exc(msg=>$message);
}

## Returns id of last (!) generated bugreport
sub last_bugreport_id { HSPC::Logger->problem_id }

###### Return short report for generate_bugreport() method, can be redefined 
###### in child class
## >> self
## << report
sub _generate_bugreport_common { DEPRECATED() }

## >> self
## >> message
## << problem ID
sub generate_problemreport {
	my $t = HSPC::Logger->no_trace;
	HSPC::Logger->error($_[1]);
	HSPC::Logger->problem_id;
}

## >> self
## >> message
sub generate_bugreport {
	$ENV{layout}->set_error_app if $ENV{layout};
	my $t = HSPC::Logger->no_trace;
	HSPC::Logger->fatal($_[1]);
	HSPC::Logger->problem_id;
}

# =>> mode : /echo/debug/error
# =>  value : 1/0
# <  current mode value
sub text_mode {
	my ($self,%h) = @_;
	$self->{__textout_modes}->{$h{mode}} = $h{value} if $h{mode};
	$self->{__textout_modes}->{$h{mode}};
}

# >  inc : increment offset value to this valye
# <  current mode value
sub text_offset {
	my ($self, $inc) = @_;
	$self->{__textout_offset} += $inc;
	$self->{__textout_offsetc} = '.' x $self->{__textout_offset};
}

# > text
sub text_echo {
	my ($self, $value) = @_;
	return unless $self->{__textout_modes}->{echo};
	print $self->{__textout_offsetc} . "$value\n";
}

## Returns true, if the database is
## inside a transaction now, false otherwise
## => dbname -- name of the database to check.
sub is_transaction {
	my $self = shift;
	my $dbname = shift;

	return ($ENV{HSPC_DB_TRANS} 
			&& $ENV{HSPC_DB_TRANS}->{$dbname} > 0 
			&& (!$ENV{HSPC_DB_ROLLBACKED}
				|| !$ENV{HSPC_DB_ROLLBACKED}->{$dbname}));
}

## must override
sub is_console { }

sub _template_broken {
	my %args = @_;
	my $ref_arg = $args{arg};
	return "ERROR : template '$$ref_arg' broken at line number $args{lineno}: $args{error}";
}

## >> self
##=>> lang
sub set_current_lang {
	my $self = shift;
	my $lang = shift;

	unless($lang) {
		HSPC::Logger->error("lang is required");
		return;
	}
	$self->{_lang} = $lang;
	$ENV{session}->{LANG} = $self->{_lang} if ( $ENV{session} );
}

sub provider_id { 1 }
sub locale_timezone { SW_GM_DEF_TIMEZONE }

####### store any persistent data
## >> self
## =>> key
## =>> data : scalar or ref to any data
## Returns undef on error
sub store_data {
	my $self = shift;
	my %h = (key => undef, @_);

	unless (defined $h{key}) {
		HSPC::Logger->warn('No key defined!');
		return;
	}

	$ENV{session}->{$h{key}} = $h{data};
}

####### restore any persistent data
## >> self
## =>> key
## < scalar or ref to any data
## returns undef on error
sub restore_data {
	my $self = shift;
	my %h = ( key => undef, @_);

	unless (defined $h{key}) {
		HSPC::Logger->warn('key undefined!');
		return;
	}

	$ENV{session}->{$h{key}};
}

1;

__END__
