# -*- perl -*-
# HSPC::DataCheck
#
# common functions for checking data of standard type
#
# Author: Oleg Shaikhatarov
# (c) 2001 SWsoft, All rights reserved


package HSPC::DataCheck;
use strict;
require 5.000;
use Exporter;
BEGIN {
	our @ISA = qw(Exporter);
	our @EXPORT =
		qw(is_http_url is_email is_local_email is_email_with_at is_email_with_dot
		   is_num is_int is_fqdn is_ip is_netmask is_zip is_phone chk_passwd
		   is_unix_username is_uk_postal_code
		   is_uri is_http_uri is_https_uri is_web_uri
		   split_uri
		);
};

use SWsoft::protect;	# Mark this module for encryption upon build
use HSPC::Localization;

use HSPC::Logger qw(sw_die);
use HSPC::ParamsValidate;
use HSPC::MT::DM::DataCheck;
use Encode;

#-------------------------------- is_email --------------------------------#
# sub is_email(mail_address) - check E-mail address for conformity to RFCs
#
# Arguments:
#	mail_address - E-mail address to be checked
#   local_email => 1|0 (def. 0) # address can be local email address
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_email {
	my $email_addr = shift;
	my %h =(@_);

	## Check if mail not contains multiple dots, beginning and ending dot
	return 0 if $email_addr =~ m|\.{2,}|;
	return 0 if $email_addr =~ m|^\.|;
	return 0 if $email_addr =~ m|\.$|;

	my $esc		= '\\\\';	my $Period	= '\.';
	my $space	= '\040';	my $tab		= '\t';
	my $OpenBR	= '\[';		my $CloseBR	= '\]';
	my $OpenParen	= '\(';		my $CloseParen	= '\)';
	my $NonASCII	= '\x80-\xff';	my $ctrl	= '\000-\037';
	my $CRlist	= '\n\015';

	my $let		= '[A-Za-z]';
	my $let_dig	= '[0-9A-Za-z]';
	my $let_dig_hyp	= '[-0-9A-Za-z]';

	my $quoted_pair	= qq{$esc [^$NonASCII]};
	my $dtext	= qq{[^$esc$NonASCII$CRlist$OpenBR$CloseBR]};
	my $qtext	= qq{[^$esc$NonASCII$CRlist]};
	my $atom_char	= qq{[^($space)\@/,;:"<>$esc$OpenBR$CloseBR$ctrl$NonASCII]};
	my $atom	= qq{$atom_char+(?!$atom_char)};
	my $quoted_str	= qq{" (?:$qtext | $quoted_pair)* "};
	my $word	= qq{(?: $atom | $quoted_str)};
	my $local_part	= qq{$word($Period $word)*};
	my $domain_lit	= qq{$OpenBR (?:$dtext | $quoted_pair) $CloseBR};
#	my $sub_domain	= qq{(?: $atom | $domain_lit)};
	my $sub_domain	= qq{ $let_dig (?:$let_dig_hyp* $let_dig )? };
	my $domain	= qq{$sub_domain (?:$Period $sub_domain)*};
	my $addr_spec	= qq{$local_part \@ $domain};

	return $h{local_email} ? (($email_addr   =~ m/^$addr_spec$/xo) || ($email_addr =~ m/^$local_part$/xo)) :
		($email_addr   =~ m/^$addr_spec$/xo);
}


## check that e-mail is valid and domain at least second level
## >> e-mail
## < 1 - Yes / 0 - No
sub is_email_with_dot
{
	my $email_addr = shift;
	return 0 unless is_email($email_addr);
	return ($email_addr =~ /\@[^\.]+(\.[^\.]+)+/ && $1 =~ /[^\.]{2,}$/);
}## is_email_with_dot


#-------------------------------- is_local_email --------------------------------#
# sub is_local_email(mail_address) - check E-mail address for conformity to RFCs
# 		and check that E-mail address contains local address part only
# Arguments:
#	mail_address - E-mail address to be checked
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_local_email {
	my $esc		= '\\\\';	my $Period	= '\.';
	my $space	= '\040';	my $tab		= '\t';
	my $OpenBR	= '\[';		my $CloseBR	= '\]';
	my $OpenParen	= '\(';		my $CloseParen	= '\)';
	my $NonASCII	= '\x80-\xff';	my $ctrl	= '\000-\037';
	my $CRlist	= '\n\015';

	my $quoted_pair	= qq{$esc [^$NonASCII]};
	my $dtext	= qq{[^$esc$NonASCII$CRlist$OpenBR$CloseBR]};
	my $qtext	= qq{[^$esc$NonASCII$CRlist]};
	my $atom_char	= qq{[^($space)\@/,;:"<>$esc$OpenBR$CloseBR$ctrl$NonASCII]};
	my $atom	= qq{$atom_char+(?!$atom_char)};
	my $quoted_str	= qq{" (?:$qtext | $quoted_pair)* "};
	my $word	= qq{(?: $atom | $quoted_str)};
	my $local_part	= qq{$word($Period $word)*};
	my $domain_lit	= qq{$OpenBR (?:$dtext | $quoted_pair) $CloseBR};
	my $sub_domain	= qq{(?: $atom | $domain_lit)};
	my $domain	= qq{$sub_domain (?:$Period $sub_domain)*};
	my $addr_spec	= qq{$local_part \@ $domain};

	return $_[0] =~ m/^$local_part$/xo && $_[0] !~m![A-Z]!;
}

#-------------------------------- is_email_with_at --------------------------------#
# sub is_email_with_at(mail_address) - check E-mail address for conformity to RFCs
# for adresses such as user@ (e.g. used in sendmail config)
#
# Arguments:
#	mail_address - E-mail address to be checked
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_email_with_at {
	my $esc		= '\\\\';	my $Period	= '\.';
	my $space	= '\040';	my $tab		= '\t';
	my $OpenBR	= '\[';		my $CloseBR	= '\]';
	my $OpenParen	= '\(';		my $CloseParen	= '\)';
	my $NonASCII	= '\x80-\xff';	my $ctrl	= '\000-\037';
	my $CRlist	= '\n\015';

	my $quoted_pair	= qq{$esc [^$NonASCII]};
	my $dtext	= qq{[^$esc$NonASCII$CRlist$OpenBR$CloseBR]};
	my $qtext	= qq{[^$esc$NonASCII$CRlist]};
	my $atom_char	= qq{[^($space)\@/,;:"<>$esc$OpenBR$CloseBR$ctrl$NonASCII]};
	my $atom	= qq{$atom_char+(?!$atom_char)};
	my $quoted_str	= qq{" (?:$qtext | $quoted_pair)* "};
	my $word	= qq{(?: $atom | $quoted_str)};
	my $local_part	= qq{$word($Period $word)*};
	my $domain_lit	= qq{$OpenBR (?:$dtext | $quoted_pair) $CloseBR};
	my $sub_domain	= qq{(?: $atom | $domain_lit)};
	my $domain	= qq{$sub_domain (?:$Period $sub_domain)*};
	my $addr_spec	= qq{$local_part \@};

	return $_[0]   =~ m/^$addr_spec$/xo;
}

#-------------------------------- is_num --------------------------------#
# sub is_num(string)- check string to be a valid number
#
# Arguments:
#	string - string to be checked
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_num{
	my $value = shift;
	if (length($value)
		&& ($value =~ /^[+-]?\d+\.?\d*$/ 	## is a real number
		|| $value =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/ 	## is a C float
	)) {
		return 1;
	} else { return 0; }
}

#-------------------------------- is_int --------------------------------#
# sub is_int(string)- check string to be an integer number
#
# Arguments:
#	string - string to be checked
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_int{
	my $value = shift;
	if ($value =~ /^\d+$/) {
		return 1;
	}else { return 0; }
}

## internal URI spitter method - direct from RFC 3986
sub _split_uri{
	my $value = shift;

	my @bits = $value =~ m|(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?|;

	return @bits;
}

## split URI to its parts
sub split_uri {
	return _split_uri(shift);
}

## is_uri - is the value a well-formed uri?
##
## Returns the untainted URI if the test value appears to be well-formed.
## Note that you may really want one of the more practical methods like
## is_http_uri or is_https_uri, since the URI standard (RFC 3986) allows
## a lot of things you probably don't want.
##
## This function does not make any attempt to check whether the URI is
## accessible or 'makes sense' in any meaningful way.  It just checks
## that it is formatted correctly.
##
## Returns the untainted URI on success, undef on failure.
sub is_uri
{
	my $uri = shift;

	return 0 unless defined($uri);

	## check for illegal characters
	return 0 if $uri =~ /[^a-z0-9\:\/\?\#\[\]\@\!\$\&\'\(\)\*\+\,\;\=\.\-\_\~\%]/i;

	## from RFC 3986
	my($scheme, $authority, $path, $query, $fragment) = _split_uri($uri);

	## scheme and path are required, though the path can be empty
	return 0 unless (defined($scheme) && length($scheme) && defined($path));

	## if authority is present, the path must be empty or begin with a /
	if(defined($authority) && length($authority)){
		return 0 unless(length($path) == 0 || $path =~ m!^/!);

	} else {
		## if authority is not present, the path must not start with //
		return 0 if $path =~ m!^//!;
	}

	## scheme must begin with a letter, then consist of letters, digits, +, ., or -
	return 0 unless lc($scheme) =~ m!^[a-z][a-z0-9\+\-\.]*$!;

	## re-assemble the URL per section 5.3 in RFC 3986
	my $out = $scheme . ':';
	if ( defined $authority && length($authority) ) {
		$out .= '//' . $authority;
	}
	$out .= $path;
	if ( defined $query && length($query) ) {
		$out .= '?' . $query;
	}
	if ( defined $fragment && length($fragment) ) {
		$out .= '#' . $fragment;
	}

	return $out;
}


## is_https_uri - is the value a well-formed HTTPS uri?
##
## See is_http_uri() for details.  This version only likes the https URI scheme.
## Otherwise it's identical to is_http_uri()
sub is_https_uri
{
	my $url = shift;

	return is_http_uri($url, 1);
}


## is_http_uri - is the value a well-formed HTTP uri?
##
## Specialized version of is_uri() that only likes http:// urls.  As a result,
## it can also do a much more thorough job validating.  Also, unlike is_uri()
## it is more concerned with only allowing real-world URIs through.  Things
## like relative hostnames are allowed by the standards, but probably aren't wise.
## Conversely, null paths aren't allowed per RFC 2616 (should be '/' instead),
## but are allowed by this function.
##
## This function only works for fully-qualified URIs.  /bob.html won't work.
## See RFC 3986 for the appropriate method to turn a relative URI into an absolute
## one given its context.
##
## Returns the untainted URI if the test value appears to be well-formed.
##
## Note that you probably want to either call this in combo with is_https_uri(). i.e.
##
## print "Good" if(is_http_uri($uri) || is_https_uri($uri));
##
## or use the convenience method is_web_uri which is equivalent.
sub is_http_uri
{
	my $uri = shift;
	my $allow_https = shift;

	return 0 unless is_uri($uri);

	my ($scheme, $authority, $path, $query, $fragment) = _split_uri($uri);

	return unless $scheme;

	if ( $allow_https ) {
		return 0 unless lc($scheme) eq 'https';
	} else {
		return 0 unless lc($scheme) eq 'http';
	}

	## fully-qualified URIs must have an authority section that is
	## a valid host
	return 0 unless($authority);

	## allow a port component
	my($port) = $authority =~ /:(\d+)$/;
	$authority =~ s/:\d+$//;

	return 0 unless HSPC::MT::DM::DataCheck::is_fqdn($authority);

	## re-assemble the URL per section 5.3 in RFC 3986
	my $out = $scheme . ':';
	$out .= '//' . $authority;

	$out .= ':' . $port if $port;

	$out .= $path;

	if ( defined $query && length($query) ) {
		$out .= '?' . $query;
	}
	if ( defined $fragment && length($fragment) ) {
		$out .= '#' . $fragment;
	}

	return $out;

}

## is_web_uri - is the value a well-formed HTTP or HTTPS uri?
##
## This is just a convinience method that combines is_http_uri and is_https_uri
## to accept most common real-world URLs.
sub is_web_uri
{
	my $uri = shift;

	my $h = is_http_uri($uri);
	return $h if $h;

	return is_https_uri($uri);
}

#-------------------------------- is_fqdn --------------------------------#
# sub is_fqdn(string, [first_level => 1])- check string to be a
# Fully Qualified Domain Name
# i.e. domain has at least one dot ('.') and each part of domain name
# between dots is valid (contains only alphanumeric values and '-', but
# not begin with '-'.
# String can be 1st level domain (without dot) if argument 'first_level'
#  was set to 1
##________________________________________________________
##	temporal wrapper for HSPC::MT::DM::DataCheck::is_fqdn |
##-------------------------------------------------------/
# Arguments:
#	string - string to be checked
#
# Return values:
#	1 - Yes / 0 - No
#

sub is_fqdn {
	return HSPC::MT::DM::DataCheck::is_fqdn(@_);
}

#-------------------------------- is_ip --------------------------------#
# sub is_ip(string)- check string to be a valid IPv4 address
#
# Arguments:
#	string - string to be checked
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_ip
{
	return ($_[0] =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ &&
			$1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255) ?
			1 : 0;
}

#------------------------------ is_netmask ------------------------------#
# sub is_netmask(string,strict_checking) - check string to be a valid subnetwork mask
#
# Arguments:
#	string - string to be checked
#	strict_checking - if we should check order of 1 and 0 bits (=0 by default)
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_netmask
{
	my ($netmask,$strict_checking) = @_;
	# basic checking
	return 0 unless ($netmask =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ &&
			$1 > 127 && $1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255);
	# bits checking
	return 0 if($strict_checking &&
		unpack( 'B32', pack( 'C4C4C4C4', split (/\./, $netmask))) !~ /^1+0+$/);
	# return 1 if it's ok
	return 1;
}


sub is_zip
{
	return ( $_[0] =~ /^[-0-9A-Za-z\s'\(\)\+,\.\/:=\?]{1,15}$/ ? 1 : 0 );
}#is_zip


sub is_phone
{
	return ( $_[0] =~ /^.{1,30}$/ ? 1 : 0 );
}#is_phone

##----------------------------- is_unix_username ---------------------------##
## sub is_unix_username(string) - check string to be a valid linux login name
##
## Arguments:
##	>> string - string to be checked
##
## Return values:
##	1 - Yes / 0 - No
##
sub is_unix_username {
	return ( $_[0] =~ /^[a-z][\w-]*$/ ? 1 : 0 );
}

## sub chk_passwd - check string to be a valid linux login name
##	=>> passwd
##	<  error_string - Failure / undef - Success
sub chk_passwd {
	my %h = (
				passwd => undef,
				@_
			);
	my $pass = $h{passwd};

	return string('password_cant_contain_non_asci_err') if (
		scalar(grep($_ < 0, unpack("c*", $pass)))
	);

	return undef;
}

sub _err_minleng_passwd {
	return (length ($_[0]) >= $_[1]) ? undef : 1;
}

sub _err_mixcase_passwd {
	return ($_[0] =~ /[a-z]+/ && $_[0] =~ /[A-Z]+/) ? undef : 1;
}

sub _err_specsym_passwd {
	return ($_[0] =~ /[\W_\s_]+/) ? undef : 1;
}

sub _err_craclib_passwd {
	return undef;
}

## validate UK Royal mail postcode
## source: http://aspn.activestate.com/ASPN/Cookbook/Rx/Recipe/65254
sub is_uk_postal_code{
	my $pcode = shift;
	my @patterns = (
		'AN NAA', 'ANN NAA', 'AAN NAA', 'AANN NAA',
		'ANA NAA', 'AANA NAA', 'AAA NAA'
	);

	foreach (@patterns) {
		s/A/[A-Z]/g;
		s/N/\\d/g;
		s/ /\\s/g;
	}

	my $re = join '|', @patterns;

	return ($pcode =~ /^(?:$re)$/o) ? 1 : 0;
}

1;
