package HSPC::Error;
use strict;
use SWsoft::protect;
use HSPC::Localization qw(string);

use Exporter;

use HSPC::Logger qw(sw_die);

use vars qw (%hspc_error_constants $constants_loaded @ISA @EXPORT);
@ISA	= qw(Exporter);
@EXPORT	= qw (last_error error error_ext error_plugin all_errors last_error_message get_err_stack);

use constant MOD_VZ => 1;
use constant MOD_VZAGENT => 2;
use constant MOD_TM => 5;
use constant MOD_COLL => 10;
use constant MOD_OD_DM => 50;
use constant MOD_MT_OM => 55;

use constant MOD_DM_PLUGIN => 56;

use constant MOD_AD => 57;
use constant MOD_DM => 58;
use constant SMOD_DM_NS => 1;

use constant MOD_IM => 60;
use constant MOD_LM => 63;
use constant MOD_CORE => 65;

use constant MOD_CP => 66;
use constant SMOD_FIREWALL => 1;
use constant SMOD_SUBSCR => 2;
use constant SMOD_UGM => 3;

use constant MOD_BACKUP	=> 81;
use constant MOD_BILLING => 82;
use constant MOD_REMHIS	=> 125;
use constant MOD_MG => 126;

use constant MOD_UM => 174;

use constant MOD_SB => 180;

use constant MOD_PLUGIN => 190;

use constant MOD_LM_PLUGIN => 200;

use constant MOD_CUSTOM => 210;

push @EXPORT, qw(
MOD_VZ
MOD_TM
MOD_COLL
MOD_OD_DM
MOD_MT_OM
MOD_DM_PLUGIN
MOD_BILLING
MOD_BACKUP
MOD_REMHIS
MOD_VZAGENT
MOD_UM
MOD_MG
MOD_IM
MOD_LM
MOD_CORE
MOD_CP
MOD_AD
MOD_DM

SMOD_DM_NS
SMOD_FIREWALL
SMOD_SUBSCR
SMOD_UGM

MOD_SB

MOD_PLUGIN

MOD_LM_PLUGIN
MOD_CUSTOM
);


sub get_error_message_id {
	my $error_id = shift;
	my $mod = shift;
	my $smod = shift;

	sw_die("Module ID not defined")
		unless $mod;

	sw_die("Error ID not defined")
		unless $error_id;

	my $sql = qq/SELECT text_id FROM error WHERE module_id = ? AND error_id = ?/;
	$sql .= qq/ AND submodule_id = ?/ if $smod;
	
	my $message_id = select_result($sql,$mod,$error_id,$smod || ());

	return $message_id;
}

sub get_error_message {
	my $error_id = shift;	
	my $mod = shift;
	my $smod = shift;
	my %params = (@_);

	my $message_id = get_error_message_id($error_id,$mod,$smod || undef);

	return $message_id ? string($message_id,%params) : '';
}

#----------------------------------------------------------------------------

#sub import
#{
#	Exporter::import();
#	_check_error_obj();
#}

#----------------------------------------------------------------------------

=pod

I<error ($module,$submodule,$error)>

I<error ($module,$error)>

register last error in the error subsystem of HSPC. 
I<$module> and I<$submodule> are module and submodule IDs, use
constants from B<HSPC::ErrorStack>. I<$error> - error code.

If I<$submodule> is not used (second variant) - submodule sets to 0 (zero).

=cut

sub error
{
    my ($module, $submodule, $error);

    if (scalar (@_) == 2) {
        ($module, $error) = @_;
        $submodule = 0;
    } elsif (scalar (@_) == 3) {
        ($module, $submodule, $error) = @_;
    } else {
        die __PACKAGE__."::error(...) - argument count should be 2 or 3.";
    }

    _check_error_obj();
    $ENV{error_obj}->set_last_error(mod => $module, smod => $submodule, err => $error);
}

#----------------------------------------------------------------------------

=pod

I<error_ext(...)> - push error code into error stack.

=over

=item => mod (integer)
=item => smod (integer)

Module and submodule IDs. Use B<HSPC::ErrorStack> constants as module/submodule ID.

=item => err (integer)

Error code inside module.

=item => params (hashref, optional)

Hash contains parameter names and values for extended error messages.

=back

=cut

sub error_ext
{
    my %arg = (
        mod => undef, 
        smod => undef, 
        err => undef,
        params => undef,
        @_
    );
    
    _check_error_obj();
    $ENV{error_obj}->set_last_error(mod => $arg{mod}, smod => $arg{smod}, 
                                    err => $arg{err}, params => $arg{params});
}

## Description:
## 		push error code into error stack for plugins which have string id.
## Parameters:
##		plugin_id - plugin id, required
##		error - error id, required
##		params - hashref, optional
## Return:
##		nothing
sub error_plugin {
	my %h = (
		plugin_id => undef, 
		error => undef,
		params => undef,
		@_
	);
	    
	sw_die("There is no plugin id!") unless ($h{plugin_id});
	sw_die("There is no error id!") unless ($h{error});
	_check_error_obj();
	$h{params}->{__plugin_id__} = $h{plugin_id};
	$ENV{error_obj}->set_last_error(
		mod => &MOD_PLUGIN, 
		smod => 0,
		err => $h{error}, 
		params => $h{params}
	);
}

#----------------------------------------------------------------------------

=pod

I<last_error()> - return last error, registered in the HSPC Error subsystem (stack).

=cut
sub last_error
{
    _check_error_obj();
    return $ENV{error_obj}->get_last_error();
}

## Get all errors from stack as single string
## => no_codes - do not add error code
## => separator - separate errors in string with this
sub get_err_stack{
	my %h = (
		no_codes => undef, 
		separator => undef, 
		@_
	);
    _check_error_obj();
	return $ENV{error_obj}->get_stack_as_text(
		no_codes => $h{no_codes},
		separator => $h{separator},
	);
}

#----------------------------------------------------------------------------

=pod

I<all_errors()> - return all error codes from error stack, 
registered in the HSPC Error subsystem. Values separated by commas ',' and
can be easily used in URL.

=cut
sub all_errors
{
    _check_error_obj();
    my @errors =  $ENV{error_obj}->get_stack();
    return join (',', @errors);
}

#----------------------------------------------------------------------------

=pod

I<last_error_message()> - return last error message, registered in the HSPC Error subsystem (stack).

=cut
sub last_error_message
{
    _check_error_obj();
    return $ENV{error_obj}->get_last_error_message();
}

#----------------------------------------------------------------------------

#
# Check if error object created and create it if not.
#
sub _check_error_obj
{
	require HSPC::ErrorStack;
    return 1 if $ENV{error_obj};
    $ENV{error_obj} = new HSPC::ErrorStack();
    return ($ENV{error_obj}) ? 1 : undef;
}

#----------------------------------------------------------------------------

=head1 SEE ALSO

L<HSPC::ErrorStack(3)>

=cut

#----------------------------------------------------------------------------

1;
