package HSPC::HTML2Text;
use strict;

use SWsoft::protect; ## mark this module for encryption upon build

use Encode;
use HTML::Entities ();
use HTML::Parser ();

our @EXPORT = qw(
	&html2text
);
use base qw(HTML::Parser Exporter);

## => out (ref to scalar or filehandle to store conversion results)
## => server (base address for URLs starting with slash
## => _result (ref to scalar for results accumulation in function mode)
## <= self
sub new($%) {
    my $class = shift;
	my %h = (
		out => undef,
		server => undef,
		_result => undef,
		@_
	);

	my $self = $class->SUPER::new(
		api_version => 3,
		start_h => ['_html2text_h', 'self,event,text'],
		end_h => ['_html2text_h', 'self,event,text'],
		text_h => ['_html2text_h', 'self,event,text'], ## require decoded text (HTML entities -> UTF-8)
		end_document_h => ['_html2text_h', 'self,event,text'],
	);
	$self->{_out} = $h{out};
	$self->{_server} = $h{server};
	$self->{_result} = $h{_result};
	$self->{_skip} = 0; ## equal to 1 when inside a <script> tag or any of this sort
	$self->{_break} = undef; ## mark first paragraph by undef
	$self->{_pre} = 0; ## is it preformatted text?

    return $self;
}

sub DESTROY {
	my $self = shift;
	$self->eof;
}

## => in (string to convert)
## => end (== 1 to flush buffers before return)
## => out, server, _result - see above
## (may be not a hash but just a value)
sub parse(@) {
	my $self = (UNIVERSAL::isa($_[0], __PACKAGE__) ? shift : undef);
	undef $self unless (defined(%$self)); ## could be "HSPC::HTML2Text->html2text" => no self
	my %h = (
		in 		=> (@_ == 1 ? shift : undef),
		end		=> 0,
		out 	=> undef,
		server	=> undef,
		_result => undef,
		@_
	);
	$h{_result} = (defined wantarray	## create storage for result if caller wants it
		&& ref($h{out}) ne 'SCALAR'		## and doesn't give us scalar reference for result
		? '' : undef) unless (defined $h{_result});
	if (defined $self) {
		## called as object method
		$self->{_result} = $h{_result} unless (defined $self->{_result});
		$self->SUPER::parse($h{in});
		$self->eof if ($h{end});
	} else {
		## called as static sub
		$self = HSPC::HTML2Text->new(out => $h{out}, server => $h{server}, _result => $h{_result});
		$self->parse(in => $h{in}, end => 1, _result => $h{_result});
	}
	return $self->{_result} || (ref($self->{_out}) eq 'SCALAR' ? ${$self->{_out}} : undef);
}

sub html2text(@) {
	&parse;
}

sub _collapse {
	my ($text, $newline) = (shift, @_);
	$$text =~ s/\n\s+\n/\n\n/gs;
	if ($newline) {
		$$text =~ s/\A\s+//g;
		$$text =~ s/\s\Z/ /g;
	}
	$$text =~ s/[ \t\r]+/ /g;
	$$text =~ s/\n{3,}/\n\n/gs;
}

sub _html2text_h {
	my ($self, $event, $data) = (shift, shift, shift);

	my %breaks = ('p' => 2, 'tr' => 2, 'ol' => 2, 'ul' => 2, 'div' => 1, 't(d|h)' => 1, 'hr' => 1);
	my $breaks_re = join('|', keys %breaks);
	if ($event eq 'start' || $event eq 'stop') {
		if ($data =~ m!^<($breaks_re)[/>\s]!i && $self->{_break} < $breaks{lc($1)}) {
			$self->{_break} = $breaks{lc($1)} if defined $self->{_break};
		} elsif ($data =~ m!^<br[/>\s]!i) {
			++$self->{_break};
		}
	}
	if ($event eq 'start') {
	## opening tags
		if ($data =~ /^<a\s[^>]*href=['"]([^'">]+)['"]/i) {
			my $url = $1;
			return if ($url =~ m/^\s*$/);
			$url =~ s#^/#http://$self->{_server}/# if ($self->{_server});
			$self->{_prefix} = "[";
			$self->{_suffix} = "] ($url)";
			($self->{_presuf_chk} = $url) =~ s!^(mailto:|(f|ht)tps?://)|/+$!!g;
			return;
		} elsif ($data =~ /^<hr[>\s]/i) {
			$data = "\n" x $self->{_break} . '----------------------------';
		} elsif ($data =~ /^<li[>\s]/i) {
			$data = "* ";
			$self->{_break} = 1 unless ($self->{_break});
			$event = 'text';
		} elsif ($data =~ /^<pre[>\s]/i) {
			$data = "\n";
			$self->{_pre} = 1;
		} elsif ($data =~ /^<script[>\s]/i) {
			$self->{_skip} = 1;
			return;
		} else {
			return;
		}
	} elsif ($event eq 'end') {
	## closing tags
		if ($data =~ /^<\/h\d[>\s]/i) {
			$data = "\n";
		} elsif ($data =~ /^<\/pre[>\s]/i) {
			$self->{_pre} = 0;
			$data = "\n";
		} elsif ($data =~ /^<\/script[>\s]/i) {
			$self->{_skip} = 0;
			return;
		} else {
			return;
		}
	}
	if ($event eq 'text') {
	## plain text
		## force UTF-8 and convert HTML entities
		Encode::_utf8_on($data);
		$data = HTML::Entities::decode($data);
		## delete prefix / suffix if data contains a value to check ('_presuf_chk')
		foreach (qw(_prefix _suffix)) {
			delete($self->{$_})
				if defined $self->{$_} && index($data, $self->{_presuf_chk}) >= $[;
		}
		delete($self->{_presuf_chk});
		## collapse text (smart spacing)
		if ($data !~ /^\s*$/ && defined $self->{_prefix}) {
			$data = $self->{_prefix} . $data;
			delete($self->{_prefix});
		}
		if ($data !~ /^\s*$/ && defined $self->{_suffix}) {
			$data .= $self->{_suffix};
			delete($self->{_suffix});
		}
		unless ($self->{_pre}) {
			_collapse(\$data, $self->{_break});
 			return unless $data;
			for (; $self->{_break} && $data =~ /\n/g; --$self->{_break}) {}
			$data =~ s/\n/ /gs unless $self->{_break};
			$data =~ s/ {2,}/ /g;
 		}
		$data = "\n" x $self->{_break} . $data;
		$self->{_break} = 0 unless ($self->{_skip});
		## unforce UTF-8
		Encode::_utf8_off($data);
	} elsif ($event eq 'end_document') {
	## end of document
		$self->{_skip} = 0;
		$data = "\n";
	}

	## accumulate output in scalar / file / call result
	unless ($self->{_skip}) {
		${$self->{_out}} .= $data
			if (ref($self->{_out}) eq 'SCALAR');
		print {${$self->{_out}}} $data
			if (ref($self->{_out}) eq 'GLOB');
		$self->{_result} .= $data
			if (defined $self->{_result});
	}
}

1;

=pod

=head1 USAGE

=over 4

=item as void, OO-style

 use HSPC::HTML2Text;
 my $result;
 my $parser = HSPC::HTML2Text->new(out => \$result); # put results to $result scalar
 $parser->parse(in => $_) while (<>); # iterate through STDIN and parse input
 $parser->parse(in => '', end => 1); # flush buffers

=item as void, non OO-style (aka nicht vobla)

 use HSPC::HTML2Text qw(html2text); # import html2text to caller's namespace
 $/ = undef; # no eof marker, i.e. read the whole file
 open OUT, '>foo.bar';
 html2text(in => <>, out => \*OUT); # parse and put results to OUT file
 close OUT;

=item as function, OO-style

 use HSPC::HTML2Text;
 my $parser = HSPC::HTML2Text->new();
 print $parser->parse(in => $_) while (<>); # iterate through STDIN, parse input and print results
 print $parser->parse(in => '', end => 1); # flush buffers

=item as function, non OO-style (aka nonus equilibrium)

 C<use HSPC::HTML2Text qw(html2text);
 print html2text(in => $_) while (<>); # iterate through STDIN, parse input and print results

=item as function, non OO-style, simple parameters list (good old C)

 C<use HSPC::HTML2Text qw(html2text);
 print html2text($html); # $html should contain some HTML :)

=back

Optional C<server> parameter to class constructor contains base URL to be used
as prefix to HREFs starting with slash.

=cut
