package HSPC::LogConfigurator;

use SWsoft::protect;

use base q(Log::Log4perl::Config::BaseConfigurator);

## This is XML::LibXML port of DOMConfigurator for Log::Log4perl
use XML::LibXML;
use Log::Log4perl::Level;
use strict;

use Params::Validate qw(:all);

use constant _INTERNAL_DEBUG => 0;

## These constants have been taken from libxml2's tree.h
## as suggested in the manual for XML::LibXML::Node::node_type
use constant ELEMENT_NODE => 1;
use constant TEXT_NODE => 3;
use constant CDATA_NODE => 4;

use constant SQL_LOG_CATEGORY	=> 'sql_log';
use constant DEBUG_LOG_CATEGORY => 'debug_log';

use constant LOG_DIR		=> '/var/log';
use constant LOG_FILES		=>
##	qw(
##		taskman.log
##		hspc/hspc.log
##		hspc/upgrade.log
##		hspc/xml_api.log
##		hspc/vzcoll.log
##		hspc/vzcoll_transport.log
##		hspc/upgrade-warnings.log
##	);
	qw(
		hspc/hspc.log
		hspc/upgrade.log
		hspc/upgrade-warnings.log
	);

our $VERSION = 0.03;

our $APPENDER_TAG = qr/^((log4j|log4perl):)?appender$/;

our $FILTER_TAG = qr/^(log4perl:)?filter$/;
our $FILTER_REF_TAG = qr/^(log4perl:)?filter-ref$/;

##can't use ValParser here because we're using namespaces? 
##doesn't seem to work - kg 3/2003 
our $PARSER_CLASS = 'XML::LibXML';

our $LOG4J_PREFIX = 'log4j';
our $LOG4PERL_PREFIX = 'log4perl';

##poor man's export
*eval_if_perl = \&Log::Log4perl::Config::eval_if_perl;
*unlog4j      = \&Log::Log4perl::Config::unlog4j;

BEGIN {
	if($ENV{MOD_PERL}) {
		require Apache;
		my $s = Apache->server;

		for (&LOG_FILES) {
			my $name = LOG_DIR . "/$_";
			open TMPF, '>>', $name
				and close TMPF;
			chown $s->uid, $s->gid, $name;
			chmod 0664, $name;
		}
	}
}

sub new {
	my $prot = shift;
	my $class = ref($prot) || $prot;

	my $self = {};
	$self->{filename} = shift;

	unless($ENV{MOD_PERL}) {
		my $uid = getpwnam('apache');
		chown $uid, $uid, $self->{filename};
	}

	bless $self, $class;
	return $self;
}

## This method modifies configuration file,
## writing new parameter values to it
## It is an error, if there is no file option
## (supplied in constructor or here)
## Parameters (passed as a hash):
## loglevel -- log level as string (XXX)
## pattern -- conversion pattern as string
## filename -- name of xml configuration file to use
## enable_sql -- enable logging of SQL clauses
## extended -- write extended report (with complete dump of all function's arguments)
## enable_debug -- write logs at DEBUG level for problem reports
sub write {
	my $self = shift;
	$self = $self->new
		unless ref($self); ## Allow this method to be called as static
 	my %h = (
 			loglevel	=> 'DEBUG',
 			pattern		=> undef,
 			filename	=> undef,
 			enable_sql	=> undef,
 			extended	=> undef,
 			enable_debug	=> undef,
 			@_,
 		);

	my $filename;
	if($h{filename}) {
		$filename = $h{filename};
	} elsif($self->{filename}) {
		$filename = $self->{filename};
	}

	my $parser = new XML::LibXML;
	my $doc;
	if(-e $filename) {
		$doc = $parser->parse_file($filename);
	}

	my $root = $doc->documentElement();
	
	## setting loglevel
	if($h{loglevel}) {
		my @root_app;
		for my $node($root->findnodes('category[@name=""]')) {
			for($node->childNodes) {
				$_->setAttribute("value", $h{loglevel})
					if $_->nodeName eq "priority";

				push @root_app, $_->getAttribute("ref")
					if $_->nodeName eq "appender-ref";
			}
		}

		for my $ap(@root_app) {
			for my $node($root->findnodes('log4perl:appender[@name="'
						.$ap.'"]/param[@name="Threshold"]')) {
				$node->setAttribute("value", $h{loglevel});
			}

			## Since we are supporting log4j configuration format
			for my $node($root->findnodes('log4perl:appender[@name="'
						.$ap.'"]/param[@name="Threshold"]')) {
				$node->setAttribute("value", $h{loglevel});
			}
		}
	}

	## Setting log message pattern
	if($h{pattern}) {
		for my $node ($doc->getElementsByTagName("param")) {
			$node->setAttribute("value", $h{pattern})
				if $node->getAttribute("name") eq "ConversionPattern";
		}
	}

	## Enable/disable sql logging
	if(defined $h{enable_sql}) {
		$self->_set_priority(
				category	=> SQL_LOG_CATEGORY,
				root		=> $root,
				loglevel	=> $h{enable_sql} ? 'DEBUG' : 'ERROR',
			);
	}

	## Enable/disable debug logging
	if(defined $h{enable_debug}) {
		$self->_set_priority(
				category	=> DEBUG_LOG_CATEGORY,
				root		=> $root,
				loglevel	=> $h{enable_debug} ? 'DEBUG' : 'ERROR',
			);
	}

	## Set/Unset extended Reports generation
	if(defined $h{extended}) {
		for my $node ($root->findnodes('log4perl:appender[@class="HSPC::ProblemReporter"]')) {
			my $flag = 0;
			for my $param ($node->findnodes('param[@name="extended"]')) {
				$param->setAttribute('value', $h{extended} ? 'yes' : 'no');
				$flag = 1;
			}

			unless($flag or !$h{extended}) {
				my $param = $doc->createElement('param');
				$param->setAttribute('name', 'extended');
				$param->setAttribute('value', 'yes');
				$node->addChild($param);
			}
		}
	}

	## Writing configuration into file
	$doc->toFile($filename);
	return undef;
}

sub _set_priority {
	my $self = shift;
	my %h = validate(@_, {
			category	=> { type => SCALAR },
			loglevel	=> { type => SCALAR },
			root		=> { type => OBJECT },
		});

	for my $node ($h{root}->findnodes('category[@name="'.$h{category}.'"]/priority')) {
		$node->setAttribute('value', $h{loglevel});
	}
}

## This method does the opposite to write:
## It reads parameters out of the XML File
## Parameters (passed as a hash):
## filename -- optional name of XML file to use
## Returns:
## {loglevel => "LOGLELVEL_STRING", pattern => "PATTERN", enable_debug => 1|0, enable_sql => 1|0, extended => 1|0}
sub read {
	my $self = shift;
	$self = $self->new
		unless ref($self);

	my %h = @_;
	my $ret = {};

	my $filename;
	my $parser = new XML::LibXML;
	if($h{filename}) {
		$filename = $h{filename};
	} elsif($self->{filename}) {
		$filename = $self->{filename};
	}
	my XML::LibXML::Document $doc;

	if($filename and -e $filename) {
		$doc = $parser->parse_file($filename);
	}

	my $root = $doc->documentElement;
	my ($node) = $root->findnodes('category[@name=""]/priority');
	$ret->{loglevel} = $node->getAttribute("value");

	my ($layout) = $doc->getElementsByTagName("layout");
	$ret->{pattern} = $layout->findvalue('param[@name="ConversionPattern"]/@value');

	## Is sql_debug enabled?
	$ret->{enable_sql} = $root->findvalue('category[@name="sql_log"]/priority/@value') eq 'DEBUG' ? 1 : 0;
	$ret->{enable_debug} = $root->findvalue('category[@name="debug_log"]/priority/@value') eq 'DEBUG' ? 1 : 0;
	$ret->{extended} = $root->findvalue('log4perl:appender[@class="HSPC::ProblemReporter"]/param[@name="extended"]/@value') eq 'yes' ? 1 : 0;
	return $ret;
}

###################################################
sub parse {
###################################################
	my($self, $newtext) = @_;

	$self->text($newtext) if defined $newtext;
	my $text = $self->{text};

	my $parser = $PARSER_CLASS->new;

	my $doc;
	if($text) {
		$doc = $parser->parse_string(join('',@$text));
	} else {
		$doc = $parser->parse_file($self->{filename});
	}

	my $l4p_tree = {};

	my @java_style;
	my @perl_style;

	eval {
		@java_style = $doc->getElementsByTagName("$LOG4J_PREFIX:configuration");
	};

	eval {
		@perl_style = $doc->getElementsByTagName("$LOG4PERL_PREFIX:configuration");
	};

	my $config;
	if(@java_style) {
		$config = shift @java_style;
	} else {
		$config = shift @perl_style;
	}

	my $threshold = uc(subst($config->getAttribute('threshold')));
	if ($threshold) {
		$l4p_tree->{threshold}{value} = $threshold;
	}

	if (subst($config->getAttribute('oneMessagePerAppender')) && subst($config->getAttribute('oneMessagePerAppender')) eq 'true') {
		$l4p_tree->{oneMessagePerAppender}{value} = 1;
	}

	for my $kid ($config->childNodes) {

		next unless $kid->nodeType == ELEMENT_NODE;

		my $tag_name = $kid->nodeName;

		if ($tag_name =~ $APPENDER_TAG) {
			&parse_appender($l4p_tree, $kid);

		}elsif ($tag_name eq 'category' || $tag_name eq 'logger'){
			&parse_category($l4p_tree, $kid);
		#Treating them the same is not entirely accurate, 
		#the dtd says 'logger' doesn't accept
		#a 'class' attribute while 'category' does.
		#But that's ok, log4perl doesn't do anything with that attribute

		}elsif ($tag_name eq 'root'){
			&parse_root($l4p_tree, $kid);

		}elsif ($tag_name =~ $FILTER_TAG){
			#parse log4perl's chainable boolean filters
			&parse_l4p_filter($l4p_tree, $kid);

		}elsif ($tag_name eq 'renderer'){
			warn "Log4perl: ignoring renderer tag in config, unimplemented";
		#"log4j will render the content of the log message according to 
		# user specified criteria. For example, if you frequently need 
		# to log Oranges, an object type used in your current project, 
		# then you can register an OrangeRenderer that will be invoked 
		# whenever an orange needs to be logged. "

		}elsif ($tag_name eq 'PatternLayout'){#log4perl only
			&parse_patternlayout($l4p_tree, $kid);
		}
	}

	return $l4p_tree;
}

##this is just for toplevel log4perl.PatternLayout tags
##holding the custome cspecs
sub parse_patternlayout {
	my ($l4p_tree, $node) = @_;

	my $l4p_branch = {};

	for my $child ($node->childNodes) {
		next unless $child->nodeType == ELEMENT_NODE;

		my $name = subst($child->getAttribute('name'));
		my $value;

		foreach my $grandkid ($child->childNodes){
			if ($grandkid->nodeType == TEXT_NODE 
					|| $grandkid->nodeType == CDATA_NODE) {
				$value .= $grandkid->data;
			}
		}
		if($value) {
			$value =~ s/^\s+//;  #just to make the unit tests pass
				$value =~ s/\s+$//;
		}
		$l4p_branch->{$name}{value} = subst($value);
	}
	$l4p_tree->{PatternLayout}{cspec} = $l4p_branch;
}


##for parsing the root logger, if any
sub parse_root {
	my ($l4p_tree, $node) = @_;

	my $l4p_branch = {};

	&parse_children_of_logger_element($l4p_branch, $node);

	$l4p_tree->{category}{value} = $l4p_branch->{value};

}


##this parses a custom log4perl-specific filter set up under
##the root element, as opposed to children of the appenders
sub parse_l4p_filter {
	my ($l4p_tree, $node) = @_;

	my $l4p_branch = {};

	my $name = subst($node->getAttribute('name'));

	my $class = subst($node->getAttribute('class'));
	my $value = subst($node->getAttribute('value'));

	if ($class && $value) {
		die "Log4perl: only one of class or value allowed, not both, "
			."in XMLConfig filter '$name'";
	}elsif ($class || $value){
		$l4p_branch->{value} = ($value || $class);

	}

	for my $child ($node->childNodes) {

		if ($child->nodeType == ELEMENT_NODE){

			my $tag_name = $child->nodeName;

			if ($tag_name =~ /^(param|param-nested|param-text)$/) {
				&parse_any_param($l4p_branch, $child);
			}
		}elsif ($child->nodeType == TEXT_NODE){
			my $text = $child->data;
			next unless $text =~ /\S/;
			if ($class && $value) {
				die "Log4perl: only one of class, value or PCDATA allowed, "
					."in XMLConfig filter '$name'";
			}
			$l4p_branch->{value} .= subst($text); 
		}
	}

	$l4p_tree->{filter}{$name} = $l4p_branch;
}

   
##for parsing a category/logger element
sub parse_category {
	my ($l4p_tree, $node) = @_;

	my $name = subst($node->getAttribute('name'));

	$l4p_tree->{category} ||= {};

	my $ptr = $l4p_tree->{category};

	for my $part (split /\.|::/, $name) {
		$ptr->{$part} = {} unless exists $ptr->{$part};
		$ptr = $ptr->{$part};
	}

	my $l4p_branch = $ptr;

	my $class = subst($node->getAttribute('class'));
	$class                       && 
		$class ne 'Log::Log4perl' &&
		$class ne 'org.apache.log4j.Logger' &&
		warn "setting category $name to class $class ignored, only Log::Log4perl implemented";

	##this is kind of funky, additivity has its own spot in the tree
	my $additivity = subst(subst($node->getAttribute('additivity')));
	if ($additivity && length $additivity) {
		$l4p_tree->{additivity} ||= {};
		my $add_ptr = $l4p_tree->{additivity};

		for my $part (split /\.|::/, $name) {
			$add_ptr->{$part} = {} unless exists $add_ptr->{$part};
			$add_ptr = $add_ptr->{$part};
		}
		$add_ptr->{value} = &parse_boolean($additivity);
	}

	&parse_children_of_logger_element($l4p_branch, $node);
}

## parses the children of a category element
sub parse_children_of_logger_element {
	my ($l4p_branch, $node) = @_;

	my (@appenders, $priority);

	for my $child ($node->childNodes) {
		next unless $child->nodeType == ELEMENT_NODE;

		my $tag_name = $child->nodeName;

		if ($tag_name eq 'param') {
			my $name = subst($child->getAttribute('name'));
			my $value = subst($child->getAttribute('value'));
			if ($value =~ /^(all|debug|info|warn|error|fatal|off|null)^/) {
				$value = uc $value;
			}
			$l4p_branch->{$name} = {value => $value};

		}elsif ($tag_name eq 'appender-ref'){
			push @appenders, subst($child->getAttribute('ref'));

		}elsif ($tag_name eq 'level' || $tag_name eq 'priority'){
			$priority = &parse_level($child);
		}
	}
	$l4p_branch->{value} = $priority.', '.join(',', @appenders);

	return;
}


sub parse_level {
	my $node = shift;

	my $level = uc (subst($node->getAttribute('value')));

	die "Log4perl: invalid level in config: $level"
		unless Log::Log4perl::Level::is_valid($level);

	return $level;
}



sub parse_appender {
	my ($l4p_tree, $node) = @_;

	my $name = subst($node->getAttribute("name"));

	my $l4p_branch = {};

	my $class = subst($node->getAttribute("class"));

	$l4p_branch->{value} = $class;

	print "looking at $name----------------------\n"  if _INTERNAL_DEBUG;

	for my $child ($node->childNodes) {
		next unless $child->nodeType == ELEMENT_NODE;

		my $tag_name = $child->nodeName();

		my $name = unlog4j(subst($child->getAttribute('name')));

		if ($tag_name =~ /^(param|param-nested|param-text)$/) {

			&parse_any_param($l4p_branch, $child);

			my $value;

		}elsif ($tag_name =~ /($LOG4PERL_PREFIX:)?layout/){
			$l4p_branch->{layout} = parse_layout($child);

		}elsif ($tag_name =~  $FILTER_TAG){
			$l4p_branch->{Filter} = parse_filter($child);

		}elsif ($tag_name =~ $FILTER_REF_TAG){
			$l4p_branch->{Filter} = parse_filter_ref($child);

		}elsif ($tag_name eq 'errorHandler'){
			die "errorHandlers not supported yet";

		}elsif ($tag_name eq 'appender-ref'){
		##dtd: Appenders may also reference (or include) other appenders. 
		##This feature in log4j is only for appenders who implement the 
		##AppenderAttachable interface, and the only one that does that
		##is the AsyncAppender, which writes logs in a separate thread.
		##I don't see the need to support this on the perl side any 
		##time soon.  --kg 3/2003
			die "Log4perl: in config file, <appender-ref> tag is unsupported in <appender>";
		}else{
			die "Log4perl: in config file, <$tag_name> is unsupported\n";
		}
	}
	$l4p_tree->{appender}{$name} = $l4p_branch;
}

sub parse_any_param {
	my ($l4p_branch, $child) = @_;

	my $tag_name = $child->nodeName();
	my $name = subst($child->getAttribute('name'));
	my $value;

	print "parse_any_param: <$tag_name name=$name\n" if _INTERNAL_DEBUG;

	##<param-nested>
	##note we don't set it to { value => $value }
	##and we don't test for multiple values
	if ($tag_name eq 'param-nested'){

		if ($l4p_branch->{$name}){
			die "Log4perl: in config file, multiple param-nested tags for $name not supported";
		}
		$l4p_branch->{$name} = &parse_param_nested($child); 

		return;

	##<param>
	}elsif ($tag_name eq 'param') {

		$value = subst($child->getAttribute('value'));

		print "parse_param_nested: got param $name = $value\n"  
			if _INTERNAL_DEBUG;

		if ($value =~ /^(all|debug|info|warn|error|fatal|off|null)$/) {
			$value = uc $value;
		}

		if ($name !~ /warp_message|filter/ &&
				$child->parentNode->getAttribute('name') ne 'cspec') {
			$value = eval_if_perl($value);
		}
	##<param-text>
	}elsif ($tag_name eq 'param-text'){

		foreach my $grandkid ($child->childNodes){
			if ($grandkid->nodeType == TEXT_NODE) {
				$value .= $grandkid->data;
			}
		}
		if ($name !~ /warp_message|filter/ &&
				$child->parentNode->getAttribute('name') ne 'cspec') {
			$value = eval_if_perl($value);
		}
	}

	$value = subst($value);

	##multiple values for the same param name
	if (defined $l4p_branch->{$name}{value} ) {
		if (ref $l4p_branch->{$name}{value} ne 'ARRAY'){
			my $temp = $l4p_branch->{$name}{value};
			$l4p_branch->{$name}{value} = [$temp];
		}
		push @{$l4p_branch->{$name}{value}}, $value;
	}else{
		$l4p_branch->{$name} = {value => $value};
	}
}

##handles an appender's <param-nested> elements
sub parse_param_nested {
	my ($node) = shift;

	my $l4p_branch = {};

	for my $child ($node->childNodes) {
		next unless $child->nodeType == ELEMENT_NODE;

		my $tag_name = $child->nodeName();

		if ($tag_name =~ /^param|param-nested|param-text$/) {
			&parse_any_param($l4p_branch, $child);
		}
	}

	return $l4p_branch;
}

##this handles filters that are children of appenders, as opposed
##to the custom filters that go under the root element
sub parse_filter {
	my $node = shift;

	my $filter_tree = {};

	my $class_name = subst($node->getAttribute('class'));

	$filter_tree->{value} = $class_name;

	print "\tparsing filter on class $class_name\n"  if _INTERNAL_DEBUG;  

	for my $child ($node->childNodes) {
		next unless $child->nodeType == ELEMENT_NODE;

		my $tag_name = $child->nodeName();

		if ($tag_name =~ 'param|param-nested|param-text') {
			&parse_any_param($filter_tree, $child);

		}else{
			die "Log4perl: don't know what to do with a ".$child->nodeName()
				."inside a filter element";
		}
	}
	return $filter_tree;
}

sub parse_filter_ref {
	my $node = shift;

	my $filter_tree = {};

	my $filter_id = subst($node->getAttribute('id'));

	$filter_tree->{value} = $filter_id;

	return $filter_tree;
}



sub parse_layout {
	my $node = shift;

	my $layout_tree = {};

	my $class_name = subst($node->getAttribute('class'));

	$layout_tree->{value} = $class_name;
	print "\tparsing layout $class_name\n"  if _INTERNAL_DEBUG;  
	for my $child ($node->childNodes) {
		next unless $child->nodeType == ELEMENT_NODE;
		if ($child->nodeName() eq 'param') {
			my $name = subst($child->getAttribute('name'));
			my $value = subst($child->getAttribute('value'));
			if ($value =~ /^(all|debug|info|warn|error|fatal|off|null)$/) {
				$value = uc $value;
			}
			print "\tparse_layout: got param $name = $value\n"
				if _INTERNAL_DEBUG;
			$layout_tree->{$name}{value} = $value;  

		}elsif ($child->nodeName() eq 'cspec') {
			my $name = subst($child->getAttribute('name'));
			my $value;
			foreach my $grandkid ($child->childNodes){
				if ($grandkid->nodeType == TEXT_NODE) {
					$value .= $grandkid->data;
				}
			}
			$value =~ s/^ +//;
			$value =~ s/ +$//;
			$layout_tree->{cspec}{$name}{value} = subst($value);  
		}
	}
	return $layout_tree;
}

sub parse_boolean {
	my $a = shift;

	if ($a eq '0' || lc $a eq 'false') {
		return '0';
	}elsif ($a eq '1' || lc $a eq 'true'){
		return '1';
	}else{
		return $a; ##probably an error, punt
	}
}


##this handles variable substitution
sub subst {
	my $val = shift;

	$val =~ s/\${(.*?)}/
	Log::Log4perl::Config::var_subst($1, {})/gex
		if $val;
	return $val;
}

42;
