package HSPC::Logger;
use strict;
use warnings;

package HSPC::Logger::Transparency;
use Log::Log4perl;

sub new {
	$Log::Log4perl::caller_depth++;
	bless {}, shift;
}

sub DESTROY { $Log::Log4perl::caller_depth-- }

package HSPC::Logger;

use SWsoft::protect;

use Encode;
use Log::Log4perl;

## HSPC Logging engine
## This package is just a proxy for Log::Log4perl::Logger object
## which does the actual logging and which is created only once
## so it is safe to use it as this: HSPC::Logger->new->error
## Every time you submit a message at "ERROR" or "FATAL" loglevel,
## problem report is authomatically generated.

use constant LOG_CONFIG_XML_FILE => '/etc/hspc/logger.conf';
use constant REPORT_LOGGER_NAME => 'debug_log';
use constant SQL_LOGGER_NAME => 'sql_log';

use HSPC::Exc::Fatal;

use base 'Exporter';
our @EXPORT_OK = qw(
	sw_die sw_log sw_log_try sw_log_ok sw_log_error
	sw_log_warn sw_log_debug
	sw_trace sw_atrace
	sw_report_problem
);

####### create bug report and die
## >> error message
sub sw_die{ throw HSPC::Exc::Fatal( msg => shift ) }

sub sw_report_problem {
	## no comments!
	my $t = HSPC::Logger->no_trace;
	HSPC::Logger->error($_[1]);
	HSPC::Logger->problem_id();
}

#################################################################################
## Usage: my $trace = sw_trace("ZZZ");
## Description:
##		create trace object
## Parameters:
##		comment >>
## Return value:
## 		HSPC::Trace object,
sub sw_trace { new HSPC::Trace(shift || '') }

#################################################################################
## Usage: my $trace = sw_atrace("ZZZ");
## Description:
##		create trace object
##		and automatically add module and function, which calls it.
## Parameters:
##		comment >
## Return value:
## 		HSPC::Trace object,
sub sw_atrace {
	my $comment = shift || '';
	(my $sub = (caller 1)[3]) =~ s/(.+)::/$1->/;
	sw_trace("$sub ($comment)");
}

## Returns problem reports appender
sub _get_pr_app {
	my $prapp = Log::Log4perl->appenders->{'REPORT'};
	$prapp &&= $prapp->{appender};
	$prapp;
}

## This method returns HSPC::Logger object
## possibly initializing and creating it before.
## Input parameters(passed in as a hash):
## => category -- logger category. If this parameter is
## omitted, returns default logger. Generally, you should not use
## this parameter unless you are sure that you know what you are doing.
my $initialized = 0;
sub new {
	my $class = shift;
	my %h     = (category => REPORT_LOGGER_NAME, @_);

	$initialized = $class->init() unless $initialized;
	Log::Log4perl->get_logger($h{category});
}

## Call this method to re-read config file.
our ($ts, $ps, $conf) = (0, 0);
sub init {
	my $lts = (stat LOG_CONFIG_XML_FILE)[9];
	require HSPC::LogConfigurator;
	$conf = new HSPC::LogConfigurator(LOG_CONFIG_XML_FILE);
	Log::Log4perl->init($conf);
	$ts = $lts; $ps = $$;
}

BEGIN {
	no strict 'refs';
	## Create aliases for level and logging subroutines
	my %aliases = (
		'debug'	=> [ qw/_debug/ ],
		'info'	=> [ '', qw/_try _ok/ ],
		'warn'	=> [ qw/_warn/ ],
		'error'	=> [ qw/_error/ ],
		'fatal'	=> [ qw/_fatal/ ],
	);
	## Create logging level aliases
	foreach my $level (keys %aliases) {
		foreach my $alias (@{ $aliases{$level} }, $level) {
			my $sub = $alias ne $level
				? 'sw_log' . $alias : $alias;
			*{__PACKAGE__ . '::' . $sub} = sub {
				shift if $sub eq $level; ## remove class info
				my $old = ${'Log::Log4perl::caller_depth'}++;
				Log::Log4perl->get_logger(REPORT_LOGGER_NAME)
					->$level(@_);
				${'Log::Log4perl::caller_depth'} = $old;
			};
		}
	}
}

## Here the entry point we can re-configure to handle next request
sub cleanup {
	my $prapp = Log::Log4perl->appenders->{'REPORT'};
	$prapp &&= $prapp->{appender};
	$prapp->cleanup() if $prapp;
	Log::Log4perl::MDC->remove();
	$initialized = 0;
}


## Returns all log strings, submitted
## for current logger's category or for category passed as a parameter.
sub get_full_log {
	my $logger = shift;
	$logger = __PACKAGE__->new($logger) unless ref $logger;
	return '' unless $logger;

	## Find appropriate appender
	my $prapp = Log::Log4perl->appenders->{'REPORT'};
	$prapp &&= $prapp->{appender};
	return $prapp->get_log_piece($logger->{category}) if $prapp;
	return '';
}

## Return conventional debug log snippet.
sub get_debug_log { get_full_log(shift, REPORT_LOGGER_NAME) }

## Return sql log snippet
sub get_sql_log { get_full_log(shift, SQL_LOGGER_NAME) }

## This method return HSPC::Logger::Transparency object.
## Use this method if you want your current "caller" to be reported
## in the logs instead of current method from which you are actually
## calling logging functions. Intended to be used with wrapper 
## functions like sw_log_debug, for example.
sub no_trace { new HSPC::Logger::Transparency }

## This function must be called before starting
## task execution in order to keep track of problem reports correctly.
## => task_id - The task which is started.
sub task_started {
	my $class = shift;
	my $task_id = shift;
	Log::Log4perl::MDC->remove();
	Log::Log4perl::MDC->put("current_task", $task_id);
}

## This function must be called after the task is finished
## in order to avoid exessive memory usage by logger.
## => task_id - The task which is started.
sub task_finished {
	my $class = shift;
	my $task_id = shift;

	my $mdc_context = Log::Log4perl::MDC->get_context();
	delete $mdc_context->{"problem_$task_id"};
}


## This function returns the filename of the
## last generated problem report or undef is there is none.
## => task_id -- id of the task to get the problem id for.
##	This parameter is optional.
sub problem_id {
	my $class = shift;
	my $task_id = shift || "default";

	my $report_name = Log::Log4perl::MDC->get("problem_$task_id") ||
		Log::Log4perl::MDC->get("last_report_id");
	Encode::_utf8_off($report_name);
	$report_name;
}

## This method loads Log Control parameters from XML File
## No input parameters.
## Returns:
## {loglevel => "LOGLELVEL_STRING", pattern => "PATTERN"}
## or undef on error
sub load { HSPC::LogConfigurator->read(filename => LOG_CONFIG_XML_FILE) }

## This method saves Log control parameters
## Parameters (passed as a hash):
## loglevel -- Log Level as string
## pattern -- Log format pattern
sub save {
	my $class = shift;
	return $_ if $_ = $class->is_wrong_settings(@_);
	HSPC::LogConfigurator->write(filename => LOG_CONFIG_XML_FILE, @_);
}

## Checks correctness of pattern and loglevel settings
## Parameters (passed as hash):
## pattern -- log pattern (format string)
## loglevel -- log level string
## Returns
## undef if everything is fine
## {error_id => "error_id_string", error_param => "error_parameter value"}
## if error occurred.
sub is_wrong_settings {
	my $class = shift;
	my %h = @_;

	if($h{loglevel}) {
		unless($h{loglevel} eq "DEBUG"
				or $h{loglevel} eq "INFO"
				or $h{loglevel} eq "WARN"
				or $h{loglevel} eq "ERROR"
				or $h{loglevel} eq "FATAL") {
			return {error_param => $h{loglevel},
				error_id => "log_loglevel_invalid"};
		}
	}

	if($h{pattern}) {
		## format string description. See PatternLayout's documentation
		## for more details.
		## Valid placeholders (except date, which is later)
		my $ph = qr/[cCFHlLmMnpPrTXx]/;
		## Maximum field width specification
		my $dotdigit = qr/\.\d+/;
		## Left justify field
		my $lj = qr/\-\d+/;
		## Some other digits (for right justification or zero paddind)
		my $pad = qr/\d+/;
		## Date placeholder
		my $datep = qr/d(\{[\w\d]+\})?/;
		## curlies handling
		my $curly = qr/\{\d\}/;

		## extract all conversion patterns out of string
		my @chunks = grep {m/%/} map {split /(?=%(?!%))/}
						grep { index($_,'%') >= 0 }
							split /\s+/, $h{pattern};

		## now, verify them one-by-one
		for (@chunks) {
			return {error_param => $_, error_id => "log_pattern_conversion_invalid"}
				unless /^%((($dotdigit|$lj|$pad|)(($ph($curly)?)|$datep))|%)/;
		}
	}

	return undef; ## No error
}

42;
