# Copyright (C) 2000, 2005  SWsoft
# All rights reserved.

package HSPC::MT::DM::Core::Phone;

use strict;
use SWsoft::protect;
use HSPC::Localization qw(:DEFAULT string string_lang);

use HSPC::Logger qw(sw_atrace sw_log_debug);
use Data::Dumper;


## >> class
## >> phone_src
## => is_keep_delimeters
sub phone_as_hash
{
	my($class,$phone_src,%h) = @_;

	my @chunks = split('\|',$phone_src);
	local $_;
	foreach my $chunk ( @chunks ) {
		$chunk =~ s/\&\&/\|/g;
		$chunk =~ s/\&;/\&/g;
		$chunk = $class->remove_delimiters($chunk) if !$h{is_keep_delimeters};
	}
	my $h ={country_code=>$chunks[0],
		area_code   =>$chunks[1],
		number      =>$chunks[2],
		extension   =>$chunks[3]
	};
	return $h;
}

## >> class
## >> phone_src
## => is_keep_delimeters
## << ref to hash with phone attributes
sub phone_src2hash {
	my $class = shift;
	return $class->phone_as_hash(@_);
}

#############################################
## >> class
## >> ref to hash with phone attributes
## << phone_src
sub hash2phone_src
{
	my($class,$hs) = @_;		
	my $phone_src = $hs->{country_code}.'|'.$hs->{area_code}.'|'
		.$hs->{number}.'|'.$hs->{extension};

	return $phone_src;
}
#############################################

## >> phone_src
## => is_keep_delimeters
sub check
{
	my($class,$phone_src, %h) = @_;
	return if !$h{not_empty} && $phone_src !~ '[^|\s]';

	my @message;
	my $h = $class->phone_as_hash($phone_src);

	unless ( $h->{country_code} =~ /^\d{1,6}$/ ) {
		push @message, string("$h{msg_prefix}cc_must_1to6_digits");
	}
	unless ( $h->{area_code} =~ /^\d{0,8}$/ ) {
		push @message, string("$h{msg_prefix}ac_must_0to8_digits");
	}
	unless ( $h->{number} =~ /^\d{1,10}$/ ) {
		push @message, string("$h{msg_prefix}num_must_1to10_digits");
	}
	unless ( $h->{extension} =~ /^\d{0,6}$/ ) {
		push @message, string("$h{msg_prefix}ext_must_0to6_digits");
	}
	return @message;
} ## check


## >> class
## >> phone_chunk
## < phone_chunk - with removed white spase and -
sub remove_delimiters
{
	my($class,$phone_chunk) = @_;
	$phone_chunk =~ s![\s+.()-]!!g;
	return $phone_chunk;
}


#############################################
## >> class
## >> phone string
## << phone_src
sub phone2phone_src
{
	my($class,$phone) = @_;
	my $hs	= $class->parse_phone($phone);
	sw_log_debug("[DM][Phone] phone_hash: ".Dumper($hs));
	return $hs ? $class->hash2phone_src($hs) : undef;
}


#############################################
## >> class
## >> phone string - HSPcomplete phone format
## << phone_src
sub phone_str2phone_src
{
	my($class,$phone) = @_;
	my $hs	= $class->phone_str2hash($phone);
	return $hs ? $class->hash2phone_src($hs) : undef;
}

#############################################
## >> class
## >> phone string
## << phone_hash
sub parse_phone {
	my $class = shift;
	my $number = shift;

	return undef unless $number; ## return if number is not specified

	my %p = (number => $number);
	sw_log_debug("Phone number to parse: $p{number}");
	
	$p{country_code} = (
		$p{number} =~ s|^\+(\d{1,3})\D*||
		? $1
		: undef
	);
	$p{extension} = (
		$p{number} =~ s!(\d+)(x|ext|\s)[\.\s]*(\d+)$!$1!
		? $3
		: ''
	);
	my $area_code_pttn = $class->get_area_code_pttn($p{country_code});
	$p{area_code} = (
		$p{number} =~ s|^(\d+)\D+(\d+)|$2|
		? $1
		: (
			$p{number} =~ s|$area_code_pttn|$2|
			 ? $1
			 : ''
		)
	);
	$p{number} =~ s|\D||g;

	sw_log_debug(
		"Parsed phone number: "
		."country_code=>$p{country_code}, "
		."area_code=>$p{area_code}, "
		."number => $p{number}, "
		."extension=>$p{extension}"
	);
	return \%p;
}

#############################################
## pattern for area code parser
## >> class
## >> country code
## << regexp pattern string. have two match parts
##    $1 - area code
##    $2 - phone number
sub get_area_code_pttn {
	my $class = shift;
	my $country_code = shift;
	my %h_pttn = (
		0 => '^(\d{3})(\d+)', ## default pattern, if country code is unknown
		1 => '^(\d{3})(\d+)',
		7 => '^(\d{3})(\d+)', ## for Russia
	);
	## If country code not defined in h_pttn hash, default pattern
	## (country code 0) will used.
	return $h_pttn{$country_code} || $h_pttn{0};
}

sub phone_src2str {
	my $class = shift;
	my $phone_src = shift;
	return $class->phone_as_str(
		phone_src => $phone_src
	);
}

## Description:
##		Returns phone as string like "+7 (095) 1234567 ext. 123"
## Usage:
##		HSPC::MT::DM::Core::Phone->phone_as_str(phone_src => $phone);
## Parameters:
##		>   class name
##		=>> phone_src - phone number as it sotred in data base
## Return value:
##		Phone number string.
sub phone_as_str {
	my $self = shift;
	my %h = (
		phone_src => undef,
		@_
	);
	my $phone_src = $h{phone_src};

	return '' if $self->is_phone_empty(phone => $phone_src);

	my $phone = $self->phone_as_hash($phone_src);
	my $str;

	$str .= "+$phone->{country_code} " if $phone->{country_code};
	$str .= "($phone->{area_code}) " if $phone->{area_code};
	$str .= "$phone->{number}";
	$str .= " ext. $phone->{extension}" if $phone->{extension};

	return $str;
}

## Description:
##		Returns phone as string in E.164 standard like "+1.8005551212123"
##
##      E.164 is an international numbering plan for public telephone
##      systems in which each assigned number contains a country code (CC),
##      a national destination code (NDC), and a subscriber number (SN).
##      There can be up to 15 digits in an E.164 number. The E.164 plan
##      was originally developed by
##      the International Telecommunication Union (ITU).
##
## Usage:
##		HSPC::MT::DM::Core::Phone->phone_as_e164(phone_src => $phone);
## Parameters:
##		>   class name
##		=>> phone_src - phone number as it sotred in data base
## Return value:
##		Phone number string.
sub phone_as_e164 {
	my $self = shift;
	my %h = (
		phone_src => undef,
		@_
	);
	my $phone_src = $h{phone_src};

	return '' if $self->is_phone_empty(phone => $phone_src);

	my $phone = $self->phone_as_hash($phone_src);
	my $str;

	$str .= "+$phone->{country_code}.";
	$str .= "$phone->{area_code}$phone->{number}";
	if ($phone->{extension}){
		my $sum_len = length($phone->{country_code})
			+ length($phone->{area_code})
			+ length($phone->{number})
			+ length ($phone->{extension});
		if ( $sum_len <= 15)
		{
			$str .= "$phone->{extension}";
		} else {
			sw_log_debug('WARNING: summary phone length: '.$sum_len
				.' E.164 phone number max length 15 digits.'
				.' Phone extension will be deleted from number');
		}
	}

	return $str;
}

## Description:
##		Returns phone as string in E.164 standard like "+1.8005551212x123"
## Usage:
##		HSPC::MT::DM::Core::Phone->phone_string_as_e164(phone => $phone);
## Parameters:
##		>   class name
##		=>> phone - phone number as string
## Return value:
##		Phone number string.
sub phone_string_as_e164 {
	my $self = shift;
	my %h = (
		phone => undef,
		@_
	);
	unless ($h{phone}){
		sw_log_debug(
			'WARNING: (HSPC::MT::DM::Core::Phone::phone_string_as_e164) '
			.'missing argument \'phone\'');
		return '';
	}
	## get phone src
	my $phone_src = $self->phone2phone_src($h{phone});
	## convert to E.164
	my $ret =  $self->phone_as_e164(phone_src => $phone_src);

	return $ret
}

## Description:
##		Returns phone as "toll-free" formatted string (used in Canada) like "1-800-555-1212"
## Usage:
##		HSPC::MT::DM::Core::Phone->phone_as_tollfree(phone_src => $phone);
## Parameters:
##		>   class name
##		=>> phone_src - phone number as it sotred in data base
## Return value:
##		Phone number string.
sub phone_as_tollfree {
	my $self = shift;
	my %h = (
		phone_src => undef,
		@_
	);
	my $phone_src = $h{phone_src};

	return '' if $self->is_phone_empty(phone => $phone_src);

	my $phone = $self->phone_as_hash($phone_src);
	my $str;

	$str .= "$phone->{country_code}-";
	$str .= "$phone->{area_code}-" if $phone->{area_code};
	$str .= "$phone->{number}";
	$str .= "x$phone->{extension}" if $phone->{extension};

	return $str;
}

## Description:
##		Returns phone as "DE" formatted string like "+49 89 12345678"
## Usage:
##		HSPC::MT::DM::Core::Phone->phone_like_de(phone => $phone);
## Parameters:
##		>   self
##		=>> phone - phone number as string
## Return value:
##		Phone number string.
sub phone_like_de {
	my $self = shift;
	my %h = (
		phone => undef,
		@_
	);

	## format some data
	## phone/fax code format example:
	## +49 89 12345678 (for DE) 
	my $phone = $self->phone_as_hash($h{'phone'});
	my $str;
	$str .= "+$phone->{country_code} " if length($phone->{country_code});
	$str .= "$phone->{area_code} " if length($phone->{area_code});
	$str .= "$phone->{number}";
	$str .= " $phone->{extension}" if length($phone->{extension});

	return $str;
}

sub check_phonecountry {
	my $class = shift;
	my %h = (
		country_code => undef,
		phone_country_code => undef,
		@_
	);
	my $trace = sw_atrace(
		"country_code => $h{country_code}; " .
		"phone_country_code => $h{phone_country_code}"
	);
	return undef
		if !(defined $h{country_code} && defined $h{phone_country_code});

##	my $pc = HSPC::MT::DM::Core::Phone::PhoneCountry->new();
##	my @req = ('id');
##	my $data = $pc->list(
##		attributes => \@req,
##		filter => {
##			'code' => $h{country_code},
##			'phone' => $h{phone_country_code}
##		}
##	);
	my $return;
	$return = 1;
##	$return = 1
##		if scalar(@$data);
##	$trace->add($return ? 'Check passed' : 'Check failed');
	return $return;
}

sub is_valid_phone_country_code {
	my $class = shift;
	my %h = (
		phone_country_code => undef,
		@_
	);
	my $trace = sw_atrace(
		"phone_country_code => $h{phone_country_code}"
	);
	return undef
		if !defined $h{phone_country_code};

##	my $pc = HSPC::MT::DM::Core::Phone::PhoneCountry->new();
##	my @req = ('id');
##	my $data = $pc->list(
##		attributes => \@req,
##		filter => {
##			'phone' => $h{phone_country_code}
##		}
##	);
	my $return = 1;
##	my $return = 0;
##	$return = 1
##		if scalar(@$data);
##	$trace->add($return ? 'Check passed' : 'Check failed');
	return $return;
}

sub is_phone_empty {
	my $class = shift;
	my %h = (
		phone => undef,
		@_
	);
	return $h{phone} && $h{phone} ne '|||' ? 0 : 1;
}

sub is_phone_valid {
	my $class = shift;
	my %h = (
		phone => undef,
		mandatory_country_code => 1,
		mandatory_area_code => 0,
		mandatory_number => 1,
		mandatory_extension => 0,
		optional => 0,
		digits_only => 1,
		@_
	);

	## if full phone are not mandatory
	if ($h{optional} && $class->is_phone_empty(phone => $h{phone})) {
		return 1;
	}

	###############################################
	## form pattern like ^\d+|\d*|\d+|\d*$
	## OR ^.+|.*|.+|.*$ in case digits_only => 0
	###############################################
	my $atom = $h{digits_only} ? '\d' : '.';
	my $cc_pattern = $atom . ($h{mandatory_country_code} ? '+' : '*');
	my $ac_pattern = $atom . ($h{mandatory_area_code} ? '+' : '*');
	my $num_pattern = $atom . ($h{mandatory_number} ? '+' : '*');
	my $ext_pattern = $atom . ($h{mandatory_extension} ? '+' : '*');
	my $pattern = join '\|', ($cc_pattern,$ac_pattern,$num_pattern,$ext_pattern);
	return ($h{phone} =~ /^$pattern$/) ? 1 : 0;
}

1;
