## HSPC::MT::DM::DataCheck
##
## common functions for checking data of standard type for Domain Manager
##

package HSPC::MT::DM::DataCheck;

use strict;

use Exporter;
use base qw(Exporter);

our @EXPORT = qw(
	is_fqdn
	is_ascii_fqdn
	is_rdata_txt
	is_valid_mx_priority
	is_utf
	is_idn_domain
	does_allow_3rd_level_tld
);

use SWsoft::protect;	# Mark this module for encryption upon build


use HSPC::Logger qw(sw_log_ok sw_log_warn);
use HSPC::MT::DM::Constants;
use Encode;
use Net::IDN::Encode;

sub is_rdata_txt {
	my $value = shift;
	## do not know any restrictions for PowerDNS yet
	return 1;
}

##------------------------------- is_valid_mx_priority --------------------#
## sub is_valid_mx_priority - check MX priority for conformity to RFCs
##
##	Arguments:
##		mx_priority
##
##	Return values:
##		1 - valid / 0 - not valid
##
sub is_valid_mx_priority {
	my $value = shift;
	if (defined $value
		and $value =~ /^\d+$/
		and $value >= 0
		and $value <= 65535)
	{
		return 1;
	} else {
		return 0;
	}
}

#-------------------------------- is_fqdn --------------------------------#
# sub is_fqdn(string, [first_level => 1])- check string to be a 
# Fully Qualified Domain Name
# i.e. domain has at least one dot ('.') and each part of domain name
# between dots is valid (contains only alphanumeric values and '-', but
# not begin with '-'.
# String can be 1st level domain (without dot) if argument 'first_level'
#  was set to 1
#
# Arguments:
#	string - string to be checked
#
# Return values:
#	1 - Yes / 0 - No
#
sub is_fqdn {
	return _is_fqdn(@_);
}

sub is_ascii_fqdn {
	return _is_ascii_fqdn(@_);
}

## Schedule of Reserved Names
## http://www.icann.org/tlds/agreements/unsponsored/registry-agmt-appk-26apr01.htm
use constant RESERVED_SLDS => [
	## ICANN:
	'aso',
	'dnso',
	'icann',
	'internic',
	'pso',

	## IANA-related names:
	'afrinic',
	'apnic',
	'arin',
	'example',
	'gtld-servers',
	'iab',
	'iana',
	'iana-servers',
	'iesg',
	'ietf',
	'irtf',
	'istf',
	'lacnic',
	'latnic',
	'rfc-editor',
	'ripe',
	'root-servers',

	## Additional Second-Level Reservations
	'aero',
	'arpa',
	'biz',
	'com',
	'coop',
	'edu',
	'gov',
	'info',
	'int',
	'mil',
	'museum',
	'name',
	'net',
	'org',
	'pro',

	## Second-Level Reservations for Registry Operations
	'nic',
	'whois',
	'www',
];

use constant EXCEPTIONS_RESERVED_SLDS => {
	'gr' => ['biz']
};

sub _is_ascii_fqdn {
	my $value = shift;
	my %h = (@_);
	my @parts = split (/\./, $value);

	return 0 if grep /^-|-$/, @parts;

	my $parts_count = scalar(@parts);
	my $first_level = ($h{first_level}) ? 1 : 0 ;
	my $min_smb = ($h{min_smb}) ? $h{min_smb} : 1 ;
	my $wildcard_enabled = ($h{wildcard_enabled}) ? 1 : 0 ;
	my $is_name;

	if($first_level){
		return 0 if ($parts_count < 1);
	} else {
		## Since the 14 january 2004 .name allows second level domains
		return 0 if ($parts_count < 2);
		$is_name = 1 if ($parts[$parts_count - 1] eq 'name');
	}

	## Check for trailing dots (bug #45284)
	return 0 if ($value =~ m#\.+$#);

	if (!$first_level){
		## Allow or forbid using 1smb or 2smb domains
		if (length($parts[$parts_count - 2]) < $min_smb){
			sw_log_warn("SLD '".$parts[$parts_count - 2]."' too short");
			return 0;
		}

		## Process restrictions for 2-nd level domains
		## http://www.afilias.info/whois_search/reserved_names
		##
		## ICANN-Reserved Names
		##
		## Names are reserved for use by ICANN, IANA, and for DNS-related reasons, 
		## as detailed in Appendix K of ICANN's agreement with Afilias. 
		## For example, ICANN requires that all one-character and two-character 
		## .INFO domain names be reserved. See also section 3.6.3 of the agreement.
		if (($parts_count == 2) && (grep {$parts[0] eq $_} @{&RESERVED_SLDS})){
			## Check on exceptions per TLD
			if (&EXCEPTIONS_RESERVED_SLDS->{$parts[1]}
				&& (grep {$parts[0] eq $_} @{&EXCEPTIONS_RESERVED_SLDS->{ $parts[1] } }) )
			{
				sw_log_ok("SLD '".$parts[0]."' is reserved by ICANN, "
					."but allowed by '".$parts[1]."' registry, allow this SLD"
				);
			} else {
				sw_log_warn("SLD '".$parts[0]."' is reserved");
				return 0;
			}
		}
	}

	## Skip if wildcard
	if ($parts[0] eq '*' && $wildcard_enabled){
		splice(@parts,0,1);
	}

	my $regex = $h{extended} ? qr/^[_a-z0-9]+(\-[_a-z0-9]*)*$/io : qr/^[a-z0-9]+(\-[a-z0-9]*)*$/io;
	foreach my $part (@parts) {
		## In compliance with RFC 952 & RFC 1034
		
		if ($part !~ $regex) {
			sw_log_warn("SLD '".$part."' include invalid character(s)");
			return 0;
		}

		## In compliance with RFC 1034 all parts must be 63 characters or less
		if (length($part) >= 64) {
			sw_log_warn("SLD '".$part."' too long");
			return 0;
		}
	}
	return 1;
}

sub _is_fqdn {
	my $value = shift;
	my %arg = @_;

	Encode::_utf8_on($value);
	my $ascii_value;

	## Added it for wildcard handle
	my $prefix = '';
	$prefix = $1 if ($value =~ s#^(\*\.)?##);

	## check trailing dots here, since 'domain_to_ascii' will remove them
	return 0 if ($value =~ m#\.+$#);

	eval {
		$ascii_value = domain_to_ascii($value);
	};

	return ($@) ? 0 : _is_ascii_fqdn($ascii_value, %arg);
}

##------------------------------- is_utf ------------------------------##
## Description:
##		Check format string is utf8
## Args:
##		string
## Return:
##		1 - if string in utf8, 0 if string in latin
sub _is_utf {
    my $string = shift;

    Encode::_utf8_off($string);
    my $off_string = $string;
    Encode::_utf8_on($string);

    return !($string eq $off_string);
}

sub is_utf {
	return _is_utf(@_);
}

##------------------------------- is_idn_domain ------------------------------##
## Description:
##		Check for IDN domain name
## Args:
##		domain name in ascii or utf format
## Return:
##		1 - if IDN domain name, 0 - else
sub is_idn_domain {
	my $value = shift;

	if ($value =~ m#^xn--# || _is_utf($value)) {
		return 1;
	}
	return 0;
}

##------------------------------- does_allow_3rd_level_tld ------------------##
## Description:
##		Check can domain be 3rd level (e.g. 'john.smith.name')
## Args:
##		domain name 
## Return:
##		1 - if domain can be 3rd level, 0 - else
sub does_allow_3rd_level_tld {
	my $domain = shift;

	my $re_obj = HSPC::MT::DM::RegEngine->new(); 
	my $shortname = $re_obj->extract_shortname_from_domain(
		domain => $domain
	);
	my $tld = $re_obj->extract_tld_from_domain(
		domain => $domain
	);
	my $is_3rd_tld = (scalar(grep {$_ eq $tld} &THIRD_LEVEL_TLDS))
		? 1 : 0;

	return ($shortname =~ m/\./ && !$is_3rd_tld) ?
		0 : 1;
}

1;
