## Copyright (C) 2007  SWsoft
## All rights reserved.
## Here should be placed MT template functions for OpenProvider plug-in
package HSPC::MT::Plugin::DM::OpenProvider;

use strict;

use HSPC::PluginToolkit::General qw(
	string
	encode_base64
	log
	log_debug
	log_warn
	purify_fromxml_data
);
use HSPC::PluginToolkit::DM qw(
	extract_shortname_from_domain
	extract_tld_from_domain
	get_contact
	set_contact_extdata
	get_domain_info
	check_phonecountry_exists
	get_domain_extdata
	set_domain_extdata
	is_phone_valid
	phone_as_hash
);
use HSPC::PluginToolkit::DM::Validate qw(
	check_data
	check_base_contacts
);
use HSPC::PluginToolkit::DM::Constants;

use HSPC::MT::Plugin::DM::OpenProvider::Constants;

use Encode;
use LWP::UserAgent;
use HTTP::Request;
use HTML::Entities;
use XML::Simple;
use Digest::MD5;
use Data::Dumper;

use constant MAP_CONTACT_TYPE => {
	owner => 'owner',
	admin => 'admin',
	technical => 'tech'
};

sub get_contact_types {
	my $class = shift;
	return [
		{type => 'owner', title => string('dm_plugin_openprovider_registrant')},
		{type => 'admin', title => string('dm_plugin_openprovider_admin')},
		{type => 'technical', title => string('dm_plugin_openprovider_tech')},
	];
}

sub can_check_register {
	my $class = shift; 
	my %h = (
		domains => [],
		@_
	);
	## plug-in can lookup any domains
	return {map {$_ => 1} @{$h{domains}}};
}

sub can_check_transfer {
	my $class = shift; 
	my %h = (
		domains => [],
		@_
	);
	## plug-in can lookup any domains
	return {map {$_ => 1} @{$h{domains}}};
}

sub register_domain {
	my $class = shift;
	my %h = (
		domain => undef,
		period => undef,
		nses => [],
		contacts => undef,
		domain_extdata => undef,
		
		config => undef,
		@_
	);
	my $config = $h{config};
	
	my $domain = $h{domain};
	my $period = $h{period};
	my $nses = $h{nses};
	my $contacts = $h{contacts};
	my $contacts_extdata = $h{contacts_extdata};
	my $domain_extdata = $h{domain_extdata};

	if (scalar @{$nses} < 2) {
		return {
			is_success => 0,
			message => string('dm_plugin_openprovider_nslimit_err'),
			domain_status => 'error'
		};
	}

	my $tld = extract_tld_from_domain($domain);
	my $shortname = extract_shortname_from_domain($domain);

	##
	## Prepare contacts
	##
	my $res = $class->_prepare_contacts(
		domain => $domain,
		contacts => $contacts,
		contacts_extdata => $contacts_extdata,
		config => $config
	);
	unless ($res->{is_success}) {
		return {
			is_success => 0,
			domain_status => 'error',
			message => $res->{message},
		};
	}
	my $stored_contacts_handles = $res->{stored_contacts_handles};

	##
	## Create NS group
	##
	my $ns_register = $class->_add_ns_group(
		login => $config->{login},
		password => $config->{password},
		remote_host => $config->{remote_host},
		nses => $nses
	);

	unless ($ns_register->{is_success}) {
		log_warn('Can\'t create NS group.');
		return {
			is_success => 0,
			message => $ns_register->{error},
			domain_status => 'error'
		};
	}

	##
	## Try to register domain
	##
	my $domainData = {
		"1_domain"		=> {name => $shortname, extension => $tld},
		"2_period"		=> $period,
		"3_ownerHandle"	=> $stored_contacts_handles->{owner},
		"4_adminHandle"	=> $stored_contacts_handles->{admin},
		"5_techHandle"	=> $stored_contacts_handles->{technical},
		"6_nsGroup"		=> $ns_register->{ns_group_info}->{ns_group_name},
	};
	$domainData->{"7_useDomicile"} = $domain_extdata->{use_op_handle} if $domain_extdata->{use_op_handle};

	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $class->_compose_request(
			username => $config->{login},
			password => $config->{password},
			data => {
				createDomainRequest => $domainData,
			},
		),
	);
	if (defined $res->{code} &&
		$res->{code} == &OPENPROVIDER_RETURN_OK) {
			if (ref($res->{data}) eq "HASH" && $res->{data}->{expirationDate}) {
				return {
					is_success => 1,
					domain_status => 'registered',
				};
			} else {
				return {
					is_success => 1,
					domain_status => 'registering',
				};
			}
	} else {
		log_warn('An error was occurred while domain registration.');
		log_warn('See the response from registrar.');
		return {
			is_success => 0,
			message => $res->{desc},
			domain_status => 'error',
		};
	}
}

sub transfer_domain {
	my $class = shift;
	my %h = (
		domain => undef,
		period => undef,
		nses => [],
		contacts => undef,
		domain_extdata => undef,
		
		config => undef,
		@_
	);
	my $config = $h{config};
	
	my $domain = $h{domain};
	my $period = $h{period};
	my $nses = $h{nses};
	my $contacts = $h{contacts};
	my $contacts_extdata = $h{contacts_extdata};
	my $domain_extdata = $h{domain_extdata};

	if (scalar @{$nses} < 2) {
		return {
			is_success => 0,
			message => string('dm_plugin_openprovider_nslimit_err'),
			domain_status => 'error'
		};
	}

	my $tld = extract_tld_from_domain($domain);
	my $shortname = extract_shortname_from_domain($domain);

	##
	## Prepare contacts
	##
	my $res = $class->_prepare_contacts(
		domain => $domain,
		contacts => $contacts,
		contacts_extdata => $contacts_extdata,
		config => $config
	);
	unless ($res->{is_success}) {
		return {
			is_success => 0,
			domain_status => 'error',
			message => $res->{message},
		};
	}
	my $stored_contacts_handles = $res->{stored_contacts_handles};

	##
	## general parameters for domain registration
	##
	my %data = (
		login => $config->{login},
		password => $config->{password},

		request_type => &SW_DM_OP_OPS->{OP_TRANS_DOMAIN},
		domain_shortname => $shortname,
		domain_tld => $tld,
		period => $period,
		contacts_handles => $stored_contacts_handles,
		domain_extdata => $domain_extdata,
	);

	##
	## Create NS group
	##
	my $ns_register = $class->_add_ns_group(
		login => $config->{login},
		password => $config->{password},
		remote_host => $config->{remote_host},
		nses => $nses
	);

	unless ($ns_register->{is_success}) {
		log_warn('Can\'t create NS group.');
		return {
			is_success => 0,
			message => $ns_register->{error},
			domain_status => 'error'
		};
	}

	my $ns_group_info = $ns_register->{ns_group_info};
	$data{ns_group} = $ns_group_info->{ns_group};
	$data{ns_group_name} = $ns_group_info->{ns_group_name};

	##
	## Try to register domain
	##
	my $xml = $class->_generate_xml(%data);
	my $response = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $xml
	);

	foreach my $result (@$response) {
		if (defined $result->{code} && 
			$result->{code} == &OPENPROVIDER_RETURN_OK
			|| $result->{code} == &OPENPROVIDER_DOMAIN_IN_ACCOUNT)
		{
				if ($result->{'expiration-date'}) {
					return {
						is_success => 1,
						domain_status => 'transferred',
					};
				} else {
					return {
						is_success => 1,
						domain_status => 'transferring',
					};
				}
		} elsif (defined $result->{code} && 
			$result->{code} == &SW_DM_OP_UNKOWN_DOMAIN_ERROR && 
			$result->{details}->{errormsg} =~ /^552/ ## Domain is already here 
		) {
			log_warn('An error was occurred while domain transferring.');
			return {
				is_success => 0,
				message => string('transfer_with_err',
					err_code => $result->{code},
					err_msg  => $result->{descr}
				),
				domain_status => 'error',
			};
		} else {
			log_warn('An unknonw error was occurred while domain transferring.');
			return {
				is_success => 0,
				message => $result->{descr},
				domain_status => 'error',
			};
		}
	}
}

sub can_transfer_domain {
	my $class = shift;
	return 1;
}


## Description:
##		The method updates nses for given domain.
## Usage:
##		$class->syncronize_domain_ns(domain => $domain, nses => $nses);
## Parameters:
##		> class
##		=> domain - domain name for which nses to be updated
##		=> nses - domain NSes info [{hostname => , ip => }, {...}, ...]
##		=> config - plugin configuration
## Return HASHREF:
##		is_success => [1|0]
##		message => localized error message if NSes synchronization completed with errors
sub synchronize_domain_ns {
	my $class = shift;
	my %h = (
		domain => undef,
		nses => undef,
		config => undef,
		@_
	);
	my $config = $h{config};

	my $result = {
		is_success => 0,
		ns_statuses => {map {$_->{hostname} => 'unsynchronized'} @{$h{nses}}},
		message => ''
	};

	my $domain = $h{domain};
	my $nses = $h{nses};
	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);
	my $contacts_extdata = {};
	my $contact_info;

	## For modify domain must get contacts handles - get it from OP
	my $contacts_handles = {};

	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $class->_compose_request(
			username => $config->{login},
			password => $config->{password},
			data => {
				retrieveDomainRequest => {
					"1_domain" => {
						name => $domain_shortname,
						extension => $domain_tld,
					},
					"2_withAdditionalData" => 1,
				},
			},
		),
	);
	if (!ref $res) {
		log_warn("Cannot get domain info!");
		$result->{message} = string('dm_plugin_openprovider_int_err');
		return $result;
	} else {
		if ($res->{code} == &OPENPROVIDER_RETURN_OK) {
			$contacts_handles->{owner} = $res->{data}->{ownerHandle};
			$contacts_handles->{admin} = $res->{data}->{adminHandle};
			$contacts_handles->{technical} = $res->{data}->{techHandle};
		} else {
			$result->{message} = string('dm_plugin_openprovider_err_code_and_msg',
				err_code => $res->[0]->{code},
				err_msg => $res->[0]->{descr}
			);
			return $result;
		}
	}
	## Add new NS group
	my $ns_register = $class->_add_ns_group(
		login => $config->{login},
		password => $config->{password},
		remote_host => $config->{remote_host},
		nses => $nses
	);
	unless ($ns_register->{is_success}) {
		log_warn('Can\'t create NS group.');
		$result->{message} = $ns_register->{error};
		return $result;
	}
	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $class->_compose_request(
			username => $config->{login},
			password => $config->{password},
			data => {
				modifyDomainRequest => {
					"1_domain" => {
						name => $domain_shortname,
						extension => $domain_tld,
					},
					"2_adminHandle" => $contacts_handles->{admin},
					"3_techHandle" => $contacts_handles->{technical},
					"4_nsGroup" => $ns_register->{ns_group_info}->{ns_group_name},
				},
			},
		),
	);
	if ($res->{code} != &OPENPROVIDER_RETURN_OK) {
		log_warn('Can\'t update domain.');
		log_warn('See response dump.');
		$result->{message} = $res->{desc};
		return $result;
	}

	$result->{is_success} = 1;
	## OpenProvider uses NS group, then
	## just mark all NSes as synchronized, if operation successful.
	$result->{ns_statuses} =
		{map {$_->{hostname} => 'synchronized'} @{$h{nses}}};

	return $result;
}

## Description:
##		The method updates contacts for given domain.
## Usage:
##		$class->update_contacts(domain => $domain, contacts => {admin=>...}, ... );
## Parameters:
##		> class
##		=> domain - domain name for which nses to be updated
##		=> contacts - contacts info {admin => HASHREF, owner => HASHREF, ... }
##		=> contacts_extdata - contacts extdata info {admin => REF, owner => REF, ... }
##		=> contacts_ids - IDs of DM contacts {admin => SCALAR, owner => SCALAR, ... }
##		=> config - plugin configuration
## Return HASHREF:
##		is_success => [1|0]
##		error_list => errors list in ARREYREF [{contact_type=>, message=>}]
sub update_contacts {
	my $class = shift;
	my %h = (
		domain => undef,
		contacts => undef,
		contacts_extdata => undef,
		contacts_ids => undef,
		config => undef,
		@_
	);
	my $config = $h{config};

	my $domain = $h{domain};
	my $contacts = $h{contacts};
	my $contacts_extdata = $h{contacts_extdata};
	my $contacts_ids = $h{contacts_ids};

	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);
	my $domain_info = get_domain_info(domain => $domain);
	my $domain_nses = $domain_info->{nses};

	my $error_list = [];
	my $stored_contacts_handles = { map {$_ => undef} keys %$contacts };

	foreach my $type_id (keys %$contacts) {
		my $contact = $contacts->{$type_id};

		$contacts_extdata->{$type_id} = {}
			unless $contacts_extdata->{$type_id};
		my $contact_extdata = $contacts_extdata->{$type_id};

		$contact_extdata->{contacts_handles} = {}
			unless $contact_extdata->{contacts_handles};
		my $contacts_handles = $contact_extdata->{contacts_handles};

		$contact_extdata = {} unless $contact_extdata;

		##
		## Try to modify this contact in registrar side
		##

		my $handle = $contacts_handles->{$type_id};

		$stored_contacts_handles->{$type_id} = $handle;

		my $request_type = $handle
			?
			(
				$contact->{is_corporate}
					? &SW_DM_OP_OPS->{OP_MODIFY_CONTACT}
					: &SW_DM_OP_OPS->{OP_MODIFY_INDIVIDUAL}
			)
			:
			(
				$contact->{is_corporate}
					? &SW_DM_OP_OPS->{OP_NEW_CONTACT}
					: &SW_DM_OP_OPS->{OP_NEW_INDIVIDUAL}
			);
		my $xml = $class->_generate_xml(
			request_type => $request_type,
			contact_type => &MAP_CONTACT_TYPE->{$type_id},
			%{$contact},
			%{$contact_extdata},
			handle => $handle,

			login => $config->{login},
			password => $config->{password}
		);
		my $response = $class->_send_request(
			remote_host => $config->{remote_host},
			data => $xml
		);

		unless (defined($response)) {
			my $message = string('dm_plugin_openprovider_connect_op_failed');
			log_warn($message);
			push @$error_list, {
				contact_type => $type_id,
				message => $message
			};
			next;
		}

		foreach my $result (@$response) {
			if(defined $result->{code}
				&& $result->{code} == &OPENPROVIDER_RETURN_OK)
			{
				$stored_contacts_handles->{$type_id} = $result->{handle}
					unless $handle;
			}
			elsif (defined $result->{code}
				&& $result->{code} == &OPENPROVIDER_COMPANY_NAME_EXISTS)
			{
				## Company name already exist
				log(
					'Company name exists. Try to add contact: '
					.&MAP_CONTACT_TYPE->{$type_id}
				);
				my $xml = $class->_generate_xml(
					request_type => &SW_DM_OP_OPS->{OP_NEW_CONTACT},
					contact_type => &MAP_CONTACT_TYPE->{$type_id},
					%{$contact},
					%{$contact_extdata},

					login => $config->{login},
					password => $config->{password}
				);
				$response = $class->_send_request(
					remote_host => $config->{remote_host},
					data => $xml
				);
		
				if(defined $response->[0]->{code}
					&& $response->[0]->{code} == &OPENPROVIDER_RETURN_OK)
				{
					$stored_contacts_handles->{$type_id} = $result->{handle};
				} else {
					my $message = 'Can\'t create contact.';
					log_warn($message);
					push @$error_list, {
						contact_type => $type_id,
						message => $message
					};
					next;
				}
			} else {
				my $message = 'Unknown response status.';
				log_warn($message);
				push @$error_list, {
					contact_type => $type_id,
					message => $message
				};
				next;
			}
		}
	}

	foreach my $type (keys %$contacts) {
		##
		## Save contact handles in contact extended data
		##
		my $contact_info = get_contact(
			domain => $domain,
			contact_type => $type
		);
		my $contact_extdata = $contact_info->{contact_extdata};
		$contact_extdata = {} unless $contact_extdata;
		$contact_extdata->{contacts_handles} = {}
			unless $contact_extdata->{contacts_handles};
		$contact_extdata->{contacts_handles}->{$type}
			= $stored_contacts_handles->{$type};
		set_contact_extdata(
			domain => $domain,
			contact_type => $type,
			contact_extdata => $contact_extdata
		);
	}

	##
	## Try to prepare NSes group set
	##
	my $ns_register = $class->_add_ns_group(
		login => $config->{login},
		password => $config->{password},
		remote_host => $config->{remote_host},
		nses => $domain_nses
	);
    
	unless ($ns_register->{is_success}) {
		log_warn('Can\'t create NS group.');
		push @$error_list, {
			contact_type => '',
			message => $ns_register->{error}
		};
		return {
			is_success => 0,
			error_list => $error_list
		};
	}
	my $ns_group_info = $ns_register->{ns_group_info};

	##
	## Try to update domain in registrar side
	##
	my %data = (
		login => $config->{login},
		password => $config->{password},

		request_type => &SW_DM_OP_OPS->{OP_MODIFY_DOMAIN},
		domain_shortname => $domain_shortname,
		domain_tld => $domain_tld,
		contacts_handles => $stored_contacts_handles,

		ns_group => $ns_group_info->{ns_group},
		ns_group_name => $ns_group_info->{ns_group_name},
	);

	my $xml = $class->_generate_xml(%data);
	my $response = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $xml
	);
	foreach (@$response) {
		if ($_->{code} != &OPENPROVIDER_RETURN_OK) {
			log_warn('Can\'t update domain.');
			log_warn('See response dump below: ');
			log_warn(Dumper($_));

			push @$error_list, {
				contact_type => '',
				message => $_->{descr}
			};
		}
	}

	return {
		is_success => scalar(@$error_list) ? 0 : 1,
		error_list => scalar(@$error_list) ? $error_list : undef
	};
}

sub can_reglock{
	return 0;
}

sub validate_data {
	my $class = shift;
	my %h = (
		domain => undef,
		action => undef,
		contacts => undef,
		contacts_extdata => undef,
		domain_extdata => undef,
		@_
	);
	my $domain = $h{domain};
	my $action = $h{action};
	my $contacts = $h{contacts};
	my $contacts_extdata = $h{contacts_extdata};
	my $domain_extdata = $h{domain_extdata};

	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);
	my $error_list = [];

	## Check base conatcs
	my $error_list_cbc = check_base_contacts(
		contacts => $h{contacts},
		mandatory_fields => [
			'org_name', 'fname','lname','email',
			'address','zip','city','country'
		],
		check_ascii => 1
	);
	push @$error_list, @$error_list_cbc;

	foreach my $type_id (keys %$contacts) {
		my $contact = $contacts->{$type_id};
		my $contact_extdata = $contacts_extdata->{$type_id};
		$contact_extdata = {} unless $contact_extdata;

		## Check Phone --------------------------------------------------------
		if (!is_phone_valid(
			phone => $contact->{phone},
			mandatory_area_code => 1))
		{
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'phone',
				message => string("dm_plugin_openprovider_phone_err")
			};
		}

		my $phone_cc = phone_as_hash($contact->{phone})->{country_code};
		if ($phone_cc && !check_phonecountry_exists($phone_cc) ) {
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'phone',
				message => string(
					'dm_plugin_openprovider_warn_country_code',
					field => string('dm_plugin_openprovider_phone_country_code')
				)
			};
		}

		## Check Mobile Phone -------------------------------------------------
		if (!is_phone_valid(
			phone => $contact_extdata->{mobile},
			mandatory_area_code => 1,
			optional => 1))
		{
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'mobile',
				message => string("dm_plugin_openprovider_mobile_err")
			};
		}

		my $mphone_cc = phone_as_hash($contact_extdata->{mobile})->{country_code};
		if ($mphone_cc && !check_phonecountry_exists($mphone_cc) ) {
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'mobile',
				message => string(
					'dm_plugin_openprovider_warn_country_code',
					field => string('dm_plugin_openprovider_mobile_country_code')
				)
			};
		}

		## Check Fax ----------------------------------------------------------
		if (!is_phone_valid(
			phone => $contact->{fax},
			mandatory_area_code => 1,
			optional => 1))
		{
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'fax',
				message => string("dm_plugin_openprovider_fax_err")
			};
		}

		my $fax_cc = phone_as_hash($contact->{fax})->{country_code};
		if ($fax_cc && !check_phonecountry_exists($fax_cc) ) {
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'fax',
				message => string(
					'dm_plugin_openprovider_warn_country_code',
					field => string('dm_plugin_openprovider_fax_country_code')
				)
			};
		}

		if ($contact->{is_corporate}) {
			if ($domain_tld eq 'it') {
				if ($contact_extdata->{coc_no} !~ /^(\d+)$/) {
					push @$error_list, {
						form => 'contact',
						contact_type => $type_id,
						field => 'coc_no',
						message => string('dm_plugin_openprovider_it_coc_no')
					};
				}
				if ($contact_extdata->{coc_region} !~ /^([a-zA-Z\-\'\ ]+)$/) {
					push @$error_list, {
						form => 'contact',
						contact_type => $type_id,
						field => 'coc_region',
						message => string('dm_plugin_openprovider_it_coc_region')
					};
				}
			}

			## Special for NL countries
			if ($contact->{country} eq 'NL') {
				## Vat-number does not mandatory
				if  ($contact_extdata->{tax_ex_number} &&
					$contact_extdata->{tax_ex_number} !~ /^NL\d{9}B\d{2}$/)
				{
					push @$error_list, {
						form => 'contact',
						contact_type => $type_id,
						field => 'tax_ex_number',
						message => string('dm_plugin_openprovider_nl_vatnumber_err')
					};
				}
				if ($contact->{zip} !~ m/^\d{4}\ [A-Z]{2}$/) {
					push @$error_list, {
						form => 'contact',
						contact_type => $type_id,
						field => 'zip',
						message => string('dm_plugin_openprovider_nl_zip_err')
					};
				}
			}
		}

		if ($domain_tld eq 'it') {
			unless ($contact_extdata->{fiscal_no} =~ /^(\d+)$/)
			{
				push @$error_list, {
					form => 'contact',
					contact_type => $type_id,
					field => 'fiscal_no',
					message => string(
						'dm_plugin_openprovider_warn_fiscal_no',
						field => string('dm_plugin_openprovider_fiscal_no')
					)
				};
			}
			unless ($contact_extdata->{birth_city} =~ /^([a-zA-Z\-\'\.\(\)\/\\\ ]+)$/)
			{
				push @$error_list, {
					form => 'contact',
					contact_type => $type_id,
					field => 'birth_city',
					message => string('dm_plugin_openprovider_it_birth_city')
				};
			}
		}
		if (!$contact_extdata->{house_num} =~ /^([\d\-\ ]+)$/) {
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'house_num',
				message => string(
					'dm_plugin_openprovider_warn_fiscal_no',
					field => string('dm_plugin_openprovider_addr_num')
				)
			};
		}
		if (!$contact_extdata->{gender}) {
			push @$error_list, {
				form => 'contact',
				contact_type => $type_id,
				field => 'gender',
				message => string(
					'dm_plugin_openprovider_field_missed',
					field => string('dm_plugin_openprovider_gender')
				)
			};
		}
	}

	if (lc($action) eq 'transfer'
		&& scalar(grep {$_ eq $domain_tld} &OP_TRANS_TLDS))
	{
		if (!$domain_extdata->{auth_code}) {
			push @$error_list, {
				form => 'domain_extdata',
				field => 'auth_code',
				message => string(
					'dm_plugin_openprovider_field_missed',
					field => string('dm_plugin_openprovider_transfer_password')
				)
			};
		}
	}

	## .de domains should either has German admin contact or domicicle on
	if(lc($action) eq 'register' && $domain =~ /\.de$/i) {
		my $admin_country = $contacts->{admin}->{country};
		if (uc($admin_country) ne 'DE' && !$domain_extdata->{use_op_handle}) {
			push @$error_list, {
				form => 'domain_extdata',
				field => 'use_op_handle',
				message => string('dm_plugin_openprovider_use_op_handle_error')
			};
		}
	}

	return {
		is_valid => scalar(@$error_list) ? 0 : 1,
		error_list => scalar(@$error_list) ? $error_list : undef
	};
}

##########################################################
###########  PRIVATE METHODS  ############################
##########################################################

## Description:
##		This method generates xml forms for OpenProvider.
## Usage:
##		$self->_generate_xml(request_type, login, passwd, etc... );
## Parameters:
##		> self
##		=>> request_type - type of action
##		=>> login - login to account.
##		=>> etc...
## Return:
##		$xml
sub _generate_xml {
	my $class = shift;
	my %h = (
		## Request specific info
		request_type => undef,
		login => undef,
		password => undef,

		## Domain specific info
		domain_shortname => undef,
		domain_tld => undef,
		domain_extdata => undef,
		period => undef,
		ns_group_name => undef,
		ns_group => undef,

		## Contact specific info
		contact_type => undef,
		handle => undef,

		is_company => undef,
		org_name => undef,
		fname => undef,
		lname => undef,
		email => undef,
		address => undef,
		zip => undef,
		city => undef,
		state => undef,
		country => undef,
		phone => undef,
		fax => undef,

		## Domains for WHOIS
		list_domains => undef,
		
		## Contact group specific info
		## {admin => handle, owner => handle, ...}
		contacts_handles => undef,
		@_
	);
	my $request_type = $h{request_type};
	unless ($request_type) {
		log_warn("Null request_type: $request_type");
		return undef;
	}
	
	my $login = $h{login}; 
	my $password = $h{password};

	my $handle = $h{handle};
	my $ns_group = $h{ns_group};
	my $auth_code = (ref $h{domain_extdata} eq 'HASH') ? $h{domain_extdata}->{auth_code} : undef;
	## Forms the header of xml form
	my $xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	$xml .= "<openXML><credentials><username>$login</username><password>$password</password></credentials>";
  
	if($request_type eq &SW_DM_OP_OPS->{OP_VIEW_CONTACT}){
		$xml .= '<overview><viewContact version="1.0">' .
			'<handle>' . $handle . '</handle>' .
			'</viewContact></overview>';
	}
	## if we create xml for deleting, modifying or creating info about customer
	## (company, contact or individual)
	elsif ($request_type =~ /Contact|Handle|Company|Individual/) {
		$xml .= '<customerRegistration>';
		$xml .= '<'.$request_type.' version="1.0">';

		## Deleting info
		if ($request_type =~ /delete/) {
			$xml .= $class->_xml_delete_customer(%h);
		} else {
			## the rest
			if ($request_type =~ /Company/) {
				$xml .= $class->_xml_company(%h);
			}

			## Add admin person for new company or add/modify contact/individual
			if ($request_type =~ /Contact|Individual/ ||
				$request_type eq 'newCompany') {
				$xml .= $class->_xml_customer(%h);
			}
		}
		$xml .= '</'.$request_type.'>';
		$xml .= '</customerRegistration>';

	} elsif ($request_type =~ /Domain/) {
		## xml form for deleting, modifying, trading, transferring, renewing 
		## or creating the domain
		my $version = '1.1';
		## Version for statusDomain and renewDomain should be 1.0
		$version = '1.0' if ($request_type =~ /status|renew|delete|info|whois/);
		$xml .= '<domainRegistration>';
		$xml .= "<$request_type version=\"$version\">";
		if($request_type =~ /info/){
			$xml .= '<type>short</type>';
		}
		elsif($request_type =~ /whois/){
			$xml .= '<type>multiple</type>';
			my $domains_to_check_list=@h{list_domains};
			foreach my $domain_in_list (@$domains_to_check_list){
				my $domain_shortname_to_check = extract_shortname_from_domain($domain_in_list);
				my $domain_tld_to_check = extract_tld_from_domain($domain_in_list);
				$xml .= '<domain>';
				$xml .= '<domainname>'.$domain_shortname_to_check.'</domainname>';
				$xml .= '<extension>'.$domain_tld_to_check.'</extension>';
				$xml .= '</domain>';
					
			}
		}
		if ($request_type !~ /whois/){
			$xml .= '<domain>';
			$xml .= '<domainname>'.$h{domain_shortname}.'</domainname>';
			$xml .= '<extension>'.$h{domain_tld}.'</extension>';

			## for new, renew and transfer domain commands
			if ($request_type =~ /new|transfer/) {
				$xml .= '<period>'.$h{period}.'</period>';

				if ($request_type =~ /transfer/ && $auth_code) {
					$xml .= '<auth-code>'.$auth_code.'</auth-code>';
				}
			}
			$xml .= '</domain>';
		}
		if ($request_type !~ /delete|status|info|whois/) {
			$xml .= $class->_xml_make_domain(%h);
	    }
		$xml .= '</'.$request_type.'>';
		$xml .= '</domainRegistration>';

	} elsif ($request_type =~ /Nameservergroup/) {
		## xml form for deleting, modifying or creating the nameservers
		$xml .= $class->_xml_make_ns_group(%h);
	} else {
		log("Unknown request_type: $request_type");
		return undef;
	}
	$xml .= '</openXML>';

	return $xml;
}

## Description:
##		This method sends request to the OpenProvider, and returns
##		response code and message.
## Usage:
##		$class->_send_request(remote_host => 'http://...', data => $data);
## Parameters:
##		>   self
##		=> remote_host - registrar api-gate URL
##		=> data - reference to xml string 
## Return:
##		[ {code => int, descr => 'string', handle => 'ID'},..] -  array of 
##		records with OpenProvider server response: code, its description and 
##		handle of registered contact in case of its registration. 
sub _send_request {
	my $class = shift;
	my %h = (
		remote_host => undef,
		data => undef,
		@_
	);
	unless ($h{data}) {
		my $err = '[OpenProvider] _send_request(): data should be defined!';
		log_warn($err);
		die($err);
	}
	log('Try to send request to remote host: '.$h{remote_host});

	my $data = $h{data};
	my $log_data = "\n".$data;

	$log_data =~ s/(>)(<)/$1\n$2/g;
	log(
		'next XML'
		." will be sent to registrar: ".$data
	);
	log(
		'next XML without "\n" symbols after "<" symbols'
		." will be sent to registrar: ".$log_data
	);

	my $url = $h{remote_host};

    ## Create a user agent object
	my $ua = LWP::UserAgent->new();
	$ua->agent("HSPC Plugin For OpenProvider" . $ua->agent);

	## Create a request
	my $req = HTTP::Request->new(POST => $url);
	$req->content($data);

	my $res = $ua->request($req);
	my $output = $res->content();
	log("OpenProvider gateway has returned the output: $output");

	my $result;
	if ($res->is_success()) {
		$output = join(' ', split(/\n/, $output));
		if ($output =~ /(\<\?xml.*)/) {
			$output = $1;
		} else {
			$output = undef;
		}
		log('Data sent successfully.');
		log('XML Recieve:'.$output);

		## Parse the server response
		$result = $class->_parse_xml( 
			xml => $output
		);
	} else {
		log_warn(
			'Error while sending data: '
			.$res->code().', '.$res->message());
		$result = undef;
	}

	return $result;
}

## Description:
##		This method parses xml response from OpenProvider.
## Usage:
##		$class->_parse_xml($xml);
## Parameters:
##		> self
##		=> xml - response of server OpenProvider
## Return:
##		[
##			{code => , descr => , handle => , 'expiration-date' =>},
##		]
sub _parse_xml {
	my $class = shift;
	my %h = (
		xml => undef,
		@_
	);
	my $xml = $h{xml};
	my $result;

	unless ($xml) {
		$result = {
			code => &OPENPROVIDER_ERROR_CODE,
			desc => "Input XML is empty",
		};
		log("PARSE result:". Dumper($result));
		return $result;
	}
	my $res = purify_fromxml_data(XMLin($xml, suppressempty => ''));

	log("PARSE:". Dumper($res));
	$result = {
		code => $res->{reply}->{code},
		desc => $res->{reply}->{desc},
		data => $res->{reply}->{data},
	};
	log("PARSE result:". Dumper($result));

	return $result;
}

## Description:
##		This method generates part of xml form for OpenProvider.
## Usage:
##		$class->_xml_company(org_name, address, etc);
## Parameters:
##		> self
##		=> org_name - company name if company is deleted 
##		=> address - address of customer
##		=> etc...
## Return:
##		$xml
sub _xml_company {
	my $class = shift;
	my %h = (
		is_company => undef,
		org_name => undef,
		gender => undef,
		fname => undef,
		lname => undef,
		email => undef,
		address => undef,
		zip => undef,
		city => undef,
		state => undef,
		country => undef,
		phone => undef,
		fax => undef,

		## extended data
		company_type => undef,
		tax_ex_number => undef,
		coc_no => undef,
		coc_region => undef,
		coc_reg_date => undef,
		house_num => undef,
		house_suff => undef,
		@_
	);
	
	my $xml;
	my $state = $h{state} || $h{country}; 
	my ($tel1, $tel2, $tel3) = $class->_format_phone($h{phone});
	my ($fax1, $fax2, $fax3) = $class->_format_phone($h{fax});

	foreach (keys %h) {
		Encode::_utf8_on($h{$_});
		HTML::Entities::encode_numeric($h{$_});
	}

	$xml = '<company>';
	$xml .= '<name>'.$h{org_name}.'</name>';
	$xml .= '<type>'.$h{company_type}.'</type>' if $h{company_type};

	$xml .= '<address-street>'.$h{address}.'</address-street>';
	$xml .= '<address-number>'.$h{house_num}.'</address-number>' ;
	$xml .= '<address-suffix>'.$h{house_suff}.'</address-suffix>' 
		if $h{house_suff};
	$xml .= '<zipcode>'.$h{zip}.'</zipcode>';
	$xml .= '<city>'.$h{city}.'</city>';
	$xml .= '<state>'.$state.'</state>' 
		if ($state);
	$xml .= '<country>'.$h{country}.'</country>';
	$xml .= '<tel1>'.$tel1.'</tel1>';
	$xml .= '<tel2>'.$tel2.'</tel2>';
	$xml .= '<tel3>'.$tel3.'</tel3>';
	if ($fax1) {
		$xml .= '<fax1>'.$fax1.'</fax1>';
	}
	if ($fax2) {
		$xml .= '<fax2>'.$fax2.'</fax2>';
	}
	if ($fax3) {
		$xml .= '<fax3>'.$fax3.'</fax3>';
	}
	if ($h{tax_ex_number}) {
		$xml .= '<vat-number>'.$h{tax_ex_number}.'</vat-number>' 
	} else {
		log_warn("[OpenProvider]: No tax exemption number!");
	}

	$xml .= '<coc-number>'.$h{coc_no}.'</coc-number>' if $h{coc_no};
	$xml .= '<coc-region>'.$h{coc_region}.'</coc-region>' if $h{coc_region};
	if ($h{coc_reg_date}) {
		my $date = $h{coc_reg_date};
		$date =~ s!^(\d+)-(\d+)-(\d+)(.+)!$3-$2-$1!;
		$xml .= '<coc-date>'.$date.'</coc-date>';
	}

	$xml .= '<main-office>'. $h{city} .'</main-office>';
	$xml .= '</company>';
	return $xml;
}

## Description:
##		This method generates part of xml form for OpenProvider.
## Usage:
##		$self->_xml_customer(request_type, contact_type, company_name, etc... );
## Parameters:
##		> self
##		=>> request_type - type of action
##		=>> contact_type - type of contact (admin, tech).
##		=>> etc...
## Return:
##		$xml
sub _xml_customer {
	my $class = shift;
	my %h = (
		request_type => undef,
		contact_type => undef,
		handle => undef,

		is_company => undef,
		org_name => undef,
		gender => undef,
		fname => undef,
		lname => undef,
		email => undef,
		address => undef,
		zip => undef,
		city => undef,
		state => undef,
		country => undef,
		phone => undef,
		fax => undef,

		## extended data
		company_type => undef,
		house_num => undef,
		house_suff => undef,
		mobile => undef,
		fiscal_no => undef,
		birth_date => undef,
		birth_city => undef,
		@_
	);
	
	my $xml;
	my $state = $h{state} || $h{country}; 
	my ($tel1, $tel2, $tel3) = $class->_format_phone( $h{phone} );
	my ($fax1, $fax2, $fax3) = $class->_format_phone( $h{fax} );
	my ($mobile1, $mobile2, $mobile3) = $class->_format_phone( $h{mobile} );

	foreach (keys %h) {
		Encode::_utf8_on($h{$_});
		HTML::Entities::encode_numeric($h{$_});
	}

	$xml = '<customer>';
	if ($h{request_type} eq &SW_DM_OP_OPS->{OP_NEW_CONTACT}) {
		$xml .= '<companyname>'.$h{org_name}.'</companyname>';
		$xml .= '<contact-type>'.$h{contact_type}.'</contact-type>';
	}

	## if we modify individual or contact person
	if ($h{request_type} =~ /modify/) {
		$xml .= '<handle>'.$h{handle}.'</handle>';
		$xml .= '<firstname>'.$h{fname}.'</firstname>' if $h{fname};
	} else {
		Encode::_utf8_on($h{fname});
		if ($h{fname} =~ /^(\w)/) {
			$h{initials} = $1 . '.';
		} else {
			log_warn("[OpenProvider]: Invalid first name!".$h{fname});
		};
		$xml .= '<initials>'.$h{initials}.'</initials>';
		$xml .= '<firstname>'.$h{fname}.'</firstname>' if $h{fname};
		$xml .= '<insertion>'.$h{insertion}.'</insertion>' if $h{insertion};
		$xml .= '<lastname>'.$h{lname}.'</lastname>';
		$xml .= '<gender>'.$h{gender}.'</gender>';
	}
	if ($h{request_type} =~ /Individual/) {
		$xml .= '<address-street>'.$h{address}.'</address-street>';
		$xml .= '<address-number>'. $h{house_num} .'</address-number>';
		$xml .= '<address-suffix>'.$h{house_suff} .'</address-suffix>' if $h{house_suff};
		$xml .= '<zipcode>'.$h{zip}.'</zipcode>';
		$xml .= '<city>'.$h{city}.'</city>';
		$xml .= '<state>'.$state.'</state>' if $state;
		$xml .= '<country>'.$h{country}.'</country>';
	}
	$xml .= '<tel1>'.$tel1.'</tel1>';
	$xml .= '<tel2>'.$tel2.'</tel2>';
	$xml .= '<tel3>'.$tel3.'</tel3>';
	if ($mobile1 && $mobile2 && $mobile3) {
		$xml .= '<mobile1>'.$mobile1.'</mobile1>';
		$xml .= '<mobile2>'.$mobile2.'</mobile2>';
		$xml .= '<mobile3>'.$mobile3.'</mobile3>';
	} 
	if ($h{request_type} eq &SW_DM_OP_OPS->{OP_NEW_INDIVIDUAL}) {
		if ($fax1) {
			$xml .= '<fax1>'.$fax1.'</fax1>';
		}
		if ($fax2) {
			$xml .= '<fax2>'.$fax2.'</fax2>';
		}
		if ($fax3) {
			$xml .= '<fax3>'.$fax3.'</fax3>';
		}
	}
	$xml .= '<email>'.$h{email}.'</email>';
	$xml .= '<fiscnumber>'.$h{fiscal_no}.'</fiscnumber>' if $h{fiscal_no};
	if ($h{birth_date}) {
		my $date = $h{birth_date};
		$date =~ s!^(\d+)-(\d+)-(\d+)(.+)!$3-$2-$1!;
		$xml .= '<birthdate>'.$date.'</birthdate>';
	}
	
	$xml .= '<birthcity>'.$h{birth_city}.'</birthcity>' if $h{birth_city};

	$xml .= '</customer>';
	
	return $xml;
}

## Description:
##		This method creates new ns group in OpenProvider.
## Usage:
##		$class->_add_ns_group(nses => $nses);
## Parameters:
##		> $class
##		=>> ns_info - ref to list of nses info
## Return:
##		hash reference - {
##			is_success => 1 | 0,
##			error => error_string_if_any,
##			ns_info => {
##				ns_group_name => string,
##				## ns_group may be omitted if we don't need to register ns group
##				ns_group_info => [{
##					ns_ip => 'any_ip_address',
##					ns_name => 'any_domain_name'
##				},..,
##				]
##			}
##		}
sub _add_ns_group {
	my $class = shift;
	my %h = (
		login => undef,
		password => undef,
		remote_host => undef,
		nses => [],
		@_
	);
	my $nses = $h{nses};
	my @ns_names_ips_array;
	my $ns_group = [];
	my $cnt = 1;

	foreach my $ns (@$nses){
		push @ns_names_ips_array, ($ns->{hostname} => $ns->{ip});
		push @$ns_group, {
			name => $ns->{hostname},
			ip => $ns->{ip}
		};
		++$cnt;

		## up to 4 DNS servers allowed
		if ($cnt > &OPENPROVIDER_NS_NUMBER) {
			log(
				"[OpenProvider]: Number of nameservers >"
				. &OPENPROVIDER_NS_NUMBER
			);
			last;
		};
	}

	my $ns_group_name;
	@ns_names_ips_array = sort @ns_names_ips_array;
	foreach (@ns_names_ips_array) {
		$ns_group_name .= $_ .'_';
	} 
	## To register nameserver group we need its name. So it is generated by 
	## Digest::MD5::md5_hex in order to keep it unique and its length less 
	## than 50 symbols
	$ns_group_name = 'ns-group-'.Digest::MD5::md5_hex($ns_group_name);

	my $data = {createNsGroupRequest => {}};
	$data->{createNsGroupRequest}->{"1_nsGroup"} = $ns_group_name;
	$data->{createNsGroupRequest}->{"2_nameServers"} = $ns_group;

	my $res = $class->_send_request(
		remote_host => $h{remote_host},
		data => $class->_compose_request(
			username	=> $h{login},
			password	=> $h{password},
			data		=> $data,
		),
	);
	my $ok = 0;
	if (defined $res->{code} &&
		($res->{code} == &OPENPROVIDER_RETURN_OK ||
		$res->{code} == &SW_DM_OP_ADDNS_EXISTS)
	) {
		$ok = 1;
	} else {
		log_warn("An error was occurred while NS creation");
		log_warn("See response from registrar.");
	}
	my $ns_info = {
		ns_group => $ns_group,
		ns_group_name => $ns_group_name,
	};
	unless ($ok) {
		return {
			is_success => 0,
			error => string('dm_plugin_openprovider_dnsadd_err'),
			ns_group_info => $ns_info,
		};
	}
	return {
		is_success => 1,
		ns_group_info => $ns_info,
	};
}

## Description:
##      Register own  NS servers group in OpenProvider system
## Usage:
##		$self->_add_ns
## Parameters:
##		> self
##      => ns_group_name - nameserver group name
##      => ns_group - array of records with NS names and ips
## Return:
##		1 | 0 [1 - is successfully done, 0 - an error was occurred]
sub _add_ns {
	my $class = shift;
	my %h = (
		login => undef,
		password => undef,
		remote_host => undef,

		ns_group => undef,
		ns_group_name => undef,
		@_
	);
	unless ($h{ns_group}) {
		log_warn(
			"[OpenProvider] _add_ns(): nameserver group "
			."should be specified!"
		);
		return 0;
	}
	
	my $request_type = &SW_DM_OP_OPS->{OP_NEW_NS_GROUP};

	my %data = (
		request_type => $request_type,
		ns_group_name => $h{ns_group_name},
		ns_group => $h{ns_group},
	);

	my $xml = $class->_generate_xml(
		login => $h{login},
		password => $h{password},
		%data,
	);
	my $resp = $class->_send_request(
		remote_host => $h{remote_host},
		data => $xml
	);

	foreach my $result (@$resp) {
		if (defined $result->{code}	&& 
			($result->{code} == &OPENPROVIDER_RETURN_OK || 
			$result->{code} == &SW_DM_OP_ADDNS_EXISTS)
		) {
			return 1;
		} else {
			log_warn("An error was occurred while NS creation");
			log_warn("See response from registrar:");
			log_warn(Dumper($resp));
			return 0;
		}
	}
}

## Description:
##		This method generates part of xml form for OpenProvider.
## Usage:
##		$class->_xml_make_ns_group(request_type, ns_group_name, ns_group);
## Parameters:
##		> class
##		=>> request_type - type of request to server
##		=>> ns_group - array of records about nses
##		=>> ns_group_name - name of that group
## Return:
##		$xml
sub _xml_make_ns_group {
	my $class = shift;
	my %h = (
		request_type => undef,
		ns_group_name => undef,
		ns_group => undef,
		@_
	);

	my $xml;
	my $ns_group = $h{ns_group};
	my $ns_group_name = $h{ns_group_name};

	$xml = '<nameserverRegistration>';
	$xml .= '<'.$h{request_type}.' version="1.0">';
	if ($h{request_type} !~ /delete/) {
		$xml .= '<nameserver>';
		$xml .= '<ns-group>'.$ns_group_name.'</ns-group>' 
			if ($ns_group_name);
		my $i = 0;
		foreach my $ns (@$ns_group){
			$xml .= '<ns>';
			$xml .= '<ns-name>'.$ns->{ns_name}.'</ns-name>';
			$xml .= '<ns-ip>'.$ns->{ns_ip}.'</ns-ip>';
			$xml .= '</ns>';
			$i++;
			if ($i >= &OPENPROVIDER_NS_NUMBER) {
				log(
					"[OpenProvider]: Number of nameservers >"
					. &OPENPROVIDER_NS_NUMBER
				);
				last;
			};
		}
		$xml .= '</nameserver>';
	} else {
		## deleting namesserver group
		$xml .= '<nameserver>';
		$xml .= '<ns-group>'.$ns_group_name.'</ns-group>'; 
		$xml .= '</nameserver>';		
	}
		$xml .= '</'.$h{request_type}.'>';
	$xml .= '</nameserverRegistration>';
	
	return $xml;
}

## Description:
##		This method generates part of xml form for OpenProvider.
## Usage:
##		$class->_xml_make_domain(ns_group, ns_group_name);
## Parameters:
##		> class
##		=>> request_type - for renew, register, transfer domain
##		=>> account_no - account ID
##		=>> extension - tld of domain 
##		=>> ns_group - array of records about nses
##		=>> ns_group_name - name of that group
## Return:
##		$xml
sub _xml_make_domain {
	my $class = shift;
	my %h = (
		request_type => undef,
		domain_shortname => undef,
		domain_tld => undef,
		domain_extdata => undef,
		ns_group_name => undef,
		ns_group => undef,
		contacts_handles => undef,
		@_
	);

	my $xml = '';
	my $request_type = $h{request_type};
	my $domain_shortname = $h{domain_shortname},
	my $domain_tld = $h{domain_tld},
	my $domain_extdata = $h{domain_extdata} || {};
	my $contacts_handles = $h{contacts_handles};
	my $ns_group = $h{ns_group};
	
	my $update_contacts = ref $contacts_handles eq 'HASH' ? 1 : 0;

	my $update_owner_contact = (
		$update_contacts
		&& $request_type ne &SW_DM_OP_OPS->{OP_MODIFY_DOMAIN}
	);
	my $update_admin_contact = (
		$update_contacts
		&& $request_type !~ /renew/
	);
	my $update_tech_contact = (
		$update_contacts
		&& $request_type !~ /renew/
	);
	my $update_nses = (
		$ns_group
		&& $request_type !~ /renew/
	);

	##
	## TODO: how to get actual usedomicile value for renewal???
	##
	my $usedomicile;
	if ($domain_extdata->{use_op_handle}) {
		$usedomicile = $domain_extdata->{use_op_handle};
	} else {
		$usedomicile = 0;
	}
	
	$xml .= "<handle usedomicile=\"$usedomicile\">"
		if $update_contacts;
	$xml .= '<owner>'.$contacts_handles->{owner}.'</owner>'
		if $update_owner_contact;
	$xml .= '<admin>'.$contacts_handles->{admin}.'</admin>'
		if $update_admin_contact;
	$xml .= '<tech>'.$contacts_handles->{technical}.'</tech>'
		if $update_tech_contact;
	$xml .= '</handle>'
		if $update_contacts;
	if ($update_nses) {
		$xml .= '<nameserver>';
		$xml .= '<ns-group>'.$h{ns_group_name}.'</ns-group>' if $h{ns_group_name};
		my $i = 0;
		foreach my $ns (@$ns_group){
			$xml .= '<ns>';
			$xml .= '<ns-name>'.$ns->{ns_name}.'</ns-name>';
			$xml .= '<ns-ip>'.$ns->{ns_ip}.'</ns-ip>';
			$xml .= '</ns>';
			$i++;
			if ($i >= &OPENPROVIDER_NS_NUMBER) {
				log(
					"[OpenProvider]: Number of nameservers >"
					. &OPENPROVIDER_NS_NUMBER
				);
				last;
			};
		}
		$xml .= '</nameserver>';
	}
	return $xml;
}

sub _prepare_contacts {
	my $class = shift;
	my %h = (
		domain => undef,
		contacts => undef,
		contacts_extdata => undef,
		config => undef,
		@_
	);
	my $domain = $h{domain};
	my $config = $h{config};
	my $contacts = $h{contacts};
	my $contacts_extdata = $h{contacts_extdata};
	my $contact_types = ['admin', 'owner', 'technical'];

	my $contact;
	my $contact_extdata;
	my $handle;
	my $request_type;

	if ($contacts->{owner}->{is_corporate}) {
		##
		## Admin contact should be the same as owner for corporate registrant
		##
		$contacts->{admin} = $contacts->{owner};
		$contacts_extdata->{admin} = $contacts_extdata->{owner};
	}

	my $stored_contacts_handles = { map {$_ => undef} @$contact_types };
	foreach my $contact_type (@$contact_types) {
		$contact = $contacts->{$contact_type};

		$contacts_extdata->{$contact_type} = {}
			unless $contacts_extdata->{$contact_type};
		my $contact_extdata = $contacts_extdata->{$contact_type};

		$contact_extdata->{contacts_handles} = {}
			unless $contact_extdata->{contacts_handles};
		my $contacts_handles = $contact_extdata->{contacts_handles};

		$handle = $contacts_handles->{$contact_type};
		unless ($handle) {
			##
			## Create new contact
			##
			my $res = $class->_send_request(
				remote_host => $config->{remote_host},
				data => $class->_compose_request(
					username	=> $config->{login},
					password	=> $config->{password},
					data		=> $class->_prepare_contact_data(
						contact 		=> $contact,
						contact_type	=> $contact_type,
						contact_extdata => $contact_extdata,
					),
				),
			);

			unless (defined($res)) {
				my $message = string('dm_plugin_openprovider_connect_op_failed', host => $config->{remote_host});
				log_warn($message);
				return {
					is_success => 0,
					message => $message,
				};
			}

			if(defined $res->{code}
				&& $res->{code} == &OPENPROVIDER_RETURN_OK)
			{
				$stored_contacts_handles->{$contact_type} = $res->{data}->{handle};
				$contacts_handles->{$contact_type} = $res->{data}->{handle};
			}
			elsif (defined $res->{code}
				&& $res->{code} == &OPENPROVIDER_COMPANY_NAME_EXISTS)
			{
				## Company name already exist
				log(
					'Company name exists. Try to add contact: '
					.&MAP_CONTACT_TYPE->{$contact_type}
				);
				$res = $class->_send_request(
					remote_host => $config->{remote_host},
					data => $class->_compose_request(
						username	=> $config->{login},
						password	=> $config->{password},
						data		=> $class->_prepare_contact_data(
							contact 		=> $contact,
							contact_type	=> $contact_type,
							contact_extdata => $contact_extdata,
							without_company	=> 1,
						),
					),
				);
				if(defined $res->{code}
					&& $res->{code} == &OPENPROVIDER_RETURN_OK)
				{
					$stored_contacts_handles->{$contact_type} = $res->{data}->{handle};
				} else {
					log_warn('Can\'t create contact.');
					return {
						is_success => 0,
						message => 'Can\'t create contact. ' . $res->{desc},
					};
				}
			} else {
				log_warn('Unknown response status.');
				return {
					is_success => 0,
					message => 'Unknown response status or invalid data: ' . $res->{desc},
				};
			}
		} else {
			$stored_contacts_handles->{$contact_type} = $handle;
		}
	}

	foreach my $type (@$contact_types) {
		##
		## Save contact handles in contact extended data
		##
		my $contact_info = get_contact(
			domain => $domain,
			contact_type => $type
		);
		$contact_extdata = $contact_info->{contact_extdata};
		$contact_extdata = {} unless $contact_extdata;
		$contact_extdata->{contacts_handles} = {}
			unless $contact_extdata->{contacts_handles};
		$contact_extdata->{contacts_handles}->{$type}
			= $stored_contacts_handles->{$type};
		set_contact_extdata(
			domain => $domain,
			contact_type => $type,
			contact_extdata => $contact_extdata
		);
	}

	return {
		is_success => 1,
		stored_contacts_handles => $stored_contacts_handles,
	}
}

sub _prepare_contact_data
{
	my $class = shift;
	my %h = (
		contact			=> undef,
		contact_type	=> undef,
		contact_extdata	=> undef,
		without_company	=> 0,
		@_,
	);
	my $contact			= $h{contact};
	my $contact_type	= $h{contact_type};
	my $contact_extdata	= $h{contact_extdata};

	my ($phoneConutryCode, $phoneAreaCode, $phoneNumber) = $class->_format_phone($contact->{phone});
	my ($faxConutryCode, $faxAreaCode, $faxNumber) = $class->_format_phone($contact->{fax});
	my $data = {createCustomerRequest => {}};

	$data->{createCustomerRequest}->{"1_companyName"} = $contact->{org_name} if $contact->{org_name} && ! $h{without_company};
	$data->{createCustomerRequest}->{"2_vat"} = $contact_extdata->{tax_ex_number};
	$data->{createCustomerRequest}->{"3_name"} = {
		initials	=> substr($contact->{fname}, 0, 1).substr($contact->{lname}, 0, 1),
		firstName	=> $contact->{fname},
		lastName	=> $contact->{lname},
	};
	$data->{createCustomerRequest}->{"4_gender"} = $contact_extdata->{gender};
	$data->{createCustomerRequest}->{"5_address"} = {
		street	=> $contact->{address},
		number	=> $contact_extdata->{house_num},
		suffix	=> $contact_extdata->{house_suff},
		zipcode	=> $contact->{zip},
		city	=> $contact->{city},
		state	=> $contact->{state},
		country	=> $contact->{country},
	};
	$data->{createCustomerRequest}->{"6_phone"} = {
		countryCode		 => $phoneConutryCode,
		areaCode		 => $phoneAreaCode,
		subscriberNumber => $phoneNumber,
	};
	$data->{createCustomerRequest}->{"7_fax"} = {
		countryCode		 => $faxConutryCode,
		areaCode		 => $faxAreaCode,
		subscriberNumber => $faxNumber,
	} if $faxNumber;
	$data->{createCustomerRequest}->{"8_email"} = $contact->{email};
	$data->{createCustomerRequest}->{"9_additionalData"} = {
		companyRegistrationCity => $contact->{city},
	} if $contact_type eq "admin" && $contact->{is_corporate};

	return $data;
}

## Description:
##		The method domain registration renewal.
## Usage:
##		$self->renew_domain(domain => "qwerqwer.cc", period => 2);
## Parameters:
##		>    self
##		=> domain - domain name to be renewed
##		=> period - renewal period
##		=> config - plugin configuration
## Return:
##		result hash
sub renew_domain{
	my $class = shift;
	my %h = (
		domain => undef,
		period => undef,
		config => undef,
		@_
	);
	my $config = $h{config};

	my $domain = $h{domain};
	my $period = $h{period};

	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);
	
	my $contact_info = get_contact(
		domain => $domain,
		contact_type => 'owner'
	);
	my $contact_extdata = $contact_info->{contact_extdata}
		? $contact_info->{contact_extdata}
		: undef;
	my $contacts_handles = {
		owner => $contact_extdata->{contacts_handles}->{owner}
	};

	my %data = (
		login => $config->{login},
		password => $config->{password},

		request_type => &SW_DM_OP_OPS->{OP_RENEW_DOMAIN},
		domain_shortname => $domain_shortname,
		domain_tld => $domain_tld,
		period => $period,
		contacts_handles => $contacts_handles
	);

	my $xml = $class->_generate_xml(%data);
	my $resp = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $xml
	);

	foreach my $result (@$resp) {
		if(defined $result->{code} && $result->{code} == &OPENPROVIDER_RETURN_OK) {
			return {
				is_success => 1,
				domain_status => 'renewed'
			};
		} else  {
			return {
				is_success => 0,
				message => $result->{descr},
				domain_status => 'error'
			};
		}
	}

	log_warn('Unknown error.');
	return {
		is_success => 0,
		message => 'Unknown error',
		domain_status => 'error'
	};
}

## Description: 
## 		This method gets info from OpenProvider about given domain
## Usage:
##		$class->get_domain_status(domain => $domain_id, action => 'register', ...);
## Parameters:
##		> class
##		=> domain - domain name
##		=> action - domain action 'register|transfer'
##		=> config - plugin configuration
## Return HASHREF:
##		is_success => [1 | 0]
##		message => localized message string
##		domain_status => 'completed|cancelled|inprogress'
sub get_domain_status {
	my $class = shift;
	my %h = (
		domain => undef,
		action => undef,
		config => undef,
		@_
	);
	my $config = $h{config};

	my $domain = $h{domain};
	my $action = $h{action};
	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);

	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $class->_compose_request(
			username => $config->{login},
			password => $config->{password},
			data => {
				retrieveDomainRequest => {
					"1_domain" => {
						name => $domain_shortname,
						extension => $domain_tld,
					},
					"2_withAdditionalData" => 1,
				},
			},
		),
	);

	## handle error situations
	if ($res && $res->{code} == &OPENPROVIDER_RETURN_OK) {
		my $status = $res->{data}->{status};
		## domain is ok
		## ACT means 'active'
		if ($status eq 'ACT') {
			return { 
				is_success => 1, 
				domain_status => 'registered'
			};
		## PEN - 'pending'
		## REQ - 'being processed'
		} elsif ($status eq 'PEN' || $status eq 'REQ') {
			return { 
				is_success => 1,
				message => string("dm_plugin_openprovider_regtr_inprogress"),
				domain_status => $action eq 'register'
					? 'registering'
					: ($action eq 'transfer' ? 'transferring' : undef)
			};
		## FAI - 'failed'
		} elsif ($status eq 'FAI') {
			return { 
				is_success => 1,
				message => string("dm_plugin_openprovider_registrar_ret_fail_status"),
				domain_status => 'error'
			};
		}
	} elsif ($res && $res->{code} == &OPENPROVIDER_DOMAIN_NOT_KNOWN) {
		return { 
			is_success => 1, 
			domain_status => 'error'
		};
	}

	my $error_msg = string('dm_plugin_openprovider_err_code_and_msg', 
		err_code => &OPENPROVIDER_ERROR_CODE,
		err_msg => string('dm_plugin_openprovider_net_error')
	);
	return { 
		is_success => 0,
		message => $error_msg
	};
} 

## Description: 
## 		This method gets details from OpenProvider about given domain
## Usage:
##		$class->get_domain_details(domain => $domain_id;
## Parameters:
##		> class
##		=> domain - domain name
##		=> config - plugin configuration
## Return HASHREF:
##		is_success => [1 | 0]
##		message => localized message string
##		expiration_date  => 'YYYY-MM-DD	HH:MM:SS'

sub get_domain_details {
	my $class = shift;
	my %h = (
		domain => undef,
		config => undef,
		@_
	);
	my $config = $h{config};
	my $domain = $h{domain};

	my $result = {
		is_success => 0,
		message => string('dm_plugin_openprovider_int_err'),
	};

	## Divide domain name in name and extension	
	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);
	my %data = (
		login => $config->{login},
		password => $config->{password},

		request_type => &SW_DM_OP_OPS->{OP_DOMAIN_INFO},
		domain_shortname => $domain_shortname,
		domain_tld => $domain_tld
	);
	
	my $xml = $class->_generate_xml(%data);
	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $xml
	);

	## handle response
	if (ref $res) {
		if ($res->[0]->{code} == &OPENPROVIDER_RETURN_OK) {
			my $status = $res->[0]->{status};
			## domain is ok
			$result->{is_success} = 1;
			$result->{message} = '';
			$result->{expiration_date} = $res->[0]->{details}->{exp_date};
		} else {
			$result->{message} = string('dm_plugin_openprovider_err_code_and_msg',
				err_code => $res->[0]->{code},
				err_msg => $res->[0]->{descr}
			);
		}
	}

	return $result;
}

sub can_terminate_domain {
	my $class = shift;
	return 1;
}

## Description:
##		      The method domain registration termination.
## Usage:
##		      $self->terminate_domain(domain => "qwerqwer.cc", period => 2);
## Parameters:
##		      >    self
##		      => domain - domain name to be renewed
##		      => config - plugin configuration
## Return:
##		      result hash
sub terminate_domain {
	my $class = shift;
	my %h = (
		domain => undef,
		config => undef,
		@_
	);
	my $config = $h{config};
	my $domain = $h{domain};

	my $result = {
		is_success => 0,
		message => string('dm_plugin_openprovider_int_err'),
	};

	my $domain_shortname = extract_shortname_from_domain($domain);
	my $domain_tld = extract_tld_from_domain($domain);

	my %data = (
		login => $config->{login},
		password => $config->{password},

		request_type => &SW_DM_OP_OPS->{OP_TERMINATE_DOMAIN},
		domain_shortname => $domain_shortname,
		domain_tld => $domain_tld,
	);
	my $xml = $class->_generate_xml(%data);
	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $xml
	);

	if (ref $res) {
		if ($res->[0]->{code} == &OPENPROVIDER_RETURN_OK) {
			$result->{is_success} = 1;
			$result->{domain_status} = 'terminated';
		} else {
			log_warn('[OpenProvider->terminate_domain()] Error while send terminate request: $domain');
			$result->{message} = string('dm_plugin_openprovider_err_code_and_msg',
				err_code => $res->[0]->{code},
				err_msg => $res->[0]->{descr}
			);
		}
	}

	return $result;
}

## Description: 
## 		This method checks in OpenProvider aviability of Domains
## Parameters:
##		> class
##		=> domains lit of domains
##		=> config - plugin configuration
## Return HASHREF:
##		domain => [3 | 2 | 1 | 0]

sub check_register {
	my $class = shift;
	my %h = (
		config => undef,
		domains => undef,
		@_
	);
	my $domains = $h{domains};
	my $config = $h{config};

	## Set default error for all domains
	my %result = ();
	@result{ @{$domains} } = map {2} @{$domains};

	my $res = $class->_send_request(
		remote_host => $config->{remote_host},
		data => $class->_compose_request(
			username => $config->{login},
			password => $config->{password},
			data => {
				checkDomainRequest => {
					domains => [
						map {
							name => extract_shortname_from_domain($_),
							extension => extract_tld_from_domain($_),
						}, @{$domains}
					],
					withWhois => 1,
				}
			}
		),
	);

	if ($res->{code} == &OPENPROVIDER_RETURN_OK) {
		my $items = $res->{data}->{array}->{item}; $items = [$items] unless ref($items) eq "ARRAY";

		## Process each domain from status data
		foreach my $item (@{$items}) {
			my ($domain, $status) = ($item->{domain}, $item->{status});
			if($status eq 'free') {
				log_debug('[OpenProvider->check_register()]'
					."Domain $domain it's free for registration"
				);
 				$result{$domain} = 1;
			} else {
				log_debug(
					'[OpenProvider->check_register()]'
					."Domain $domain it's not free for registration"
				);
				$result{$domain} = 0;
			}
		}
	} else {
		## We didn't received info, let's return error on lookup
		log_debug(
			'[OpenProvider->check_register()]'
			."Some error occurred while checking domains in OpenProvider"
		);
	}

	return \%result;
}

## Description: 
##		Create a list of formatted phone string and phone extension 
##			('+12','345','67890') from hspc formatted phone string
## Usage:
##		$class->_format_phone($phone)
## Parameters:
##		>   self
##		>>  phone string ('12|34|5678|90')
## Return:
##		<<  list formatted phone string ('+12','345','67890') or undef
sub _format_phone {
	my $class = shift;
	my $phone_source = shift;

	my ($phone_parts, $country_code, $area_code, $number);

	$phone_source =~ s![^0-9\|]!!g;
	$phone_parts = [split(/\|/, $phone_source)];

	$country_code = $phone_parts->[0];
	$area_code = $phone_parts->[1];
	$number = $phone_parts->[2].$phone_parts->[3];
	$country_code =~ s!^0*!!;
	$country_code = "+$country_code" if ($country_code);
	$area_code =~ s!^0*!!;

	return ($country_code, $area_code, $number);
}

sub get_domains_contacts {
	my $class = shift;
	my %h = (
		domains => undef,
		config => undef,
		@_
	);

	log_debug('OpenProvider->get_domains_contacts()');

	my $domains = $h{domains};
	my $config = $h{config};

	my $result;

	foreach my $domain (keys(%$domains)){
		my $plugin_data = $domains->{$domain}->{plugin_data};
		my $hspc_data = $domains->{$domain}->{hspc_data};
		my $contact = $class->get_domain_contacts(
			domain => $domain,
			plugin_data => $plugin_data,
			hspc_data => $hspc_data,
			config => $config
		);
		$result->{$domain} = $contact;
	}

	return $result;
}

sub add_domains_extdata {
	my $class = shift;
	my %h = (
		domains_extdata => undef,
		config => undef,
		@_
	);

	log_debug('OpenProvider->add_domains_extdata()');

	my $domains_extdata = $h{domains_extdata};
	my $config = $h{config};

	foreach my $domain (keys(%$domains_extdata)){
		my $domain_extdata = $domains_extdata->{$domain};
		$class->add_domain_extdata(
			domain => $domain,
			domain_extdata => $domain_extdata,
		);
	}

	return undef;
}

sub get_domain_contacts {
	my $class = shift;
	my %h = (
		domain => undef,
		plugin_data => undef,
		hspc_data => undef,
		config => undef,
		@_
	);

	my $domain = $h{domain};
	my $config = $h{config};
	my $plugin_data = $h{plugin_data};
	my $hspc_data = $h{hspc_data};
	my $account_details = $hspc_data->{account_details};

	my $handles = $plugin_data;

	my $contacts = {};
	my $contacts_extdata = {};
	my $domain_extdata = {};
	$domain_extdata->{domain_login} = $hspc_data->{domain_login}
		if(defined($hspc_data->{domain_login}));
	$domain_extdata->{domain_password} = $hspc_data->{domain_password}
		if(defined($hspc_data->{domain_password}));
	$domain_extdata->{order_id} = $hspc_data->{domain_order}
		if(defined($hspc_data->{domain_order}));

	my $is_success = 0;
	foreach my $contact_type (keys(%{&MAP_CONTACT_TYPE})){
		my $registrar_contact_type = &MAP_CONTACT_TYPE->{$contact_type};

		my $contact = {};
		my $contact_extdata = {};
		my $handle = $handles->{$registrar_contact_type};
		if($handle){
			my %data = (
				request_type => &SW_DM_OP_OPS->{OP_VIEW_CONTACT},
				handle => $handle
			);
			my $xml = $class->_generate_xml(
				login => $config->{login},
				password => $config->{password},
				%data,
			);
			my $responses = $class->_send_request(
				remote_host => $config->{remote_host},
				data => $xml
			);
			foreach my $response (@$responses) {
				if(defined($response->{code}) &&
					($response->{code} == &OPENPROVIDER_RETURN_OK)){

					$is_success ||= 1;

					my $details = $response->{details};

					$contact->{org_name} = $account_details->{org_name};
					$contact->{is_corporate} = $account_details->{is_corporate};

					$contact->{fname} = $details->{firstname};
					$contact->{lname} = $details->{lastname};
					$contact->{country} = $details->{country};
					$contact->{state} = $details->{state};
					$contact->{city} = $details->{city};
					$contact->{zip} = $details->{zipcode};
					$contact->{address} = $details->{address_street};
					$contact->{email} = $details->{email};
					$contact->{phone} = $details->{tel1} . '|' .
						$details->{tel2} . '|' . $details->{tel3} . '|';
					$contact->{phone} =~ s/^\+//;
					$contact->{fax} = $details->{fax1} . '|' .
						$details->{fax2} . '|' . $details->{fax3} . '|';
					$contact->{fax} =~ s/^\+//;

					$contact_extdata->{gender} = $details->{sex};
					$contact_extdata->{house_num} = $details->{address_number};
					$contact_extdata->{house_suff} = $details->{address_suffix};
					$contact_extdata->{mobile} = $details->{mobile1} . '|' .
						$details->{mobile2} . '|' . $details->{mobile3} . '|';
					$contact_extdata->{mobile} =~ s/^\+//;
					$contact_extdata->{fiscal_no} = $details->{fiscnr};
					$contact_extdata->{birth_date} = $details->{birthdate};
					$contact_extdata->{birth_city} = $details->{birthcity};
					$contact_extdata->{initials} = $details->{initials};
					$contact_extdata->{insertion} = $details->{insertion};

					## it can't be tested but added because it don't break anything.
##					$contact_extdata->{contact_type} = $details->{contact_type};
					$contact_extdata->{company_type} = $details->{company_type};
					$contact_extdata->{coc_no} = $details->{coc_no};
					$contact_extdata->{coc_region} = $details->{coc_region};
					$contact_extdata->{coc_reg_date} = $details->{coc_reg_date};
					$contact_extdata->{tax_ex_number} = $details->{tax_ex_number};

					while(my ($ctype, $reg_ctype) = each(%{&MAP_CONTACT_TYPE})){
						$contact_extdata->{contacts_handles}->{$ctype} = $handles->{$reg_ctype};
					}
				} else {
					log_warn("Error occurred during contact information retrieving from registrar for handle '$handle' and type '$contact_type' for domain '$domain'.");
				}
			}
		} else {
			log_warn("Handle not found for contact type '$contact_type' for domain '$domain'.");
		}

		$contacts->{$contact_type} = $contact;
		$contacts_extdata->{$contact_type} = $contact_extdata;
	}

	my $result = {
		is_success => $is_success,
		message => (
			$is_success ?
			'Some contacts was successfully received.' :
			'Error occurred during contact information retrieving from registrar.'
		),
		contacts => $contacts,
		contacts_extdata => $contacts_extdata,
		domain_extdata => $domain_extdata
	};

	return $result;
}

sub add_domain_extdata {
	my $class = shift;
	my %h = (
		domain => undef,
		domain_extdata => undef,
		@_
	);

	my $domain = $h{domain};
	my $domain_extdata = $h{domain_extdata};

	if(!$domain){
		log_warn('Domain Extended Data not updated, Domain undefined!');
		return undef;
	}

	return undef if(!$domain_extdata);

	my $extdata = get_domain_extdata(domain => $domain);
	my $old_domain_extdata = !defined $extdata->{domain_extdata} ?
		{} : $extdata->{domain_extdata};
	%$old_domain_extdata = (%$old_domain_extdata, %$domain_extdata);

	set_domain_extdata(
		domain => $domain,
		domain_extdata => $old_domain_extdata,
	);

	return undef;
}

sub _compose_request($$) {
	my $class = shift;
	my %h = (
		username => undef,
		password => undef,
		data => undef,
		@_
	);

	return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" . $class->_compose_xml({
		openXML => {
			"1_credentials" => {
				"username" => $h{username},
				"password" => $h{password},
			},
			%{$h{data}},
		}
	});
}

sub _compose_xml($$) {
	my $class = shift;
	my $data = shift;
	my $xml = "";

	if (ref($data) eq "HASH") {
		foreach my $key (sort keys %{$data}) {
			my $tag_name = $key;
			$tag_name =~ s/^\d+_//;
			$xml .= "<$tag_name>".$class->_compose_xml($data->{$key})."</$tag_name>";
		}
	} elsif (ref($data) eq "ARRAY") {
		$xml .= "<array>";
		foreach my $item (@{$data}) {
			$xml .= "<item>".$class->_compose_xml($item)."</item>";
		}
		$xml .= "</array>";
	} else {
		$xml .= $data;
	}
	return $xml;
}

1;
