package HSPC::MT::Plugin::PM::BT_Norma19;

use strict;

use HSPC::PluginToolkit::General qw(throw_exception);

sub provided_payment_method_types {
	my $class = shift;

	return [
		{type => 'BT_Norma19', title_id => 'bapm_norma19_type_norma19', image_id => ''},
	];
}

sub title_id {
	return 'bapm_norma19_ba_uc';
}

sub get_public_data {
	my $class = shift;
	my %h = (
		data             => undef,
		@_
	);

	my %public_data = (
		holder_name    => $h{data}->{holder_name},
	);
	return \%public_data;
}

sub get_secure_data {
	my $class = shift;
	my %h = (
		data             => undef,
		keep_secure_code => undef,
		@_
	);

	my %secure_data = (
		account_number     => $h{data}->{account_number},
		bank_agency_number => $h{data}->{bank_agency_number},
		bank_branch_number => $h{data}->{bank_branch_number},
		stored_control_dig => $h{data}->{stored_control_dig},
		cif                => $h{data}->{cif},

	);

	return \%secure_data;
}

sub get_public_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return substr($h{secure_data}->{account_number},-4);
}

sub get_secure_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return $h{secure_data}->{bank_agency_number} .
		$h{secure_data}->{bank_branch_number}    .
		$h{secure_data}->{stored_control_dig}    .
		$h{secure_data}->{cif}                   .
		$h{secure_data}->{account_number};
}

sub validate {
	my $class = shift;
	my %h = (
		expire_date => undef,
		secure_data => undef,
		public_data => undef,
		type        => undef,
		@_
	);

	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_norma19_holder_name_not_defined'}
	) unless $h{public_data}->{holder_name};

	return unless $h{secure_data};

	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_norma19_agency_number_required'}
	) unless $h{secure_data}->{bank_agency_number};

	if (
		$h{secure_data}->{bank_agency_number} == 0 ||
		$h{secure_data}->{bank_agency_number} =~ /\D/ ||
		length($h{secure_data}->{bank_agency_number}) > 4
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_agency_number_is_wrong',
				string_args => {
					number => $h{secure_data}->{bank_agency_number}
				}
			}
		);
	}

	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_norma19_branch_number_required'}
	) unless $h{secure_data}->{bank_branch_number};

	if (
		$h{secure_data}->{bank_branch_number} =~ /\D/ ||
		length($h{secure_data}->{bank_branch_number}) > 4
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_branch_number_is_wrong',
				string_args => {
					number => $h{secure_data}->{bank_branch_number}
				}
			}
		);
	}

	throw_exception(
		type    => 'parameter',
		message => {string_id => 'bapm_norma19_account_number_required'}
	) unless $h{secure_data}->{account_number};

	if (
		$h{secure_data}->{account_number} =~ /\D/ ||
		length($h{secure_data}->{account_number}) > 10
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_account_number_is_wrong',
				string_args => {
					acc => $h{secure_data}->{account_number}
				}
			}
		);
	}

	my $control_dig = $class->get_checksumm(
		agency_number  => $h{secure_data}->{bank_agency_number},
		branch_number  => $h{secure_data}->{bank_branch_number},
		account_number => $h{secure_data}->{account_number},
	);

	if ( $control_dig != $h{secure_data}->{stored_control_dig} ) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'bapm_norma19_control_digits_mismatch',
			}
		);
	}

	return undef;
}

sub get_paymethod_type {
	return 'BT_Norma19';
}

sub get_checksumm {
	my $class = shift;
	my %h = (
		agency_number  => undef,
		branch_number  => undef,
		account_number => undef,
		@_
	);

	## these weights are specified by spanish bank
	my $weights = [6,3,7,9,10,5,8,4,2,1];
	my $agency_num = sprintf("%04s%04s", $h{agency_number}, $h{branch_number});
	my @agency_num_array = reverse unpack("A1" x length($agency_num),$agency_num); 

	my $account_num = sprintf("%010s", $h{account_number});
	my @account_num_array = reverse unpack("A1" x length($account_num),$account_num);
	
	my $sum_agency = 0;
	my $sum_acc = 0;
	for( my $i=0; $i < scalar(@{$weights}); $i++ ) {
		$sum_agency += @agency_num_array[$i] * $weights->[$i];
		if ($i < scalar(@account_num_array)) {
			$sum_acc += @account_num_array[$i] * $weights->[$i];
		}
	}

	## modulus 11 is used to obtain each control digit
	my $control_digit_agency = 11 - ($sum_agency % 11);
	if ($control_digit_agency == 10) {
		$control_digit_agency = 1;
	} 
	elsif ($control_digit_agency == 11) {
		$control_digit_agency = 0;
	}

	my $control_digit_acc = 11 - ($sum_acc % 11);
	if ($control_digit_acc == 10) {
		$control_digit_acc = 1;
	} 
	elsif ($control_digit_acc == 11) {
		$control_digit_acc = 0;
	}

	my $control_sum = sprintf("%1d%1d", $control_digit_agency, $control_digit_acc);

	return $control_sum;
}

1;
