package HSPC::MT::Plugin::PM::OP_CCard;

use strict;

use HSPC::PluginToolkit::General qw(datetime_gmt_now compare_dates throw_exception);

sub provided_payment_method_types {
	my $class = shift;

	return [
		{type => 'A', title_id => 'ccpmp_type_a', image_id => 'amex_card'},
		{type => 'B', title_id => 'ccpmp_type_b', image_id => 'cb_card'},
		{type => 'C', title_id => 'ccpmp_type_c', image_id => 'diners_card'},
		{type => 'D', title_id => 'ccpmp_type_d', image_id => 'discover_card'},
		{type => 'E', title_id => 'ccpmp_type_e', image_id => 'enroute_card'},
		{type => 'J', title_id => 'ccpmp_type_j', image_id => 'jcb_card'},
		{type => 'L', title_id => 'ccpmp_type_l', image_id => 'visa_e_card'},
		{type => 'M', title_id => 'ccpmp_type_m', image_id => 'mastercard_card'},
		{type => 'O', title_id => 'ccpmp_type_o', image_id => 'optima_card'},
		{type => 'S', title_id => 'ccpmp_type_s', image_id => 'solo_card'},
		{type => 'T', title_id => 'ccpmp_type_t', image_id => 'visa_delta_card'},
		{type => 'V', title_id => 'ccpmp_type_v', image_id => 'visa_card'},
		{type => 'W', title_id => 'ccpmp_type_w', image_id => 'maestro_card'},
	];
}

sub title_id {
	return 'ccpmp_credit_card_uc';
}

sub get_public_data {
	my $class = shift;
	my %h = (
		data => undef,
		@_
	);

	my %public_data = (
		expire_date => $h{data}->{expire_date},
		type        => $h{data}->{type},
		valid_from  => $h{data}->{valid_from},
	);

	return \%public_data;
} ## (mandatory) return reference to an array of public attributes of a payment method type;

sub get_secure_data {
	my $class = shift;
	my %h = (
		data             => undef,
		keep_secure_code => undef,
		@_
	);

	my %secure_data = (
		card_number    => $h{data}->{card_number},
		holder_name    => $h{data}->{holder_name},
		holder_name2   => $h{data}->{holder_name2},
		cvv            => $h{keep_secure_code} ? $h{data}->{cvv} : '',
		issue_number   => $h{data}->{issue_number},
		address1       => $h{data}->{address1},
		address2       => $h{data}->{address2},
		city           => $h{data}->{city},
		state_uscanada => $h{data}->{state_uscanada},
		state          => $h{data}->{state},
		country        => $h{data}->{country},
		zip            => $h{data}->{zip},
		phone_src      => $h{data}->{phone_src},
		phone          => $h{data}->{phone},
	);

	return \%secure_data;
} ## (mandatory) return reference to an array of attributes of a payment method type that will be encrypted on save operation;

sub validate {
	my $class = shift;
	my %h = (
		expire_date => undef,
		secure_data => undef,
		public_data => undef,
		type        => undef,
		@_
	);

	my $expire_date = $h{expire_date};

	my $type = $h{type} ||
		throw_exception(
			type        => 'error',
			log_message => 'The type must be provided'
		);

	if ($expire_date) {
		my $current_date = datetime_gmt_now();
		if ( &compare_dates($current_date, $expire_date) > 0 ) {
			throw_exception(
				type    => 'parameter',
				message => {string_id => 'cc_expired'}
			);
		}
	}
	return undef unless $h{secure_data};

	throw_exception(
		type    => 'parameter',
		message => {
			string_id   => 'wrong_cc_number_format',
			string_args => { num => $h{secure_data}->{card_number} }
		}
	) unless $h{secure_data}->{card_number};

	throw_exception(
		type    => 'parameter',
		message => {
			string_id   => 'pm_no_holder_name_uc',
		}
	) unless ($h{secure_data}->{holder_name}) ;##&& $h{secure_data}->{holder_name2});

	my $result = $class->__get_ccard_properties(
		card_number => $h{secure_data}->{card_number}
	);

	unless ( $result->{type} ) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'cannot_determine_cc_type_explained',
			}
		);
	}

	unless ( $result->{type} && $result->{luhn} ) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'wrong_cc_number_format',
				string_args => { num => $h{secure_data}->{card_number} }
			}
		);
	}

## comment for fix 75885
##	unless ( $result->{type} eq $type ) {
##		throw_exception(
##			type    => 'parameter',
##			message => {
##				string_id => 'cc_type_not_match',
##			}
##		);
##	}

	## check CVV
	if ($h{secure_data}->{cvv}
		&& $h{secure_data}->{cvv} !~ /^\d{2,4}$/
	) {
		throw_exception(
			type    => 'parameter',
			message => {
				string_id => 'cvv_should_be_number',
			}
		);
	}

	return undef;
} ## (optional) method that validates values of all payment method's attributes for correctness, values are provided to the method via parameter;

sub get_public_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return substr($h{secure_data}->{card_number},-4);
} ## (mandatory) returns public number representation of the payment method;

sub get_secure_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return $h{secure_data}->{card_number};
} ## (mandatory) returns full set of identification characters (secure number) of the payment method. For a credit card it should return full credit card number, for a bank account it should return concatination of all account number fields (including bank code and branch identification number);

sub get_expiration_date {
	my $class = shift;
	my %h = (
		secure_data => undef,
		@_
	);

	return $h{public_data}->{expire_date};
} ## (optional) if exists, should return expiration date of the payment method in format of MM/YY;

sub get_paymethod_type {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return $h{public_data}->{type} if $h{public_data}->{type};

	return unless $h{secure_data};

	my $result = $class->__get_ccard_properties(
		card_number => $h{secure_data}->{card_number}
	);

	return $result->{type};
} ## (mandatory) returns type of the payment method object (Ex. Visa, Switch, ....). This method gets payment method information hash as parameter. This is required only for plug-ins that supported few payment method type in one code and type of payment method can be defined only by using some sort of analysing of payment method properties. 

sub __get_ccard_properties {
	my $class = shift;
	my %h = (
		card_number => undef,
		@_
	);

	my $card_no = $h{card_number} ||
		throw_exception(
			type        => 'error',
			log_message => 'The card_no parameter must be defined'
		);

	my $result = {
		type   => undef,
		length => undef,
		luhn   => 0,
	};

	my $card_prefix = substr($card_no, 0, 6);

	my $card_matrix = $class->__get_ccard_type_rule();

	foreach my $rule (@$card_matrix) {
		if ( $card_prefix >= $rule->{lo} && $card_prefix <= $rule->{hi} ) {
			next if ( $rule->{num_len} && length($card_no) != $rule->{num_len} );

			$result->{type} = $rule->{type};
			$result->{length} = $rule->{num_len};
			last;
		}
	}

	my $no_length = length($card_no);

	## Checking of CC number based on LUHN method
	if ( $result->{type} ) {
		if ( $result->{type} ne 'W' ) {
			my $checksum = 0;
    	    # Add even digits in even length strings
        	# or odd digits in odd length strings.
	        for (my $pos = 1 - ($no_length % 2); $pos < $no_length; $pos += 2) {
    	        $checksum += substr($card_no, $pos, 1);
	        }

	        # Analyze odd digits in even length strings
    	    # or even digits in odd length strings.
        	for (my $pos = ($no_length % 2); $pos < $no_length; $pos += 2) {
            	my $value = substr($card_no, $pos, 1) * 2;
	            $checksum += $value < 10 ? $value : $value - 9;
    	    }

        	# Checksums divisible by 10 are good.
	        if ($checksum % 10 == 0) {
				$result->{luhn} = 1;
        	}
        } else {
			$result->{luhn} = 1;
        }
	}

	return $result;
}

sub __get_ccard_type_rule {
	my $class = shift;

	my %h = (
		for_javascript => undef,
		@_
	);

	my @card_matrix = (
		{ lo => 180000, hi => 180099, type => 'J', num_len => 15},
		{ lo => 201400, hi => 201499, type => 'E', num_len => 15},
		{ lo => 213100, hi => 213199, type => 'J', num_len => 15},
		{ lo => 214900, hi => 214999, type => 'E', num_len => 15},
		{ lo => 300000, hi => 305999, type => 'C', num_len => 14},
		{ lo => 340000, hi => 349999, type => 'A', num_len => 15},
		{ lo => 352800, hi => 358999, type => 'J', num_len => 16},
		{ lo => 360000, hi => 369999, type => 'C', num_len => 14},
		{ lo => 370000, hi => 379999, type => 'A', num_len => 15},
		{ lo => 380000, hi => 388999, type => 'C', num_len => 14},
		{ lo => 389000, hi => 389999, type => 'B', num_len => 14},
		{ lo => 300000, hi => 399999, type => 'J', num_len => 16},
		{ lo => 413733, hi => 413737, type => 'T', num_len => 16},
		{ lo => 431930, hi => 431979, type => 'T', num_len => 16},
		{ lo => 446200, hi => 446299, type => 'T', num_len => 16},
		{ lo => 450875, hi => 450875, type => 'L', num_len => 16},
		{ lo => 453978, hi => 453979, type => 'T', num_len => 16},
		{ lo => 454313, hi => 454313, type => 'T', num_len => 16},
		{ lo => 454432, hi => 454435, type => 'T', num_len => 16},
		{ lo => 454742, hi => 454742, type => 'T', num_len => 16},
		{ lo => 456725, hi => 456745, type => 'T', num_len => 16},
		{ lo => 465830, hi => 465879, type => 'T', num_len => 16},
		{ lo => 465901, hi => 465950, type => 'T', num_len => 16},
		{ lo => 475110, hi => 475159, type => 'T', num_len => 16},
		{ lo => 475710, hi => 475759, type => 'T', num_len => 16},
		{ lo => 484406, hi => 484408, type => 'L', num_len => 16},
		{ lo => 484409, hi => 484410, type => 'T', num_len => 16},
		{ lo => 484411, hi => 484455, type => 'L', num_len => 16},
		{ lo => 490302, hi => 490309, type => 'W', num_len => 18},
		{ lo => 490335, hi => 490339, type => 'W', num_len => 18},
		{ lo => 490960, hi => 490979, type => 'T', num_len => 16},
		{ lo => 491101, hi => 491102, type => 'W', num_len => 16},
		{ lo => 491174, hi => 491182, type => 'W', num_len => 18},
		{ lo => 491730, hi => 491759, type => 'L', num_len => 16},
		{ lo => 491880, hi => 491880, type => 'L', num_len => 16},
		{ lo => 492181, hi => 492182, type => 'T', num_len => 16},
		{ lo => 493600, hi => 493699, type => 'W', num_len => 19},
		{ lo => 498824, hi => 498824, type => 'T', num_len => 16},
		{ lo => 400000, hi => 499999, type => 'V', num_len => 16},
		{ lo => 400000, hi => 499999, type => 'V', num_len => 13},
		{ lo => 500000, hi => 509999, type => 'W', num_len => 0},
		{ lo => 510000, hi => 559999, type => 'M', num_len => 16},
		{ lo => 564182, hi => 564182, type => 'W', num_len => 16},
		{ lo => 560000, hi => 589999, type => 'W', num_len => 0},
		{ lo => 601100, hi => 601199, type => 'D', num_len => 16},
		{ lo => 633300, hi => 633300, type => 'W', num_len => 16},
		{ lo => 633301, hi => 633301, type => 'W', num_len => 19},
		{ lo => 633302, hi => 633349, type => 'W', num_len => 16},
		{ lo => 633450, hi => 633453, type => 'S', num_len => 16},
		{ lo => 633454, hi => 633457, type => 'S', num_len => 16},
		{ lo => 633458, hi => 633460, type => 'S', num_len => 16},
		{ lo => 633461, hi => 633461, type => 'S', num_len => 18},
		{ lo => 633462, hi => 633472, type => 'S', num_len => 16},
		{ lo => 633473, hi => 633473, type => 'S', num_len => 18},
		{ lo => 633474, hi => 633475, type => 'S', num_len => 16},
		{ lo => 633476, hi => 633476, type => 'S', num_len => 19},
		{ lo => 633477, hi => 633477, type => 'S', num_len => 16},
		{ lo => 633478, hi => 633478, type => 'S', num_len => 18},
		{ lo => 633479, hi => 633480, type => 'S', num_len => 16},
		{ lo => 633481, hi => 633481, type => 'S', num_len => 19},
		{ lo => 633482, hi => 633489, type => 'S', num_len => 16},
		{ lo => 633490, hi => 633493, type => 'S', num_len => 16},
		{ lo => 633494, hi => 633494, type => 'S', num_len => 18},
		{ lo => 633495, hi => 633497, type => 'S', num_len => 16},
		{ lo => 633498, hi => 633498, type => 'S', num_len => 19},
		{ lo => 633499, hi => 633499, type => 'S', num_len => 18},
		{ lo => 675900, hi => 675900, type => 'W', num_len => 16},
		{ lo => 675901, hi => 675901, type => 'W', num_len => 19},
		{ lo => 675902, hi => 675904, type => 'W', num_len => 16},
		{ lo => 675905, hi => 675905, type => 'W', num_len => 19},
		{ lo => 675906, hi => 675917, type => 'W', num_len => 16},
		{ lo => 675918, hi => 675918, type => 'W', num_len => 19},
		{ lo => 675919, hi => 675937, type => 'W', num_len => 16},
		{ lo => 675938, hi => 675940, type => 'W', num_len => 18},
		{ lo => 675941, hi => 675949, type => 'W', num_len => 16},
		{ lo => 675950, hi => 675962, type => 'W', num_len => 19},
		{ lo => 675963, hi => 675997, type => 'W', num_len => 16},
		{ lo => 675998, hi => 675998, type => 'W', num_len => 19},
		{ lo => 675999, hi => 675999, type => 'W', num_len => 16},
		{ lo => 676700, hi => 676700, type => 'S', num_len => 16},
		{ lo => 676701, hi => 676701, type => 'S', num_len => 19},
		{ lo => 676702, hi => 676702, type => 'S', num_len => 16},
		{ lo => 676703, hi => 676703, type => 'S', num_len => 18},
		{ lo => 676704, hi => 676704, type => 'S', num_len => 16},
		{ lo => 676705, hi => 676705, type => 'S', num_len => 19},
		{ lo => 676706, hi => 676707, type => 'S', num_len => 16},
		{ lo => 676708, hi => 676711, type => 'S', num_len => 16},
		{ lo => 676712, hi => 676715, type => 'S', num_len => 16},
		{ lo => 676716, hi => 676717, type => 'S', num_len => 16},
		{ lo => 676718, hi => 676718, type => 'S', num_len => 19},
		{ lo => 676719, hi => 676739, type => 'S', num_len => 16},
		{ lo => 676740, hi => 676740, type => 'S', num_len => 18},
		{ lo => 676741, hi => 676749, type => 'S', num_len => 16},
		{ lo => 676750, hi => 676762, type => 'S', num_len => 19},
		{ lo => 676763, hi => 676769, type => 'S', num_len => 16},
		{ lo => 676770, hi => 676770, type => 'S', num_len => 19},
		{ lo => 676771, hi => 676773, type => 'S', num_len => 16},
		{ lo => 676774, hi => 676774, type => 'S', num_len => 18},
		{ lo => 676775, hi => 676778, type => 'S', num_len => 16},
		{ lo => 676779, hi => 676779, type => 'S', num_len => 18},
		{ lo => 676780, hi => 676781, type => 'S', num_len => 16},
		{ lo => 676782, hi => 676782, type => 'S', num_len => 18},
		{ lo => 676783, hi => 676794, type => 'S', num_len => 16},
		{ lo => 676795, hi => 676795, type => 'S', num_len => 18},
		{ lo => 676796, hi => 676797, type => 'S', num_len => 16},
		{ lo => 676798, hi => 676798, type => 'S', num_len => 19},
		{ lo => 676799, hi => 676799, type => 'S', num_len => 16},
		{ lo => 600000, hi => 699999, type => 'W', num_len => 0},
		{ lo => 940000, hi => 949999, type => 'B', num_len => 14},
		{ lo => 950000, hi => 959999, type => 'B', num_len => 14},
	);

	## For future use
	## Optima
	## return 'O' if ($card_no =~ /^37\d7\d{11}$/);
	##	{ lo => 561000, hi => 561099, type => "Australian BankCard", num_len => 16},

	if ( $h{for_javascript} ) {
		@card_matrix = map { qq{"$_->{lo},$_->{hi},$_->{type},$_->{num_len}"} } @card_matrix;
	}
	return \@card_matrix;
}

1;
