## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
##
package HSPC::MT::Plugin::PP::BT_Clieop03::Batch;
use strict;

use HSPC::PluginToolkit::General qw(round throw_exception);
use HSPC::MT::Plugin::PM::BT_Clieop03;

####################################################
## Module is responsible for building a batch record
#################################################### 

sub __new {
	my $class = shift;

	my $self = {};
	bless ($self,$class);

	return $self;
}

sub create {
	my $class = shift;
	my %h = (
		account_num_ordering => undef,
		batch_sequence_num   => undef, 
		fixed_desc           => undef,
		date                 => undef,
		order_party_name     => undef,
		@_
	);		
	my $batch = $class->__new();
	
	$batch->{account_num_ordering} = $h{account_num_ordering} ||
		throw_exception(
			type        => 'error',
			log_message => 'account number ordering party is mandatory'
		);
	$batch->{batch_sequence_num} = $h{batch_sequence_num} ||
		throw_exception(
			type        => 'error',
			log_message => 'batch sequence number is mandatory'
		);
	$batch->{fixed_desc} = $h{fixed_desc} ||
		throw_exception(
			type        => 'error',
			log_message => 'fixed description is mandatory'
		);
	$batch->{date} = $h{date} ||
		throw_exception(
			type        => 'error',
			log_message => 'date is mandatory'
		);
	$batch->{order_party_name} = $h{order_party_name} ||
		throw_exception(
			type        => 'error',
			log_message => 'batch sequence number is mandatory'
		);

	## batch header fields
	$batch->{record_code}          = '0010';
	$batch->{variant_code}         = 'B';
	$batch->{transactiongroup}     = '10'; ## we support only direct debits
	$batch->{account_num_ordering} = $h{account_num_ordering};
	$batch->{batch_sequence_num}   = $h{batch_sequence_num};
	$batch->{delivery_currency}    = 'EUR';
	$batch->{filler}               = ' ' x 26;

	## fixed description record
	$batch->{fixed_desc_record_code}  = '0020';
	$batch->{fixed_desc_variant_code} = 'A';
	$batch->{fixed_desc}              = $h{fixed_desc};
	$batch->{fixed_desc_filler}       = ' ' x 13;

	## ordering party record
	$batch->{order_party_record_code}  = '0030';
	$batch->{order_party_variant_code} = 'B';
	$batch->{name_code}                = 1; ## it equals 1 for direct debit batch

	my ($day,$mon,$year) = (localtime(time+86400))[3,4,5];
	my $processingdate = sprintf("%.2d%.2d%.2d", $day, $mon+1, $year%100);
	
	$batch->{desired_processing_date} = $processingdate;
	$batch->{order_party_name}        = $h{order_party_name};
	$batch->{test_code}               = $h{test_code} ? 'T' : 'P';
	$batch->{order_party_filler}      = ' ' x 2;

	## batch trailer fields
	$batch->{trailer_record_code}  = '9990';
	$batch->{trailer_variant_code} = 'A';	
	$batch->{total_amount}         = 0;
	$batch->{total_account_nums}   = 0;
	$batch->{items_num}            = 0;
	$batch->{trailer_filler}       = ' ' x 10;

	$batch->__check();

	return $batch;
}

sub process_content {
	my $self = shift;
	my $txt;

	$self->{number_of_items} = scalar(@{$self->{__item}});

	if ($self->{number_of_items} > 100000) {
		throw_exception(
			type        => 'error',
			log_message => "Number of items in batch should be less than 100000, it's ".$self->{number_of_items}." now"
		);
	}

	$txt .= $self->__prepare_batchheader;
	$txt .= $self->__fixed_description_record;
	$txt .= $self->__ordering_party_record;

	$self->{total_amount} = 0 unless (exists $self->{total_amount});
	$self->{account_numbers} = 0 unless (exists $self->{account_numbers});

	## prepare item
	foreach (@{$self->{__item}}) {
		my ($line, $acc_num, $total_amount) = $_->prepare_content();
		$txt .= $line;
		
		## calculate total amount
		$self->{total_amount} += $total_amount;

		## account numbers
		$self->{account_numbers} += $self->{account_num_ordering} + $acc_num;
	}

	$txt .= $self->__batchtrailer_record;
}

sub __prepare_batchheader {
	my $self = shift;

	my $txt = "";

	##  Record code, 4 digits
	$txt .= sprintf("%04d", $self->{record_code});

	## Variant code, 1 character
	$txt .= sprintf("%1s", $self->{variant_code} );

	## Transaction group, 2 characters
	$txt .= sprintf("%2s", $self->{transactiongroup} );

	## Account number ordering party, 10 digits
	$txt .= sprintf("%010d", $self->{account_num_ordering} );

	## Batch sequence number 4 digits
	$txt .= sprintf("%04d", $self->{batch_sequence_num});

	## Delivery currency
	$txt .= sprintf("%3s", $self->{delivery_currency});

	## filer
	$txt .= sprintf("%26s", $self->{filler});

	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of batch header is wrong: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub __fixed_description_record {
	my $self = shift;

	my $txt = "";

	for ( my $i=0; $i < length($self->{fixed_desc}); $i+=32 ) {
		my $description = substr($self->{fixed_desc}, $i, 32);

		##  Record code, 4 digits
		my $current_line = sprintf("%04d", $self->{fixed_desc_record_code});

		## Variant code, 1 character
		$current_line .= sprintf("%1s", $self->{fixed_desc_variant_code} );

		## Fixed description, 32 characters
		$current_line .= sprintf("%-32s", $description );

		## filer
		$current_line .= sprintf("%13s", $self->{fixed_desc_filler});

		if (length($current_line) != 50) {
			throw_exception(
				type        => 'error',
				log_message => "Length of fixed description record is wrong: '$current_line'"
			);
		}

		$txt .= "$current_line\r\n";
	}
	return $txt;
}


sub __ordering_party_record {
	my $self = shift;

	my $date = $self->{desired_processing_date};

	my $txt = "";

	##  Record code, 4 digits
	$txt .= sprintf("%04d", $self->{order_party_record_code});

	## Variant code, 1 character
	$txt .= sprintf("%1s", $self->{order_party_variant_code});

	## Name code 1 digit
	$txt .= sprintf("%01d", $self->{name_code});

	## Desired processing date, 6 diigits ddmmyy
	$txt .= sprintf("%06d", $date);

	## Name of ordering party, 35 characters
	$txt .= sprintf("%-35s", $self->{order_party_name});

	## Test code 1 character
	$txt .= sprintf("%1s", $self->{test_code});

	## filer
	$txt .= sprintf("%2s", $self->{order_party_filler});
	
	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of ordering party record is wrong: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub __batchtrailer_record{
	my $self = shift;

	my $txt = "";

	##  Record code, 4 digits
	$txt .= sprintf("%04d", $self->{trailer_record_code});

	## Variant code, 1 character
	$txt .= sprintf("%1s", $self->{trailer_variant_code});

	## Total amount, 18 digits
	$txt .= sprintf("%018d", round($self->{total_amount} * 100, 0));

	## Total account numbers, 10 digits
	$txt .= sprintf("%010s", substr($self->{account_numbers},-10));

	## Total account numbers, 7 digits
	$txt .= sprintf("%07d", $self->{number_of_items});

	## filler
	$txt .= sprintf("%10s", $self->{trailer_filler});
			
	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of batchtrailer record is incorrect: '$txt'"
		);
	}					
	
	$txt .= "\r\n";
	return $txt;
}

sub add_item {
	my $self = shift;
	my %h = (
		item => undef,
		@_
	);
	
	my $item = $h{item} || throw_exception(
		type        => 'error',
		log_message => "Item is not specified"
	);

	$self->{__item} = [] unless (exists $self->{__item});
	push @{$self->{__item}}, $item;
}

sub __check {
	my $self = shift;
	
	my $account_num_ordering = $self->{account_num_ordering};

 	throw_exception(
		type        => 'error',
		log_message => {
			string_id => 'bapm_clieop03_account_number_is_wrong',
			string_args => {acc => $account_num_ordering}
		}
	) unless HSPC::MT::Plugin::PM::BT_Clieop03->account_number_check(
		$account_num_ordering
	);

	return undef;
}


1;

__END__
