## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
##
package HSPC::MT::Plugin::PP::BT_Clieop03::File;
use strict;

use HSPC::PluginToolkit::General qw(throw_exception);

######################################################
## This module is responsible for creating File record
######################################################

sub __new {
	my $class = shift;

	my $self = {};
	bless ($self,$class);

	return $self;
}

###
## usage HSPC::MT::Plugins::PP::BT_Clieop03::File->create( ... )
## < file object,

sub create {
	my $class = shift;
	my %h = (
		sender_id    => undef, ## id assigned to sender by himself
		file_date    => undef, ## File creation date, in DDMMYY format
		sequence_num => undef,
		@_
	);
	
	throw_exception(
		type        => 'error',
		log_message => 'sender_id parameter is mandatory'
	) unless $h{sender_id};

	throw_exception(
		type        => 'error',
		log_message => 'sequence_num parameter is mandatory'
	) unless $h{sequence_num};

	my $date = $h{file_date} || 
	throw_exception(
		type        => 'error',
		log_message => 'file_date parameter is mandatory'
	);
	
	$date =~ s|^\d\d(\d\d)-(\d+)-(\d+)\s[\d:]+|$3$2$1|;
	my $day = $3;

	my $file = $class->__new();

	## File Header fields
	$file->{record_code} = '0001';
	$file->{variant_code} = 'A';
	$file->{file_date} = $date;
	$file->{file_name} = 'CLIEOP03';
	$file->{sender_id} = $h{sender_id};
	$file->{file_id} = sprintf("%02d%02d", $day, $h{sequence_num});
	$file->{duplicate_code} = 1; ## unique for now
	$file->{filler} = ' ' x 21;

	## File Trailer fields
	$file->{trailer_record_code} = '9999';
	$file->{trailer_variant_code} = 'A';
	$file->{trailer_filler} = ' ' x 45;

	$file->__check();

	return $file;
}

###
## Prepare record methods
##

sub prepare_content {
	my $self = shift;

	my $txt = "";
	my $batch = $self->{__batch} || 
		throw_exception(
			type        => 'error',
			log_message => 'Add batch content first'
		);

	$txt .= $self->__fileheader();
	$txt .= $batch->process_content();
	$txt .= $self->__filetrailer(); 

	return $txt;
}

sub __fileheader {
	my $self = shift;

	my $txt = "";

	##  Record code: initial position 1, length 4, numerical
	$txt .= sprintf("%04d", $self->{record_code}); 

	## Variant code: initial position 5, length 1, alphabetical
	$txt .= sprintf("%1s", $self->{variant_code} ); 

	## File Creation date: initial position 6, length 6, numeric
	$txt .= sprintf("%06d", $self->{file_date}); 

	## File Name: initial position 12, length 8, alphabetical
	$txt .= sprintf("%8s", $self->{file_name}); 

	## Sender Identification: initial position 20, length 5, alphabetical
	$txt .= sprintf("%5s", $self->{sender_id}); 

	## File Identification: initial position 25, length 4, alphabetical
	$txt .= sprintf("%04d", $self->{file_id});
	
	## Duplicate Code: initial position 29, length 1, alphabetical
	$txt .= sprintf("%01d", $self->{duplicate_code}); 
	
	## filer
	$txt .= sprintf("%21s", $self->{filler}); 

	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of submitter header is wrong: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub __filetrailer {
	my $self = shift;
	
	my $txt = "";

	## Record code: initial position 1, length 4, numerical
	$txt .= sprintf("%04d", $self->{trailer_record_code}); 

	## Data code: initial position 5, length 1, alphanumeric
	$txt .= sprintf("%1s", $self->{trailer_variant_code}); 

	## Filler. length 45, alphanumeric
	$txt .= sprintf("%-45s", $self->{trailer_filler});

	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of Filetrailer record is incorrect: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub add_batch {
	my $self = shift;

	my %h = (
		batch => undef,
		@_
	);
	
	my $batch = $h{batch} || 
		throw_exception(
			type        => 'error',
			log_message => "batch is not specified"
		);

	$self->{__batch} = $batch;
}

###
## check file record data consistency
##

sub __check {
	my $self = shift;

	my $sender_id = $self->{sender_id};

	## check sender identification. According to documentation the length is 5 alphanumeric characters
	unless (($sender_id =~ /w+/) || (length($sender_id) == 5)) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of sender identification '$sender_id'!"
				." It should be alphanumerical and not longer than 5 characters."
		);
	}

	return undef;
}

1;

__END__
