## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
##
package HSPC::MT::Plugin::PP::BT_Clieop03::Item;
use strict;

use HSPC::MT::Plugin::PM::BT_Clieop03;
use HSPC::PluginToolkit::General qw(round round2 throw_exception);

####################################################
## Responsible for building Item record
####################################################
 
sub __new {
	my $class = shift;

	my $self = {};
	bless ($self,$class);

	return $self;
}

sub create {
	my $class = shift;
	my %h = (	
		amount                     => undef,
		account_number             => undef,
		account_number_beneficiary => undef,
		name_payer                 => undef,
		payment_reference         => undef,
		description                => undef,
		@_
	);
	my $item = $class->__new();

	$item->{amount} = $h{amount} ||
		throw_exception(
			type        => 'error',
			log_message => 'transaction record amount is mandatory'
		);

	$item->{account_number} = $h{account_number} ||
		throw_exception(
			type        => 'error',
			log_message => 'transaction record payer account number is mandatory'
		);

	$item->{account_number_beneficiary} = $h{account_number_beneficiary} ||
		throw_exception(
			type        => 'error',
			log_message => 'transaction account number beneficiary is mandatory'
		);

	$item->{name_payer} = $h{name_payer} ||
		throw_exception(
			type        => 'error',
			log_message => 'name payer is mandatory'
		);

	$item->{payment_reference} = $h{payment_reference} ||
		throw_exception(
			type        => 'error',
			log_message => 'payment reference is mandatory'
		);

	$item->{description} = $h{description} ||
		throw_exception(
			type        => 'error',
			log_message => 'description for description record is mandatory field'
		);

	## transaction record
	$item->{record_code}     = '0100';
	$item->{variant_code}    = 'A';
	$item->{transactiontype} = '1002'; ## unchecked direct debit of a postbank account
	$item->{filler}          = ' ' x 9;

	## name payer record
	$item->{payer_record_code}  = '0110';
	$item->{payer_variant_code} = 'B';
	$item->{payer_filler}       = ' ' x 10;

	## payment reference record
	$item->{payment_record_code}  = '0150';
	$item->{payment_variant_code} = 'A';
	$item->{payment_filler}       = ' ' x 29;

	## description record
	$item->{desc_record_code}  = '0160';
	$item->{desc_variant_code} = 'A';	
	$item->{desc_filler}       = ' ' x 13;

	$item->__check();

	return $item;
}

sub prepare_content {
	my $self = shift;

	my $txt = '';

	$txt .= $self->__transaction_record;
	$txt .= $self->__namepayer_record;
	$txt .= $self->__payment_reference_record;
	$txt .= $self->__description_record;

	return ($txt, $self->{account_number}, $self->{amount});
}

sub __transaction_record {	
	my $self = shift;
	my $txt = "";

	## Record code, 4 digits
	$txt .= sprintf("%04d", $self->{record_code});

	## Variant code, 1 character
	$txt .= sprintf("%1s", $self->{variant_code});

	## Transactiontype, 4 characters
	$txt .= sprintf("%4s", $self->{transactiontype});

	## Amount, 12 digits
	## "%012s" used here, because numbers like 2710001001 cause overflow in case %012d
	$txt .= sprintf( "%012s", round($self->{amount} * 100, 0) );

	## Account number payer
	$txt .= sprintf("%010s", $self->{account_number});

	## Account number beneficiary
	$txt .= sprintf("%010s", $self->{account_number_beneficiary});

	## filer
	$txt .= sprintf("%9s", $self->{filler});

	if (length($txt) != 50) {
		sw_die ("Length of transaction record is wrong: '$txt'");
	}

	$txt .= "\r\n";
	return $txt;
}

sub __namepayer_record{
	my $self = shift;

	my $txt = "";

	# Name payer record code, 4 digits
	$txt .= sprintf("%04d", $self->{payer_record_code});

	# Name payer record variant code, 1 character
	$txt .= sprintf("%1s", $self->{payer_variant_code});

	# Name payer, 35 characters
	$txt .= sprintf("%-35s", substr($self->{name_payer},0,35));

	## Payer filer
	$txt .= sprintf("%10s", $self->{payer_filler});

	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of namepayer record is wrong: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub __payment_reference_record {
	my $self = shift;
	my $txt = "";

	# Payment reference record code, 4 digits
	$txt .= sprintf("%04d", $self->{payment_record_code});

	# Payment reference variant code, 1 character
	$txt .= sprintf("%1s", $self->{payment_variant_code});

	# Payment reference, 16 characters
	$txt .= sprintf("%-16s", $self->{payment_reference});

	## Payment filler
	$txt .= sprintf("%29s", $self->{payment_filler});

	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of payment reference record is wrong: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub __description_record {
	my $self = shift;

	my $txt = "";

	# Description record code, 4 digits
	$txt .= sprintf("%04d", $self->{desc_record_code});

	# Description record variant code, 1 character
	$txt .= sprintf("%1s", $self->{desc_variant_code});

	# Description record: Description, 32 characters
	$txt .= sprintf("%-32s", $self->{description});

	## Payment filler
	$txt .= sprintf("%13s", $self->{desc_filler});

	if (length($txt) != 50) {
		throw_exception(
			type        => 'error',
			log_message => "Length of description record is wrong: '$txt'"
		);
	}

	$txt .= "\r\n";
	return $txt;
}

sub __check {
	my $self = shift;

	my $amount = $self->{amount};

	if ($amount != round2($amount) || $amount <= 0 || $amount > 453780216.08) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong amount '$amount'"
		);
	}

	throw_exception(
		type        => 'error',
		log_message => "Wrong account number specified in Item record\n".$self->{account_number}
	) unless HSPC::MT::Plugin::PM::BT_Clieop03->account_number_check($self->{account_number});

	throw_exception(
		type        => 'error',
		log_message => "Wrong account number beneficiary in Item record\n".$self->{account_number_beneficiary}
	) unless HSPC::MT::Plugin::PM::BT_Clieop03->account_number_check($self->{account_number_beneficiary});

	return undef;
}

1;

__END__
