package HSPC::MT::Plugin::PP::BT_Norma19;

use HSPC::PluginToolkit::General qw(datetime_gmt_now string throw_exception round2 gmt_to_local);
use strict;

use HSPC::MT::Plugin::PP::BT_Norma19::Submitter;
use HSPC::MT::Plugin::PP::BT_Norma19::Issuer;
use HSPC::MT::Plugin::PP::BT_Norma19::Order;

sub get_title {
	my $class = shift;
	
	return string('bapm_norma19_plugin_title_uc');
}

sub process_batch_content {
	my $class = shift;
	my %h = (
		config           => undef,
		transaction_list => undef,
		vendor_info      => undef,
		file_id          => undef,
		@_
	);

	my $config = $h{config};
	my $file_id = $h{file_id};
	my $transaction_list = $h{transaction_list} ||
		throw_exception(
			type        => 'error',
			log_message => 'Transactions list is not specified'
		);

	my $cdate = gmt_to_local( datetime_gmt_now() );

	my $submitter = HSPC::MT::Plugin::PP::BT_Norma19::Submitter->create(
		submtt_code => $config->{issuer_code},     ## Submitter Code.
		submtt_name => $config->{issuer_name},     ## Name of Submitting Client.
		agency_code => $config->{issuer_agency},   ## Agency receiving the file.
		branch_code => $config->{issuer_branch} ,  ## Receiving Branch. Branch of Receiving Agency, where the file is delivered.
		cdate       => $cdate                      ## File creation date, in YYYY-MM-DD format
	);

	my $issuer = HSPC::MT::Plugin::PP::BT_Norma19::Issuer->create(
		issuer_code    => $config->{issuer_code},    ## Issuing Client Code.
		issuer_name    => $config->{issuer_name},    ## Name of Issuing Client.
		agency_code    => $config->{issuer_agency},  ## Agency Number
		branch_code    => $config->{issuer_branch},  ## Branch Nubmer, where Issuing Client account is being held
		control_digits => $config->{issuer_control_dig}, ## Control Digits of the Agency, Branch and Account Number.
		account_code   => $config->{issuer_account}, ## Account Number of Issuing Client.
        cdate          => $cdate, ## File creation date, in YYYY-MM-DD format
		charge_date    => $cdate
	);

	$submitter->add_issuer( issuer => $issuer );

	foreach my $t (@$transaction_list) {
		my $b_order = HSPC::MT::Plugin::PP::BT_Norma19::Order->create(
			issuer_code          => $config->{issuer_code}, ## Issuing Client Code.
			reference_code       => $t->{transaction_id}, ## Reference Code.
			debit_account_name   => $t->{payment_method}->{public_data}->{holder_name}, ## Name or company name of the direct payment holder.
			debit_agency_code    => $t->{payment_method}->{secure_data}->{bank_agency_number}, ## Debitor Agency Number
			debit_branch_code    => $t->{payment_method}->{secure_data}->{bank_branch_number}, ## Debitor Branch Number.
			debit_control_digits => $t->{payment_method}->{secure_data}->{stored_control_dig}, ## Debitor Control Digits
			debit_account_code   => $t->{payment_method}->{secure_data}->{account_number}, ## Debitor Account Number.
			amount               => $t->{transaction_amount}, ## Amount to charge in euros, i.e  123.12
			internal_reference   => $t->{transaction_id},     ## Internal reference number
			returns_code         => "",  ## Returns code (for internal use?)
		);

		{ ## fill optional fields
			my $date = $t->{document_info}->{date};
			$date =~ s#^(\d{4})-(\d{2})-(\d{2}).*$#$3/$2/$1#g;
			my %placeholders = (
				CIF     => $t->{payment_method}->{secure_data}->{cif},
				DOCNUM  => $t->{document_info}->{number},
				DOCID   => $t->{document_info}->{id},
				TAXSUM  => round2( $t->{document_info}->{total} - $t->{document_info}->{subtotal} ),
				TOTSUM  => round2( $t->{document_info}->{total} ),
				CLRSUM  => round2( $t->{document_info}->{subtotal} ),
				DOCDESC => $t->{document_info}->{description},
				DATE     => $date,
				COMPANY  => $h{vendor_info}->{name},
				NIF      => $t->{document_info}->{vendor_tax_number},
				HP       => $t->{document_info}->{plan_name},
				DOMAIN   => $t->{document_info}->{domain},
			);

			for (my $i = 1; $i <= 16; $i++) {
				my $opt_field = $config->{"opt_field_$i"};
				while (my ($key, $value) = each( %placeholders ) ) {
					$opt_field =~ s/\%$key\%/$value/g;
				}
				$b_order->add_opt_field( $i, $opt_field );
			}
		}

		if ($config->{opt_address_add}) {
			$b_order->set_optional_address(
				name        => $t->{payment_method}->{public_data}->{holder_name},
				town        => $t->{account_info}->{address_city}, ## Town of account holder address
				address     => $t->{account_info}->{address_address1} ||
					$t->{account_info}->{address_address2}, ## Account holder address.
				postal_code => $t->{account_info}->{address_zip}, ## Post code.
				@_
			);
		}
		$issuer->add_order( order => $b_order );
	}
	
    my $txt = $submitter->prepare_content();
	return {
		BATCH_CONTENT   => $txt,
		BATCH_FILE_NAME => sprintf("norma19_file_%06d.txt", $file_id),
	};
}

sub get_supported_payment_method_types{
	my $class = shift;
	
	return [
		'BT_Norma19',
	];
}

sub get_currencies_supported{
	return [
		'EUR', # Euro
	];
}


1;