## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
##
package HSPC::MT::Plugin::PP::BT_Norma19::Issuer;
use strict;

use HSPC::PluginToolkit::General qw(round2 round throw_exception);

use HSPC::MT::Plugin::PP::BT_Norma19::Tools;
use HSPC::MT::Plugin::PP::BT_Norma19::Order;

##--------------------------------------------------
## Constants
##-------------------------------------------------- 
use constant ISSUER_HEADER_RECORD_CODE => 53;
use constant ISSUER_TOTAL_RECORD_CODE  => 58;

use constant ISSUER_DIRECT_DATA_CODE   => 80;
use constant ISSUER_RETURN_DATA_CODE   => 90;

##--------------------------------------------------
## Issuer is organization, which charge customer. Several Issuers may present for one file
## 
## Issuer is responsible for forming issuer header record,
## Issuer should fill order content by calling each included order
## Issuer is responsible for Issuer Total record.

##-------------------------------------------------- 

##--------------------------------------------------
## Methods
##-------------------------------------------------- 
sub __new {
    my $class = shift;

    my $self = {};
    bless ($self,$class);

    $self->{orders} = [];

    return $self;
}

## usage HSPC::MT::Plugin::PP::BT_Norma19::Issuer->create( ... )
## < Issuer object,
sub create {
	my $class = shift;
	my %h = (
		issuer_code    => undef,  ## Issuing Client Code.
		issuer_name    => undef,  ## Name of Issuing Client.
		cdate          => undef,  ## File creation date, in YYYY-MM-DD format
		charge_date    => undef , ## Date charged. Date on which the Direct Payment Agency must debit the amounts from its clients.in YYYY-MM-DD format, if empty, will set to cdate
		agency_code    => undef,  ## Agency Number
		branch_code    => undef,  ## Branch Nubmer, where Issuing Client account is being held
		control_digits => undef,  ## Control Digits of the Agency, Branch and Account Number.
		account_code   => undef,  ## Account Number of Issuing Client.
		@_
	);

	my $issuer = $class->__new();

	$issuer->{issuer_code}    = $h{issuer_code}    ||
		throw_exception(
			type        => 'error',
			log_message => 'issuer_code parameter is mandatory'
		);
	$issuer->{issuer_name}    = $h{issuer_name}    ||
		throw_exception(
			type        => 'error',
			log_message => 'issuer_name parameter is mandatory'
		);
	$issuer->{agency_code}    = $h{agency_code}    ||
		throw_exception(
			type        => 'error',
			log_message => 'agency_code parameter is mandatory'
		);
	$issuer->{branch_code}    = $h{branch_code}    ||
		throw_exception(
			type        => 'error',
			log_message => 'branch_code parameter is mandatory'
		);
	$issuer->{account_code}   = $h{account_code}   ||
		throw_exception(
			type        => 'error',
			log_message => 'account_code parameter is mandatory'
		);
	$issuer->{cdate}          = $h{cdate}          ||
		throw_exception(
			type        => 'error',
			log_message => 'cdate parameter is mandatory'
		);
	$issuer->{control_digits} = $h{control_digits} ||
		throw_exception(
			type        => 'error',
			log_message => 'control_digits parameter is mandatory'
		);
	$issuer->{charge_date}    = $h{charge_date} || $h{cdate};

	$issuer->__check();

	return $issuer;
};

##--------------------------------------------------
## Parse methods
##-------------------------------------------------- 
## usage HSPC::MT::Plugin::PP::BT_Norma19::Issuer->parse( ... )
sub parse {
	my $class = shift;
	my %h = (
		lines => undef,
		@_
	);

	my $lines = $h{lines} ||
		throw_exception(
			type        => 'error',
			log_message => 'lines parameter is mandatory'
		);

	my $issuer_total_line_beg = &ISSUER_TOTAL_RECORD_CODE.&ISSUER_RETURN_DATA_CODE; ## "5890"
	
	my $issuers_lines = [];
	my $issuer_num = 0;
	while (my $line = shift( @$lines )) {
		$issuers_lines->[$issuer_num] ||= [];
		my $issuer = $issuers_lines->[$issuer_num];
		push @$issuer, $line;

		## if it was issuer total record
		if ($line =~ /^$issuer_total_line_beg/) {
			++$issuer_num;
		}
	}

	my @issuers;
	foreach my $issuer_line_coll (@$issuers_lines) {
		my $issuer = $class->__parse_one_issuer( lines => $issuer_line_coll );
		push @issuers, $issuer;
	}

	return \@issuers;
};

sub __parse_one_issuer {
	my $class = shift;
	my %h = (
		lines => undef,
		@_
	);

	my $lines = $h{lines} ||
		throw_exception(
			type        => 'error',
			log_message => 'lines parameter is mandatory'
		);

	my $issuer = $class->__new();

	my $header_line = shift(@$lines);
	my $total_line  = pop(@$lines);
	my $order_lines = $lines;

	$issuer->__parse_header( line => $header_line );
	$issuer->__parse_orders( lines => $order_lines );
	$issuer->__parse_total( line => $total_line );

	return $issuer;
}


sub __parse_orders {
	my $self = shift;
	my %h = (
		lines => undef,
		@_
	);

	my $lines = $h{lines} ||
		throw_exception(
			type        => 'error',
			log_message => 'lines parameter is mandatory'
		);

	my @orders;
	foreach my $line (@$lines) {
		my $order = HSPC::MT::Plugin::PP::BT_Norma19::Order->parse(
			line => $line
		);
		push @orders, $order;
	}

	$self->{orders} = \@orders;

	return undef;
}

sub __parse_header {
	my $self = shift;
	my %h = (
		line => undef,
		@_
	);

	my $line = $h{line};

	my $field_definition = {
		A1 => { start_pos =>   1, len =>  2 }, ## Record Code
		A2 => { start_pos =>   3, len =>  2 }, ## Data code

		B1 => { start_pos =>   5, len => 12 }, ## Receiving Client Code
		B3 => { start_pos =>  23, len =>  6 }, ## Date on which Direct Payment Agency had instructions to make the debits

		C  => { start_pos =>  29, len => 40 }, ## Name receiving client

		D1 => { start_pos =>  69, len =>  4 }, ## Agency No. (4 numerical positions)
		D2 => { start_pos =>  73, len =>  4 }, ## Branch No. (4 numerical positions)
		D3 => { start_pos =>  77, len =>  2 }, ## Control digits
		D4 => { start_pos =>  79, len => 10 }, ## Issuing Client Account No.
	};

	my $fld = HSPC::MT::Plugin::PP::BT_Norma19::Tools->splitline(
		line             => $line,
		field_definition => $field_definition
	);

	if ($fld->{A1} != &ISSUER_HEADER_RECORD_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_recode_code',
				string_args => {
					new_code => $fld->{A1},
					must_code => &ISSUER_HEADER_RECORD_CODE
				}
			}
		);
	}

	if ($fld->{A2} != &ISSUER_RETURN_DATA_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_data_code',
				string_args => {
					new_code => $fld->{A2},
					must_code => &ISSUER_RETURN_DATA_CODE
				}
			}
		);
	}

	$self->{issuer_code} = $fld->{B1};
	$self->{issuer_name} = HSPC::MT::Plugin::PP::BT_Norma19::Tools->trim(
		$fld->{C}
	);

	$self->{agency_code}    = $fld->{D1};
	$self->{branch_code}    = $fld->{D2};
	$self->{account_code}   = $fld->{D4};
	$self->{control_digits} = $fld->{D3};

	$self->{cdate} = HSPC::MT::Plugin::PP::BT_Norma19::Tools->date_DDMMYY_to_hspc_format(
		$fld->{B3}
	);
	unless ( $self->{cdate} ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_date_format',
				string_args => {
					date => $fld->{B3}
				}
			}
		);
	}
	return undef;
};

sub __parse_total {
	my $self = shift;
	my %h = (
		line => undef,
		@_
	);

	my $line = $h{line};

	my $field_definition = {
		A1 => { start_pos =>   1, len =>  2 }, ## Record Code
		A2 => { start_pos =>   3, len =>  2 }, ## Data code

		B1 => { start_pos =>   5, len => 12 }, ## Receiving Client Code

		E1 => { start_pos =>  89, len =>  10 }, ## Amounts Total 

		F1 => { start_pos => 105, len => 10 }, ## No. of Returns contained in the medium or file
		F2 => { start_pos => 115, len => 10 }, ## No. of Records contained in the medium
	};

	my $fld = HSPC::MT::Plugin::PP::BT_Norma19::Tools->splitline(
		line             => $line,
		field_definition => $field_definition
	);

	if ($fld->{A1} != &ISSUER_TOTAL_RECORD_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_total_recode_code',
				string_args => {
					new_code => $fld->{A1},
					must_code => &ISSUER_TOTAL_RECORD_CODE
				}
			}
		);
	}

	if ($fld->{A2} != &ISSUER_RETURN_DATA_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_data_code',
				string_args => {
					new_code => $fld->{A2},
					must_code => &ISSUER_RETURN_DATA_CODE
				}
			}
		);
	}

	if ($fld->{B1} ne $self->{issuer_code}) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_code_not_equal_total',
				string_args => {
					issuer_code => $self->{issuer_code},
					total => $fld->{B1}
				}
			}
		);
	}

	my $total_amount = round2( $fld->{E1} / 100 );
	if ( $total_amount != $self->documents_sum ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_docs_amount_dont_match',
				string_args => {
					total_amount => $total_amount,
					documents_sum => $self->documents_sum
				}
			}
		);
	}
	
	my $num_documents = $fld->{F1};
	if ( $num_documents != $self->num_documents ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_docs_count_dont_match',
				string_args => {
					num_documents => $num_documents,
					document_nums => $self->num_documents
				}
			}
		);
	}
	
	my $num_records = $fld->{F2};
	if ( $num_records != $self->num_records ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_recs_count_dont_match',
				string_args => {
					num_records => $num_records,
					record_nums => $self->num_records
				}
			}
		);
	}

	return undef;
};

##--------------------------------------------------
## Prepare record methods
##-------------------------------------------------- 
sub prepare_content {
	my $self = shift;

	my $txt = $self->__issuer_header();
	$txt .= $self->__orders_content();
	$txt .= $self->__issuer_total();

	return $txt;
}

sub __issuer_header {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ISSUER_HEADER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ISSUER_DIRECT_DATA_CODE); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	{ ## B2. Medium creation date (DDMMYY), initial position 17, length 6
		my $cdate = $self->{cdate};
		if ($cdate =~ /^(\d\d)(\d\d)-(\d\d)-(\d\d)/) {
			my $yy = $2;
			my $mm = $3;
			my $dd = $4;
			$txt .= sprintf("%02d%02d%02d", $dd, $mm, $yy);
		} else {
			throw_exception(
				type        => 'error',
				log_message => "Wrong cdate: $cdate"
			);
		}
	}

	{ ## B3. Date charged (DDMMYY), initial position 23, length 6
		my $charge_date = $self->{charge_date};
		if ($charge_date =~ /^(\d\d)(\d\d)-(\d\d)-(\d\d)/) {
			my $yy = $2;
			my $mm = $3;
			my $dd = $4;
			$txt .= sprintf("%02d%02d%02d", $dd, $mm, $yy);
		} else {
			throw_exception(
				type        => 'error',
				log_message => "Wrong charge_date: $charge_date"
			);
		}
	}

	## C. Name of issuing client. Initial position 29, length 40.
	$txt .= sprintf("%-40s", substr($self->{issuer_name}, 0, 40) );

	## D1. Depository Agency Number. Initial position 69, length 4. Numerical
	$txt .= sprintf( "%04d", $self->{agency_code} );

	## D2. Branch No. Initial position 73, length 4. Numerical
	$txt .= sprintf( "%04d", $self->{branch_code} );

	## D3. Control Digits. Initial position 77, length 2. Numerical
	$txt .= sprintf( "%02d", $self->{control_digits} );

	## D4. Account No. Initial position 79, length 10. Numerical
	$txt .= sprintf( "%010s", $self->{account_code} );

	## E1. Free. Initial position: 89, length 8. 
	$txt .= (" " x 8);

	## E2. Debit Procedure: 01 corresponds to the First, 02 corresponds to the Second
	## Initial position 97, len 2.
	$txt .= "01";

	## E3. Free. Initial position: 99, length 10. 
	$txt .= (" " x 10);

	## F. Free. Initial position: 109, length 40. NUMERICAL!
	$txt .= ("0" x 40);

	## G. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of issuer header is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
};

sub __issuer_total {
	my $self = shift;
	
	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ISSUER_TOTAL_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ISSUER_DIRECT_DATA_CODE); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2. Free. Initial position: 17, length 12. 
	$txt .= (" " x 12);

	## C. Free. Initial position: 29, length 40. 
	$txt .= (" " x 40);

	## D. Free. Initial position: 69, length 20. 
	$txt .= (" " x 20);

	## E1. Sum of amounts for the Issuing Client
	## Initial pos 89, length 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", round($self->documents_sum * 100, 0) );

	## E2. Free. Initial position: 99, length 6. 
	$txt .= (" " x 6);

	## F1. Total No. of compulsory individual records. (really documents)
	## Initial position 105, len 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", $self->num_documents );

	## F2. No. of records contained in the Issuer,
	## including the Issuer Header, the Issuer Total
	## Initial position 115, len 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", $self->num_records );

	## F3. Free. Initial position: 125, length 20. 
	$txt .= (" " x 20);

	## G. Free. Initial position: 145, length 18. 
	$txt .= (" " x 18);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of issuer total is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
};

sub __orders_content {
	my $self = shift;

	my $txt;
	my $orders_count = 0;
	foreach my $order (@{$self->{orders}}) {
		$txt .= $order->prepare_content();
		$orders_count++;
	}

	throw_exception(
		type        => 'error',
		log_message => 'No orders specified'
	) if $orders_count <= 0;

	return $txt;
};


##--------------------------------------------------
## fill object methods
##-------------------------------------------------- 
sub add_order {
    my $self = shift;
    my %h = (
        order => undef,
        @_
    );
    
	my $order = $h{order} ||
		throw_exception(
			type        => 'error',
			log_message => 'order does not specified'
		);

    push @{$self->{orders}}, $order;
    
	return undef;
} 

##--------------------------------------------------
## Accessors
##-------------------------------------------------- 
sub num_records {
	my $self = shift;

	my $numrecs = 2; ## Issuer header + issuer total.
	foreach my $order (@{$self->{orders}}) {
		$numrecs += $order->num_records();
	}

	return $numrecs;
};

sub num_documents {
	my $self = shift;

	my $numdocs = scalar( @{$self->{orders}} ); 

	return $numdocs;
};


sub documents_sum {
	my $self = shift;

	my $sum = 0.00; ## Submitter heade + total.
	foreach my $order (@{$self->{orders}}) {
		$sum += $order->amount();
	}

	$sum = round2($sum);
	return $sum;
};

## check issuer data consistency
sub __check {
	my $self = shift;

	## prepare shortcuts
	my $issuer_code    = $self->{issuer_code};
	my $issuer_name    = $self->{issuer_name};
	my $agency_code    = $self->{agency_code};
	my $branch_code    = $self->{branch_code};
	my $account_code   = $self->{account_code};
	my $cdate          = $self->{cdate};
	my $charge_date    = $self->{charge_date};
	my $control_digits = $self->{control_digits};

	## issuer code check.
	if ( ( $issuer_code =~ /\W|_/ ) || ( length($issuer_code) > 12 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Issuer code '$issuer_code'!" .
				' It should be alfanumerical and not bigger than 12 chars.'
		);
	}

    if (length($issuer_name) > 40) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Issuer name '$issuer_name'!" .
				' It should be alfanumerical and not bigger than 40 chars.'
		);
    }

    if (($agency_code =~ /\D/) || (length($agency_code) > 4)) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Agency code '$agency_code'!" .
				' It should be numerical and not bigger than 4 chars.'
		);
    }

	if ( ( $branch_code =~ /\D/ ) || ( length($branch_code) > 4 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Branch code '$branch_code'!" .
				' It should be numerical and not bigger than 4 chars.'
		);
	}

	if ( ( $account_code =~ /\D/ ) || ( length($account_code) > 10 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Account code '$account_code'!" .
				' It should be numerical and not bigger than 10 chars.'
		);
	}

	if ( ( $control_digits =~ /\D/ ) || ( length($control_digits) > 2 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of control digits '$control_digits'!" .
				' It should be numerical and not bigger than 2 chars.'
		);
	}

	## check file creation date, receiving the file
    if ($cdate !~ /(\d\d\d\d)-(\d\d)-(\d\d)/) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of file creation date '$cdate'!"
		);
    }

	## check charge_date, receiving the file
    if ($charge_date !~ /(\d\d\d\d)-(\d\d)-(\d\d)/) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of file charge date '$charge_date'!"
		);
    }
}

1;

__END__
