## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
##
## 
##
package HSPC::MT::Plugin::PP::BT_Norma19::Order;
use strict;

use HSPC::PluginToolkit::General qw(round2 round throw_exception);
use HSPC::MT::Plugin::PP::BT_Norma19::Tools;
use Encode qw(from_to);

##--------------------------------------------------
## Constants
##-------------------------------------------------- 
use constant ORDER_RECORD_CODE => 56;

use constant ORDER_COMPULSORY_DATA_CODE => 80;
use constant ORDER_OPTREC_DATA_CODE_1   => 81;
use constant ORDER_OPTREC_DATA_CODE_2   => 82;
use constant ORDER_OPTREC_DATA_CODE_3   => 83;
use constant ORDER_OPTREC_DATA_CODE_4   => 84;
use constant ORDER_OPTREC_DATA_CODE_5   => 85;
use constant ORDER_OPTREC_DATA_CODE_6   => 86;

use constant ORDER_RETURN_DATA_CODE => 90;

use constant OPT_ITEM_MAX_NUM => 16;
use constant OPT_ITEM_MIN_NUM => 1;
use constant OPT_ITEM_MAX_LEN => 40;

## optiona items distribution by "optional individual records"
use constant OPT_ITEM_DISTRIBUTION => {
	 1 => 0, ## mean "First Optiona Item" belongs to "Compulsory Individual Record"
	 2 => 1, ## mean "Second Optional Item" belongs to "First Optional Record"
	 3 => 1,
	 4 => 1,
	 5 => 2,
	 6 => 2,
	 7 => 2,
	 8 => 3,
	 9 => 3,
	10 => 3,
	11 => 4,
	12 => 4,
	13 => 4,
	14 => 5,
	15 => 5,
	16 => 5,
};

##--------------------------------------------------
## Methods
##-------------------------------------------------- 
sub __new {
    my $class = shift;

    my $self = {};
    bless ($self,$class);

	$self->{optional_items} = {};

    return $self;
}

sub create {
	my $class = shift;
	my %h = (
		issuer_code          => undef, ## Issuing Client Code. The same entered in this field in the "Issuer Header"
		reference_code       => undef, ## Reference Code. service; it might be the contract number, policy number, etc.
		debit_account_name   => undef, ## Name or company name of the direct payment holder.
		debit_agency_code    => undef, ## Debitor Agency Number, in four numerical positions.
		debit_branch_code    => undef, ## Debitor Branch Number.
		debit_control_digits => undef, ## Debitor Control Digits
		debit_account_code   => undef, ## Debitor Account Number.
		amount               => undef, ## Amount to charge in euros, i.e  123.12
		internal_reference   => undef, ## Internal reference number
		returns_code         => "",    ## Returns code (for internal use?)
		@_
	);

	my $order = $class->__new();

	$order->{issuer_code} = $h{issuer_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'issuer_code parameter is mandatory'
		);
	$order->{reference_code} = $h{reference_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'reference_code parameter is mandatory'
		);
	$order->{debit_account_name} = $h{debit_account_name} ||
		throw_exception(
			type        => 'error',
			log_message => 'debit_account_name parameter is mandatory'
		);

	if (length($order->{debit_account_name}) > 40) {
		$order->{debit_account_name} = substr($order->{debit_account_name},0,40);
    }

	$order->{debit_agency_code} = $h{debit_agency_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'debit_agency_code parameter is mandatory'
		);
	$order->{debit_branch_code} = $h{debit_branch_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'debit_branch_code parameter is mandatory'
		);
	$order->{debit_control_digits} = $h{debit_control_digits} ||
		throw_exception(
			type        => 'error',
			log_message => 'debit_control_digits parameter is mandatory'
		);
	$order->{debit_account_code} = $h{debit_account_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'debit_account_code parameter is mandatory'
		);
	$order->{amount} = $h{amount} ||
		throw_exception(
			type        => 'error',
			log_message => 'amount parameter is mandatory'
		);
	$order->{internal_reference} = $h{internal_reference} ||
		throw_exception(
			type        => 'error',
			log_message => 'internal_reference parameter is mandatory'
		);

	$order->{returns_code} = $h{returns_code};

	$order->__check();

	return $order;
}

##--------------------------------------------------
## Parse methods
##-------------------------------------------------- 
## usage HSPC::MT::Plugin::PP::BT_Norma19::Order->parse( ... )
sub parse {
	my $class = shift;
	my %h = (
		line => undef,
		@_
	);

	my $line = $h{line} ||
		throw_exception(
			type        => 'error',
			log_message => 'line parameter is mandatory'
		);

	my $field_definition = {
		A1 => { start_pos =>   1, len =>  2 }, ## Record Code
		A2 => { start_pos =>   3, len =>  2 }, ## Data code

		B1 => { start_pos =>   5, len => 12 }, ## Issuer Client Code
		B2 => { start_pos =>  17, len => 12 }, ## Reference Code

		C  => { start_pos =>  29, len => 40 }, ## Debit Client Name

		D1 => { start_pos =>  69, len =>  4 }, ## Debit Agency No. (4 numerical positions)
		D2 => { start_pos =>  73, len =>  4 }, ## Debit Branch No. (4 numerical positions)
		D3 => { start_pos =>  77, len =>  2 }, ## Debit Control digits
		D4 => { start_pos =>  79, len => 10 }, ## Debit Client Account No.

		E  => { start_pos =>  89, len => 10 }, ## Amount

		F1 => { start_pos =>  99, len =>  6 }, ## Returns code
		F2 => { start_pos => 105, len => 10 }, ## Internal Reference Code

		H  => { start_pos => 155, len =>  1 }, ## Reason for the Return
	
	};

	my $fld = HSPC::MT::Plugin::PP::BT_Norma19::Tools->splitline(
		line             => $line,
		field_definition => $field_definition
	);

	if ($fld->{A1} != &ORDER_RECORD_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_order_recode_code',
				string_args => {
					new_code => $fld->{A1},
					must_code => &ORDER_RECORD_CODE
				}
			}
		);
	}

	if ($fld->{A2} != &ORDER_RETURN_DATA_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_order_data_code',
				string_args => {
					new_code => $fld->{A2},
					must_code => &ORDER_RETURN_DATA_CODE
				}
			}
		);
	}

	my $order = $class->__new();

	$order->{issuer_code}          = $fld->{B1};
	$order->{reference_code}       = $fld->{B2};
	$order->{debit_account_name}   = $fld->{C};
	$order->{debit_agency_code}    = $fld->{D1};
	$order->{debit_branch_code}    = $fld->{D2};
	$order->{debit_control_digits} = $fld->{D3};
	$order->{debit_account_code}   = $fld->{D4};

	$order->{amount} = round2( $fld->{E} / 100 );

	$order->{internal_reference} = $fld->{F2};
	$order->{returns_code}       = $fld->{F1};
	$order->{reason_for_return}  = $fld->{H};

	return $order;
};

##--------------------------------------------------
## Prepare record methods
##-------------------------------------------------- 
sub prepare_content {
	my $self = shift;

	my $txt = $self->compulsory_record();
	$txt .= $self->first_optional_record()  if $self->is_need_opt_record(1);
	$txt .= $self->second_optional_record() if $self->is_need_opt_record(2);
	$txt .= $self->third_optional_record()  if $self->is_need_opt_record(3);
	$txt .= $self->fourth_optional_record() if $self->is_need_opt_record(4);
	$txt .= $self->fifth_optional_record()  if $self->is_need_opt_record(5);
	$txt .= $self->sixth_optional_record()  if $self->is_need_opt_record(6);

	return $txt;
}

sub compulsory_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf( "%02d", &ORDER_RECORD_CODE );

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf( "%02d", &ORDER_COMPULSORY_DATA_CODE );

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C: Name or Company Name of Direct Payment Holder
	## initial postion 29, length 40

	from_to($self->{debit_account_name},'utf8','latin1');
	$txt .= sprintf("%-40s", substr($self->{debit_account_name}, 0, 40));

	## D1. Agency Number. Initial position 69, length 4. Numerical
	$txt .= sprintf( "%04d", $self->{debit_agency_code} );

	## D2. Branch No. Initial position 73, length 4. Numerical
	$txt .= sprintf( "%04d", $self->{debit_branch_code} );

	## D3. Control Digits. Initial position 77, length 2. Numerical
	$txt .= sprintf( "%02d", $self->{debit_control_digits} );

	## D4. Account No. Initial position 79, length 10. Numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", $self->{debit_account_code} );

	## E. Direct Payment Amount
	## Initial pos 89, length 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", round($self->{amount} * 100, 0) );

	## F1. Returns code ## Initial pos 99, length 6
	$txt .= sprintf( "%-6s", $self->{returns_code} );

	## F2. Internal Reference Code. Initial pos 105, length 10
	$txt .= sprintf( "% 10s", $self->{internal_reference} );

	## G. First Optional Items filed. Initial pos 115, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{1} );

	## H. Free, Initial pos 155, length 8
	$txt .= (" " x 8);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of compulsory record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
}

sub first_optional_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ORDER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ORDER_OPTREC_DATA_CODE_1); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C. Second Optional Items filed. Initial pos 29, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{2} );

	## D. Third Optional Items filed. Initial pos 69, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{3} );

	## E. Fourth Optional Items filed. Initial pos 109, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{4} );

	## F. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of first optional record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
}

sub second_optional_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ORDER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ORDER_OPTREC_DATA_CODE_2); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C. Fifth Optional Items filed. Initial pos 29, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{5} );

	## D. Sixth Optional Items filed. Initial pos 69, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{6} );

	## E. Seventh Optional Items filed. Initial pos 109, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{7} );

	## F. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of first optional record is wrong: '$txt'"
		);
	}
	$txt .= "\n";
	return $txt;
}

sub third_optional_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ORDER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ORDER_OPTREC_DATA_CODE_3); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C. Eighth Optional Items filed. Initial pos 29, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{8} );

	## D. Ninth Optional Items filed. Initial pos 69, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{9} );

	## E. Tenth Optional Items filed. Initial pos 109, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{10} );

	## F. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of first optional record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
}

sub fourth_optional_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ORDER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ORDER_OPTREC_DATA_CODE_4); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C. Eleventh Optional Items filed. Initial pos 29, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{11} );

	## D. Twelfth Optional Items filed. Initial pos 69, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{12} );

	## E. Thirteenth Optional Items filed. Initial pos 109, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{13} );

	## F. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of first optional record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
}

sub fifth_optional_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ORDER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ORDER_OPTREC_DATA_CODE_5); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C. Fourteenth Optional Items filed. Initial pos 29, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{14} );

	## D. Fifteenth Optional Items filed. Initial pos 69, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{15} );

	## E. Sixteenth Optional Items filed. Initial pos 109, length 40
	$txt .= sprintf( "%-40s", $self->{optional_items}->{16} );

	## F. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of first optional record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
}

sub sixth_optional_record {
	my $self = shift;

	## A1: Record code: initial position 1, length 2, numerical
	my $txt = sprintf("%02d", &ORDER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &ORDER_OPTREC_DATA_CODE_6); 

	## B1: Issuer client code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{issuer_code}
	);

	## B2: Reference code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 17, length 12
	$txt .= sprintf( "%012s", $self->{reference_code} );

	## C. Name of Direct Payment Account Holder. Initial pos 29, length 40
	$txt .= sprintf( "%-40s", $self->{_opt6_name});

	## D. Account Holder Address. Initial pos 69, length 40
	$txt .= sprintf( "%-40s", $self->{_opt6_address} );

	## E1. Town of Account Holder Address. Initial pos 109, length 35
	$txt .= sprintf( "%-35s", $self->{_opt6_town} );

	## E2. Post code of Account Holder Address. Initial pos 144, length 5, numerical
	$txt .= sprintf( "%05d", $self->{_opt6_postal_code} );

	## F. Free. Initial position: 149, length 14. 
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of first optional record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
}

##--------------------------------------------------
## Accessors
##-------------------------------------------------- 
sub num_records {
	my $self = shift;

	my $numrecs = 1; ## Compulsory record
	for (my $i = 1; $i <= 6; $i++) {
		$numrecs++ if $self->is_need_opt_record($i);
	}

	return $numrecs;
};

sub amount {
	my $self = shift;
	return $self->{amount};
};

## used to add optional item fields 
## every optional filed item may contain up to 40 alphanumerical symbols.
sub add_opt_field {
	my $self = shift;
	my $field_num = shift ||
		throw_exception(
			type        => 'error',
			log_message => 'field_num parameter is mandatory'
		);
	my $field_value = shift;

	if ($field_num < &OPT_ITEM_MIN_NUM or $field_num > &OPT_ITEM_MAX_NUM) {
		throw_exception(
			type        => 'error',
			log_message => "field number $field_num is out of range " .
				&OPT_ITEM_MIN_NUM . '..' . &OPT_ITEM_MAX_NUM
		);
	}

	if (length($field_value) > &OPT_ITEM_MAX_LEN) {
		$field_value = substr($field_value, 0, &OPT_ITEM_MAX_LEN);
	}

	$self->{optional_items}->{$field_num} = $field_value;

	return undef;
};

## do we need to form "Optional individual record #X".
sub is_need_opt_record {
	my $self = shift;
	my $record_num = shift ||
		throw_exception(
			type        => 'error',
			log_message => 'record_num is a mandatory parameter'
		);

	my $result = 0;
	if ($record_num >= 1 && $record_num <= 5) {
		for (my $i = &OPT_ITEM_MIN_NUM; $i <= &OPT_ITEM_MAX_NUM; $i++) {
			if (&OPT_ITEM_DISTRIBUTION->{$i} == $record_num
				&& $self->{optional_items}->{$i}
			) {
				$result = 1;
				last;
			}
		}
	} elsif ($record_num == 6) {
		if ($self->{_opt6_name}
			|| $self->{_opt6_town}
			|| $self->{_opt6_address}
			|| $self->{_opt6_postal_code}
		) {
			$result = 1;
		}
	} else {
		throw_exception(
			type        => 'error',
			log_message => 'Opt record num $record_num is out of range 1..6'
		);
	}

	return $result;
}

## set "Account holder address"
sub set_optional_address {
	my $self = shift;
	my %h = (
		name        => undef, ## Name of direct payment account holder
		town        => undef, ## Town of account holder address
		address     => undef, ## Account holder address.
		postal_code => undef, ## Post code.
		@_
	);

	$self->{_opt6_name}        = $h{name};
	$self->{_opt6_town}        = $h{town};
	$self->{_opt6_address}     = $h{address};
	$self->{_opt6_postal_code} = $h{postal_code};

	if (length($self->{_opt6_name}) > 40) {
		$self->{_opt6_name} = substr($self->{_opt6_name}, 0, 40);
	}
	if (length($self->{_opt6_address}) > 40) {
		$self->{_opt6_address} = substr($self->{_opt6_address}, 0, 40);
	}
	if (length($self->{_opt6_town}) > 35) {
		$self->{_opt6_town} = substr($self->{_opt6_address}, 0, 35);
	}
	if ($self->{_opt6_postal_code} =~ /\D/ || length($self->{_opt6_postal_code}) > 5) {
		throw_exception(
			type           => 'error',
			vendor_message => {
				string_id   => 'norma19_wrong_pcode_uc',
				string_args => { pcode => $self->{_opt6_postal_code} }
			},
			log_message => "Wrong format of Postal code '"
			  . $self->{_opt6_postal_code} . "'!"
			  . ' It should be numerical and not bigger than 5 chars.'
		);
	}

	return undef;
}

sub __check {
	my $self = shift;

	## shortcuts
	my $issuer_code          = $self->{issuer_code};
	my $reference_code       = $self->{reference_code};
	my $debit_account_name   = $self->{debit_account_name};
	my $debit_agency_code    = $self->{debit_agency_code};
	my $debit_branch_code    = $self->{debit_branch_code};
	my $debit_control_digits = $self->{debit_control_digits};
	my $debit_account_code   = $self->{debit_account_code};
	my $amount               = $self->{amount};
	my $internal_reference   = $self->{internal_reference};
	my $returns_code         = $self->{returns_code};


	if ( ( $issuer_code =~ /\W|_/ ) || ( length($issuer_code) > 12 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Issuer code '$issuer_code'!" .
				' It should be alfanumerical and not bigger than 12 chars.'
		);
	}

	if ( ( $reference_code =~ /\W|_/ ) || ( length($reference_code) > 12 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of reference code '$reference_code'!" .
				' It should be alfanumerical and not bigger than 12 chars.'
		);
	}

    if (($debit_agency_code =~ /\D/) || (length($debit_agency_code) > 4)) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Agency code '$debit_agency_code'!" .
				' It should be numerical and not bigger than 4 chars.'
		);
    }

	if ( ( $debit_branch_code =~ /\D/ ) || ( length($debit_branch_code) > 4 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Branch code '$debit_branch_code'!" .
				' It should be numerical and not bigger than 4 chars.'
		);
	}

	if ( ( $debit_account_code =~ /\D/ ) || ( length($debit_account_code) > 10 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Account code '$debit_account_code'!" .
				' It should be numerical and not bigger than 10 chars.'
		);
	}

	if ( ( $debit_control_digits =~ /\D/ ) || ( length($debit_control_digits) > 2 ) ) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of control digits '$debit_control_digits'!" .
				' It should be numerical and not bigger than 2 chars.'
		);
	}

    if (length($internal_reference) > 10) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of internal_reference '$internal_reference'!" .
				' It should be alphanumerical and not bigger than 10 chars.'
		);
    }

    if (length($returns_code) > 6) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of returns_code '$returns_code'!" .
				' It should be alphanumerical and not bigger than 6 chars.'
		);
    }

	if ($amount != round2($amount) || $amount <= 0 || $amount >= 100000000) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong amount '$amount'"
		);
	}

	return undef;
}

1;

__END__
