## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
##
package HSPC::MT::Plugin::PP::BT_Norma19::Submitter;
use strict;

use HSPC::PluginToolkit::General qw(round2 round throw_exception);

use HSPC::MT::Plugin::PP::BT_Norma19::Tools;
use HSPC::MT::Plugin::PP::BT_Norma19::Issuer;

##--------------------------------------------------
## Constants
##-------------------------------------------------- 
use constant SUBMITTER_HEADER_RECORD_CODE => 51;
use constant GENERAL_TOTAL_RECORD_CODE    => 59;

use constant SUBMITTER_DIRECT_DATA_CODE => 80;
use constant SUBMITTER_RETURN_DATA_CODE => 90;

##--------------------------------------------------
## Submitter is top level, finally it forms whole file
## 
## Submitter is responsible for forming submitter header record,
## Submitter should fill issuer records by calling each issuer.
## Submitter is responsible for General Total record.
##-------------------------------------------------- 

sub __new {
    my $class = shift;

    my $self = {};
    bless ($self,$class);

    $self->{issuers} = [];

    return $self;
}

## usage HSPC::MT::Plugin::PP::BT_Norma19::Submitter->create( ... )
## < Submitter object,
sub create {
	my $class = shift;
    my %h = (
        submtt_code => undef, ## Submitter Code.
        submtt_name => undef, ## Name of Submitting Client.
        agency_code => undef, ## Agency receiving the file.
        branch_code => undef, ## Receiving Branch. Branch of Receiving Agency, where the file is delivered.
		cdate       => undef, ## File creation date, in YYYY-MM-DD format
        @_
    );

	my $submitter = $class->__new();

	$submitter->{submtt_code} = $h{submtt_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'submtt_code parameter is mandatory'
		);
	$submitter->{submtt_name} = $h{submtt_name} ||
		throw_exception(
			type        => 'error',
			log_message => 'submtt_name parameter is mandatory'
		);
	$submitter->{agency_code} = $h{agency_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'agency_code parameter is mandatory'
		);
	$submitter->{branch_code} = $h{branch_code} ||
		throw_exception(
			type        => 'error',
			log_message => 'branch_code parameter is mandatory'
		);
	$submitter->{cdate}       = $h{cdate}       ||
		throw_exception(
			type        => 'error',
			log_message => 'cdate parameter is mandatory'
		);

	$submitter->__check();

	return $submitter;
};

##--------------------------------------------------
## Parse methods
##-------------------------------------------------- 
## usage HSPC::MT::Plugin::PP::BT_Norma19::Submitter->parse( ... )
sub parse {
	my $class = shift;
	my %h = (
		lines => undef,
		@_
	);

	my $lines = $h{lines} ||
		throw_exception(
			type        => 'error',
			log_message => 'lines parameter is mandatory'
		);

	my $submitter = $class->__new();

	my $header_line   = shift(@$lines);
	my $total_line    = pop(@$lines);
	my $issuers_lines = $lines;

	$submitter->__parse_header( line => $header_line );
	$submitter->__parse_issuers( lines => $issuers_lines );
	$submitter->__parse_total( line => $total_line );

	return $submitter;
};

sub __parse_issuers {
	my $self = shift;
	my %h = (
		lines => undef,
		@_
	);

	$self->{issuers} = HSPC::MT::Plugin::PP::BT_Norma19::Issuer->parse(
		lines => $h{lines}
	);

	return undef;
}

sub __parse_header {
	my $self = shift;
	my %h = (
		line => undef,
		@_
	);

	my $line = $h{line};

	my $field_definition = {
		A1 => { start_pos =>   1, len =>  2 }, ## Record Code
		A2 => { start_pos =>   3, len =>  2 }, ## Data code

		B1 => { start_pos =>   5, len => 12 }, ## Receiving Client Code
		B2 => { start_pos =>  17, len =>  6 }, ## Medium creation date (DDMMYY)

		C  => { start_pos =>  29, len => 40 }, ## Name receiving client

		E1 => { start_pos =>  89, len =>  4 }, ## Agency submitting the medium
		E2 => { start_pos =>  93, len =>  4 }, ## Submitting Branch of the Submitting Agency

		F  => { start_pos => 109, len => 40 }, ## Name of Submitting Depository Agency
	};

	my $fld = HSPC::MT::Plugin::PP::BT_Norma19::Tools->splitline(
		line             => $line,
		field_definition => $field_definition
	);

	if ($fld->{A1} != &SUBMITTER_HEADER_RECORD_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_submitter_recode_code',
				string_args => {
					new_code => $fld->{A1},
					must_code => &SUBMITTER_HEADER_RECORD_CODE
				}
			}
		);
	}

	if ($fld->{A2} != &SUBMITTER_RETURN_DATA_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_data_code',
				string_args => {
					new_code => $fld->{A2},
					must_code => &SUBMITTER_RETURN_DATA_CODE
				}
			}
		);
	}

	$self->{submtt_code} = $fld->{B1};
	$self->{submtt_name} = HSPC::MT::Plugin::PP::BT_Norma19::Tools->trim( $fld->{C} );
	$self->{agency_code} = $fld->{E1};
	$self->{branch_code} = $fld->{E2};
	$self->{cdate}       = HSPC::MT::Plugin::PP::BT_Norma19::Tools->date_DDMMYY_to_hspc_format($fld->{B2});
	unless ($self->{cdate}) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_date_format',
				string_args => {
					date => $fld->{B2}
				}
			}
		);
	}

	return undef;
};

sub __parse_total {
	my $self = shift;
	my %h = (
		line => undef,
		@_
	);

	my $line = $h{line};

	my $field_definition = {
		A1 => { start_pos =>   1, len =>  2 }, ## Record Code
		A2 => { start_pos =>   3, len =>  2 }, ## Data code

		B1 => { start_pos =>   5, len => 12 }, ## Receiving Client Code

		E1 => { start_pos =>  89, len =>  10 }, ## Amounts Total 

		F1 => { start_pos => 105, len => 10 }, ## No. of Returns contained in the Medium or File
		F2 => { start_pos => 115, len => 10 }, ## No. of Records contained in the medium
	};

	my $fld = HSPC::MT::Plugin::PP::BT_Norma19::Tools->splitline(
		line             => $line,
		field_definition => $field_definition
	);

	if ($fld->{A1} != &GENERAL_TOTAL_RECORD_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_submitter_total_recode_code',
				string_args => {
					new_code => $fld->{A1},
					must_code => &SUBMITTER_HEADER_RECORD_CODE
				}
			}
		);
	}

	if ($fld->{A2} != &SUBMITTER_RETURN_DATA_CODE) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_wrong_submitter_total_data_code',
				string_args => {
					new_code => $fld->{A2},
					must_code => &SUBMITTER_RETURN_DATA_CODE
				}
			}
		);
	}

	if ($fld->{B1} ne $self->{submtt_code}) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_submitter_code_not_equal_total',
				string_args => {
					submit_code => $self->{submtt_code},
					recieved_submit_code => $fld->{B1},
				}
			}
		);
	}

	my $total_amount = round2( $fld->{E1} / 100 );
	if ( $total_amount != $self->documents_sum ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_docs_amount_dont_match',
				string_args => {
					total_amount => $total_amount,
					documents_sum => $self->documents_sum
				}
			}
		);
	}
	
	my $num_documents = $fld->{F1};
	if ( $num_documents != $self->num_documents ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_docs_count_dont_match',
				string_args => {
					num_documents => $num_documents,
					document_nums => $self->num_documents
				}
			}
		);
	}
	
	my $num_records = $fld->{F2};
	if ( $num_records != $self->num_records ) {
		throw_exception(
			type        => 'error',
			log_message => {
				string_id => 'btpp_norma19_recs_count_dont_match',
				string_args => {
					num_records => $num_records,
					record_nums => $self->num_records
				}
			}
		);
	}

	return undef;
};

##--------------------------------------------------
## Prepare record methods
##-------------------------------------------------- 
sub prepare_content {
	my $self = shift;

	my $txt = "";
	$txt .= $self->__submitter_header();
	$txt .= $self->__issuers_content();
	$txt .= $self->__general_total();

	return $txt;
}

sub __submitter_header {
	my $self = shift;

	my $txt = "";

	## A1: Record code: initial position 1, length 2, numerical
	$txt .= sprintf("%02d", &SUBMITTER_HEADER_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &SUBMITTER_DIRECT_DATA_CODE); 

	## B1: Submitter code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{submtt_code}
	);

	{ ## B2. Medium creation date (DDMMYY), initial position 17, length 6
		my $cdate = $self->{cdate};
		if ($cdate =~ /^(\d\d)(\d\d)-(\d\d)-(\d\d)/) {
			my $yy = $2;
			my $mm = $3;
			my $dd = $4;
			$txt .= sprintf("%02d%02d%02d", $dd, $mm, $yy);
		} else {
			throw_exception(
				type        => 'error',
				log_message => "Wrong cdate: $cdate"
			);
		}
	}

	## B3. Free. Initial position: 23, length 6. 
	$txt .= (" " x 6);

	## C. Name of submitting client. Initial position 29, length 40.
	$txt .= sprintf("%-40s", substr($self->{submtt_name}, 0, 40) );

	## D: Free. Initial pos: 69, lengtn 20
	$txt .= (" " x 20);

	## E1: Agency receiving the medium. Initial pos: 89, lengtn 4, numerical
	$txt .= sprintf( "%04d", $self->{agency_code} );
	
	## E2: REceiving  branch of receiving Agency. Initial pos: 93, lengtn 4, numerical
	$txt .= sprintf( "%04d", $self->{branch_code} );

	## E3: Free. Initial pos: 97, lengtn 12
	$txt .= (" " x 12);

	## F: Free. Initial pos: 109, lengtn 40
	$txt .= (" " x 40);

	## G: Free. Initial pos: 149, lengtn 14
	$txt .= (" " x 14);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of submitter header is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
};

sub __general_total {
	my $self = shift;
	
	my $txt = "";

	## A1: Record code: initial position 1, length 2, numerical
	$txt .= sprintf("%02d", &GENERAL_TOTAL_RECORD_CODE); 

	## A2: Data code: initial position 3, length 2, numerical
	$txt .= sprintf("%02d", &SUBMITTER_DIRECT_DATA_CODE); 

	## B1: Submitter code. Aligned to right, wihte zeroes to the left if necessary
	## initial postion 5, length 12
	$txt .= HSPC::MT::Plugin::PP::BT_Norma19::Tools->format_client_identifier(
		$self->{submtt_code}
	);

	## B2. Free. Initial position: 17, length 12. 
	$txt .= (" " x 12);

	## C. Free. Initial position: 29, length 40. 
	$txt .= (" " x 40);

	## D1. Number of issuers, contained in the medium or file.
	## Initial position 69, len 4, numerical.
	$txt .= sprintf( "%04d", scalar(@{$self->{issuers}}) );

	## D2. Free. Initial position: 73, length 16. 
	$txt .= (" " x 16);

	## E1. Amounts Total (Amounts Total (Sum of Records Amounts of Record Code 56 and Data Code 80)
	## Initial position 89, len 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", round($self->documents_sum * 100, 0) );
	
	## E2. Free. Initial position: 99, length 6. 
	$txt .= (" " x 6);

	## F1. Total No. of compulsory individual records. (really documents)
	## Initial position 105, len 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", $self->num_documents );

	## F2. No. of records contained in the Medium,
	## including the Header, the Issuer Total and the
	## General Total record itself
	## Initial position 115, len 10, numerical
	## I use "%010s" here, because of numbers like 2710001001 cause overflow in case %010d
	$txt .= sprintf( "%010s", $self->num_records );

	## F3. Free. Initial position: 125, length 20. 
	$txt .= (" " x 20);

	## G. Free. Initial position: 145, length 18. 
	$txt .= (" " x 18);

	if (length($txt) != 162) {
		throw_exception(
			type        => 'error',
			log_message => "Length of general total record is wrong: '$txt'"
		);
	}

	$txt .= "\n";
	return $txt;
};

sub __issuers_content {
	my $self = shift;

	my $txt;
	my $issuer_count = 0;
	foreach my $issuer (@{$self->{issuers}}) {
		$txt .= $issuer->prepare_content();
		$issuer_count++;
	}

	throw_exception(
		type        => 'error',
		log_message => 'No issuers specified'
	) if $issuer_count <= 0;

	return $txt;
};


##--------------------------------------------------
## fill object methods
##-------------------------------------------------- 
sub add_issuer {
    my $self = shift;
    my %h = (
        issuer => undef,
        @_
    );

	my $issuer = $h{issuer} ||
		throw_exception(
			type        => 'error',
			log_message => 'issuer is a mandatory field'
		);

    push @{$self->{issuers}}, $issuer;
    
	return undef;
} 

##--------------------------------------------------
## Accessors
##-------------------------------------------------- 
sub num_records {
	my $self = shift;

	my $numrecs = 2; ## Submitter heade + total.
	foreach my $issuer (@{$self->{issuers}}) {
		$numrecs += $issuer->num_records();
	}

	return $numrecs;
};

sub num_documents {
	my $self = shift;

	my $numdocs = 0; 
	foreach my $issuer (@{$self->{issuers}}) {
		$numdocs += $issuer->num_documents();
	}

	return $numdocs;
};

sub documents_sum {
	my $self = shift;

	my $sum = 0.00; ## Submitter heade + total.
	foreach my $issuer (@{$self->{issuers}}) {
		$sum += $issuer->documents_sum();
	}

	$sum = round2($sum);
	return $sum;
};

## check submitter data consistency
sub __check {
	my $self = shift;

    my $submtt_code = $self->{submtt_code};
    my $submtt_name = $self->{submtt_name};
    my $agency_code = $self->{agency_code};
    my $branch_code = $self->{branch_code};
    my $cdate = $self->{cdate};

	## check submitter code. According documentation it length max 12, alphanumerical
    if (($submtt_code =~ /\W|_/) || (length($submtt_code) > 12)) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Submitter code '$submtt_code'!" .
				' It should be alfanumerical and not bigger than 12 chars.'
		);
    }

	## check Submitting Client
    if (length($submtt_name) > 40) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Submiting client '$submtt_name'!" .
				' It should be alfanumerical and not bigger than 40 chars.'
		);
    }

	## check Agency, receiving the file
    if (($agency_code =~ /\D/) || (length($agency_code) > 4)) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Agency code '$agency_code'!" .
				' It should be numerical and not bigger than 4 chars.'
		);
    }

	## check Branch, receiving the file
    if (($branch_code =~ /\D/) || (length($branch_code) > 4)) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of Branch code '$branch_code'!" .
				' It should be numerical and not bigger than 4 chars.'
		);
    }

	## check date
    if ($cdate !~ /(\d\d\d\d)-(\d\d)-(\d\d)/) {
		throw_exception(
			type        => 'error',
			log_message => "Wrong format of file creation date '$cdate'!"
		);
    }

	return undef;
};

1;

__END__
