package HSPC::MT::Plugin::PP::OP_BBVA;

use strict;

use HSPC::PluginToolkit::General qw(get_data_dir string log_warn log_debug throw_exception);

use Digest::SHA1 qw(sha1_hex);
use Encode::Byte qw(encode);
use XML::Simple qw(XMLin);
use SOAP::Lite;

use constant PAYMETHOD_MAP => {
	'V' => 'VISA'
};

use constant CURRENCY_MAP => {
	EUR => 978,
};

## Fields to be shadow while logging
use constant FIELD2SHADOW => [qw{}];## todo

sub get_title {
	my $class = shift;

	return string('bbva_title');
}

sub get_supported_payment_method_types{
	my $self = shift;

	return [
		'V'
	];
}

sub process_sale {
	my $self = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $document_info = $h{document_info};

 	throw_exception(
 		type        => 'error',
 		log_message => 'Internal error:  No order_id'
 	) unless defined($document_info->{id});

	return $self->_process_transaction(
				%h,
				trans_type => 'SALE'
			);
}

sub _compose_refnum{
	my $self = shift;
	my %h = (
				order_id      => undef,
				pr_trans_data => undef,
				@_
			);

	my $ret;
	my $order_id      = $h{order_id};

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(time);

	$ret = sprintf(
		"%02d%02d%02d%02d%02d%02d",
		$min, $hour, $mday, $mon,
		substr( $year,     length($year) - 2,     2 ),
		substr( $order_id, length($order_id) - 2, 2 )
	);

	return $ret;
}

## Description:
##	The metod composes transaction specific data to be sent to gateway.
## Usage:
##		$plugin->_compose_data(desc => 'bla-bla-bla', ccard => $ccard, etc.);
## Parameters:
##	>>  self
##	=>> desc
##	=>> ccard
##	=>> refnum
##	=>> account_info
##	=>> total_paid
##	=>> trans_type
##  =>> transaction_id
##  =>> currency_iso
## Return:
##		data hash
sub _compose_request {
	my $self = shift;
	my %h = (
				  desc             => undef,
				  ccard            => undef,
				  config           => undef,
				  refnum           => undef,
				  order_id         => undef,
				  total_paid       => undef,
				  trans_type       => undef,
				  currency_iso     => undef,
				  secure_signature => undef,
				  @_
			);
	my $desc             = $h{desc};
	my $ccard            = $h{ccard};
	my $config           = $h{config};
	my $refnum           = $h{refnum};
	my $order_id         = $h{order_id};
	my $total_paid       = $h{total_paid};
	my $trans_type       = $h{trans_type};
	my $currency_iso     = $h{currency_iso};
	my $secure_signature = $h{secure_signature};

	throw_exception(
		type        => 'error',
		log_message => string('total_summ_undef', order_id => $order_id)
	) unless ( $total_paid && $total_paid > 0 );
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No ccard'
	) unless $ccard;
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No trans_type'
	) unless $trans_type;

	my $secure_data = $ccard->{secure_data};
	my $public_data = $ccard->{public_data};

	my $terminal_id         = $config->{terminal_id};
	my $merchant_id         = $config->{merchant_id};
	my $currency            = &CURRENCY_MAP->{$currency_iso};
	my $private_card_number = $secure_data->{card_number};
	my $expire_date         = $self->_expiry_format($ccard->{expire_date});
	$total_paid             = sprintf('%.2f', $total_paid);
	$total_paid             =~ s/\./,/;

	my $data;
	my $ccard_type = &PAYMETHOD_MAP->{$ccard->{type}};


	## merchant info
	if ($trans_type eq "SALE")
	{
		$data = <<XML;
		<tpv>
			<oppago>
				<idterminal>$terminal_id</idterminal>
				<idcomercio>$merchant_id</idcomercio>
				<idtransaccion>$refnum</idtransaccion>
				<moneda>$currency</moneda>
				<importe>$total_paid</importe>
				<numtarjeta>$private_card_number</numtarjeta>
				<fechacaducidad>$expire_date</fechacaducidad>
				<localizador>$order_id</localizador>
				<firma>$secure_signature->{secure_signature}</firma>
			</oppago>
		</tpv>
XML
	}
	else {
		throw_exception(
			type        => 'error',
			log_message => 'Internal error: Wrong trans_type submited!'
		);
	}

	return $data;
}

## Description:
##		The method cuts off confidential info from XML-message.
## Usage:
##		$self->_safe_dump($xml);
## Parameters:
##	>>  self
##	>>  xml message
## Return:
##		xml message without confidential information.
sub _safe_dump {
	my $self = shift;
	my $data = shift;
	$data =~ s/<numtarjeta>\d{1}\d*(\d{4})<\/numtarjeta>/<numtarjeta>*****$1<\/numtarjeta>/;

	return $data;
}


## Description:
##	The metod composes transaction specific data to be sent to gateway.
## Usage:
##		$plugin->_compose_data(desc => 'bla-bla-bla', ccard => $ccard, etc.);
## Parameters:
##	>>  self
##	=>> data
##	trans_type
## Return:
##		reply hash
sub _send_request {
	my $self = shift;
	my %h = (
				data       => undef,
				config     => undef,
				trans_type => undef, ## unused here
				@_
			);
	my $response;
	my $reply;
	my $request    = $h{data};
	my $config     = $h{config};
	my $trans_type = $h{trans_type};

	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No data'
	) unless $request;

	my $trans_type   = $h{trans_type};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No trans_type'
	) unless $trans_type;

	my $config       = $h{config};
	throw_exception(
		type        => 'error',
		log_message => 'Internal error: No config'
	) unless $config;

	my $host       = $config->{payment_url};
	my $xmlns_pp   = 'PeticionTPVSoapS';
	my $error;

	log_debug(
		"Request to gateway ("
		. $host
		. "):\n"
		. $self->_safe_dump($request)
	);
	my $meth     = SOAP::Data->name('procesarPago')->uri($xmlns_pp);
	my $soap     = SOAP::Lite->proxy($host)->uri($xmlns_pp);
	my $response = $soap->call($meth => $request);

	if (!$response) {
		$reply->{error} .= string('srv_comm_err');
		sw_log_error($reply->{error});
	} elsif ($response->fault) {
		$reply->{error} .= string('fault_detail') . $response->faultdetail . "\n";
		$reply->{error} .= string('fault_code')   . $response->faultcode . "\n";
		$reply->{error} .= string('fault_string') . $response->faultstring . "\n";
		$reply->{error} .= string('fault_actor')  . $response->faultactor . "\n";
		sw_log_error($reply->{error});
	} else {
		$response = $response->match('//return');
		log_debug(
			"Reply from gateway ("
			. $host
			. "):\n"
			. $response->valueof
		);
	}

	if (!$reply->{error}) {
		$reply->{data} = XMLin($response->valueof);
	}

	return $reply;
}

## Description:
##		The method composes details hash for subsequent
##		storing them in TransLog.
##		completed or not.
## Usage:
##		$self->_get_details(reply => $reply, refnum => $refnum);
## Parameters:
##	>>  self
##	=>> reply
##	=>  trans_type (is not used here)
## Return:
##		details hash
sub _get_details {
	my $self = shift;
	my %h = (
				reply      => undef,
				refnum     => undef, ## only for preauth and sale
				trans_type => undef, ## dummy here
				config     => undef,
				@_
			);
	my $refnum     = $h{refnum};
	my $trans_type = $h{trans_type};
	my $reply      = $h{reply};
	my $config     = $h{config};
	my $details    = {};

	if ($reply->{error}){
		$details->{error} .= $reply->{error};
	} elsif ($reply->{data}->{coderror}) {
		$details->{error} .= $reply->{data}->{coderror} . "\n";
		$details->{error} .= $reply->{data}->{deserror} . "\n";
	} else {
		my $check_response = $self->_check_response_signature(
			reply  => $reply,
			config => $config,
		);
		if ($check_response->{error}) {
			$details->{error} .= $check_response->{error};
		} elsif ($reply->{data}->{respago}->{coderror} != 0) {
			$details->{declined_tr} = $reply->{data}->{respago}->{coderror} . " " .
									$reply->{data}->{respago}->{deserror};
			$details->{transaction_id} = $reply->{data}->{respago}->{idtransaccion};
		} else {
			$details->{transaction_id} = $reply->{data}->{respago}->{idtransaccion};
		}
	}

	$details->{error} = $reply->{error} if $reply->{error};

	return $details;
}

## Description:
##		The method checks if the current transaction has been successfully
##		completed or not.
## Usage:
##		$self->_safe_dump($details);
## Parameters:
##	>>  self
##	>>  details
## Return:
##		0 on error, 1 on success
sub _is_trans_ok {
	my $self = shift;
	my $details = shift;

	return !$details->{declined_tr} ? 1 : 0;
}

## Description:
##		The method composes result to be returned
##		outside the plugin.
## Usage:
##		$self->_create_result(details => $details, total_paid => $total);
## Parameters:
##	>>  self
##	=>> details
##	=>> total_paid
## Return:
##		result_hash
sub _create_result {
	my $self = shift;
	my %h = (
				details    => undef,
				total_paid => undef,
				@_
			);
	my $details    = $h{details};
	my $total_paid = $h{total_paid};

	if (!$details || ($details && $details->{error})) {
		throw_exception(
			type        => 'error',
			log_message => "Internal error: " . $details->{error},
		);
	}

	if($self->_is_trans_ok($details)) {
		return {
			STATUS               => 'APPROVED',
			TRANSACTION_DETAILS  => $details,
		};
	} else {
		return {
			STATUS               => 'DECLINED',
			TRANSACTION_DETAILS  => $details,
			TEXT => {
				vendor_message   => string('trans_declined'),
			},
		};
	}
}

## Description:
##		process_* methods wrap this one. All common parameters
##		checkings are here.
## Usage:
##		$self->_process_transaction(
##			@_,
##			trans_type => &SW_CCP_TRANS_TYPE_SALE
##		);
## Parameters:
##	>>  self
#### Parameter for wrapper
##	=>> trans_type
#### Parameters from caller to plugin methods
##	=>> ccard
##	=>> order_id
##	=>> trans_id          ## needed in process_capture only
##	=>> trans_type
##	=>> account_no
##	=>> total_paid
##	=>  description
##	=>> now_trans_id
##	=>> is_card_verified
## Return:
##		result hash
sub _process_transaction {
	my $self = shift;
	my %h = (
				config                    => undef,
				document_info             => undef,
				account_info              => undef,
				payment_method            => undef,
				transaction_id            => undef,
				transaction_amount        => undef,
				trans_type                => undef,
				previous_transaction_data => undef,
				currency_iso              => undef,
				@_
			);
	my $ccard         = $h{payment_method};
	my $trans_type    = $h{trans_type};
	my $total_paid    = $h{total_paid};
	my $document_info = $h{document_info};
	my $total_paid    = $h{transaction_amount};
	my $currency_iso  = $h{currency_iso};
	my $config        = $h{config};

	my $desc     = $document_info->{description};
	my $order_id = $document_info->{id};

 	throw_exception(
 		type        => 'error',
		log_message => 'Internal error: No ccard'
	) unless $ccard;

	throw_exception(
		type        => 'error',
		log_message => string('total_summ_undef', order_id => $order_id)
	) unless ( $total_paid && $total_paid > 0 );

	my $reply;
	my $refnum = $self->_compose_refnum(
		order_id => $order_id,
	);

	my $secure_signature = $self->_get_secure_signature(
		refnum       => $refnum,
		total_paid   => $total_paid,
		order_id     => $order_id,
		config       => $config,
		currency_iso => $currency_iso,
	);

	if ($secure_signature->{error}) {
		return $self->_create_result(
					details    => {error => $secure_signature->{error}},
					total_paid => $total_paid,
			   );
	}

	my $data = $self->_compose_request(
					desc             => $desc,
					ccard            => $ccard,
					config           => $config,
					refnum           => $refnum,
					order_id         => $order_id,
					total_paid       => $total_paid,
					trans_type       => $trans_type,
					currency_iso     => $currency_iso,
					secure_signature => $secure_signature
			   );

	$reply = $self->_send_request(
						data       => $data,
						config     => $config,
						trans_type => $trans_type,
					);

	my $details = $self->_get_details(
					reply      => $reply,
					config     => $config,
					refnum     => $refnum,
					trans_type => $trans_type,
				  );

	return $self->_create_result(
				details => $details,
				total_paid => $total_paid,
		   );
}

sub get_currencies_supported{

	return [
		'EUR', ##Euro
	];
}

sub _xor_bytes {
	my $self = shift;
	my $first_array  = shift;
	my $second_array = shift;

	return {error => ('secure key more then "xor admin key".')} if length($first_array) > length($second_array);

	my @xor_bytes_array;

	my $index = 0;
	foreach (@{$first_array}) {
		$xor_bytes_array[$index] = int($first_array->[$index]) ^ int($second_array->[$index]);
		$index++;
	}

	return {xor_bytes_array => \@xor_bytes_array};
}

sub _decode_secure_word {
	my $self = shift;
	my $adm_secure_key_str = shift;
	my $xor_join_str = shift;

	$xor_join_str      = Encode::Byte::encode("iso-8859-1", $xor_join_str);

	$xor_join_str      =~ s/(.)/ord("$1") . "," \./g;
	$xor_join_str      =~ s/\.$//g;
	$xor_join_str      = eval $xor_join_str;

	my @xor_join_bytes = split ("," , $xor_join_str);

	$adm_secure_key_str      =~ s/([^;])$/$1;/;
	$adm_secure_key_str      =~ s/([0-9,A-F]*);/hex("$1") . "," \./ig;
	$adm_secure_key_str      =~ s/\.$//ig;
	$adm_secure_key_str      = eval $adm_secure_key_str;

	my @adm_secure_key_bytes = split ("," , $adm_secure_key_str);

	my $xor_bytes            = $self->_xor_bytes(\@adm_secure_key_bytes, \@xor_join_bytes);

	my $decodrd_secure_word_bytes;

	if ($xor_bytes->{error}) {
		return {error => $xor_bytes->{error}};
	} else {
		$decodrd_secure_word_bytes = $xor_bytes->{xor_bytes_array};
	}

	@{$decodrd_secure_word_bytes}    = map (chr, @{$decodrd_secure_word_bytes});

	my $decodrd_secure_word       = join('' , @{$decodrd_secure_word_bytes});
	$decodrd_secure_word          = Encode::Byte::encode("iso-8859-1", $decodrd_secure_word);

	return {decoded_secure_word => $decodrd_secure_word};
}

sub _get_secure_signature{
	my $self   = shift;
	my %h      = (
		refnum           => undef,
		total_paid       => undef,
		order_id         => undef,
		config           => undef,
		currency_iso     => undef,
		@_
	);

	my $refnum             = $h{refnum};
	my $total_paid         = $h{total_paid};
	my $order_id           = $h{order_id};
	my $config             = $h{config};
	my $currency_iso       = $h{currency_iso};

	my $secure_word        = $config->{secure_word};
	my $secure_admin_key   = $config->{secure_admin_key};
	my $merchant_id        = $config->{merchant_id};

	my $total_only_numbers = sprintf('%.2f', $total_paid);
	$total_only_numbers    =~ s/\D//;

	my $xor_join_str = $secure_admin_key . substr($merchant_id, 0 ,9) . "***";

	my $decode_secure_word = $self->_decode_secure_word($secure_word, $xor_join_str);

	if ($decode_secure_word->{error}) {
		return {error => $decode_secure_word->{error}};
	} else {
		return {
				secure_signature => uc(Digest::SHA1::sha1_hex(
											$config->{terminal_id}                           .
											$config->{merchant_id}                           .
											$refnum                                          .
											$total_only_numbers                              .
											&CURRENCY_MAP->{$currency_iso} .
											$order_id                                        .
											$decode_secure_word->{decoded_secure_word}
				))
		};
	}
}

## Description:
##		The method composes exparity date
##		in appropriate for BBVA format.
## Usage:
##		$self->_expiry_format($date);
## Parameters:
##	>>  self
##	>>  date
## Return:
##		BBVA date
sub _expiry_format {
	my $self = shift;
	my $date = shift;

	my ($yy,$mm) = $date =~ /^\s*(\d{4})\D(\d{2})/;

	return $yy . sprintf("%02d", $mm);
}

sub _check_response_signature {
	my $self  = shift;
	my %h      = (
		reply  => undef,
		config => undef,
		@_
	);

	my $reply  = $h{reply};
	my $config = $h{config};
	my $data   = $reply->{data};
	my $secure_signature;

	my $secure_word        = $config->{secure_word};
	my $secure_admin_key   = $config->{secure_admin_key};
	my $merchant_id        = $config->{merchant_id};

	my $total_only_numbers = $data->{respago}->{importe};
	$total_only_numbers    =~ s/\D//;

	my $xor_join_str = $secure_admin_key . substr($merchant_id, 0 ,9) . "***";

	my $decode_secure_word = $self->_decode_secure_word($secure_word, $xor_join_str);

	if ($decode_secure_word->{error}) {
		return {error => $decode_secure_word->{error}};
	} else {
		$secure_signature = uc(Digest::SHA1::sha1_hex(
											$data->{respago}->{idterminal}      .
											$data->{respago}->{idcomercio}      .
											$data->{respago}->{idtransaccion}   .
											$total_only_numbers                 .
											$data->{respago}->{moneda}          .
											$data->{respago}->{estado}          .
											$data->{respago}->{coderror}        .
											$data->{respago}->{codautorizacion} .
											$decode_secure_word->{decoded_secure_word}
		));
	}

	if ($data->{respago}->{firma} ne $secure_signature){
		return {error => string('secure_signature_not_match')};
	} else {
		return undef;
	}
}

1;