package HSPC::MT::Plugin::PP::OP_BancaSella;
use strict;
use HSPC::MT::Plugin::PP::OP_BancaSella::OP_BancaSellaAPI;
use HSPC::PluginToolkit::General qw(throw_exception log log_debug log_warn string round2 argparam uriparam);
use Data::Dumper;

use constant CURRENCIES => {
	EUR => 242,
	ITL => 18,
	USD => 1,
	GBP => 2,
};

use constant MAP_LOCALES => {
	IT => { mapping => 1, default => 1 },
	EN => { mapping => 2 },
	ES => { mapping => 3 },
	FR => { mapping => 4 },
	GR => { mapping => 5 },
};

sub get_title {
	my $class = shift;

	return string("bancasella_naming");
}

sub get_currencies_supported{
	return [keys %{&CURRENCIES}];
}

sub map_locale {
	my $class  = shift;
	my $locale = uc(shift);
	my $ret;
	my $t = &MAP_LOCALES;

	$ret = $t->{$locale}->{mapping} if $t->{$locale};

	## There is no appropriate mapping, use mapping for default
	## locale.
	if ( !$ret ) {
		my $l;
		for $l ( keys %$t ) {
			if ( $t->{$l}->{default} ) {
				$ret = $t->{$l}->{mapping};
				last;
			}
		}
	}
	return $ret;
}

sub process_sale {
	my $self = shift;
	my %h = (
		transaction_amount => undef,
		account_info       => undef,
		config             => undef,
		currency_iso       => undef,
		current_lang       => undef,
		payment_method     => undef,
		document_info      => undef,
		ref_no             => undef,
		@_
	);
	my $trans_amount   = $h{transaction_amount};
	my $account_info   = $h{account_info};
	my $config         = $h{config};
	my $currency_iso   = $h{currency_iso};
	my $current_lang   = $h{current_lang};
	my $payment_method = $h{payment_method};
	my $document_info  = $h{document_info};
	my $ref_no         = $h{ref_no};

	my $objCrypt = new HSPC::MT::Plugin::PP::OP_BancaSella::OP_BancaSellaAPI;
	$objCrypt->{DomainName} = $self->_get_domain_from_url($h{config}->{server_url});
	$objCrypt->SetShopLogin($config->{merchant_id});
	$objCrypt->SetCurrency(	&CURRENCIES->{$currency_iso} );
	$objCrypt->SetAmount( round2($trans_amount) );
	$objCrypt->SetShopTransactionID( $ref_no );

	if ( !$config->{basic_agreement} ) {
		$objCrypt->SetLanguage(
				$self->map_locale( $current_lang )
		);
		$objCrypt->SetBuyerName( $account_info->{billing_fname}
				. " " . $account_info->{billing_lname} );
		$objCrypt->SetBuyerEmail( $account_info->{billing_email} );
	}

	$objCrypt->Encrypt();

	my $dat;
	if ( $objCrypt->GetErrorCode() ) {
		throw_exception(
			type           => 'error',
			log_message    => $objCrypt->GetErrorCode() . ' '
					. $objCrypt->GetErrorDescription(),
			vendor_message => $objCrypt->GetErrorCode() . ' '
					. $objCrypt->GetErrorDescription(),
		);
	} else {
		$dat->{a} = $objCrypt->GetShopLogin();
		$dat->{b} = $objCrypt->GetEncryptedString();
	}

	log_debug( "Data to be posted to BancaSella gateway.: "
		. Dumper($dat) );

	return {
		STATUS        => 'REDIRECT',
		REDIRECT_HASH => {
			url     => $config->{server_url},
			attrs   => $dat,
			method  => "POST",
		},
	};
}

sub collect_transaction_refno{
	my $class = shift;

	my $a = argparam('a');
	my $b = argparam('b');

	my $objCrypt = new HSPC::MT::Plugin::PP::OP_BancaSella::OP_BancaSellaAPI;
	$objCrypt->{DomainName} = uriparam("server_url") if uriparam("server_url");
	$objCrypt->SetShopLogin($a);
	$objCrypt->SetEncryptedString($b);
	$objCrypt->Decrypt();

	return $objCrypt->GetShopTransactionID();
}

sub process_callback{
	my $class = shift;
	my $action = uriparam('action');
	my %h = (
		config             => undef,
		document_info      => undef,
		transaction_amount => undef,
		@_
	);

	my $config    = $h{config};
	my $doc_info  = $h{document_info};
	my $amount    = $h{transaction_amount};

	if ($action eq 'result') {
		my $result = uriparam('result');

		my $status;
		my $desc;

		if ($result eq 'Success') {
			$status = "APPROVED";
		} else {
			$status = "DECLINED";
			$desc   = string('err_trans_explained');
		}

		return {
			STATUS => $status,
			TEXT                 => {
				customer_message => $desc,
			},
			ACTION => 'restore_session',
		};
	} elsif ($action eq 'notify') {
		return $class->_process_notification(
			doc_info => $doc_info,
			config   => $config,
			amount   => $amount,
		);
	}
}

sub _process_notification {
	my $class     = shift;
	my %h = (
			doc_info => undef,
			config   => undef,
			amount   => undef,
			@_
		);
	my $doc_info = $h{doc_info};
	my $config   = $h{config};
	my $amount   = $h{amount};

	my $a = argparam('a');
	my $b = argparam('b');

	my $objCrypt = new HSPC::MT::Plugin::PP::OP_BancaSella::OP_BancaSellaAPI;
	$objCrypt->{DomainName} = $class->_get_domain_from_url($h{config}->{server_url});
	$objCrypt->SetShopLogin($a);
	$objCrypt->SetEncryptedString($b);
	$objCrypt->Decrypt();

	my $merch_id = $objCrypt->GetShopLogin();
	my $total    = $objCrypt->GetAmount();
	my $err;
	if ( $merch_id ne $config->{merchant_id} ) {
		## Bad total
		$err = "bancasella_bad_total_received";
	}
	if ( $total != round2($amount) ) {
		## Bad merchant id recieved
		$err = "bancasella_bad_merchant_recieved";
	}
	my $details;
	my $status;

	if ($err) {
		$details->{ccp_error} = $err;
		$status = 'ERROR';
	} else {
		$details = $class->_get_details(data => $objCrypt);
		if ($objCrypt->GetTransactionResult() eq "OK") {
			$status = "APPROVED";
		} else {
			$status = "DECLINED";
		}
	}

	return {
		STATUS               => $status,
		ACTION               => 'update',
		TRANSACTION_DETAILS  => $details,
	};
}

sub _get_details {
	my $self = shift;
	my %h = (
		data => undef,
		@_
	);
	my $ret;
	my $data = $h{data} || sw_die('data undefined');

	$ret->{refnum}	  = $data->GetBankTransactionID();
	$ret->{error_code}= $data->GetErrorCode();
	$ret->{error}     = $data->GetErrorDescription();
	$ret->{authcode}  = $data->GetAuthorizationCode();

	return $ret;
}

sub _get_domain_from_url {
	my $class = shift;
	my $url = shift;
	return $1 if $url =~ m#.+://([^/:]+)(/|:|$)#;
	return "";
}

1;
