package HSPC::MT::Plugin::PP::OP_BancaSellaSSL;

use strict;

use HSPC::PluginToolkit::General qw(round2 throw_exception log log_debug log_warn string split_date_string uriparam argparam);
use HSPC::MT::Plugin::PP::OP_BancaSellaSSL::OP_BancaSellaAPI;

use Data::Dumper;

use Digest::MD5  qw(md5_hex);
use LWP::UserAgent;
use HTTP::Request;
use HTTP::Request::Common;
use HTTP::Cookies;

use constant CONNECTION_TIMEOUT => 30;
use constant TIMEOUT => 30;

use constant MAP_CURRENCIES => {
	EUR => { mapping => 242 },
	ITL => { mapping => 18 },
	USD => { mapping => 1 },
	GBP => { mapping => 2 },
};

use constant MAP_LOCALES => {
	IT => { mapping => 1, default => 1 },
	EN => { mapping => 2 },
	ES => { mapping => 3 },
	FR => { mapping => 4 },
	GR => { mapping => 5 },
};

sub get_title {
	my $class = shift;

	return string("bancasellassl_naming");
}

sub get_supported_payment_method_types{
	my $class = shift;

	return [
		'V',
		'M',
		'A',
		'C',
		'L',
	];
}

sub process_sale {
	my $self = shift;
	my %h = (
		transaction_amount => undef,
		account_info       => undef,
		config             => undef,
		currency_iso       => undef,
		current_lang       => undef,
		payment_method     => undef,
		document_info      => undef,
		transaction_refnum => undef,
		@_
	);
	my $refno          = $h{transaction_refnum};
	my $trans_amount   = $h{transaction_amount};
	my $account_info   = $h{account_info};
	my $config         = $h{config};
	my $currency_iso   = $h{currency_iso};
	my $current_lang   = $h{current_lang};
	my $payment_method = $h{payment_method};
	my $document_info  = $h{document_info};

	my $data = $self->_compose_request(
		transaction_amount => $trans_amount,
		account_info       => $account_info,
		config             => $config,
		currency_iso       => $currency_iso,
		refno              => $refno,
		current_lang       => $current_lang,
		payment_method     => $payment_method,
	);

	my $resp = $self->_send_request(
		data   => $data,
		config => $config,
	);

	return  {
		STATUS        => 'PENDING',
	};
}

sub collect_transaction_refno{
	my $self = shift;
	my %h = (
		config => undef,
	);

	my $a = argparam('a');
	my $b = argparam('b');

	my $objCrypt = new HSPC::MT::Plugin::PP::OP_BancaSellaSSL::OP_BancaSellaAPI;
	$objCrypt->{DomainName} = uriparam("server_url") if uriparam("server_url");
	$objCrypt->SetShopLogin($a);
	$objCrypt->SetEncryptedString($b);
	$objCrypt->Decrypt();

	return $objCrypt->GetShopTransactionID();
}

sub process_callback{
	my $class = shift;
	my $action = uriparam('action');
	my %h = (
		config             => undef,
		document_info      => undef,
		transaction_amount => undef,
		@_
	);

	my $config    = $h{config};
	my $doc_info  = $h{document_info};
	my $amount    = $h{transaction_amount};

	if ($action eq 'notify') {
		return $class->_process_notification(
			doc_info => $doc_info,
			config   => $config,
			amount   => $amount,
		);
	}
}

sub _process_notification {
	my $class     = shift;
	my %h = (
			doc_info => undef,
			config   => undef,
			amount   => undef,
			@_
		);
	my $doc_info = $h{doc_info};
	my $config   = $h{config};
	my $amount   = $h{amount};

	my $a = argparam('a');
	my $b = argparam('b');

	my $objCrypt = new HSPC::MT::Plugin::PP::OP_BancaSellaSSL::OP_BancaSellaAPI;
	$objCrypt->{DomainName} = $class->_get_domain_from_url($h{config}->{server_url});
	$objCrypt->SetShopLogin($a);
	$objCrypt->SetEncryptedString($b);
	$objCrypt->Decrypt();

	my $merch_id = $objCrypt->GetShopLogin();
	my $total    = $objCrypt->GetAmount();
	my $err;
	if ( $merch_id ne $config->{merchant_id} ) {
		## Bad total
		$err = "bancasellassl_bad_total_received";
	}
	if ( $total != round2($amount) ) {
		## Bad merchant id recieved
		$err = "bancasellassl_bad_merchant_recieved";
	}
	my $details;
	my $status;

	if ($err) {
		$details->{ccp_error} = $err;
		$status = 'ERROR';
	} else {
		$details = $class->_get_details(data => $objCrypt);
		if ($objCrypt->GetTransactionResult() eq "OK") {
			$status = "APPROVED";
		} else {
			$status = "DECLINED";
		}
	}

	return {
		STATUS               => $status,
		ACTION               => 'update',
		TRANSACTION_DETAILS  => $details,
	};
}

sub _get_details {
	my $self = shift;
	my %h = (
		data => undef,
		@_
	);
	my $ret;
	my $data = $h{data} || sw_die('data undefined');

	$ret->{refnum}	   = $data->GetBankTransactionID();
	$ret->{error_code} = $data->GetErrorCode();
	$ret->{error}      = $data->GetErrorDescription();
	$ret->{authcode}   = $data->GetAuthorizationCode();

	return $ret;
}

sub _compose_request {
	my $class = shift;
	my %h = (
		transaction_amount => undef,
		account_info       => undef,
		config             => undef,
		currency_iso       => undef,
		refno              => undef,
		current_lang       => undef,
		payment_method     => undef,
		@_
	);
	my $transaction_amount = $h{transaction_amount};
	my $account_info       = $h{account_info};
	my $config             = $h{config};
	my $currency_iso       = $h{currency_iso};
	my $refno              = $h{refno};
	my $current_lang       = $h{current_lang};
	my $payment_method     = $h{payment_method};

	throw_exception (
		type        => 'error',
		log_message => '_compose_request: config undefined'
	) unless $config;

	my $objCrypt = new HSPC::MT::Plugin::PP::OP_BancaSellaSSL::OP_BancaSellaAPI;
	$objCrypt->{DomainName} = $class->_get_domain_from_url($h{config}->{server_url});

	$objCrypt->SetShopLogin( $config->{merchant_id} );

	$objCrypt->SetCurrency( $class->map_currency($currency_iso) );

	$objCrypt->SetAmount( round2($transaction_amount) );
	$objCrypt->SetShopTransactionID( $refno );

	$objCrypt->SetLanguage( $class->map_locale($current_lang) );

	$objCrypt->SetBuyerName(
		$account_info->{billing_fname} . ' ' . $account_info->{billing_lname} );
	$objCrypt->SetBuyerEmail( $account_info->{billing_email} );

	my %date = split_date_string( $payment_method->{expire_date} );

	$objCrypt->SetExpMonth($date{month});
	$objCrypt->SetExpYear( substr($date{year} , 2, 2 ) );
	$objCrypt->SetCardNumber( $payment_method->{secure_data}->{card_number} );
	## $objCrypt->SetCVV($payment_method->{secure_data}->{cvv}); ## currently cvv not supported by gateway.
	## Will be uncomented when Banca Sella starts support cvv codes
	$objCrypt->Encrypt();

	my $dat;
	if ( $objCrypt->GetErrorCode() ) {
		throw_exception(
			type           => 'error',
			log_message    => $objCrypt->GetErrorCode() . " " . $objCrypt->GetErrorDescription(),
			vendor_message => $objCrypt->GetErrorDescription(),
		);
	} else {
		$dat->{a} = $objCrypt->GetShopLogin();
		$dat->{b} = $objCrypt->GetEncryptedString();
	}

	return $dat;
}

sub _send_request {
	my $self = shift;
	my %h = (
		data   => undef,
		config => undef,
		@_
	);

	my $data   = $h{data};
	my $config = $h{config};

	log_debug(
		"Sent to gateway: ("
		. $config->{server_url}
		. ")\n '"
		. Dumper($data)
		. "'"
	);

	my $content;
	my $ua = LWP::UserAgent->new(timeout => &CONNECTION_TIMEOUT);
	$ua->cookie_jar({});
	my $req = POST $config->{server_url}, $data;
	my $resp;

	eval {
		local $SIG{ALRM} = sub { die "alarm\n" };
		alarm &TIMEOUT;
		$resp = $ua->request($req);
		alarm 0;
	};

	if ($@) {
		log_warn("Request Error: $@");
		throw_exception(
			type           => 'error',
			log_message    => string(
				'bancasellassl_request_error',
				error => $@
			),
			vendor_message => string(
				'bancasellassl_request_error',
				error => $@
			)
		);
	} else {
		my $location = $resp->headers->{location};
		if (!$location) {
			my $content = $resp->content();
			$content =~ /location.href = "(.*)"/;
			$location = $1;
		}
		if ($location =~ s/^.*(\?.*)$/$1/) {
			$req = GET $config->{server_url} . $location;
			eval {
				local $SIG{ALRM} = sub { die "alarm\n" };
				alarm &TIMEOUT;
				$resp = $ua->request($req);
				alarm 0;
			};

			if ($@) {
				log_warn("Request Error: $@");
				throw_exception(
					type           => 'error',
					log_message    => string(
						'bancasellassl_request_error',
						error => $@
					),
					vendor_message => string(
						'bancasellassl_request_error',
						error => $@
					)
				);
			}
		}
	}
}

sub map_currency {
	my $class = shift;
	my $curr  = shift;
	my $ret;

	$ret = &MAP_CURRENCIES->{$curr}->{mapping} if &MAP_CURRENCIES->{$curr};

	return $ret;
}

sub map_locale {
	my $class  = shift;
	my $locale = uc(shift);
	my $ret;
	my $t = &MAP_LOCALES;

	$ret = $t->{$locale}->{mapping} if $t->{$locale};

	## There is no appropriate mapping, use mapping for default
	## locale.
	if ( !$ret ) {
		my $l;
		for $l ( keys %$t ) {
			if ( $t->{$l}->{default} ) {
				$ret = $t->{$l}->{mapping};
				last;
			}
		}
	}
	return $ret;
}

sub get_currencies_supported {
	my $class = shift;
	return [ 'EUR', 'ITL', 'USD', 'GBP', ];
}

sub _get_domain_from_url {
	my $class = shift;
	my $url = shift;
	return $1 if $url =~ m#.+://([^/:]+)(/|:|$)#;
	return "";
}

1;