package HSPC::MT::Plugin::PP::OP_Nordea;

use strict;

use HSPC::PluginToolkit::General
			qw(currency_precision secs_to_time get_data_dir throw_exception log log_debug log_warn string round2 argparam uriparam);
use HSPC::PluginToolkit::PP qw(callback_url);
use Digest::MD5 qw(md5_hex);
use Data::Dumper;

###############################################################################
## Nordea specified constants
###############################################################################

use constant ND_PAYMENT_GATEWAY_VERSION => '0003';
use constant ND_REFUND_GATEWAY_VERSION => '0
001';

## date of payment to be processed. if 'EXPRESS' - processed immidiately
use constant ND_PAYMENT_DUE_DATE => 'EXPRESS';

## receive additional info about Payment result
use constant ND_PAYMENT_CONFIRM => 'YES';

## receive additional info about Refund result
use constant ND_REFUND_CONFIRM => 'Y';

## strange mandatory parameter for Refund
use constant ND_REFUND_ALG => '01';

## constants for testing
use constant ND_MERCH_ID_TST => '12345678';
use constant ND_MERCH_MAC_TST => 'LEHTI';
use constant ND_MERCH_MAC_VER_TST => '0001';

## only EURos supported
use constant ND_CURRENCIES_SUPPORTED => ['EUR'];

## languages supported
use constant ND_LANGUAGES => [
	[1, string('lang_fi_uc')],
	[2, string('lang_sv_uc')],
	[3, string('lang_en_uc')]
];

## default language
use constant ND_DEFAULT_LANGUAGE => 1;

## responce parameters to add into return request
use constant ND_RESPONSE => {
	'ACCEPTED' => 'APPROVED',
	'CANCELED' => 'DECLINED',
	'REJECTED' => 'DECLINED'
};

## MAC calculation parameters and sequence
use constant ND_MAC_CALC_SEQ => {
	'SALE' => [qw|
		SOLOPMT_VERSION
		SOLOPMT_STAMP
		SOLOPMT_RCV_ID
		SOLOPMT_AMOUNT
		SOLOPMT_REF
		SOLOPMT_DATE
		SOLOPMT_CUR|
	],
	'CREDIT' => [qw|
		SOLOPMT_VERSION
		SOLOPMT_TIMESTMP
		SOLOPMT_RCV_ID
		SOLOPMT_LANGUAGE
		SOLOPMT_RESPTYPE
		SOLOPMT_RESPDATA
		SOLOPMT_RESPDETL
		SOLOPMT_STAMP
		SOLOPMT_REF
		SOLOPMT_AMOUNT
		SOLOPMT_CUR
		SOLOPMT_REF2
		SOLOPMT_KEYVERS
		SOLOPMT_ALG|
	]
};

## structure and sequence of Payment request parameters
use constant ND_PAYMENT_REQUEST => [
	{
		SOLOPMT_VERSION => &ND_PAYMENT_GATEWAY_VERSION,
	}, {
		SOLOPMT_STAMP => undef,
	}, {
		SOLOPMT_RCV_ID => undef,
	}, {
		SOLOPMT_LANGUAGE  => undef,
	}, {
		SOLOPMT_AMOUNT  => undef,
	}, {
		SOLOPMT_REF => undef,
	}, {
		SOLOPMT_DATE => &ND_PAYMENT_DUE_DATE,
	}, {
		SOLOPMT_MSG => undef,
	}, {
		SOLOPMT_RETURN => undef,
	}, {
		SOLOPMT_CANCEL => undef,
	}, {
		SOLOPMT_REJECT => undef,
	}, {
		SOLOPMT_CONFIRM => &ND_PAYMENT_CONFIRM,
	}, {
		SOLOPMT_KEYVERS => undef,
	}, {
		## TODO: depend on vendor selected currency, if it is not only one
		SOLOPMT_CUR => &ND_CURRENCIES_SUPPORTED->[0],
	}
];

## structure and sequence of Refund request parameters
use constant ND_REFUND_REQUEST => [
	{
		SOLOPMT_VERSION => &ND_REFUND_GATEWAY_VERSION,
	}, {
		SOLOPMT_TIMESTAMP => undef, ##YYYYMMDDHHMMSSnnnn
	}, {
		SOLOPMT_RCV_ID => undef,
	}, {
		SOLOPMT_LANGUAGE  => undef,
	}, {
		SOLOPMT_RESPTYPE => 'html' ## or 'xml'
	}, {
		SOLOPMT_RESPDATA => undef,
	}, {
		SOLOPMT_RESPDETL => &ND_REFUND_CONFIRM,
	}, {
		SOLOPMT_STAMP => undef,
	}, {
		SOLOPMT_REF => undef,
	}, {
		SOLOPMT_AMOUNT  => undef,
	}, {
		## TODO: depend on vendor selected currency, if it is not only one
		SOLOPMT_CUR => &ND_CURRENCIES_SUPPORTED->[0],
	}, {
		SOLOPMT_REF2 => undef,
	}, {
		SOLOPMT_KEYVERS => undef,
	}, {
		SOLOPMT_ALG => &ND_REFUND_ALG,
	}
];

###############################################################################
## PUBLIC METHODS
###############################################################################

sub get_title {
	my $class = shift;

	return string('nordea_title');
}

sub process_sale{
	my $class = shift;
	return $class->_process_redirect(@_);
}

sub collect_transaction_refno{
	my $class = shift;
	my $ref_no = uriparam('ref_no');

	return $ref_no;
}

## verifies responce data correctness by comparing MAC codes
## >self
## >%responce hash
## < 1 | 0
sub verify {
	my ($self, %h) = @_;
	my $config = $h{config};
	my $str_to_encode;

	$str_to_encode =
		$h{SOLOPMT_RETURN_VERSION}.'&'
		.$h{SOLOPMT_RETURN_STAMP}.'&'
		.$h{SOLOPMT_RETURN_REF}.'&';
	if ($h{SOLOPMT_RETURN_PAID}){
		$str_to_encode .= $h{SOLOPMT_RETURN_PAID};
	}
	$str_to_encode .= '&';
	$str_to_encode .= $config->{test_mode} ?
		&ND_MERCH_MAC_TST : $config->{merchant_mac};
	$str_to_encode .= '&';
	$str_to_encode = uc(Digest::MD5::md5_hex($str_to_encode));

	return ($str_to_encode eq uc($h{SOLOPMT_RETURN_MAC})) ? 1 : 0;
}

sub process_callback{
	my $class = shift;
	my %h = (
			config        => undef,
			@_
	);

	my $config = $h{config};
	my %resp   = %{$ENV{args}};
	my $url    = $ENV{q}->url(-path_info => 1);
	my $customer_desc;
	my $details;
	my $status;

	log_debug('Reply from gateway:');
	my @pars_name = $ENV{q}->param;
	my %pars = map {$_ => $ENV{q}->param($_)} @pars_name;
	foreach (@pars_name) {
		log_debug($_. " => " .argparam($_));
	}

	if ($url =~ m#/result(.*)$#) {
		my $result     = uriparam('result');

		## Check data received
		if ($result eq 'Cancel'){
			$status        = 'ERROR';
		} elsif ($result eq 'Failure') {
			$status        = 'ERROR';
		} elsif ($result eq 'Success') {
			if (
				$class->verify(
					config => $config,
					%resp
				)
			){
				$status = 'APPROVED';
			} else {
				$status = 'ERROR';
				$customer_desc = string('nordea_resp_invalid');
			}
		}
	} else {
		$status = 'ERROR';
	}

	return {
		STATUS               => $status,
		TEXT                 => {
			customer_message => $customer_desc,
		},
		TRANSACTION_DETAILS  => $details,
	};
}

sub get_currencies_supported{

	return &ND_CURRENCIES_SUPPORTED;
}

## calculate MAC verification code
sub calculate_mac {
	my $self = shift;
	my %h = (@_);
	my $config = $h{config};
	my $str_to_encode;
	## concatenates values of parameters in order
	## as described in ND_MAC_CALC_SEQ divided by '&' sign
	foreach my $item (@{ND_MAC_CALC_SEQ()->{$h{trans_type}}}){
		throw_exception(
			type        => 'error',
			log_message => "Internal error: calculate_mac: cannot calculate - $item is undefined"
		) unless defined $h{$item};
		$str_to_encode .= "$h{$item}&";
	}
	$str_to_encode .= $config->{test_mode} ?
		&ND_MERCH_MAC_TST : $config->{merchant_mac};
	return uc(Digest::MD5::md5_hex($str_to_encode.'&'));
}

## generate Payment reference number by Order number
## algorithm provided by Nordea
## 1. reverse payment number (e. g. 123456 -> 654321)
## 2. multiply each digit by 7-3-1 in loop and summarize results (42+15+4+21+6+1 = 89)
## 3. substract summ for nearest ten (90 - 89 = 1)
## 4. add this digit to the end of original Order number

sub _compose_refnum {
	my @multip = (7, 3, 1);
	my $self = shift;
	my %h = (
		pnum => undef,
		@_
	);

	throw_exception(
		type        => 'error',
		log_message => "Internal error: ref_number(): please, provide payment number!"
	) unless $h{pnum};

	my $pnum = $h{pnum};
	## 1.
	my $rev = reverse ($pnum);
	my $i = 0; my $sum = 0;
	while ($rev =~ /(\d)/g){
		## 2.
		$sum += $1 * $multip[$i%3];
		$i++;
	}
	## 3. and 4.
	return $pnum*10 + (10 - $sum%10)%10;

}
sub _process_redirect {
	my $class = shift;
	my %h = (
			document_info => undef,
			ref_no        => undef,
			config        => undef,
			@_
	);

	my $document_info      = $h{document_info};
	my $description        = $document_info->{description};
	my $document_id        = $document_info->{id};
	my $currency_iso       = $h{currency_iso};
	my $transaction_amount = $h{transaction_amount};
	my $config             = $h{config};
	my $ref_no             = $h{ref_no};

	my $nordea_ref_no = $class->_compose_refnum(
		pnum => $document_id
	);

	## create description for transaction
	my $msg = 'Document #' . $document_id;
	if (length($description)){
		$msg .= '. ' . $description;
	}
	## nordea requires message to be not more than 6 lines
	## 35 characters each (by spec)
	## insert \n after each 35 symbol
	$msg =~ s/(.{35})/$1\n/g;
	## crop $msg after 6 lines, if it is bigger
	$msg =~ s/^((.{35}\n){6})(.+)/$1/s;
	## remove last \n if it is
	chomp $msg;

	my $dat;
	## To escape from mass recoding the plugin we should accept the little
	## piece of shit
	for my $i (@{&ND_PAYMENT_REQUEST}) {
		for my $k (keys %$i) {
			$dat->{$k} = $i->{$k} if $i->{$k};
		}
	}

	$dat->{'SOLOPMT_LANGUAGE'} = $config->{language};
	$dat->{'SOLOPMT_STAMP'}    = time;# . int(rand(9999999999));
	$dat->{'SOLOPMT_REF' }     = $nordea_ref_no;
	$dat->{'SOLOPMT_AMOUNT'}   = sprintf ("%.*f", currency_precision(), $transaction_amount);
	$dat->{'SOLOPMT_MSG'}      = $msg;
	$dat->{'SOLOPMT_RCV_ID'}   = $config->{test_mode}
		? &ND_MERCH_ID_TST
		: $config->{merchant_id};
	$dat->{'SOLOPMT_KEYVERS'}  = $config->{test_mode}
		? &ND_MERCH_MAC_VER_TST
		: $config->{mac_ver};
	$dat->{'SOLOPMT_RETURN'}   = callback_url(
		suffix => 'result',
		result => 'Success',
		ref_no => $ref_no,
	);
	$dat->{'SOLOPMT_CANCEL'}   = callback_url(
		suffix => 'result',
		result => 'Cancel',
		ref_no => $ref_no,
	);
	$dat->{'SOLOPMT_REJECT'}   = callback_url(
		suffix => 'result',
		result => 'Failure',
		ref_no => $ref_no,
	);

	## after request hash is initialized calcutate MAC verification code
	$dat->{'SOLOPMT_MAC'} = $class->calculate_mac(
		%{$dat},
		trans_type => 'SALE',
		config     => $config
	);

	my $redirect_hash = {
		url    => $config->{payment_url},
		method => "POST",
		attrs  => $dat,
	};
	log_debug("Prepared request's params: \n".Dumper($dat));

	return {
		STATUS               => 'REDIRECT',
		REDIRECT_HASH        => $redirect_hash
	};
}


1;