package HSPC::MT::Plugin::PP::OP_Ogone;
use strict;

use HSPC::PluginToolkit::General
  qw(get_data_dir string log_warn log_debug throw_exception split_date_string);

use XML::Simple;
use Data::Dumper;
use LWP::UserAgent;
use HTTP::Request::Common qw(POST);
use POSIX qw(strftime);


## Fields to be shadow while logging
use constant FIELD2SHADOW => [qw{CardNo ED PspID UserID Pswd}];

sub get_title {
	return string('ogone_plugin_title');
}

sub get_supported_payment_method_types {
	return [
		'M',
		'V',
		'A',
		'C',
		'J',
	];
}

sub get_currencies_supported {
	return [
		'AUD',
		'BEF',
		'CAD',
		'CHF',
		'CNY',
		'CZK',
		'DKK',
		'EEK',
		'EUR',
		'XBA',
		'GBP',
		'HKD',
		'HRK',
		'HUF',
		'ILS',
		'ISK',
		'JPY',
		'LTL',
		'LVL',
		'MAD',
		'MXN',
		'NOK',
		'NZD',
		'PLN',
		'RUR',
		'SEK',
		'SGD',
		'SKK',
		'THB',
		'TRY',
		'UAH',
		'USD',
		'XAF',
		'XOF',
		'ZAR',
	];
}

sub process_sale {
	my $class = shift;
	my %h = @_;

	log_debug(__PACKAGE__.'::process_sale called');
	return $class->_process_transaction(%h, trans_type => 'SAL');
}

sub process_credit {
	my $class = shift;
	my %h = @_;

	log_debug(__PACKAGE__.'::process_credit called');
	return $class->_process_transaction(%h, trans_type => 'RFD');
}


sub _compose_request {
	my $self = shift;
	my %h = @_;

	my $total_paid   = $h{transaction_amount};

	throw_exception(type => 'error', log_message => "Total sum ($total_paid) for doc to be paid is undefind or zero")
		unless ( $total_paid && $total_paid > 0 );

	my $data;
	my $date = $h{payment_method}->{expire_date};
	my %expire_date = split_date_string($date);

	my $mm = sprintf("%02d", $expire_date{month});
	my $yy = sprintf("%02d", $expire_date{year});

	$data->{Operation} = $h{trans_type};
	$data->{PspID    } = $h{config}->{merchant_id};
	$data->{UserID   } = $h{config}->{user_id};
	$data->{Pswd     } = $h{config}->{password};
	$data->{Currency } = $h{currency_iso};
	$data->{OrderID  } = $h{document_info}->{id}.":".time;
	$data->{Amount   } = $total_paid*100;
	$data->{ALIAS    } = $h{payment_method}->{secure_data}->{card_number};
	$data->{ED       } = "$mm/$yy";

	if ("" ne $h{payment_method}->{secure_data}->{cvv}) {
		$data->{CardNo   } = $h{payment_method}->{secure_data}->{card_number};
		$data->{CVC      } = $h{payment_method}->{secure_data}->{cvv};
	} else {
		$data->{ECI      } = 9; ## Use for recurring peyments
	}

	return $data;
}


sub _masker {
	my $self = shift;
	$_ = shift;
	(length > 10)
	? s/(.*)/"*****".substr($1,-4)/e
	: s/(.*)/("*"x(length $1))/e;
	return $_;
}

sub _safe_dump {
	my $self = shift;
	my %h = (
		data => undef,
		@_
	);
	my %data = %{$h{data}};
	for my $field (@{&FIELD2SHADOW}) {
		if ($data{$field}) {
			$data{$field} =~ s/(.*)/$self->_masker($1)/e;
		}
	}
	return \%data;
}


sub _send_request {
	my $self = shift;
	my %h = @_;

	my $data = $h{data} || throw_exception(type => 'error', log_message => 'No data');
	log_debug(__PACKAGE__." Request: ".Dumper($self->_safe_dump(data => $data)));

	my $server_url = $h{config}->{server_url};
	my $agent = LWP::UserAgent->new;
	my $request = POST  $server_url, $data;
	my $response = $agent->request($request);
	log_debug(__PACKAGE__." Response: ".Dumper($response->content));
	my $details = eval {XMLin($response->content)};
	return undef if $@;
	return $details;
}


sub _get_details {
	my $self = shift;
	my %h = @_;

	my $reply    = $h{reply};

	return undef if (!$reply || !defined $reply->{STATUS});
	my $details = {};

	$details->{error		 } = $reply->{NCERRORPLUS } unless ($reply->{NCERRORPLUS } eq '!');
	$details->{error_code    } = $reply->{NCERROR     } unless ($reply->{NCERRORPLUS } eq '!');

	$details->{transaction_id} = $reply->{PAYID};
	$details->{status} = $reply->{STATUS};

	return $details;
}


sub _is_trans_ok {
	my $self = shift;
	my $details = shift;
	if (!defined $details->{error}) {
		return 1;
	} else {
		return 0;
	}
}


sub _create_result {
	my $self = shift;
	my %h = @_;

	my $details    = $h{details};
	my $total_paid = $h{transaction_amount};

	if (!$details) {

		throw_exception(
			type        => 'error',
			user_message => {string_id => 'err_trans_explained'},
			log_message => $details->{ccp_error} || $details->{ccp_message} ||
				{ string_id => 'unexpected_error_trying_to_process_payment'}
		);

	} elsif ($self->_is_trans_ok($details)) {

		return {
			STATUS               => 'APPROVED',
			TRANSACTION_DETAILS  => $details,
		};

	} else {

		return {
			STATUS => 'DECLINED',
			TEXT   => {
				customer_message => string('err_trans_explained'),
				vendor_message   => string('err_trans_explained'),
			},
			TRANSACTION_DETAILS  => $details,
		};

	}
}


sub _process_transaction {
	my $self = shift;
	my %h = @_;

	my $data = $self->_compose_request(%h);
	my $reply = $self->_send_request(%h, data => $data);
	my $details = $self->_get_details(%h, reply => $reply);

	return $self->_create_result(%h, reply => $reply, details => $details);
}


1;

__END__
