package HSPC::Mailer;

use strict;
use HSPC::Config;

use HSPC::Util qw(sw_load_file);
use HSPC::Logger qw(sw_log_debug sw_log_warn sw_log sw_die);
use HSPC::WebSystemLib;
use HSPC::Localization qw(current_lang);
use HSPC::Localization::Date qw(gmt_now);
use HSPC::LocResource;
use Mail::Sendmail;
use MIME::Entity;
use MIME::Words qw(:all);
use MIME::Base64;
use SWsoft::protect;	# Mark this module for encryption upon build

use HSPC::DataRequest;
use HSPC::MT::Core;
use HSPC::MT::GM;
use HSPC::MT::GM::Constants qw(:lang);
use HSPC::MT::Core::EMail;

use Encode qw(resolve_alias);
use OpenCA::OpenSSL;
use OpenCA::OpenSSL::SMIME;
use OpenCA::X509;

use constant SENDMAILCMD=>'/usr/sbin/sendmail -t';


use constant CHARSET_MAP=>{
	'utf8'=>'UTF-8',
	'UTF8'=>'UTF-8'
};##/CHARSET_MAP


sub _email_charset {
	my $ch = shift;
	if ( exists(&CHARSET_MAP->{$ch}) ) {
		return &CHARSET_MAP->{$ch};
	}
	return $ch;
}##/_email_charset


## >> class
## =>> handler_id
## => lang : default = 'en'
## => provider_id 
## => charset
## => content_type
## < ref to HSPC::Mail instance
sub new{
	my $class=shift;
	my $self={};
	bless($self,$class);

	my %h=(
		handler_id => undef,
		lang => &current_lang(),
		content_type => 'text/plain',
		content_protect => 0,
		@_
	);		
	
	unless( defined $h{provider_id} ){
		$h{provider_id} = $ENV{system_obj}->provider_id;
	}

	$self->{_lang} = HSPC::Localization::get_language($h{lang});	
	$self->{_provider_id} = $h{provider_id} || sw_die (
		'No provider_id specified for new Mailer object!'
	);
	$self->{_handler_id} = $h{handler_id} || sw_die(
		"HSPC::Mailer->new(): handler_id is required"
	);
	$self->{_content_type} = $h{content_type};
	$self->{_charset} = $h{charset};
	$self->{_content_protect} = $h{content_protect};
	## Hash internal_name => local_file_name
	$self->{_attachments} = {};
	## initialize to,cc,bcc
	$self->{_to} = [];
	$self->{_cc} = [];
	$self->{_bcc} = [];
	
	return $self;
}

## > ref to array
## < string
sub _addresses{
	my ($self, $address_list, $do_not_encode)=@_;

	my @addresses;
	my $charset = resolve_alias($self->{_charset}) ? $self->{_charset} : SW_GM_DEF_MAIL_CHARSET;
	$charset = _email_charset($charset);
	foreach my $addr ( @$address_list ) {
		my $name = $addr->{name};
		my $email = $addr->{email};
		$name =~ s/([\\"\r])/\\$1/gs;##"
		$name = encode_mimeword($name,'b',$charset) unless ($do_not_encode);
		push @addresses,$name." <$email>";
	}
	return join(', ',@addresses);	
}


sub send{
	my $self = shift;		
	
	## load provider config
	my $setup = HSPC::MT::GM->find_provider_config(id=>$self->{_provider_id});				
	
	## fill mail data
	if(!$self->{_from}){				
		$self->from(name=>$setup->mailer_from_name,email=>$setup->mailer_from_email);
	}	
	
	### is mailer enabling ?
	$setup->enable_notify()	|| return undef;

	my $charset = resolve_alias($self->{_charset}) ? $self->{_charset} : SW_GM_DEF_MAIL_CHARSET;
	
	## Create e-mail message
	## Strip same email recipients in email notifications
	$self->normalize_recipients();
	## create hash for composing email function
	my %email_params = (
		from_address => $self->_addresses($self->{_from}),
		to_address => $self->_addresses($self->{_to}),
		cc_address => $self->_addresses($self->{_cc}),
		bcc_address => $self->_addresses($self->{_bcc}),
		subject => $self->{_subject},
		body => $self->{_body},
		html_body => ($self->{_html_body} ? $self->{_html_body} : undef),
		charset => $charset,
		content_type => $self->{_content_type},
		content_protect => $self->{_content_protect},
		provider_id => $self->{_provider_id},
		attachments => $self->{_attachments},
	);
	
	
	my $email = HSPC::MT::Core::EMail->new();
	$email->provider_id( $self->{_provider_id} );
	$email->is_sent( 0 );
	$email->message( $self->compose_message(\%email_params) );
	$email->direction(&HSPC::MT::Core::EMail::OUT);
	$email->mail_from($self->_addresses($self->{_from}, 1));
	$email->mail_to($self->_addresses($self->{_to}, 1));
	$email->subject($self->{_subject});
	$email->handler_id($self->{_handler_id});
	$email->save();
	return undef;
}

## >> class
## >> email_params (hashref)
##
## Special note for html_body param: if {html_body}=undef {body} will be used,
## if {html_body}='' no text/html part will be added to message
##
sub compose_message {
	my $self = shift;
	my $email_params = shift;
	sw_die('Invalid argument email_params') unless(ref($email_params) eq 'HASH');
	my %email = %{$email_params};

	##
	## Prepare mail body
	my %ready = (%email);
	my @keys = qw(from_address to_address cc_address bcc_address subject body html_body);
	
	$ready{html_body} ||= $ready{body};

	$ready{body} =~ s!\r!!g;
	$ready{html_body} =~ s!\r!!g;

	my $charset = resolve_alias($email{charset}) ? $email{charset} : SW_GM_DEF_MAIL_CHARSET;

	##bug 42622 workaround
	if (!$email{content_protect}) {
		my %tmp = %ready;
		eval {
			foreach (@keys) {
				$ready{$_} = HSPC::LocResource->convert_from_utf8(
					value => $ready{$_}, charset => $charset
				);
			}
		};

		if($@) {
			sw_log_warn("UTF8 conversion: $@; using UTF8 instead.");
			$charset = SW_GM_DEF_MAIL_CHARSET;
			%ready = %tmp;
		}
	}
	$charset = _email_charset($charset);
	
	## Additional body preparation, if from plain text
	$ready{html_body} = "<pre>".&sw_web_prepare4browser('text' => $ready{html_body})."</pre>"
		unless(defined($email{'html_body'}));

	### Create the top-level, and set up the mail headers:
	my $encoding = '8bit';
	$encoding = '7bit' if (
		uc($charset) eq 'JIS' || uc($charset) =~ /^ISO-2022/
	);
	my $type = 'multipart/alternative';
	$type = 'multipart/mixed' if ( scalar(keys(%{$email{attachments}})) );
	my $mime = MIME::Entity->build(
		Type    => $type,
		From    => $ready{from_address},
		To      => $ready{to_address},
		Cc      => $ready{cc_address},
		Bcc     => $ready{bcc_address},
		Subject => $ready{subject},
		Charset => $charset,
		Encoding=> $encoding
	);
	my $amime;
	if ( scalar(keys(%{$email{attachments}})) ) {
		$amime = $mime->attach(
			Type=>'multipart/alternative',Charset=>$charset,Encoding=>$encoding
		);
	} else {
		$amime = $mime;
	}
	my $id = time();
	$amime->attach(
		Id          => $id,
		Type        => "text/plain",
		Data        => $ready{body} || '',
		Encoding    => $encoding,
		Charset     => $charset,
	);
	my $att_count = 1;
	
	if( length($email{html_body}) ) {
		$amime->attach(
			Id          => $id,
			Type        => "text/html",
			Data        => $ready{html_body} || '',
			Encoding    => "quoted-printable",
			Charset     => $charset,
		);
		$att_count++;
	}

	## Attach files, if any
	foreach my $int_name (keys(%{$email{attachments}})) {
		my $attach = $email{attachments}->{$int_name};
		$mime->attach(
			Id 					=> $int_name,
			Type        => $attach->{content_type},
			Path        => $attach->{file_name},
			Encoding    => "base64",
		);
		$att_count++;
	}
	$mime->make_singlepart() if ( $att_count == 1 );
	
	##
	my $head_obj = $mime->head;
	$head_obj->unfold();
	my %mail;
	$mail{Sender} = $email{from_address}; ## to use an envelope sender address
	for (my $i = 0; $i < @{$head_obj->{mail_hdr_list}}; ++$i) {
		my ($h_name, $h_val) = split(/:\s*/, $head_obj->{mail_hdr_list}->[$i], 2);

		if($h_name =~ /^subject$/i) {
			chomp($h_val);
			if ( uc($charset) =~ /^ISO-2022/ ) {
				$h_val = encode_mimeword($h_val,'b',$charset);
			} else {
				## The following part of code is a
				## workaround of encode_mimewords bug
				## See http://rt.cpan.org/Public/Bug/Display.html?id=5462
				my $NONPRINT = "\\x00-\\x1F\\x7F-\\xFF";
				my $result = '';
				my @raw_words = split(/\s+/, $h_val);
				my $to_encode = '';
				foreach my $raw_word (@raw_words) {
						my $is_2b_enc;
						if ($raw_word =~ m/([$NONPRINT]+)/) {
							$is_2b_enc = 1;
						} else {
							$is_2b_enc = 0;
						}
						if(!$is_2b_enc) {
							$result .= encode_mimeword($to_encode,'b',$charset) . ' ' if $to_encode;
							$to_encode = '';
							$result .= $raw_word . ' ';
						} else {
							$to_encode .= $raw_word . ' ';
						}
					}
					$result .= ' ' . encode_mimeword($to_encode,'b',$charset)
						if $to_encode;
				$h_val = $result;
			}
			$head_obj->{mail_hdr_list}->[$i] = "$h_name: $h_val";
		} elsif($h_name =~ /^(from|to|cc|bcc)$/i) {
			chomp($h_val);
			$head_obj->{mail_hdr_list}->[$i] = "$h_name: $h_val";
		}
	}
	my $config = HSPC::MT::GM->find_provider_config(id=>$self->{_provider_id});
	if (
		!$email{content_protect} &&
		$config->is_emails_signed() &&
		$config->is_email_cert_present() &&
		$config->is_email_cert_valid()
	) {
		my $shell = OpenCA::OpenSSL->new();
		my $smime = OpenCA::OpenSSL::SMIME->new(
			ENTITY=>$mime,SHELL=>$shell
		);
		my $x509 = new OpenCA::X509(SHELL=>$shell,INFILE=>$config->email_cert_path());
		$smime->sign(
			CERTIFICATE=>$x509,
			PRIVATE_KEY=>sw_load_file($config->email_key_path())
		);
		$mime = $smime->get_mime();
	}

	return $mime->stringify();
}

## >> class
sub task_mailer{
	my $class = shift;
	sw_log_debug('[Mailer] task_mailer() called');
	my $req = HSPC::DataRequest->new(cols=>[qw|id is_sent direction|]);
	$req->set_where('is_sent',0);
	$req->set_where('direction',&HSPC::MT::Core::EMail::OUT);
	HSPC::MT::Core->find_emails(request=>$req);
	## cycle
	foreach my $row( @{$req->data} ){
		my $m='[Mailer] task_mailer() /mail_id='.$row->[0].'/';
		### send email
		my $email = HSPC::MT::Core->find_email(id=>$row->[0]);
		next unless $email;
		$class->send_real_email($email);
		sw_log("$m sent email");
		### change email status
		$email->is_sent(1);
		$email->sent_time( gmt_now() );
		$email->save();
		sw_log("$m updated email status");
	}
}

## >> class
## >> email : ref to HSPC::MT::Core::EMailer
sub send_real_email{
	my ($class,$email) = @_;
	open MAIL, "|".&SENDMAILCMD;
	my $is_ok = print MAIL $email->message();
	close MAIL;
	unless ( $is_ok ) {
		sw_log_warn("Error while sending mail: $!");
	}
}

# >> self
# =>> name 
# =>> email
sub from{
	my ($self,%h) = @_;
	
	sw_die('ERROR: you must define email=> argument in HSPC::Mailer from() method') unless $h{email};
	sw_die('ERROR: you must define name=> argument in HSPC::Mailer from() method') unless $h{name};
	
	push @{$self->{_from}},{name=>$h{name},email=>$h{email}};
}

# >> self
# =>> name 
# =>> email
sub to{
	my ($self,%h) = @_;
	
	sw_die('ERROR: you must define email=> argument in HSPC::Mailer to() method') unless $h{email};
	
	push @{$self->{_to}},{name=>$h{name},email=>$h{email}};
}

# >> self
# =>> name
# =>> email
sub cc
{
	my ($self,%h) = @_;

	sw_die('ERROR: you must define email=> argument in HSPC::Mailer cc() method') unless $h{email};

	push @{$self->{_cc}},{email=>$h{email}||''};
}

## >> self
## =>> name
## =>> email
sub bcc
{
	my ($self,%h) = @_;

	sw_die('ERROR: you must define email=> argument in HSPC::Mailer cc() method') unless $h{email};

	push @{$self->{_bcc}},{email=>$h{email}||''};
}

# >> self
# >> subject
sub subject{
	my ($self,$subject) = @_;
	
	$self->{_subject} = $subject;
}

# >> self
# >> body
sub body{
	my ($self,$body) = @_;
	
	sw_die( 'ERROR: you must provide email body as argument in HSPC::Mailer body() method') unless $body;
	
	$self->{_body} = $body;
}

## >> self
## >> html_body
sub html_body {
	my ($self,$body) = @_;
	sw_die( 'body argument should be provided') unless $body;
	## attach images
	my $regexp = qr/(<img\s+[^>]*)src\s*=\s*"(\/[^"]+)"/is;
	my ($img_n, %images) = (0);
	while ($body =~ /$regexp/g) {
		my $file = $2;
		$images{$file} = {
			int_name		=> 'hspc_img_' . (++$img_n),
			content_type	=> ($file =~ /\.(gif|jpe?g)$/i
				? 'image/' . $1
				: 'application/octet-stream'),
		}
	}
	$body =~ s/$regexp/$1 src=\"cid:$images{$2}->{int_name}\"/g;
	foreach my $img (keys %images) {
		$self->add_attachment(
			internal_name	=> $images{$img}->{int_name},
			file_name		=> '/var/opt/hspc-root' . $img,
			content_type	=> $images{$img}->{content_type}
		);
	}
	## save HTML body
	$self->{_html_body} = $body;
}

## >> self
## =>> internal_name -- name of attachment in message
## =>> file_name
## =>> content_type
sub add_attachment {
	my $self = shift;
	my %h = (
		internal_name => undef,
		file_name => undef,
		content_type => undef,
		@_
	);
	sw_die('Internal name of attachment should be provided') unless($h{internal_name});
	sw_die('Local file name of attachment should be provided') unless($h{file_name});
	sw_die('Content type of attachment should be provided') unless($h{content_type});
	my %attach = (
		file_name => $h{file_name},
		content_type=> $h{content_type},
	);
	$self->{_attachments}->{$h{internal_name}} = \%attach;
}

## >> self
## >> internal_name -- name of attachment in message
sub remove_attachment {
	my $self = shift;
	my $int_name = shift;
	sw_die('internal_name should be provided') unless($int_name);
	$self->{_attachments}->{$int_name} = undef;
}


## >> self
## remove from to,cc,bcc the same mails
sub normalize_recipients {
	my $self = shift;
	my %exists;
	my (@to,@cc,@bcc);
	foreach my $to ( @{$self->{_to}} ) {
		next if ( exists($exists{$to->{email}}) );
		$exists{$to->{email}} = 1;
		push @to,$to;
	}
	foreach my $cc ( @{$self->{_cc}} ) {
		next if ( exists($exists{$cc->{email}}) );
		$exists{$cc->{email}} = 1;
		push @cc,$cc;
	}
	foreach my $bcc ( @{$self->{_bcc}} ) {
		next if ( exists($exists{$bcc->{email}}) );
		$exists{$bcc->{email}} = 1;
		push @bcc,$bcc;
	}
	$self->{_to} = \@to;
	$self->{_cc} = \@cc;
	$self->{_bcc} = \@bcc;
}##/normalize_recipients


## XXX TODO AMARK move it into common utilities
## => provider_id
## => account_no
## < charset
sub find_charset {
	my $class = shift;
	my %h = @_;
	my $provider_id = $h{provider_id};
	my $account_no = $h{account_no};

	return SW_GM_DEF_MAIL_CHARSET unless (
		$provider_id || $account_no
	);
	my $lang;
	if ( $account_no ) {
		my $account = HSPC::MT::Core->find_account(
			account_id=>$account_no
		) || sw_die("Cannot find account #$account_no.");

		return $account->charset() if ( $account->charset() );

		$lang = $account->lang();
		$provider_id = $account->vendor_id() || sw_die(
			"Account #$account_no does not have vendor."
		);
	}
	my $pc = HSPC::MT::GM->find_provider_config(id=>$provider_id) || sw_die(
		"Cannot find provider config #$provider_id."
	);
	$lang = $pc->def_lang() unless ( $lang );
	my $langs = $pc->langs() || [];
	
	my @charset = map {$_->{charset}} grep {$_->{lang} eq $lang} @{$langs};
	
	return $charset[0] if ( $charset[0] );
	
	return $pc->mail_charset() if ( $pc->mail_charset() );

	return SW_GM_DEF_MAIL_CHARSET;
}##/sw_find_charset

1;
__END__
