package HSPC::Plugin::API::Common;
use strict;
use Data::Dumper;

package HSPC::Plugin::API::Common::Request;

use base qw(Exporter);

sub new
{
	my $self = {};
	bless ($self, shift);
	return $self;
}

sub url($)
{
	my $self = shift;
	return $self->{url};
}

sub param($)
{
	my ($self, %params) = @_;
	my @keys = keys %params;

	return keys %{$self->{params}} unless %params;
	return $self->{params}->{$params{@keys[0]}} if 1 == length(@keys);

	foreach my $key (@keys) {
		$self->{params}->{$key} = $params{$key};
	}
	return undef;
}

package HSPC::Plugin::API::Common;

$ENV{MOD_PERL} = 0;

use Data::Dumper;
use HSPC::System;
use HSPC::Config;
use HSPC::Logger;
use HSPC::Localization;
use HSPC::MT::Core::Constants;
use HSPC::MT::PluginConfig;
use HSPC::MT::PluginTools;
use HSPC::Plugin::API::PP;

use base qw(Exporter);
use vars qw(@EXPORT_OK %ENV);

@EXPORT_OK = qw(
	prepare_account_from_soap
	prepare_hash_from_soap
	APIResult
	APIResultErrors
);

sub new
{
	$ENV{system_obj} = HSPC::System->new();

	HSPC::Logger->init();
	HSPC::Localization::load_all_languages();

	return bless {} => shift;
}

sub init_request
{
	my $me = shift;
	my $params = shift;
	my $template_id = shift;

	foreach my $key (keys %{$params}) {
		if (ref($params->{$key}) eq "HASH" && $params->{$key}->{content}) {
			use File::Temp qw(tempfile);
			use MIME::Base64;

			my ($fh, $filename) = tempfile();
			print $fh decode_base64($params->{$key}->{content});
			close $fh;

			open FILE, "<".$filename;
			$params->{$key} = \*FILE;
		}
	}
	$ENV{q} = HSPC::Plugin::API::Common::Request->new();
	$ENV{q}->{url} = HSPC::MT::PluginTools::get_request_data("referer_url");
	$ENV{q}->param(%{$params});

	HSPC::MT::PluginTools::set_request_data(plugin_config_id => $template_id);
	HSPC::MT::PluginTools->init_plugin_argparams(plugin_argparams => $params);
	$ENV{args} = $params;
}

sub get_plugins($$)
{
	my $me = shift;
	my $plugin_type = shift;

	my $templates = HSPC::MT::PluginConfig->find_templates(
		plugin_type			=> $plugin_type,
		vendor_id			=> 1,
		force_permission	=> 1,
	);
	my $result = {};
	foreach my $template_id (@$templates) {
		if ("PP" eq $plugin_type) {
			my $method_types = HSPC::Plugin::API::PP->get_supported_payment_method_types($template_id);
			if ($method_types->{data}->[0] eq $template_id) {
				$result->{$template_id} = "BT";
				next;
			}
			if ($method_types->{data}->[0] ne "") {
				$result->{$template_id} = "CC";
				next;
			} else {
				$result->{$template_id} = "TP";
			}
			next;
		}
		if ("DM" eq $plugin_type) {
			$result->{$template_id} = "DM";
		}
	}
	return APIResult($result);
}

sub get_plugin_templates($$$)
{
	my $me = shift;
	my $plugin_type = shift;
	my $template_id = shift;

	my $plugin_config = HSPC::MT::PluginConfig->new();
	$plugin_config->vendor_id(SW_HSP_ID);
	$plugin_config->plugin_type($plugin_type);
	$plugin_config->template_id($template_id);

	my $pr_template = $plugin_config->find_presentation_template();
	my $mt_template = $plugin_config->find_mt_template();

	eval("require $pr_template");
	eval("require $mt_template");

	return {
		"pr_template" => $pr_template,
		"mt_template" => $mt_template,
	};
}

sub get_help_page($$)
{
	my $me = shift;
	my $args = shift;

	my $templates = HSPC::Plugin::API::Common->get_plugin_templates($args->{plugin_type}, $args->{template_id});

	my $page = $templates->{pr_template}->get_help_page(
		action => $args->{action},
		language => $args->{language},
		customer_view => $args->{customer_view},
	);

	return APIResult(HSPC::Plugin::SOAP::Struct::HelpPageResult->new(page => $page));
}

sub prepare_account_from_soap
{
	my $account = shift;
	my $result = {};

	foreach my $contact_type qw(admin billing technical)
	{
		foreach my $param (keys %{$account->{$contact_type}})
		{
			$result->{$contact_type . "_" . $param} = $account->{$contact_type}->{$param};
		}
	}

	foreach my $address_param qw(address_address1 address_address2 address_city address_country address_state address_zip address_phone address_fax address_mobile)
	{
		$result->{$address_param} = $result->{"admin_" . $address_param};
	}

	return $result;
}

sub add_common_js($)
{
	my $me = shift;
	my $html = shift;

	return undef unless $html;

	$html =~ s/getElementById\('item'\)/getElementById\('form'\)/g;
	return $html . "
	<script language='javascript'>
	//<!--
		function disable_field(field,disable) {
			if(typeof(field) != 'object') {
				return;
			}
			if(disable) {
				field.disabled = true;
				if (field.style){
					if(field.type == 'text' || field.type == 'select-one' || field.type == 'password'){
						field.style.backgroundColor = '#F0F0F0';
						field.style.borderColor = '#B7B7C6';
						field.style.color = '#555555';
					}
				}
			} else {
				field.disabled = false;
				if (field.style){
					if(field.type == 'text' || field.type == 'select-one' || field.type == 'password'){
						field.style.backgroundColor = '#FFFFFF';
						field.style.borderColor = '#B7B7C6';
						field.style.color = '#000000';
					}
				}
			}
		}
	//-->
	</script>";
}

sub prepare_hash_from_soap
{
	my $hash = shift;
	my $default = shift;
	my $output;

	if (ref($hash) eq "ARRAY") {
		return $default unless scalar @{$hash};
	}
	if (ref($hash) eq "HASH" || ref($hash) eq "Struct" ) {
		return $default unless scalar keys %{$hash};
	}
	return $default unless $hash;

	use HTML::Entities;

	if (ref($hash) eq "ARRAY") {
		my @arr_output;
		foreach my $value (@{$hash})
		{
			if(ref($value) ne "") {
				push @arr_output, prepare_hash_from_soap($value, (ref($value) eq "ARRAY" ? [] : {}));
				next;
			}
			decode_entities($value);
			$value =~ s/\\\"/\"/g;
			$value =~ s/\\\'/\'/g;
			$value =~ s/\\\\/\\/g;
			encode_entities($value);
		}
		$output = \@arr_output;
	} else {
		my $hash_output = {};
		foreach my $key (keys %{$hash}) {
			$hash_output->{$key} = $hash->{$key};
		}
		foreach my $key (keys %{$hash_output})
		{
			my $value = $hash_output->{$key};
			if(ref($value) ne "") {
				$hash_output->{$key} = prepare_hash_from_soap($value, (ref($value) eq "ARRAY" ? [] : {}));
				next;
			}
			decode_entities($hash_output->{$key});
			$hash_output->{$key} =~ s/\\\"/\"/g;
			$hash_output->{$key} =~ s/\\\'/\'/g;
			$hash_output->{$key} =~ s/\\\\/\\/g;
			encode_entities($hash_output->{$key});
		}
		$output = $hash_output;
	}
	return $output ? $output : $default;
}

sub APIResult
{
	return HSPC::Plugin::SOAP::Struct::APIResult->new(@_);
}

sub APIResultErrors
{
	return APIResult()->errors(@_);
}

1;
