package HSPC::Plugin::API::DM;

use strict;

use Data::Dumper;
use Error qw(:try);

use HSPC::System;
use HSPC::Config;
use HSPC::Logger;
use HSPC::Localization qw(:DEFAULT set_language);
use HSPC::PluginToolkit::General qw(argparam);
use HSPC::Plugin::API::Common qw(prepare_account_from_soap prepare_hash_from_soap APIResult APIResultErrors);

use HSPC::MT::Core::Constants;
use HSPC::MT::PluginConfig;
use HSPC::MT::PluginTools;
##use HSPC::MT::CCP::Constants qw(:DEFAULT :status :permissions :process :credit_types :trans_types);

use HSPC::Plugin::SOAP::Struct::APIResult;
use HSPC::Plugin::SOAP::Struct::FormResult;
use HSPC::Plugin::SOAP::Struct::CollectedResult;
use HSPC::Plugin::SOAP::Struct::HelpPageResult;

use HSPC::Exception::Plugin::InvalidParameter;
use HSPC::Exception::Plugin::GeneralError;

use constant PLUGIN_TYPE => "DM";
use constant SW_CCP_TRANS_TYPE_CALLBACK => 10;

sub view_config_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$form_content = $templates->{pr_template}->view_config_form(config => $plugin_data);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => "",
		user_error => $user_error,
	));
}

sub edit_config_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		#use Data::Dumper;
		#die(Dumper($args->{request_context}));
		#die(Dumper($request));

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$form_content = $templates->{pr_template}->edit_config_form(config => $plugin_data)
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => "",
		user_error => $user_error,
	));
}

sub collect_config_data($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $collected_data;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$collected_data = $templates->{pr_template}->collect_config_data(config => $plugin_data)->{config};
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::CollectedResult->new(
		plugin_data => $collected_data,
		user_error => $user_error,
	));
}

sub check_capability($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $supported;

	try
	{
		my $template_id	= $args->{template_id};
		my $capability = $args->{capability};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);

		my $mt_template = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id)->{mt_template};
		if ("check_transfer" eq $capability) {
			$supported = (UNIVERSAL::can($mt_template, "check_transfer") || UNIVERSAL::can($mt_template, "check_register")) ? 1 : 0;
		} else {
			$supported = UNIVERSAL::can($mt_template, $capability) ? 1 : 0;
		}
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		supported => $supported,
	});
}

sub check_register($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my %domains_statuses;

	try
	{
		my $template_id	= $args->{template_id};
		my $domains = $args->{domains};
		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);

		my $mt_template = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id)->{mt_template};
		my $result = $mt_template->check_register(config => $plugin_data, domains => [@{$domains}]);
		for my $domain (@{$domains}) {
			$domains_statuses{$domain} = "avail" if 1 == $result->{$domain};
			$domains_statuses{$domain} = "not-avail" if 0 == $result->{$domain};
			next if "" ne $domains_statuses{$domain};

			$domains_statuses{$domain} = "cant-check";
		}
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		domains => \%domains_statuses,
	});
}

sub get_contact_types($$)
{
	my $me = shift;
	my $template_id = shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $contact_types;

	try
	{
		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		$contact_types = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id)->{mt_template}->get_contact_types();
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult($contact_types);
}

sub edit_contact_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $old_contacts = prepare_hash_from_soap($args->{old_contacts}, {});
		my $old_contacts_extdata = prepare_hash_from_soap($args->{old_contacts_extdata}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(old_contacts => $old_contacts);
		HSPC::MT::PluginTools::set_request_data(old_contacts_extdata => $old_contacts_extdata);

		my $plugin_data			= prepare_hash_from_soap($args->{plugin_data}, {});
		my $contact				= prepare_hash_from_soap($args->{contact}, {});
		my $contact_extdata		= prepare_hash_from_soap($args->{contact_extdata}, {});
		my $request				= prepare_hash_from_soap($args->{request_context}, {}, 1);

#		use Data::Dumper;
#		die(Dumper($args));

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$form_content = $templates->{pr_template}->edit_contact_form(
			config			=> $plugin_data,
			domain			=> $args->{domain},
			action			=> $args->{action},
			contact			=> $contact,
			contact_type	=> $args->{contact_type},
			contact_extdata	=> $contact_extdata,
		);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => "",
		user_error => $user_error,
	));
}

sub collect_contacts_data($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $collected_data;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$collected_data = $templates->{pr_template}->collect_contacts_data(
			config => $plugin_data,
			domain => $args->{domain},
			action => $args->{action},
		);
#		use Data::Dumper;
#		die(Dumper($collected_data));
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		contacts => $collected_data->{contacts},
		contacts_extdata => $collected_data->{contacts_extdata},
		user_error => $user_error,
	});
}

sub edit_domain_extdata_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $old_domain_extdata = prepare_hash_from_soap($args->{old_domain_extdata}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(old_domain_extdata => $old_domain_extdata);

		my $plugin_data			= prepare_hash_from_soap($args->{plugin_data}, {});
		my $domain_extdata		= prepare_hash_from_soap($args->{domain_extdata}, {});
		my $request				= prepare_hash_from_soap($args->{request_context}, {}, 1);

#		use Data::Dumper;
#		die(Dumper($args));

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$form_content = $templates->{pr_template}->edit_domain_extdata_form(
			config			=> $plugin_data,
			domain			=> $args->{domain},
			action			=> $args->{action},
			domain_extdata	=> $domain_extdata,
		);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => "",
		user_error => $user_error,
	));
}

sub collect_domain_extdata($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $collected_data;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $old_domain_extdata = prepare_hash_from_soap($args->{old_domain_extdata}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(old_domain_extdata => $old_domain_extdata);

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$collected_data = $templates->{pr_template}->collect_domain_extdata(
			config => $plugin_data,
			domain => $args->{domain},
			action => $args->{action},
		);
#		use Data::Dumper;
#		die(Dumper($collected_data));
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		domain_extdata => $collected_data->{domain_extdata},
		user_error => $user_error,
	});
}

sub validate_data($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $plugin_result;

	set_language($args->{language});

	try
	{
		my $template_id			= $args->{template_id};
		my $plugin_data			= prepare_hash_from_soap($args->{plugin_data}, {});
		my $domain				= $args->{domain};
		my $action				= $args->{action};
		my $contacts			= prepare_hash_from_soap($args->{contacts}, {});
		my $contacts_extdata	= prepare_hash_from_soap($args->{contacts_extdata}, {});
		my $domain_extdata		= prepare_hash_from_soap($args->{domain_extdata}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(old_contacts => $contacts);
		HSPC::MT::PluginTools::set_request_data(old_contacts_extdata => $contacts_extdata);
		HSPC::MT::PluginTools::set_request_data(old_domain_extdata => $domain_extdata);

		HSPC::Plugin::API::Common->init_request({}, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$plugin_result = $templates->{mt_template}->validate_data(
			config			=> $plugin_data,
			domain			=> $domain,
			action			=> $action,
			contacts		=> $contacts,
			contacts_extdata=> $contacts_extdata,
			domain_extdata	=> $domain_extdata,
		);
#		my $result = {
#			is_valid => 0,
#			error_list => [
#				{
#					form => "contact",
#					contact_type => "owner",
#					field => "fiscal_no",
#					message => "Any error message",
#				},
#				{
#					...
#				}
#			],
#		};
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult($plugin_result);
}

sub register_domain($$)
{
	my $me = shift;
	return $me->_register_domain("register", @_);
}

sub transfer_domain($$)
{
	my $me = shift;
	return $me->_register_domain("transfer", @_);
}

sub _register_domain($$$)
{
	my $me		= shift;
	my $action	= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $plugin_result;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $plugin_data			= prepare_hash_from_soap($args->{plugin_data}, {});
		my $domain				= $args->{domain};
		my $period				= $args->{period};
		my $contacts			= prepare_hash_from_soap($args->{contacts}, {});
		my $contacts_extdata	= prepare_hash_from_soap($args->{contacts_extdata}, {});
		my $domain_extdata		= prepare_hash_from_soap($args->{domain_extdata}, {});
		my $nses				= prepare_hash_from_soap($args->{nses}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(old_contacts => $contacts);
		HSPC::MT::PluginTools::set_request_data(old_contacts_extdata => $contacts_extdata);
		HSPC::MT::PluginTools::set_request_data(old_domain_extdata => $domain_extdata);
		HSPC::MT::PluginTools::set_request_data(old_nses => $nses);

#		use Data::Dumper;
#		die(Dumper($args));

		HSPC::Plugin::API::Common->init_request({}, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		my $method = $action . "_domain";
		$plugin_result = $templates->{mt_template}->$method(
			config			=> $plugin_data,
			domain			=> $domain,
			period			=> $period,
			nses			=> $nses,
			contacts		=> $contacts,
			contacts_extdata=> $contacts_extdata,
			domain_extdata	=> $domain_extdata,
		);
#		my $result = {
#			is_success => 1,
#			domain_status => "registered",
#		};
#		my $result = {
#			is_success => 0,
#			domain_status => "error",
#			message => "Error message",
#		};
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		plugin_result => $plugin_result,
		contacts => HSPC::MT::PluginTools::get_request_data("old_contacts"),
		contacts_extdata => HSPC::MT::PluginTools::get_request_data("old_contacts_extdata"),
		domain_extdata => HSPC::MT::PluginTools::get_request_data("old_domain_extdata"),
	});
}

sub get_domain_status($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $result;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});

		HSPC::Plugin::API::Common->init_request({}, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$result = $templates->{mt_template}->get_domain_status(
			config => $plugin_data,
			domain => $args->{domain},
			action => $args->{action},
		);
#		use Data::Dumper;
#		die(Dumper($collected_data));
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		plugin_result => $result,
		user_error => $user_error,
	});
}

sub synchronize_domain_ns($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $plugin_result;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $plugin_data			= prepare_hash_from_soap($args->{plugin_data}, {});
		my $domain				= $args->{domain};
		my $nses				= prepare_hash_from_soap($args->{nses}, {});

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);

#		use Data::Dumper;
#		die(Dumper($args));

		HSPC::Plugin::API::Common->init_request({}, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$plugin_result = $templates->{mt_template}->synchronize_domain_ns(
			config			=> $plugin_data,
			domain			=> $domain,
			nses			=> $nses,
		);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		plugin_result => $plugin_result,
		user_error => $user_error,
	});
}

sub get_domain_status($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $result;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});

		HSPC::Plugin::API::Common->init_request({}, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$result = $templates->{mt_template}->get_domain_status(
			config => $plugin_data,
			domain => $args->{domain},
			action => $args->{action},
		);
#		use Data::Dumper;
#		die(Dumper($collected_data));
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		plugin_result => $result,
		user_error => $user_error,
	});
}

1;

__END__
