package HSPC::Plugin::API::PM;

use strict;

use Data::Dumper;
use Error qw(:try);

use HSPC::System;
use HSPC::Config;
use HSPC::Logger;
use HSPC::Localization qw(:DEFAULT set_language);
use HSPC::PluginToolkit::General qw(argparam);
use HSPC::Plugin::API::Common qw(prepare_account_from_soap prepare_hash_from_soap APIResult APIResultErrors);

use HSPC::MT::Core::Constants;
use HSPC::MT::PluginConfig;
use HSPC::MT::PluginTools;
use HSPC::MT::PM::Tools;

use HSPC::Plugin::SOAP::Struct::APIResult;
use HSPC::Plugin::SOAP::Struct::FormResult;
use HSPC::Plugin::SOAP::Struct::CollectedResult;

use HSPC::Exception::Plugin::InvalidParameter;
use HSPC::Exception::Plugin::GeneralError;

use constant PLUGIN_TYPE => "PM";

sub view_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;
	my $form_onsubmit_js;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $secure_data	= prepare_hash_from_soap($args->{secure_data});
		my $public_data	= prepare_hash_from_soap($args->{public_data});
		my $expire_date	= $args->{expire_date};
		my $name		= $args->{name};
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		HSPC::MT::PM::Tools->init();
		my $type = $templates->{mt_template}->get_paymethod_type();
		my ($type_info) = @{HSPC::MT::PM::Tools->get_pm_template_description(types => [$type])};

		my $form = $templates->{pr_template}->view_form(
			secure_data	=> $secure_data,
			public_data	=> $public_data,
			expire_date	=> $expire_date,
			name		=> $name,

			is_admin	=> 1,
			type		=> $type,
			type_info	=> $type_info,
		);

		$form_content = $form->{content};
		$form_onsubmit_js = $form->{js_onsubmit};
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => $form_onsubmit_js,
		user_error => $user_error,
	));
}

sub edit_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;
	my $form_onsubmit_js;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};
		my $secure_data	= prepare_hash_from_soap($args->{secure_data});
		my $public_data	= prepare_hash_from_soap($args->{public_data});
		my $expire_date	= $args->{expire_date};
		my $name		= $args->{name};
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		HSPC::MT::PM::Tools->init();
		my $type = $templates->{mt_template}->get_paymethod_type();
		my ($type_info) = @{HSPC::MT::PM::Tools->get_pm_template_description(types => [$type])};

		my $form = $templates->{pr_template}->edit_form(
			secure_data	=> $secure_data,
			public_data	=> $public_data,
			expire_date	=> $expire_date,
			name		=> $name,

			is_admin	=> 1,
			type		=> $type,
			type_info	=> $type_info,
			allowed_types => [[],[[],string($templates->{mt_template}->provided_payment_method_types()->[0]->{title_id})]],
			keep_secure_code => 1,
		);

		$form_content = $form->{content};
		$form_onsubmit_js = $form->{js_onsubmit};
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => $form_onsubmit_js,
		user_error => $user_error,
	));
}

sub add_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;
	my $form_onsubmit_js;

	set_language($args->{language});

	try
	{
		my $template_id		= $args->{template_id};
		my $secure_data		= prepare_hash_from_soap($args->{secure_data});
		my $public_data		= prepare_hash_from_soap($args->{public_data});
		my $expire_date		= $args->{expire_date};
		my $name			= $args->{name};
		my $account_data	= prepare_account_from_soap($args->{account_data});
		my $request			= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		HSPC::MT::PM::Tools->init();
		my $type = $templates->{mt_template}->get_paymethod_type();
		my ($type_info) = @{HSPC::MT::PM::Tools->get_pm_template_description(types => [$type])};

		my $form = $templates->{pr_template}->add_form(
			secure_data		=> $secure_data,
			public_data		=> $public_data,
			expire_date		=> $expire_date,
			account_data	=> $account_data,
			name			=> $name,

			is_admin		=> 1,
			type			=> $type,
			type_info		=> $type_info,
			keep_secure_code => 1,
		);

		$form_content = $form->{content};
		$form_onsubmit_js = $form->{js_onsubmit};
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => $form_onsubmit_js,
		user_error => $user_error,
	));
}

sub validate($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;

	set_language($args->{language});

##	return APIResultErrors(Dumper($args));

	try {
		my $template_id		= $args->{template_id};
		my $secure_data		= prepare_hash_from_soap($args->{secure_data}, undef);
		my $public_data		= prepare_hash_from_soap($args->{public_data});

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$templates->{mt_template}->validate(
			secure_data => $secure_data,
			public_data => $public_data,
		);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::CollectedResult->new(
		plugin_data => {},
		user_error => $user_error,
	));
}

sub collect_data($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $collected_data;

	set_language($args->{language});

	try {
		my $template_id		= $args->{template_id};
		my $secure_data		= prepare_hash_from_soap($args->{secure_data});
		my $public_data		= prepare_hash_from_soap($args->{public_data});
		my $account_data	= prepare_account_from_soap($args->{account_data});
		my $request			= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		$collected_data = $templates->{pr_template}->collect_data(
			secure_data => $secure_data,
			public_data => $public_data,
			account_data => $account_data,
			keep_secure_code => 1,
		);
		$collected_data = {
			secure_data => $templates->{mt_template}->get_secure_data(data => $collected_data, keep_secure_code => 1),
			public_data => $templates->{mt_template}->get_public_data(data => $collected_data),
		}
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = $user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = $user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::CollectedResult->new(
		plugin_data => $collected_data,
		user_error => $user_error,
	));
}

sub get_secure_data($$)
{
	my $me = shift;
	my $template_id = shift;
	my $plugin_data = prepare_hash_from_soap(shift, {});
	my $data;

	my $fatal_error;
	my $other_error;

	try
	{
		$data =
			HSPC::Plugin::API::Common
				->get_plugin_templates(PLUGIN_TYPE, $template_id)
				->{mt_template}
				->get_secure_data(
					data => $plugin_data,
					keep_secure_code => 1,
				);
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult($data);
}

sub get_public_data($$)
{
	my $me = shift;
	my $template_id = shift;
	my $plugin_data = prepare_hash_from_soap(shift, {});
	my $data;

	my $fatal_error;
	my $other_error;

	try
	{
		$data =
			HSPC::Plugin::API::Common
				->get_plugin_templates(PLUGIN_TYPE, $template_id)
				->{mt_template}
				->get_public_data(data => $plugin_data);
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult($data);

}

sub get_public_number($$$)
{
	my $me = shift;
	my $template_id = shift;
	my $plugin_secure_data = prepare_hash_from_soap(shift, {});
	my $plugin_public_data = prepare_hash_from_soap(shift, {});
	my $data;

	my $fatal_error;
	my $other_error;

	try
	{
		$data =
			HSPC::Plugin::API::Common
				->get_plugin_templates(PLUGIN_TYPE, $template_id)
				->{mt_template}
				->get_public_number(
					secure_data => $plugin_secure_data,
					public_data => $plugin_public_data,
				);
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult($data);

}

1;
