package HSPC::Plugin::API::PP;

use strict;

use Data::Dumper;
use Error qw(:try);

use HSPC::System;
use HSPC::Config;
use HSPC::Logger;
use HSPC::Localization qw(:DEFAULT set_language);
use HSPC::PluginToolkit::General qw(argparam);
use HSPC::Plugin::API::Common qw(prepare_account_from_soap prepare_hash_from_soap APIResult APIResultErrors);

use HSPC::MT::Core::Constants;
use HSPC::MT::PluginConfig;
use HSPC::MT::PluginTools;
use HSPC::MT::PM::Tools;
use HSPC::MT::CCP::Constants qw(:DEFAULT :status :permissions :process :credit_types :trans_types);

use HSPC::Plugin::SOAP::Struct::APIResult;
use HSPC::Plugin::SOAP::Struct::FormResult;
use HSPC::Plugin::SOAP::Struct::CollectedResult;
use HSPC::Plugin::SOAP::Struct::HelpPageResult;

use HSPC::Exception::Plugin::InvalidParameter;
use HSPC::Exception::Plugin::GeneralError;

use constant PLUGIN_TYPE => "PP";
use constant SW_CCP_TRANS_TYPE_CALLBACK => 10;

use constant API_TRANS_TYPE => {
	'refund'			=> &SW_CCP_TRANS_TYPE_REFUND,
	'preauth'			=> &SW_CCP_TRANS_TYPE_PREAUTH,
	'preauth_reversal'	=> &SW_CCP_TRANS_TYPE_PREAUTH_REVERSAL,
	'capture'			=> &SW_CCP_TRANS_TYPE_CAPTURE,
	'capture_reversal'	=> &SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL,
	'sale'				=> &SW_CCP_TRANS_TYPE_SALE,
	'credit'			=> &SW_CCP_TRANS_TYPE_CREDIT,
	'check_status'		=> &SW_CCP_TRANS_TYPE_CHECK_STATUS,
	'callback'			=> &SW_CCP_TRANS_TYPE_CALLBACK,
};

use constant API_TRANS_STATUS => {
	'APPROVED'	=> 'approved',
	'FRAUD'		=> 'fraudulent',
	'DECLINED'	=> 'declined',
	'DELETED'	=> 'deleted',
	'UNKNOWN'	=> 'unknown',
	'PENDING'	=> 'pending',
	'AUTHCALL'	=> 'authcall',
	'ERROR'		=> 'errored',
	'REFUNDED'	=> 'refunded',
	'3DSECURE'	=> '3dsecure',
	'REDIRECT'	=> 'redirect',
};

use constant TRANS_TYPE_TO_METHOD => {
	&SW_CCP_TRANS_TYPE_PREAUTH          => "process_preauthorize",
	&SW_CCP_TRANS_TYPE_PREAUTH_REVERSAL => "process_preauthorize_void",
	&SW_CCP_TRANS_TYPE_CAPTURE          => "process_capture",
	&SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL => "process_capture_void",
	&SW_CCP_TRANS_TYPE_SALE             => "process_sale",
	&SW_CCP_TRANS_TYPE_CREDIT           => "process_credit",
	&SW_CCP_TRANS_TYPE_CHECK_STATUS     => "process_check_status",
	&SW_CCP_TRANS_TYPE_CALLBACK         => "process_callback",
};

use constant CUSTOMERS_CONST_TO_PP_CONST => {
	'APPROVED' => &SW_CCP_TRANS_APPROVED,
	'FRAUD'    => &SW_CCP_TRANS_FRAUD,
	'DECLINED' => &SW_CCP_TRANS_DECLINED,
	'PENDING'  => &SW_CCP_TRANS_PENDING,
	'AUTHCALL' => &SW_CCP_TRANS_AUTHCALL,
	'ERROR'    => &SW_CCP_TRANS_ERROR,
	'REFUNDED' => &SW_CCP_TRANS_REFUNDED,
	'3DSECURE' => &SW_CCP_TRANS_3DSECURE,
};

use constant PLUGINS_TRANS_TYPE_TO_ENGINE => {
	"capture"      => &SW_CCP_TRANS_TYPE_CAPTURE,
	"capture_void" => &SW_CCP_TRANS_TYPE_CAPTURE_REVERSAL,
};

sub view_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$me->_require_supported_pms($templates->{mt_template});

		$form_content = $templates->{pr_template}->view_form(config => $plugin_data);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => "",
		user_error => $user_error,
	));
}

sub edit_form($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $form_content;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		#use Data::Dumper;
		#die(Dumper($args->{request_context}));
		#die(Dumper($request));

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$me->_require_supported_pms($templates->{mt_template});

		$form_content = $templates->{pr_template}->edit_form(config => $plugin_data)
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::FormResult->new(
		form_content => HSPC::Plugin::API::Common->add_common_js($form_content),
		form_onsubmit_js => "",
		user_error => $user_error,
	));
}

sub collect_data($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $collected_data;

	set_language($args->{language});

	try
	{
		my $template_id	= $args->{template_id};

		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});

		my $plugin_data	= prepare_hash_from_soap($args->{plugin_data}, {});
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$me->_require_supported_pms($templates->{mt_template});

		$collected_data = $templates->{pr_template}->collect_data(config => $plugin_data);
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult(HSPC::Plugin::SOAP::Struct::CollectedResult->new(
		plugin_data => $collected_data,
		user_error => $user_error,
	));
}

sub get_supported_payment_method_types($$)
{
	my $me			= shift;
	my $template_id	= shift;

	my $fatal_error;
	my $other_error;
	my $pm_method_types;

	try
	{
		$pm_method_types =
			HSPC::Plugin::API::Common
				->get_plugin_templates(PLUGIN_TYPE, $template_id)
				->{mt_template}
				->get_supported_payment_method_types();
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult($pm_method_types);
}

sub is_trans_type_supported($$)
{
	my $me = shift;

	my $user_error;
	my $fatal_error;
	my $other_error;

	my $supported = 0;

	my $api_trans_type = &API_TRANS_TYPE;
	my $trans_type_to_method = &TRANS_TYPE_TO_METHOD;

	my $template_id	= shift;
	my $trans_type = shift;

	try
	{

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);

		if (UNIVERSAL::can($templates->{mt_template}, $trans_type_to_method->{$api_trans_type->{$trans_type}})) {
			$supported = 1;
		}
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		supported => $supported,
		user_error => $user_error,
	});

}

sub process_batch_content($$)
{
	my $me		= shift;
	my $args	= shift;

	my $fatal_error;
	my $other_error;
	my $result_data;
	my $user_error;

	try
	{
		my $template_id	= $args->{template_id};
		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$me->_require_supported_pms($templates->{mt_template});

		$result_data = $templates->{mt_template}->process_batch_content(%{$args});
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = {
			"message"	=> "".$user_error->{vendor_message},
			"error_dump"=> Dumper({
				"arguments" => $args,
				"exception" => $user_error,
			}),
		};
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	$result_data->{user_error} = $user_error if $user_error;

	return APIResult($result_data);
}

sub process_transaction($$)
{
	my $me		= shift;
	my $args	= shift;

	my $fatal_error;
	my $other_error;
	my $result_data;
	my $user_error;

#	return APIResultErrors(
#		"DebugException",
#		"Exception's dump: ".Dumper($args)
#	);

	try
	{

		my $template_id	= $args->{template_id};
		HSPC::MT::PluginTools::set_request_data(template_id => $template_id);
		HSPC::MT::PluginTools::set_request_data(callback_url => $args->{callback_url});
		HSPC::MT::PluginTools::set_request_data(referer_url => $args->{request_uri});
		HSPC::Plugin::API::Common->init_request(prepare_hash_from_soap($args->{request_context}, {}, 1), $template_id);

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$me->_require_supported_pms($templates->{mt_template});
		$result_data = $me->_process_transaction($templates->{mt_template}, $args);
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = {
			"message"	=> "".$user_error->{vendor_message}." ".$user_error->{log_message}." ".$user_error->{message},
			"error_dump"=> Dumper({
				"arguments" => $args,
				"exception" => $user_error,
			}),
		};
	}
	otherwise
	{
		$other_error = shift;
	};


	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	if ($user_error) {
		$result_data->{user_error} = $user_error;
		$result_data->{text} = "".$user_error->{message};
		$result_data->{status} = &API_TRANS_STATUS->{ERROR};
	};

	return APIResult($result_data);
}

sub collect_reference_number($$)
{
	my $me		= shift;
	my $args	= shift;

	my $user_error;
	my $fatal_error;
	my $other_error;
	my $refno;

	try
	{
		my $template_id	= $args->{template_id};
		my $request		= prepare_hash_from_soap($args->{request_context}, {}, 1);

		HSPC::Plugin::API::Common->init_request($request, $template_id);
		HSPC::MT::PluginTools::set_request_data(referer_url => $args->{referer_url});

		my $templates = HSPC::Plugin::API::Common->get_plugin_templates(PLUGIN_TYPE, $template_id);
		$me->_require_supported_pms($templates->{mt_template});

		$refno = $templates->{mt_template}->collect_transaction_refno();
	}
	catch HSPC::Exception::Plugin::InvalidParameter with
	{
		$user_error = shift;
		$user_error = "".$user_error->{message};
	}
	catch HSPC::Exception::Plugin::GeneralError with
	{
		$user_error = shift;
		$user_error = "".$user_error->{vendor_message};
	}
	catch HSPC::Exc::Fatal with
	{
		$fatal_error = shift;
	}
	otherwise
	{
		$other_error = shift;
	};

	return APIResultErrors(
		"Fatal error has occured: ".$fatal_error->{msg}.". Calls stack: ".$fatal_error->{calls_stack},
		"Exception's dump: ".Dumper($fatal_error)
	) if $fatal_error;

	return APIResultErrors(
		"Unknown error has occured: {".$other_error->{-text}." in ".$other_error->{-file}."} at line ".$other_error->{-line},
		"Exception's dump: ".Dumper($other_error)
	) if $other_error;

	return APIResult({
		reference_number => $refno,
		user_error => $user_error,
	});
}


###
### PRIVATE METHODS
###

sub _process_transaction($$)
{
	my $me = shift;
	my $mt_template = shift;
	my $args = shift;
	my $call_method = &TRANS_TYPE_TO_METHOD->{&API_TRANS_TYPE->{$args->{transaction_type}}};

	my $result = $mt_template->$call_method(%{$args});
	my $return = {};

	$return->{text} = $result->{TEXT}->{vendor_message} if $result->{TEXT};
	$return->{status} = &API_TRANS_STATUS->{$result->{STATUS}} if $result->{STATUS};
	$return->{transaction_details} = $result->{TRANSACTION_DETAILS} if $result->{TRANSACTION_DETAILS};
	$return->{next_transaction_gap} = $result->{NEXT_TRANSACTION_GAP} if $result->{NEXT_TRANSACTION_GAP};
	$return->{redirect_hash} = $result->{REDIRECT_HASH} if $result->{REDIRECT_HASH};

	return $return;
}

sub _require_supported_pms($$)
{
	my $me = shift;
	my $mt_template = shift;

	return unless $mt_template->can("get_supported_payment_method_types");

	my $supported_pms = $mt_template->get_supported_payment_method_types();
	foreach my $pm_template_id (@$supported_pms)
	{
		my $pm_mt_template = HSPC::MT::PluginConfig->find_mt_template(
		    plugin_type => "PM",
		    template_id => $pm_template_id,
		);
		eval "require $pm_mt_template";
	}
}

1;
