package HSPC::PluginToolkit::DM;

use strict;
use SWsoft::protect;

use HSPC::MT::DM::Constants;
use HSPC::MT::DM::ToolsInt;
use HSPC::MT::DM::RegEngine;
use HSPC::MT::DM::DataCheck(qw{is_fqdn is_ascii_fqdn is_idn_domain});
use HSPC::MT::DM::WhoisClient;
use HSPC::MT::DM::Core::Phone;
use HSPC::MT::PluginTools qw(get_request_data set_request_data);
use HSPC::PluginToolkit::General qw(server_name);
use HSPC::PluginToolkit::General qw(geo_get_state_name);
use HSPC::DataCheck qw(is_uk_postal_code is_email);
use HSPC::Util::DM;

use Params::Validate qw(:all);

use base qw(Exporter);
our @EXPORT_OK = qw(
	domain2utf domain2ascii extract_tld_from_domain
	extract_shortname_from_domain
	is_ascii_fqdn is_fqdn is_idn_domain
	check_phonecountry phone_as_tollfree phone_as_e164 phone_as_str
	phone_as_hash is_phone_empty
	get_contact_related_domains
	get_contact set_contact_extdata
	get_domain_extdata set_domain_extdata
	get_domain_info
	make_login make_password
	is_uk_postal_code is_email
	get_whois_info
	send_mail callback_name callback_email
	check_phonecountry_exists
	get_help_url
	is_phone_valid
	check_country_state

	## undocumented functions
	select_run select_row select_hash select_hashrows
);

sub domain2utf {
	return HSPC::Util::DM::domain2utf(@_);
}

sub domain2ascii {
	return HSPC::MT::DM::ToolsInt->domain2ascii(@_);
}

sub is_fqdn {
	return HSPC::MT::DM::ToolsInt->domain2ascii(@_);
}

sub check_country_state {
	my %h = (
		country => undef,
		state => undef,
		@_
	);

	if($h{country} =~ m/^(US|CA)$/i){
		return geo_get_state_name(
			$h{country},
			$h{state}
		) ? 1 : 0;
	}

	return 1;
}

sub is_ascii_fqdn {
	return HSPC::MT::DM::ToolsInt->domain2ascii(@_);
}

sub is_idn_domain {
	return HSPC::MT::DM::DataCheck::is_idn_domain(@_);
}

sub extract_shortname_from_domain {
	my $domain = shift;
	return HSPC::MT::DM::RegEngine->extract_shortname_from_domain(domain => $domain);
}

sub extract_tld_from_domain {
	my $domain = shift;
	return HSPC::MT::DM::RegEngine->extract_tld_from_domain(domain => $domain);
}

sub check_phonecountry {
	my $country_code = shift;
	my $phone_country_code = shift;
	return HSPC::MT::DM::Core::Phone->check_phonecountry(
		country_code => $country_code,
		phone_country_code => $phone_country_code
	);
}

sub phone_as_e164 {
	my $phone_src = shift;
	return HSPC::MT::DM::Core::Phone->phone_as_e164(
		phone_src => $phone_src,
	);
}

sub phone_as_tollfree {
	my $phone_src = shift;
	return HSPC::MT::DM::Core::Phone->phone_as_tollfree(
		phone_src => $phone_src,
	);
}

sub phone_as_str {
	my $phone_src = shift;
	return HSPC::MT::DM::Core::Phone->phone_as_str(
		phone_src => $phone_src,
	);
}

sub is_phone_empty {
	my $phone = shift;
	warn "is_phone_empty(): this function is deprecated, use is_phone_valid() instead";
	return !is_phone_valid(phone => $phone, digits_only => 0);
}

sub is_phone_valid {
	return HSPC::MT::DM::Core::Phone->is_phone_valid(@_);
}

sub phone_as_hash {
	my $phone = shift;
	return HSPC::MT::DM::Core::Phone->phone_as_hash($phone);
}

sub check_phonecountry_exists {
	my $phone_country_code = shift;
	return HSPC::MT::DM::Core::Phone->is_valid_phone_country_code(
		phone_country_code => $phone_country_code
	);
}

#sub get_contact_related_domains {
#	my %h = (
#		domain => undef,
#		contact_type => undef,
#		@_
#	);
#	my $regdomain = HSPC::MT::DM->find_regdomain(
#		domain => $h{domain}
#	);
#	my $plugin_id = $regdomain->plugin_id();
#	my $contact = HSPC::MT::DM::ContactsFactory->find_domain_contact(
#		domain => $h{domain},
#		contact_type => $h{contact_type}
#	);

#	return [] unless $contact;
#	return $contact->get_related_domains(plugin_id => $plugin_id);
#}

sub get_contact {
	my %h = (
		domain => undef,
		contact_type => undef,
		@_
	);
	my $old_contacts = get_request_data("old_contacts");
	my $old_contacts_extdata = get_request_data("old_contacts_extdata");
	return {
		contact => $old_contacts->{$h{contact_type}},
		contact_extdata => $old_contacts_extdata->{$h{contact_type}},
	};
}

sub set_contact_extdata {
	my %h = (
		domain => undef,
		contact_type => undef,
		contact_extdata => undef,
		@_
	);
	return undef unless $h{contact_extdata};

	my $old_contacts_extdata = get_request_data("old_contacts_extdata");
	$old_contacts_extdata->{$h{contact_type}} = $h{contact_extdata};
	set_request_data("old_contacts_extdata", $old_contacts_extdata);
	return undef;
}

sub get_domain_extdata {
	my %h = (
		domain => undef,
		@_
	);
	my $old_domain_extdata = get_request_data("old_domain_extdata");
	return {
		domain_extdata => $old_domain_extdata,
	};
}

sub set_domain_extdata {
	my %h = (
		domain => undef,
		domain_extdata => undef,
		@_
	);
	return undef unless $h{domain_extdata};
	set_request_data("old_domain_extdata", $h{domain_extdata});
	return undef;
}

sub make_login {
	my $login = shift;

	$login =~ s|\s||gi;
	$login =~ s|\W||gi;
	$login =~ s|\_||gi;

	## If login contain only "bad" symbols then it will be empty.
	## We make login at least working. It non unique, but
	## it's normal, because this login used for unique domain
	## and we allow only one login per domain
	$login = 'autogen' unless $login;

	$login = substr $login, 0, 20;

	return $login;
}

sub make_password {
	## Each character is selected from the range of ASCII characters delimited 
	## by (lo_char,hi_char)
	my %h = (
		minlen => 8,
		maxlen => 12,
		lo_char => 'a',
		hi_char => 'z',
		@_
	);

	require Crypt::RandPasswd;
	my $password = Crypt::RandPasswd::random_chars_in_range(
		$h{minlen}, $h{maxlen}, $h{lo_char} => $h{hi_char}
	);

	return $password;
}

##
## get_domain_info method will returned some domain information in HASHREF
##
#sub get_domain_info {
#	my %h = (
#		domain => undef,
#		@_
#	);
#	my $domain = $h{domain};
#	my $domain_info = {};
#	my $regdomain = HSPC::MT::DM->find_regdomain(
#		domain => $domain
#	);
#	return undef unless $regdomain;

#	## Preparing of NSes info
#	my $nses = [];
#	foreach my $ns (@{$regdomain->ns_info()}){
#		push(@$nses,{hostname => $ns->{hostname}, ip => $ns->{ip}});
#	}
#
#	my $domain_status = $regdomain->reg_status();
#	my $action = $regdomain->action();
#	$action = DM_ACTION_MAP_NUM_STRING->{$action};
#
#	my $general_status_map = {
#		&REG_STATUS_UNREGISTERED => &DM_DOMAIN_STATUS_UNREGISTERED,
#		&REG_STATUS_ERROR => &DM_DOMAIN_STATUS_ERROR,
#		&REG_STATUS_RENEWING => &DM_DOMAIN_STATUS_RENEWING,
#		&REG_STATUS_TERMINATING => &DM_DOMAIN_STATUS_TERMINATING,
#		&REG_STATUS_TERMINATED => &DM_DOMAIN_STATUS_TERMINATED,
#		&REG_STATUS_ZONESTARTING => &DM_DOMAIN_STATUS_ZONESTARTING,
#	};
#	my $register_status_map = {
#		%$general_status_map,
#		&REG_STATUS_REGISTERING => &DM_DOMAIN_STATUS_REGISTERING,
#		&REG_STATUS_REGISTERED => &DM_DOMAIN_STATUS_REGISTERED,
#	};
#	my $transfer_status_map = {
#		%$general_status_map,
#		&REG_STATUS_TRANSFERRING => &DM_DOMAIN_STATUS_TRANSFERRING,
#		&REG_STATUS_REGISTERED => &DM_DOMAIN_STATUS_TRANSFERRED,
#	};

#	$domain_status = $regdomain->action() == &DM_DOMAIN_ACTION_REG
#		? $register_status_map->{$domain_status}
#		: $transfer_status_map->{$domain_status};
#
#	return {
#		domain => $domain,
#		domain_status => $domain_status,
#		registration_date => $regdomain->reg_time(),
#		expiration_date => $regdomain->expire_time(),
#		nses => $nses,
#		action => $action
#	};
#}

sub is_uk_postal_code{
	my $code = shift;
	return HSPC::DataCheck::is_uk_postal_code($code);
}

sub is_email {
	my $email = shift;
	return HSPC::DataCheck::is_email_with_dot($email);
}

sub get_whois_info {
	my %h = (
		domain	=> undef,
		server	=> undef,
		port	=> 43,
		query	=> undef,
		@_
	);
	my $domain = $h{domain};
	my $server = $h{server};
	my $port = $h{port};
	my $query = $h{query};

	my $info = HSPC::MT::DM::WhoisClient->whois_query(
		domain	=> $domain,
		server	=> $server,
		port	=> $port,
		query	=> $query
	);

	return $info;
}

##TODO: prepare Email context to return it into Plesk Billing
#sub send_mail {
#	my %h = validate(@_,{
#		handler_id	=> {type => SCALAR, optional => 1},
#		charset     => {type => SCALAR, default => 'utf8'},
#		content_protect => {type => SCALAR},
#		from		=> {type => HASHREF},
#		to 			=> {type => HASHREF},
#		bcc			=> {type => HASHREF, optional => 1},
#		subject 	=> {type => SCALAR},
#		body	 	=> {type => SCALAR},
#		attachments => {type => ARRAYREF, optional => 1},
#	});

#	my $handler_id = $h{handler_id} || 'HSPC::MT::DM::PluginEngine';
#	my $provider_id = get_request_data('plugin_vendor_id');
	
#	my $mailer = HSPC::Mailer->new(
#		%h,
#		handler_id => $handler_id,
#		provider_id => $provider_id
#	);

#	$mailer->from(
#		email => $h{from}->{email},
#		name => $h{from}->{name},
#	);
#	$mailer->to(
#		email => $h{to}->{email},
#		name => $h{to}->{name},
#	);
#	if (defined $h{bcc}) {
#		$mailer->bcc(
#			email => $h{bcc}->{email},
#			name => $h{bcc}->{name},
#		);
#	}
#	$mailer->subject($h{subject});
#	$mailer->body($h{body});
#	if (defined($h{attachments}) && scalar(@{$h{attachments}})) {
#		foreach my $attach (@{$h{attachments}}) {
#			$mailer->add_attachment(
#				internal_name	=>	$attach->{internal_name},
#				file_name		=>	$attach->{file_name},
#				content_type	=>	$attach->{content_type}
#			);
#		}
#	}
#
#	$mailer->send();
#}

sub callback_name {
	my $plugin_id = get_request_data('plugin_config_id');
	my $plugin_vendor_id = get_request_data('plugin_vendor_id');
	my $plugin_template_id = get_request_data('plugin_template_id');
	my $callback_name = lc($plugin_template_id);
	my $n = $plugin_vendor_id;
	$callback_name .= $n*($n + 1);
	$n = $plugin_id;
	$callback_name .= 2*$n*$n + $n + 100000;
	return $callback_name;
}

sub callback_email {
	return callback_name() . '@' . server_name();
}

sub get_help_url {
	my %h = validate(@_,{
		action => {type => SCALAR},
		language => {type => SCALAR, optional => 1}
	});

	my $action = $h{action};
	my $language = $h{language};
	my $plugin_id = get_request_data('plugin_helper_config_id') || get_request_data('plugin_config_id');
	my $template_id = get_request_data('plugin_helper_template_id') || get_request_data('plugin_template_id');
	my $vendor_id = get_request_data('plugin_helper_vendor_id') || get_request_data('plugin_vendor_id');

	my $help_url = HSPC::MT::DM::RegEngine->get_help_url(
		plugin_id => $plugin_id,
		action => $action,
		language => $language,
		template_id => $template_id,
		vendor_id => $vendor_id
	);

	return $help_url;
}

## Undocumented functions
sub select_run {
	return undef;
##	return HSPC::WebDB::select_run(@_);
}

sub select_row {
	return undef;
##	return HSPC::WebDB::select_row(@_);
}

sub select_hash {
	return undef;
##	return HSPC::WebDB::select_hash(@_);
}

sub select_hashs {
	return undef;
##	return HSPC::WebDB::select_hashrows(@_);
}

1;
