package HSPC::PluginToolkit::DM::Protocol::EPP;

use strict;

use SWsoft::protect;

use HSPC::PluginToolkit::General qw(string);

use XML::Simple;
use XML::LibXML;
use HSPC::PluginToolkit::DM::Protocol::EPP::Constants;

use HSPC::PluginToolkit::DM qw (
	phone_as_e164
);

## ------------------------- compose -------------------------------------------

sub compose_request {
	my $class = shift;
	my %h = (
		command => undef,
		options => undef,
		@_
	);
	
	my $result;
	if ($h{command} =~ m/login|logout/) {
		$result = $class->_compose_auth_request(%h);
	}
	elsif ($h{command} =~ m/domain/) {
		$result = $class->_compose_domain_request(%h);
	}
	elsif ($h{command} =~ m/contact/) {
		$result = $class->_compose_contact_request(%h);
	}
	elsif ($h{command} =~ m/host/) {
		$result = $class->_compose_host_request(%h);
	}
	elsif ($h{command} =~ m/hello/) {
		$result->{request} = $class->_hello_request();
		$result->{is_success} = 1;
	}
	else {
		$result->{message} = string(
			'dm_epp_unknown_command_err',
			command => $h{command}
		);
	}
	return $result;
}

## ------------------------- domain -------------------------------------------

sub _compose_domain_request {
	my $class = shift;
	my %h = (
		command => undef,
		options => {},
		@_
	);
	
	my $options = $h{options} || {};
	
	my $domains = $options->{domains} || [];
	my $extension = $options->{extension} || {};
	my $domain = $options->{domain};
	my $period = $options->{period};
	my $nses = $options->{nses} || [];
	my $contacts	= $options->{contacts} || {};
	my $password = $options->{password};
	my $expiration_date = $options->{expiration_date};
	my $new_nses = $options->{new_nses} || [];
	my $old_nses = $options->{old_nses} || [];
	my $new_contacts = $options->{new_contacts} || {};
	my $old_contacts = $options->{old_contacts} || {};
	
	my $result = {
		is_success => 1,
	};
	
	if ($h{command} =~ m/check/) {
		$result->{request} = $class->_check_domains_request(
			domains => $domains,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/create/) {
		$result->{request} = $class->_create_domain_request(
			domain => $domain,
			period => $period,
			nses => $nses,
			contacts => $contacts,
			password => $password,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/delete/) {
		$result->{request} = $class->_delete_domain_request(
			domain => $domain,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/transfer/) {
		$result->{request} = $class->_transfer_domain_request(
			command => $h{command},
			domain => $domain,
			period => $period,
			password => $password,
			extension => $extension,
		);
	}
	elsif ($h{command} =~ m/update/) {
		$result->{request} = $class->_update_domain_request(
			domain => $domain,
			new_nses => $new_nses,
			old_nses => $old_nses,
			new_contacts => $new_contacts,
			old_contacts => $old_contacts,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/info/) {
		$result->{request} = $class->_info_domain_request(
			domain => $domain,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/renew/) {
		$result->{request} = $class->_renew_domain_request(
			domain => $domain,
			extension => $extension,
			expiration_date => $expiration_date,
			period => $period,
		);
	}
	else {
		$result->{message} = string(
			'dm_epp_unknown_command_err',
			command => $h{command}
		);
		$result->{is_success} = 0;
	}

	return $result;
}

sub _check_domains_request {
	my $class = shift;
	my %h = (
		domains => [],
		extension => {},
		@_
	);
	my $domains = $h{domains};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("check");
	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:check");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);

	foreach my $domain (@$domains) {
		$domain_action->appendChild(
			$class->__add_element($doc, "domain:name", $domain)
		);
	}

	my $domain_extension = $h{extension};
	my $extension = $class->_check_domains_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _create_domain_request {
	my $class = shift;
	my %h = (
		domain => undef,
		period => undef,
		nses => [],
		contacts => {},
		password => undef,
		extension => {},
		@_
	);

	my $domain = $h{domain};
	my $period = $h{period};
	my $nses = $h{nses};
	my $contacts	= $h{contacts};
	my $password = $h{password};
	my $domain_extension = $h{extension};

	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("create");
	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:create");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:name", $domain)
	);

	my $period = $class->__add_element($doc, "domain:period", $period);
	$period->setAttribute("unit", "y");
	$domain_action->appendChild($period);

	$class->__add_nses($doc, $domain_action, $domain, $nses);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:registrant", $contacts->{owner})
	);
	delete $contacts->{owner};

	my $contact_types = $class->__contact_map();
	foreach my $type (keys %$contacts) {
		my $domain_contact = $class->__add_element(
			$doc, "domain:contact", $contacts->{$type}
		);
		$domain_contact->setAttribute("type", $contact_types->{$type});
		$domain_action->appendChild($domain_contact);
	}

	my $auth = $doc->createElement("domain:authInfo");
	$auth->appendChild($class->__add_element($doc, "domain:pw", $password));
	$domain_action->appendChild($auth);
	
	my $extension = $class->_create_domain_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _delete_domain_request {
	my $class = shift;
	my %h = (
		domain => undef,
		extension => {},
		@_
	);
	
	my $domain = $h{domain};
	my $domain_extension = $h{extension};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("delete");
	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:delete");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:name", $domain)
	);
	
	my $extension = $class->_delete_domain_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _info_domain_request {
	my $class = shift;
	my %h = (
		domain => undef,
		extension => {},
		@_
	);
	
	my $domain = $h{domain};
	my $domain_extension = $h{extension};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("info");
	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:info");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:name", $domain)
	);
	
	my $extension = $class->_info_domain_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _renew_domain_request {
	my $class = shift;
	my %h = (
		domain => undef,
		extension => {},
		expiration_date => undef,
		period => undef,
		@_
	);
	
	my $domain = $h{domain};
	my $domain_extension = $h{extension};
	my $expiration_date = $h{expiration_date};
	my $period = $h{period};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("renew");
	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:renew");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:name", $domain)
	);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:curExpDate", $expiration_date)
	);

	my $period = $class->__add_element($doc, "domain:period", $period);
	$period->setAttribute("unit", "y");
	$domain_action->appendChild($period);
	
	my $extension = $class->_renew_domain_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _transfer_domain_request {
	my $class = shift;
	my %h = (
		domain => undef,
		period => undef,
		password => undef,
		extension => {},
		@_
	);
	
	my $domain = $h{domain};
	my $period = $h{period};
	my $password = $h{password};
	my $domain_extension = $h{extension};
	my $command = $h{command};

	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );
	
	my $op = "request";

	if ($command =~ m/query/) {
		$op = "query";
	}

	my $action = $doc->createElement("transfer");
	
	$action->setAttribute("op", $op);

	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:transfer");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);

	$domain_action->appendChild(
		$class->__add_element($doc, "domain:name", $domain)
	);

	if ($period) {
		my $period = $class->__add_element($doc, "domain:period", $period);
		$period->setAttribute("unit", "y");
		$domain_action->appendChild($period);
	}

	my $auth = $doc->createElement("domain:authInfo");
	$auth->appendChild($class->__add_element($doc, "domain:pw", $password));
	$domain_action->appendChild($auth);

	my $extension = $class->_transfer_domain_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _update_domain_request {
	my $class = shift;
	my %h = (
		domain => undef,
		new_nses => [],
		old_nses => [],
		new_contacts => {},
		old_contacts => {},
		extension => {},
		@_
	);
	
	my $domain = $h{domain};
	my $new_nses = $h{new_nses} || [];
	my $old_nses = $h{old_nses} || [];
	my $old_hostnames =  {map {$_->{hostname} => 1}  @$old_nses};
	my $uniq_new_ns = [];
	foreach my $ns (@$new_nses) {
	  	push @$uniq_new_ns, $ns unless ($old_hostnames->{$ns->{hostname}});
	}
	my $new_hostnames =  {map {$_->{hostname} => 1}  @$new_nses};
	my $uniq_old_ns = [];
	foreach my $ns (@$old_nses) {
		push @$uniq_old_ns, $ns unless ($new_hostnames->{$ns->{hostname}});
	}

	my $contact_types = $class->__contact_map();
	my $old_contacts = $h{old_contacts} || {};
	my $new_contacts = $h{new_contacts} || {};
	my $registrant	= $new_contacts->{owner};
	my $domain_extension = $h{extension};
	delete $new_contacts->{owner};	
	delete $old_contacts->{owner};	

	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("update");
	$doc->setDocumentElement($action);

	my $domain_action = $doc->createElement("domain:update");
	$domain_action->setNamespace($class->get_nses()->{domain} , "domain", 0 );
	$action->appendChild($domain_action);
	
	$domain_action->appendChild(
		$class->__add_element($doc, "domain:name", $domain)
	);

	if (scalar @$uniq_new_ns || (scalar keys %$new_contacts)) {
		my $domain_add = $doc->createElement("domain:add");
		$class->__add_nses($doc, $domain_add, $domain, $uniq_new_ns);
		$domain_action->appendChild($domain_add);

		foreach my $ctype (keys %$new_contacts) {
			my $domain_contact = $class->__add_element(
				$doc, "domain:contact", $new_contacts->{$ctype}
			);
			$domain_contact->setAttribute("type", $contact_types->{$ctype});
			$domain_add->appendChild($domain_contact);
		}
	}
	if ($registrant) {
		my $chg = $doc->createElement("domain:chg");
		$chg->appendChild(
			$class->__add_element($doc, "domain:registrant", $registrant)
		);
		$domain_action->appendChild($chg);
	}
	if (scalar @$uniq_old_ns || (scalar keys %$old_contacts)) {
		my $domain_del = $doc->createElement("domain:rem");
		$class->__add_nses($doc, $domain_del, $domain, $uniq_old_ns);
		$domain_action->appendChild($domain_del);

		foreach my $ctype (keys %$old_contacts) {
			my $domain_contact = $class->__add_element(
				$doc, "domain:contact", $old_contacts->{$ctype}
			);
			$domain_contact->setAttribute("type", $contact_types->{$ctype});
			$domain_del->appendChild($domain_contact);
		}
	}
	my $extension = $class->_update_domain_extension_request(
		$doc, $domain_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _check_domains_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _create_domain_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _delete_domain_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _info_domain_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _renew_domain_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _transfer_domain_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _update_domain_extension_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

## -------------------------  host  -------------------------------------------
sub _compose_host_request {
	my $class = shift;
	my %h = (
		command => undef,
		options => {},
		@_
	);
	
	my $options = $h{options} || {};
	
	my $hostnames = $options->{hostnames} || [];
	my $extension = $options->{extension} || {};
	my $ns = $options->{ns} || {};
	
	my $result = {
		is_success => 1,
		message => undef,
		request => ''
	};
	
	if ($h{command} =~ m/check/) {
		$result->{request} = $class->_check_hosts_request(
			hostnames => $hostnames,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/create/) {
		$result->{request} = $class->_create_host_request(
			ns => $ns,
			extension => $extension
		);
	}
	else {
		$result->{message} = string(
			'dm_epp_unknown_command_err',
			command => $h{command}
		);
		$result->{is_success} = 0;
	}
	return $result;
}

sub _check_hosts_request {
	my $class = shift;
	my %h = (
		hostnames => [],
		extension => {},
		@_
	);
	my $hostnames = $h{hostnames};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );
	
	my $action = $doc->createElement("check");
	$doc->setDocumentElement($action);
	
	my $host_action = $doc->createElement("host");
	my $host_action = $doc->createElement("host:check");
	$host_action->setNamespace($class->get_nses()->{host} , "host", 0 );
	$action->appendChild($host_action);
	
	foreach my $hostname (@$hostnames) {
		my $hostname_element = $class->__add_element(
			$doc, "host:name", $hostname
		);
		$host_action->appendChild($hostname_element);
	}
	my $host_extension = $h{extension};
	my $extension = $class->_check_hosts_extension_request(
		$doc, $host_extension
	);
	
	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _create_host_request {
	my $class = shift;
	my %h = (
		ns => {},
		extension => {},
		@_
	);
	my $ns = $h{ns};
	
	my $doc = XML::LibXML::Document->new( &VERSION, &ENCODING );

	my $action = $doc->createElement("create");
	$doc->setDocumentElement($action);

	my $host_action = $doc->createElement("host:create");
	$host_action->setNamespace($class->get_nses()->{host} , "host", 0 );
	$action->appendChild($host_action);

	$host_action->appendChild(
		$class->__add_element($doc, "host:name", $ns->{hostname})
	);

#	my $host_addr = $class->__add_element($doc, "host:addr", $ns->{ip});
#	$host_addr->setAttribute("ip", "v4");
#	$host_action->appendChild($host_addr);

	my $host_extension = $h{extension};
	my $extension = $class->_create_host_extension_request(
		$doc, $host_extension
	);
	
	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _check_hosts_extension_request {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _create_host_extension_request {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

## ------------------------- contact -------------------------------------------

sub _compose_contact_request {
	my $class = shift;
	my %h = (
		command => undef,
		options => {},
		@_
	);
	
	my $options = $h{options} || {};
	
	my $contact_id = $options->{contact_id};
	my $contact = $options->{contact} || {};
	my $postal_type = $options->{postal_type};
	my $password = $options->{password};
	my $extension = $options->{extension} || {};
	
	my $result = {
		is_success => 1,
		message => undef,
		request => ''
	};
	
	if ($h{command} =~ m/create/) {
		$result->{request} = $class->_create_contact_request(
			contact_id => $contact_id,
			contact => $contact,
			postal_type => $postal_type,
			password => $password,
			extension => $extension
		);
	}
	elsif ($h{command} =~ m/info/) {
		$result->{request} = $class->_info_contact_request(
			contact_id => $contact_id,
			password => $password,
			extension => $extension
		);
	} else {
		$result->{message} = string(
			'dm_epp_unknown_command_err',
			command => $h{command}
		);
		$result->{is_success} = 0;
	}
	return $result;
}

sub _create_contact_request {
	my $class = shift;
	my %h = (
		contact_id => undef,
		contact => {},
		postal_type => undef,
		password => undef,
		extension => {},
		@_
	);
	my $contact_id = $h{contact_id};
	my $contact = $h{contact};
	my $postal_type = $h{postal_type};
	my $password = $h{password};
	my $contact_extension = $h{extension};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );
	
	my $action = $doc->createElement("create");
	$doc->setDocumentElement($action);
	
	my $contact_action = $doc->createElement("contact:create");
	$contact_action->setNamespace(
		$class->get_nses()->{contact} , "contact", 0 
	);
	$action->appendChild($contact_action);
	
	$contact_action->appendChild(
		$class->__add_element($doc, "contact:id", $contact_id)
	);
	
	my $postal = $doc->createElement("contact:postalInfo");
	$postal->setAttribute("type", $postal_type);
	
	##create name element of postalInfo
	my $name_val;
	if ($contact->{fname} && $contact->{lname}) {
		$name_val = $contact->{fname}. ' '. $contact->{lname};
	} elsif ($contact->{name}) {
		$name_val = $contact->{org_name};
	}
	
	if ($name_val) {
		my $name = $class->__add_element(
			$doc,
			"contact:name",
			$name_val
		);
		$postal->appendChild($name);
	}
	
	$postal->appendChild(
		$class->__add_element($doc, "contact:org", $contact->{org_name})
	);
	
	my $addr = $doc->createElement("contact:addr");
	$addr->appendChild(
		$class->__add_element($doc, "contact:street", $contact->{address})
	);
	$addr->appendChild(
		$class->__add_element($doc, "contact:city", $contact->{city})
	);
	$addr->appendChild(
		$class->__add_element($doc, "contact:sp", $contact->{state})
	);
	$addr->appendChild(
		$class->__add_element($doc, "contact:pc", $contact->{zip})
	);
	$addr->appendChild(
		$class->__add_element($doc, "contact:cc", $contact->{country})
	);
	$postal->appendChild($addr);
	
	$contact_action->appendChild($postal);
	
	my $phone = $class->__add_element(
		$doc,
		"contact:voice",
		phone_as_e164($contact->{phone})
	);
	$contact_action->appendChild($phone);
	
	my $fax = $class->__add_element(
		$doc,
		"contact:fax",
		phone_as_e164($contact->{fax})
	);
	$contact_action->appendChild($fax);
	
	$contact_action->appendChild(
		$class->__add_element($doc, "contact:email", $contact->{email})
	);
	
	my $auth = $doc->createElement("contact:authInfo");
	$auth->appendChild(
		$class->__add_element($doc, "contact:pw", $password)
	);
	$contact_action->appendChild($auth);
	
	my $extension = $class->_create_contact_extension_request(
		$doc, $contact_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _info_contact_request {
	my $class = shift;
	my %h = (
		contact_id => undef,
		password => undef,
		extension => {},
		@_
	);
	my $contact_id = $h{contact_id};
	my $password = $h{password};
	my $contact_extension = $h{extension};
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("info");
	$doc->setDocumentElement($action);

	my $contact_action = $doc->createElement("contact:info");
	$contact_action->setNamespace(
		$class->get_nses()->{contact},
		"contact",
		0
	);
	$action->appendChild($contact_action);

	$contact_action->appendChild(
		$class->__add_element($doc, "contact:id", $contact_id)
	);

	my $auth = $doc->createElement("contact:authInfo");
	$auth->appendChild(
		$class->__add_element($doc, "contact:pw", $password)
	);
	$contact_action->appendChild($auth);

	my $extension = $class->_info_contact_extension_requets(
		$doc, $contact_extension
	);

	return $class->__form_request(
		command => $action,
		extension => $extension
	);
}

sub _create_contact_extension_request_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}

sub _info_contact_extension_request_request {
	my $class = shift;
	my $doc = shift;
	my $extension = shift;
	return undef;
}


## -------------------------  auth  -------------------------------------------


sub _compose_auth_request {
	my $class = shift;
	my %h = (
		command => undef,
		options => {},
		@_
	);
	
	my $options = $h{options} || {};
	
	my $config = $options->{config} || {};
	
	my $result = {
		is_success => 1,
		message => undef,
		request => ''
	};
	
	if ($h{command} =~ m/login/) {
		$result->{request} = $class->_login_request(
			config => $config
		);
	}
	elsif ($h{command} =~ m/logout/) {
		$result->{request} = $class->_logout_request();
	}
	else {
		$result->{message} = string(
			'dm_epp_unknown_command_err',
			command => $h{command}
		);
		$result->{is_success} = 0;
	}
	return $result;
}

sub _hello_request {
	my $class = shift;
	my %h = (
		@_
	);

	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );
	$doc->setStandalone(0);

	my $root = $doc->createElementNS($class->get_nses()->{epp}, 'epp');
	$doc->setDocumentElement($root);
	$root->setAttributeNS(
		$class->get_nses()->{xml_schema_instance},
		"xsi:schemaLocation",
		$class->get_nses()->{epp} . ' ' . $class->get_xsds()->{epp}
	);

	my $command = $doc->createElement("hello");
	$root->appendChild($command);

	return $doc->toString(1);
}

sub _login_request {
	my $class = shift;
	my %h = (
		config => {},
		@_
	);
	my $config = $h{config};

	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );

	my $action = $doc->createElement("login");
	$doc->setDocumentElement($action);

	$action->appendChild(
		$class->__add_element($doc, "clID", $config->{login})
	);

	$action->appendChild(
		$class->__add_element($doc, "pw", $config->{password})
	);

	my $options = $doc->createElement("options");
	$options->appendChild(
		$class->__add_element($doc, "version", VERSION)
	);	
	$options->appendChild(
		$class->__add_element($doc, "lang", LANG)
	);	
	$action->appendChild($options);

	my $svcs = $doc->createElement("svcs");
	$svcs->appendChild(
		$class->__add_element($doc, "objURI", $class->get_nses()->{contact})
	);	
	$svcs->appendChild(
		$class->__add_element($doc, "objURI", $class->get_nses()->{host})
	);	
	$svcs->appendChild(
		$class->__add_element($doc, "objURI", $class->get_nses()->{domain})
	);	
	my $svcs_ext = $class->_svc_extension_request($doc);
	$svcs->appendChild($svcs_ext) if $svcs_ext;
	$action->appendChild($svcs);

	return $class->__form_request(
		command => $action
	);
}

sub _logout_request {
	my $class = shift;
	my %h = (
		@_
	);
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );
	my $action = $doc->createElement("logout");
	return $class->__form_request(command => $action);
}

sub _svc_extension_request {
	my $class = shift;
	my $doc = shift;
	return undef;
}

## ---------------------------------- parse ------------------------------------

sub parse_response {
	my $class = shift;
	my %h = (
		command => undef,
		response => undef,
		@_
	);

	my $response = $class->__parse(%h);
	return $response unless $response->{is_success};
	
	my $result = {
		is_success => 1
	};

	if ($h{command} =~ m/login|logout/) {
		$result = $class->_parse_auth_response(
			command => $h{command},
			response => $response->{resData},
			extension => $response->{extension}
		);
	}
	elsif ($h{command} =~ m/domain/) {
		$result = $class->_parse_domain_response(
			command => $h{command},
			response => $response->{resData},
			extension => $response->{extension},
			code => $response->{code}
		);
	}
	elsif ($h{command} =~ m/contact/) {
		$result = $class->_parse_contact_response(
			command => $h{command},
			response => $response->{resData},
			extension => $response->{extension}
		);
	}
	elsif ($h{command} =~ m/host/) {
		$result = $class->_parse_host_response(
			command => $h{command},
			response => $response->{resData},
			extension => $response->{extension}
		);
	}
	elsif ($h{command} =~ m/hello/) {
		$result = $class->_login_response(
			response => $response->{resData},
			extension => $response->{extension}
		);
	}
	else {
		$result = {
			message => string(
				'dm_epp_unknown_command_err',
				command => $h{command}
			),
			is_success => 0
		};
	}

	return $result;
}

sub __parse {
	my $class = shift;
	my %h = (
		command => undef,
		response => undef,
		@_
	);

	return {
		is_success => 0,
		message => string('dm_epp_illegal_input_params_err')
	} unless $h{response};
		
	my $response = eval {XMLin($h{response})->{response}};
	return {
		is_success => 0,
		message => string(
			'dm_epp_xml_err',
			error => $@
		)
	} if ($@);
	
	my $result = $response->{result};
	$result = [$result] if (ref $result	ne 'ARRAY');
	
	my $message = '';
	my $is_success = 1;

	foreach my $res (@$result) {
		my $code = $res->{code};

		$message .=  $code. '. '; 
		$message .= ref($res->{msg}) eq 'HASH' ? 
			$res->{msg}->{content} : 
			$res->{msg};
		$message .= '; ';
		
		$is_success = (
			$code =~ m/^1\d+$/ ? 1 : 0
		) if $is_success;
	};

	my $code = $$result[0]->{code};
	return {
		is_success => $is_success,
		message => $message,
		resData => $response->{resData},
		extension => $response->{extension},
		raw_response => $h{response},
		code => $code
	};
}

## --------------------------- Domain ------------------------------------------

sub _parse_domain_response {
	my $class = shift;
	my %h = (
		command => undef,
		response => {},
		extension => {},
		code => undef,
		@_
	);
	
	my $result = {is_success => 1};
	
	if ($h{command} =~ m/check/) {
		$result->{response} = $class->_check_domains_response(
			response => $h{response},
			extension => $h{extension},
		);
	} elsif ($h{command} =~ m/create/) {
		$result->{response} = $class->_create_domain_response(
			response => $h{response},
			extension => $h{extension},
			code => $h{code}
		);
	}	elsif ($h{command} =~ m/update/) {
		$result->{response} = $class->_update_domain_response(
			response => $h{response},
			extension => $h{extension},
			code => $h{code}
		);
	} elsif ($h{command} =~ m/info/) {
		$result->{response} = $class->_info_domain_response(
			response => $h{response},
			extension => $h{extension}
		);
	} 	elsif ($h{command} =~ m/renew/) {
		$result->{response} = $class->_renew_domain_response(
			response => $h{response},
			extension => $h{extension},
			code => $h{code}
		);
	} 	elsif ($h{command} =~ m/transfer/) {
		$result->{response} = $class->_transfer_domain_response(
			response => $h{response},
			extension => $h{extension}
		);
	}	elsif ($h{command} =~ m/delete/) {
		$result->{response} = $class->_delete_domain_response(
			response => $h{response},
			extension => $h{extension},
			code => $h{code}
		);
	}	else {
		$result = {
			message => string(
				'dm_epp_unknown_command_err',
				command => $h{command}
			),
			is_success => 0
		};
	}
	return $result;
}

sub _check_domains_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);

	my $result = {};

	my $dc = $h{response}->{"domain:chkData"}->{"domain:cd"};
	$dc = [$dc] if (defined($dc) && ref($dc)	ne 'ARRAY');
	foreach my $dn (@$dc) {
		my $availiable = $dn->{"domain:name"}->{avail};
		$availiable = ($availiable eq 'true') ? 1 : 0 
			if ($availiable !~ m/^[0-1]$/);	
		$result->{$dn->{"domain:name"}->{content}} = $availiable;
	}

	my $extension = $class->_check_domains_extension_response(%h) || {};

	return {%$result, %$extension};
	
}

sub _create_domain_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		code => undef,
		@_
	);
	my $result = {};
	
	$h{response}->{"domain:creData"}->{"domain:exDate"} =~ m/(.*)T(.*)/;
	$result->{expiration_date} = $1 if ($1);
	
	$h{response}->{"domain:creData"}->{"domain:crDate"} =~ m/(.*)T(.*)/;
	$result->{registration_date} = $1 if ($1);
	
	## 1001 is code for pending in EPP
	$result->{domain_status} = ($h{code} == 1001) ? 'registering' : 'registered';
	
	my $extension = $class->_create_domain_extension_response(%h) || {};
	
	return {%$result, %$extension};
}

sub _delete_domain_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		code => undef,
		@_
	);
	
	my $result = {};
	
	## 1001 is code for pending in EPP
	$result->{domain_status} = ($h{code} == 1001 ) ? 'terminating' : 'terminated';
	
	my $extension = $class->_delete_domain_extension_response(%h) || {};
	
	return {%$result, %$extension};
}

sub _info_domain_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);

	my $result = {};
	
	my $inf_data = $h{response}->{"domain:infData"};
		
	##Parse domain status
	$result->{status} = $inf_data->{"domain:status"}->{s};
		
	## Parse NSes list
	if(defined($inf_data->{"domain:ns"})){
		$result->{nses} = [];

		my $nses = $inf_data->{"domain:ns"}->{"domain:hostObj"};
		$nses = [$nses] if (defined($nses) && ref($nses)	ne 'ARRAY');
		foreach my $ns (@$nses){
			push @{$result->{nses}}, {
				hostname => $ns
			};	
		}
	
		$nses = $inf_data->{"domain:ns"}->{"domain:hostAttr"};
		$nses = [$nses] if (defined($nses) && ref($nses) ne 'ARRAY');
		foreach my $ns (@$nses){
			my $ip = $ns->{"domain:hostAddr"}->{content};
			$ip = '' if $ns->{"domain:hostAddr"}->{ip} ne 'v4';
			push @{$result->{nses}}, {
				hostname => $ns->{"domain:hostName"},
				ip => $ip
			};	
		}
	}

	## Parse expiration date
	$inf_data->{"domain:exDate"} =~ m/(.*)T(.*)/;
	$result->{expiration_date} = $1 if ($1);

	## Parse registration date
	$inf_data->{"domain:crDate"} =~ m/(.*)T(.*)/;
	$result->{registration_date} = $1 if ($1);

	## Parse contacts
	my $handles = {}; 	
	$handles->{owner} = defined($inf_data->{"domain:registrant"}) ?
		$inf_data->{"domain:registrant"} :
		$inf_data->{"domain:clID"};
	my $contacts = $inf_data->{"domain:contact"};
	$contacts = [$contacts] 
		if (defined($contacts) && ref($contacts) ne 'ARRAY');
	my %contact_map = reverse %{&CONTACT_MAP};
	foreach my $contact (@$contacts){
		$handles->{$contact_map{$contact->{type}}} = $contact->{content};
	}
	$result->{handles} = $handles;
	
	my $extension = $class->_info_domain_extension_response(%h) || {};
	
	return {%$result, %$extension};
}

sub _renew_domain_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		code => undef,
		@_
	);
	my $result = {};
	
	my $exp_date = $h{response}->{"domain:renData"}->{"domain:exDate"};
	if ($exp_date) {
		$exp_date =~ m/(.*)T(.*)/;
		$result->{expiration_date} = $1;
	}
	
	## 1001 is code for pending in EPP
	$result->{domain_status} = ($h{code} == 1001 ) ? 'renewing' : 'renewed';
	
	my $extension = $class->_renew_domain_extension_response(%h) || {};
	
	return {%$result, %$extension};
}

sub _transfer_domain_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);
	
	my $result = {};
	
	my $tr_data = $h{response}->{"domain:trnData"};
		
	## Parse expiration date
	$tr_data->{"domain:exDate"} =~ m/(.*)T(.*)/;
	$result->{expiration_date} = $1 if ($1);
	my $status = $tr_data->{"domain:trStatus"};
	if ($status eq "pending") {
		$result->{domain_status} = 'transferring';
	} elsif ($status eq "clientApproved" || $status eq "serverApproved") {
		$result->{domain_status} = 'transferred';
	} else { ## clientCancelled clientRejected serverCancelled
		$result->{domain_status} = 'error';
	}
	
	my $extension = $class->_transfer_domain_extension_response(%h) || {};
	
	return {%$result, %$extension};	
}

sub _update_domain_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		code => undef,
		@_
	);
	my $extension = $class->_update_domain_extension_response(%h) || {};
	return $extension;
}

sub _check_domains_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _create_domain_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _delete_domain_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _info_domain_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _renew_domain_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _transfer_domain_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _update_domain_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

## --------------------------- Contact ------------------------------------------

sub _parse_contact_response {
	my $class = shift;
	my %h = (
		command => undef,
		response => {},
		extension => {},
		@_
	);
	
	my $result = {is_success => 1};
	
	if ($h{command} =~ m/create/) {
		$result->{response} = $class->_create_contact_response(
			response => $h{response},
			extension => $h{extension}
		);
	}
	elsif ($h{command} =~ m/info/) {
		$result->{response} = $class->_info_contact_response(
			response => $h{response},
			extension => $h{extension}
		);
	}	else {
		$result = {
			message => string(
				'dm_epp_unknown_command_err',
				command => $h{command}
			),
			is_success => 0
		};
	}
	return $result;
}

sub _create_contact_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);
	my $result = {};

	$result->{handle} = $h{response}->{"contact:creData"}->{"contact:id"};
	
	my $extension = $class->_create_contact_extension_response(%h) || {};

	return {%$result, %$extension};
}

sub _info_contact_response{
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);
	my $result = {};

	my $inf_data = $h{response}->{"contact:infData"};

	$result->{handle} = $inf_data->{"contact:id"};

	my $postal_info = $inf_data->{"contact:postalInfo"};

	$result->{postal_type} = $postal_info->{"type"};
	$result->{name} = $postal_info->{"contact:name"};
	$result->{org_name} = $postal_info->{"contact:org"};
	## we have only one street
	$result->{address} = $postal_info->{"contact:street"};
	$result->{city} = $postal_info->{"contact:city"};
	$result->{state} = $postal_info->{"contact:sp"};
	$result->{zip} = $postal_info->{"contact:pc"};
	$result->{country} = $postal_info->{"contact:cc"};

	if(defined($inf_data->{"contact:voice"})){
		my $epp_phone = ref($inf_data->{"contact:voice"}) eq 'HASH' ?
			$inf_data->{"contact:voice"}->{content} :
			$inf_data->{"contact:voice"};
		my $epp_phone_ext = $inf_data->{"contact:voice"}->{x};
		$result->{phone} = $class->_parse_phone(
			phone => $epp_phone,
			extension => $epp_phone_ext
		);
	}

	if(defined($inf_data->{"contact:fax"})){
		my $epp_fax = ref($inf_data->{"contact:fax"}) eq 'HASH' ?
			$inf_data->{"contact:fax"}->{content} :
			$inf_data->{"contact:fax"};
		my $epp_fax_ext = $inf_data->{"contact:fax"}->{x};
		$result->{fax} = $class->_parse_phone(
			phone => $epp_fax,
			extension => $epp_fax_ext
		);
	}

	$result->{email} = $inf_data->{"contact:email"};
	
	if(defined($inf_data->{"contact:authInfo"})){
		$result->{password} = $inf_data->{"contact:authInfo"}->{"contact:pw"} 
			if $inf_data->{"contact:authInfo"}->{"contact:pw"};
	}

	my $extension = $class->_info_contact_extension_response(%h) || {};

	return {%$result, %$extension};
}

sub _create_contact_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _info_contact_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

## --------------------------- Auth ---------------------------------------------

sub _parse_auth_response {
	my $class = shift;
	my %h = (
		command => undef,
		response => {},
		extension => {},
		@_
	);
	
	my $result = {is_success => 1};
	
	if ($h{command} =~ m/login/ ) {
		$result->{response} = $class->_login_response(
			response => $h{response},
			extension => $h{extension}
		);
	} elsif ($h{command} =~ m/logout/) {
		$result->{response} = $class->_logout_response(
			response => $h{response},
			extension => $h{extension}
		);
	}	else {
		$result = {
			message => string(
				'dm_epp_unknown_command_err',
				command => $h{command}
			),
			is_success => 0
		};
	}
	return $result;
}

sub _login_response {
	return undef;
}

sub _logout_response {
	return undef;
}

sub _hello_response {
	return undef;
}

## --------------------------- Host ---------------------------------------------

sub _parse_host_response {
	my $class = shift;
	my %h = (
		command => undef,
		response => {},
		extension => {},
		@_
	);
	
	my $result = {is_success => 1};
	
	if ($h{command} =~ m/check/) {
		$result->{response} = $class->_check_hosts_response(
			response => $h{response},
			extension => $h{extension}
		);
	} elsif ($h{command} =~ m/create/) {
		$result->{response} = $class->_create_host_response(
			response => $h{response},
			extension => $h{extension}
		);
	}	else {
		$result = {
			message => string(
				'dm_epp_unknown_command_err',
				command => $h{command}
			),
			is_success => 0
		};
	}
	return $result;
}

sub _check_hosts_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);

	my $result = {};

	my $dc = $h{response}->{"host:chkData"}->{"host:cd"};
	$dc = [$dc] if (defined($dc) && ref($dc) ne 'ARRAY');
	foreach my $dn (@$dc) {
		$result->{$dn->{"host:name"}->{content}} = ($dn->{"host:name"}->{avail} eq 'true') ? 1 : 0;
	}

	my $extension = $class->_check_hosts_extension_response(%h) || {};

	return {%$result, %$extension};
}

sub _create_host_response {
	my $class = shift;
	my %h = (
		response => {},
		extension => {},
		@_
	);
	return undef;
}

sub _check_hosts_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

sub _create_host_extension_response {
	my $class = shift;
	my %h = (
		extension => {},
		@_
	);
	return undef;
}

#### ------ private methods -------------------------------

sub __generate_cltrid {
	return "hspc-trans-id-".time();
}

sub __schema_location {
	my $class = shift;
	my $attr = [];
	foreach (keys %{$class->get_nses()}) { 
				push(
					@$attr,
					$class->get_nses()->{$_} . ' ' . $class->get_xsds()->{$_}
				);
	}
	return join ' ', @$attr;
}

sub __form_request {
	my $class = shift;
	my %h = (
		command => undef,
		extension => undef,
		@_
	);
	
	my $doc = XML::LibXML::Document->new( VERSION, ENCODING );
	$doc->setStandalone(0);

	my $root = $doc->createElementNS($class->get_nses()->{epp}, 'epp');
	$doc->setDocumentElement($root);
	$root->setAttributeNS(
		$class->get_nses()->{xml_schema_instance},
		"xsi:schemaLocation",
		$class->__schema_location()
	);

	my $command = $doc->createElement("command");
	$root->appendChild($command);
	$command->appendChild($h{command});

	if ($h{extension}) {
		my $extension = $doc->createElement("extension");
		$extension->appendChild($h{extension});
		$command->appendChild($extension);
	}

	$command->appendChild(
		$class->__add_element($doc, 'clTRID', __generate_cltrid())
	);
	
	return $doc->toString(1);
}

sub __contact_map {
	my $class = shift;
	return &CONTACT_MAP;
};

sub __add_nses {
	my $class = shift;
	my $doc = shift;
	my $domain_action = shift;
	my $domain = shift;
	my $nses = shift || [];
	
	my $domain_ns = $doc->createElement("domain:ns");
	foreach my $ns (@$nses) {
		my $domain_host_addr = $doc->createElement("domain:hostAttr");
		$domain_host_addr->appendChild(
			$class->__add_element($doc, "domain:hostName", $ns->{hostname})
		);
		if ($ns->{hostname} =~ m/$domain$/) {
			$domain_host_addr->appendChild(
				$class->__add_element($doc, "domain:hostAddr", $ns->{ip})
			);
		}
		$domain_ns->appendChild($domain_host_addr);
	}
	$domain_action->appendChild($domain_ns);

	return undef;
}
	
#### ------ work with XML -----------------------------------

sub __add_element_if_not_empty {
	my $class = shift;
	return $class->__add_element(@_, 1);
}

sub __add_element {
	my $class = shift;
	my $doc = shift;
	my $elem_name = shift;
	my $elem_value = shift;
	my $add_if_not_empty = shift;

	if ($add_if_not_empty and ! length $elem_value){
		return undef;
	}

	my $element = $doc->createElement($elem_name);
	if (defined $elem_value){
		my $element_value = XML::LibXML::Text->new($elem_value);
		$element->appendChild($element_value);
	}

	return $element;
}

## returns hashref with {ns_prefix => ns} pairs 
## you must redefine this method to use your own nses
sub get_nses {
	my $class = shift;
	return &NS;	
}

## returns hashref with {ns_prefix => shema_instance} pairs 
## you must redefine this method to use your own shema instances
sub get_xsds {
	my $class = shift;
	return &XSD;
}

####--------------------------------other---------------------------------------

sub _parse_phone{
	my $class = shift;
	my %h = (
		phone => undef,
		extension => undef,
		@_
	);	
	my $extension = $h{extension} || '';
	
	my $hspc_phone = $h{phone};
	if($h{phone} =~ /^\+(\d+)\.(\d+)(x(\d+))?$/){
		$hspc_phone = $1 . '||' . $2 . '|' . $4;
	}
	elsif($h{phone} =~ /^\+(\d+)\.(\d+)$/){
		$hspc_phone = $1 . '||' . $2 . '|' . $extension;
	} else {
		$hspc_phone = '||' . $h{phone} . '|';
	}

	return $hspc_phone;
}
1;
