package HSPC::PluginToolkit::DM::Transport::Socket;

use strict;

use IO::Socket::INET;
use IO::Socket::SSL;
use Data::Dumper;

sub _debug {
	my $this = shift;
	my $message = shift;
	print $message;
}

sub new {
	my $class = shift;
	my %h = (
		ssl => 0,
		use_length => 0,
		socket_host => undef,
		socket_port => undef,
		socket_proto => 'tcp',
		socket_timeout => 60,
		recv_timeout => 100,
		debug => 0,
		@_
	);

	my $name = $h{ssl} ? "SSL Socket" : "INET Socket";
	my $socket_class = $h{ssl} ? 'IO::Socket::SSL' : 'IO::Socket::INET';

	my $options = {
		PeerHost => $h{socket_host},
		PeerPort => $h{socket_port},
		Proto => $h{socket_proto},
		Timeout => $h{socket_timeout}
	};

	$class->_debug("Try to connect... ") if($h{debug});

	my $client = $socket_class->new(%$options) ||
		return "Can't open $name with error: [" . $@ . "]";

	$class->_debug("OK\n") if($h{debug});

	my $self = {
		__name => $name,
		__client => $client,
		__options => \%h,
	};
	bless($self, $class);

	return $self;
}

sub name {
	my $self = shift;
	return $self->{__name};
}

sub client {
	my $self = shift;
	return $self->{__client};
}

sub options {
	my $self = shift;
	return $self->{__options};
}

sub destroy {
	my $self = shift;
	my $client = $self->client() ||
		return {
			is_success => 0,
			message => "Socket couldn't be closed, " .
				$self->name() . " should be opened first"
		};

	$client->close();

	return {
		is_success => 1
	};
}

sub send {
	my $self = shift;
	my %h = (
		data => undef,
		@_
	);
	
	
	my $client = $self->client() ||
		return {
			is_success => 0,
			message => "Data couldn't be write, " .
				$self->name() . " should be opened first"
		};
	my $data = $self->options()->{use_length} ?
		pack('N', length($h{data}) + 4).$h{data} :
		$h{data};

	$self->_debug("Send Data: [" . $h{data} . "]") if($self->options()->{debug});

	print $client "" . $data;

	return {
		is_success => 1
	};
}

sub recv {
	my $self = shift;
	my %h = (
		eop => undef,
		@_
	);
	my $client = $self->client() ||
		return {
			is_success => 0,
			message => "Data couldn't be read, " .
				$self->name() . " should be opened first"
		};

	$self->_debug("CLIENT RECV: " . Dumper($client)) if($self->options()->{debug});

	my ($data, $result);
	my $timeout_message = "READ SOCKET TIMEOUT\n";
	eval {
		local $SIG{ALRM} = sub {
			die($timeout_message);
		};
		alarm($self->options()->{recv_timeout});
		if($self->options()->{use_length}){
			my $length;
			read($client, $length, 4);
			$length = (unpack('N', $length) - 4);

			if ($length < 1) {
				die("Got a bad response length from server");
			} else {
				while (length($data) < $length) {
					my $octet;
					read($client, $octet, ($length - length($data)));
					last if (length($octet) == 0);
					$data .= $octet;
				}
			}
		} else {
			local $/ = $h{eop};
			$data = <$client>;
		}
		
		alarm(0);
	};
	if($@){
		if($@ eq $timeout_message){
			$result = {
				is_success => 0,
				message => "Read socket timeout",
				data => $data ## in this case length is present
			};
		} else {
			$result = {
				is_success => 0,
				message => "Read socket error: " . $@,
				data => $data ## in this case length is present
			};
		}
	} else {
		$result = {
			is_success => 1,
			data => $data
		};
	}

	return $result;
}

sub sendrecv {
	my $self = shift;
	my %h = (
		data => undef,
		eop => undef,
		@_
	);
	$self->_debug("Try to send data... ") if($self->options()->{debug});

	my $send_result = $self->send(data => $h{data});
	return {
		is_success => 0,
		message => "Communication error: " . $send_result->{message}
	} if(!$send_result->{is_success});

	$self->_debug("OK\nTry to receive data... ") if($self->options()->{debug});

	my $recv_result = $self->recv(eop => $h{eop});
	return {
		is_success => 0,
		message => "Communication error: " . $recv_result->{message},
		data => $recv_result->{data}
	} if(!$recv_result->{is_success});

	return $recv_result;
}

1;
