package HSPC::PluginToolkit::DM::Validate;
use strict;
use SWsoft::protect;

use HSPC::ParamsValidate qw(:DEFAULT);
use HSPC::Localization;
use HSPC::PluginToolkit::DM qw(
	is_email
	is_phone_valid
	check_country_state
);
use HSPC::PluginToolkit::General qw(geo_get_country_name);

use base qw(Exporter);
our @EXPORT_OK = qw(check_value check_data check_data_ascii check_base_contacts);

use constant REGEXP_ASCII => q|^\p{ASCII}*$|;

## Description:
##		Check HASHREF data on specified regexp
## Input params:
##		data	=> HASHREF
##		regexp	=> regexp for validation all values in data hashref
## Return:
##		HASREF with the same structure as data with boolean values,
##		that indicated valid or not valid value by regexp rules
##
sub check_data {
	my %h = validate_params(
		@_,
		{
			data	=> {type => HASHREF},
			regexp	=> {type => SCALAR},
		}
	);

	return _check_data($h{data}, $h{regexp});
}

## Description:
##		Check value on specified regexp
## Input params:
##		value	=> scalar values
##		regexp	=> regexp for validation value
## Return:
##		1 or 0, that indicated valid or not valid value by regexp rules
##
sub check_value {
	my %h = validate_params(
		@_,
		{
			value	=> {type => SCALAR},
			regexp	=> {type => SCALAR},
		}
	);

	return _check_data($h{data}, $h{regexp});
}

## Description:
##		Check HASHREF data on ASCII value
## Input params:
##		data	=> HASHREF
## Return:
##		HASREF with the same structure as data with boolean values,
##		that indicated valid or not valid value by ASCII rules
##
sub check_data_ascii {
	my %h = @_;
	return check_data(data => $h{data}, regexp => &REGEXP_ASCII);
}

sub _check_value {
	my $val = shift;
	my $re = shift;

	return ($val =~ m#$re#) ? 1 : 0;
}

sub _check_data {
	my $data = shift; ## ref to hash
	my $re = shift || '';

	my $result = {};
	sub __check_fields {
		my $data = shift; ## ref to hash
		my $result = shift;
		my $re = shift;

		foreach my $key (keys %$data) {
			my $val = $data->{$key};
			$result->{$key} = 1;
			if (ref($val) eq 'HASH') {
				my $h = {};
				__check_fields($val,$h,$re);
				if (keys %$h) {
					$result->{$key} = $h;
				}
			} else {
				$result->{$key} = _check_value($val, $re);
			}
		}
	}

	__check_fields($data, $result, $re);

	return $result;
}

sub check_base_contacts {
	my %h = validate_params(@_,{
		contacts => {type => HASHREF},
		mandatory_fields => {type => ARRAYREF},
		check_ascii => {type => SCALAR, default => 0},
		force_state_required => {type => ARRAYREF, optional => 1},
	});

	foreach my $type (keys(%{$h{contacts}})){
		$h{contacts}->{$type} = {} if(!$h{contacts}->{$type});
	}

	my %fields_loc_map = (
		fname => 'fname',
		lname => 'lname',
		email => 'email',
		phone => 'phone',
		fax => 'fax',
		address => 'address',
		city => 'city',
		state => 'state',
		zip => 'zip_code',
		country => 'country',
		org_name => 'company_name',
	);
	my @ascii_fields = qw(org_name fname lname address city state);

	my $error_list = [];
	my $is_empty = check_data(data => $h{contacts}, regexp => '^\s*$');

	my $is_ascii = undef;
	if ($h{check_ascii}) {
		$is_ascii = check_data_ascii(data => $h{contacts});
	}

	my @mand_fields = @{$h{mandatory_fields}};
	foreach my $type (keys %{$h{contacts}}){
		for my $field (@mand_fields){
			if ($is_empty->{$type}==1) {
				## all hash for this contact type is empty => show warnings for all mandatory fields
				push @$error_list, {
					form => 'contact',
					contact_type => $type,
					field => $field,
					message => string('dm_field_mandatory',
						field => string($fields_loc_map{$field})
					),
				};
			} elsif ($is_empty->{$type}->{$field}) {
				if(($field ne 'org_name') || ($h{contacts}->{$type}->{is_corporate})){
					push @$error_list, {
						form => 'contact',
						contact_type => $type,
						field => $field,
						message => string('dm_field_mandatory',
							field => string($fields_loc_map{$field})
						),
					};
				}
			} elsif ($field eq 'phone' || $field eq 'fax') {
				unless (HSPC::PluginToolkit::DM::is_phone_valid(phone => $h{contacts}->{$type}->{$field})) {
					push @$error_list, {
						form => 'contact',
						contact_type => $type,
						field => $field,
						message => string('dm_field_phone_type_mand_err',
							phone_type_name => string($fields_loc_map{$field})
						),
					};
				}
			} elsif ($field eq 'email') {
				if (!HSPC::PluginToolkit::DM::is_email($h{contacts}->{$type}->{'email'})) {
					push @$error_list, {
						form => 'contact',
						contact_type => $type,
						field => 'email',
						message => string('invalid_eaddr'),
					};
				}
			}
		}

		if (defined $is_ascii) {
			foreach my $field (@ascii_fields) {
				if ($is_ascii->{$type}->{$field}==0) {
					push @$error_list, {
						form => 'contact',
						contact_type => $type,
						field => $field,
						message => string('dm_field_ascii_err',
							field => string($fields_loc_map{$field})
						)
					};
				} ## If is_ascii
			} ## foreach
		} ## if check_ascii

		## additional default checks on not mandatory phone or fax
		foreach my $field (qw(phone fax)) {
			next if (grep {$_ eq $field} @mand_fields);
			my $is_valid = HSPC::PluginToolkit::DM::is_phone_valid(
				phone		=> $h{contacts}->{$type}->{$field},
				optional 	=> 1
			);
			if (!$is_valid) {
				push @$error_list, {
					form => 'contact',
					contact_type => $type,
					field => $field,
					message => string('dm_field_phone_type_opt_err',
						phone_type_name => string($fields_loc_map{$field})
					)
				};
			}
		}

		## additional default check on not mandatory state
		my $state = $h{contacts}->{$type}->{state};
		my $country = $h{contacts}->{$type}->{country};
		unless (grep {$_ eq 'state'} @mand_fields){
			if(grep {$_ eq $country} @{$h{force_state_required}}){
				push @$error_list, {
					form => 'contact',
					contact_type => $type,
					field => 'state',
					message => string(
						'dm_state_cant_empty',
						country => geo_get_country_name($country)
					)
				} unless $state;
			}
		}
		if($state){
			$state = check_country_state(
				country => $country,
				state => $state
			);
			push @$error_list, {
				form => 'contact',
				contact_type => $type,
				field => 'state',
				message => string('dm_state_not_match_country')
			} unless $state;
		}
	}

	return $error_list;
}

1;
