package HSPC::PluginToolkit::General;

use strict;
use SWsoft::protect;

use HSPC::Localization qw(:DEFAULT string);
use HSPC::GeoData;
use HSPC::Logger qw(sw_log_debug sw_log_warn sw_log sw_die);
use Date::Calc qw(Days_in_Month);
use HSPC::Localization::Date qw(
	split_datetime gmt_now time_gmt_to_local cmp_dates str_to_time secs_to_time
);
use HSPC::Util::String qw(sw_encode_base64 sw_encode_base64_safe sw_decode_base64 sw_escape_string);
use HSPC::MT::PluginTools qw(get_request_data);
use HSPC::Math;
use HSPC::WebUpload;

use HSPC::MT::LibXMLSimple qw(XMLin);

use HSPC::PluginToolkit::DM::Translit qw( translit_data_utf2ascii );
use HSPC::Util qw(sw_purify_data);
use HSPC::Config;

use Params::Validate qw(:all);

use Error;
use HSPC::Exception::Plugin;
use HSPC::Exception::Plugin::GeneralError;
use HSPC::Exception::Plugin::SecureKey;
use HSPC::Exception::Plugin::InvalidParameter;

use base qw|Exporter|;
our @EXPORT_OK = qw(
	string argparam uriparam geo_get_countries geo_get_states
	geo_get_country_name geo_get_state_name last_month_day
	geo_get_states_us geo_get_states_ca lang_string
	split_date_string str_to_time datetime_gmt_now gmt_to_local compare_dates secs_to_time
	encode_base64 decode_base64 encode_base64_safe
	log log_debug log_warn throw_exception get_data_dir get_common_dir
	server_name round mround round2 purify_fromxml_data
	upload_file get_help_url escape_string money_to_str
	currency_precision period_round percent_to_str
	country_letters_to_code currency_prefix currency_postfix
	convert_quotes charset argparam_list
	translit_data_utf2ascii parse_xml
);

sub charset {
	return $LOCALE{local_charset};
}

sub currency_precision {
	return undef; ## Not needed for shared PBA
}

sub period_round {
	HSPC::Math::period_round(@_);
}

sub percent_to_str {
	HSPC::Math::percent_to_str(@_);
}

sub money_to_str {
	return sw_money_to_str( @_[0], get_request_data('plugin_vendor_id'),
		@_[1] );
}

sub purify_fromxml_data{
	return HSPC::Util::sw_purify_data(@_);
}

sub round {
	return HSPC::Math::round(@_);
}

sub mround {
	return sw_mround(@_);
}

sub currency_prefix {
	return undef; ## Not needed for shared PBA
}

sub currency_postfix {
	return undef; ## Not needed for shared PBA
}

sub round2 {
	return HSPC::Math::round2(@_);
}

sub string{
	return HSPC::Localization::string(@_);
}

sub lang_string{
	my $lang = shift;
	return HSPC::Localization::string_lang(lc($lang), @_);
}

sub argparam{
	return HSPC::MT::PluginTools->get_plugin_argparam(
		param_name => $_[0],
	);
}

sub argparam_list {
	return (); ## Not needed for shared PBA
}

sub escape_string{
	return sw_escape_string(@_);
}

sub uriparam{
	my $key = shift;
	my $uri = get_request_data("referer_url");
	return $1 if $uri =~ m#[/\?]$key[,;]([^/\?]+)(/|\?|$)#;
	return '';
}

sub geo_get_countries {
	return HSPC::GeoData->get_countries();
}

sub geo_get_states {
	return HSPC::GeoData->get_states();
}

sub geo_get_country_name {
	return HSPC::GeoData::get_country_name(@_);
}

sub geo_get_state_name {
	return HSPC::GeoData::get_state_name(@_);
}

sub geo_get_states_ca {
	return HSPC::GeoData::get_states_ca();
}

sub geo_get_states_us{
	return HSPC::GeoData::get_states_us();
}

sub split_date_string {
	return HSPC::Localization::Date::split_datetime(shift);
}

sub str_to_time {
	return HSPC::Localization::Date::str_to_time(shift);
}

sub last_month_day {
	my %date = split_date_string(shift);

	return undef unless $date{year} != 0 && $date{month} != 0;

	my $ld=Days_in_Month($date{year}, $date{month});

	return sprintf("%04d-%02d-%02d 23:59:59", $date{year}, $date{month}, $ld);
}

sub datetime_gmt_now {
	return gmt_now;
}

sub gmt_to_local {
	return time_gmt_to_local(shift);
}

sub encode_base64 {
	my $str = shift;
	return sw_encode_base64($str);
}

sub decode_base64 {
	my $str = shift;
	return sw_decode_base64($str);
}

sub encode_base64_safe{
	my $str = shift;
	return sw_encode_base64_safe($str);
}

sub throw_exception {
	my %h = (
		type           => undef,
		message        => undef,
		log_message    => undef,
		vendor_message => undef,
		user_message   => undef,
		@_
	);

	my %map_type = (
		'error'     => '::GeneralError',
		'parameter' => '::InvalidParameter',
		'key'       => '::SecureKey',
	);

	my $exception = 'HSPC::Exception::Plugin' . $map_type{lc($h{type})};

	## Call HSPC::Logger::no_trace twice in order to
	## get rid of "throw" in stack as well.
	my $t1 = HSPC::Logger->no_trace;
	my $t2 = HSPC::Logger->no_trace;

	throw $exception(
		message        => $h{message},
		log_message    => $h{log_message},
		vendor_message => $h{vendor_message},
		user_message   => $h{user_message},
	);
}

sub log {
	my $nt = HSPC::Logger->no_trace();
	sw_log(@_);
}

sub log_debug {
	my $nt = HSPC::Logger->no_trace();
	sw_log_debug(@_);
}

sub log_warn {
	my $nt = HSPC::Logger->no_trace();
	sw_log_warn(@_);
}

sub compare_dates {
	return cmp_dates(@_);
}

sub secs_to_time {
	return HSPC::Localization::Date::secs_to_time(@_);
}

sub server_name {
	return get_request_data("server_name");
}

sub get_data_dir {
	my $plugin_id = get_request_data('plugin_config_id');

	return undef unless $plugin_id;

	my $path = $CONFIG{HSPCROOT_ROOT}.$CONFIG{HSPCROOT_DIR_PLUGIN}.'/';
	$path .= $plugin_id;

	unless ( -d $path ) {
		system("mkdir -p $path") == 0 || throw_exception(
			type        => 'error',
			log_message => "Can not create directory $path"
		);
	}
	return $path;
}

sub get_common_dir {
	my $plugin_template_id = get_request_data('plugin_template_id');

	return undef unless $plugin_template_id;

	my $path = $CONFIG{HSPCROOT_ROOT}.$CONFIG{HSPCROOT_DIR_PLUGIN}.'/';
	$path .= $plugin_template_id;

	unless ( -d $path ) {
		system("mkdir -p $path") == 0 || throw_exception(
			type        => 'error',
			log_message => "Can not create directory $path"
		);
	}
	return $path;
}

sub upload_file {
	my %h = (
		view_name => undef,
		base_path => undef,
		overwrite => undef,
		new_rel_path => undef,
		old_rel_path => undef,
		old_file_name => undef,
		old_file_name => undef,
		only_images => undef,
		file_name => undef,
		@_
	);

	## add new parameters if required
	my $res = HSPC::WebUpload::upload_file( %h );
	return $res;
}

sub current_lang {
	return HSPC::Localization::current_lang();
}

sub get_help_url {
	my %h = (
		action        => undef,
		customer_view => undef,
		language      => uc(current_lang()),
		template_id   => get_request_data('plugin_template_id'),
		plugin_type   => get_request_data('plugin_type'),
		vendor_id     => undef,
		@_
	);
	return "";
}

sub country_letters_to_code{
	my $code = shift;
	return &COUNTRY_LETTERSCODE2NUMCODE->{$code};
}

sub convert_quotes{
    my $line = shift;
    $line=~s/"/\&quot;/g;##"
    $line=~s/'/\&#39;/g;##'
    $line=~s|\<txt_(.+?)\>|~~txt_$1~~|g;
    $line=~s|\<|\&lt\;|g;
    $line=~s|\>|\&gt\;|g;
    $line=~s|~~txt_(.+?)~~|<txt_$1>|g;
    return $line;
}

sub parse_xml
{
	return XMLin(@_);
}

use constant COUNTRY_LETTERSCODE2NUMCODE => {
	'AF' => '004',
	'AL' => '008',
	'DZ' => '012',
	'AS' => '016',
	'AD' => '020',
	'AO' => '024',
	'AI' => '660',
	'AQ' => '010',
	'AG' => '028',
	'AR' => '032',
	'AM' => '051',
	'AW' => '533',
	'AU' => '036',
	'AT' => '040',
	'AZ' => '031',
	'BS' => '044',
	'BH' => '048',
	'BD' => '050',
	'BB' => '052',
	'BY' => '112',
	'BE' => '056',
	'BZ' => '084',
	'BJ' => '204',
	'BM' => '060',
	'BT' => '064',
	'BO' => '068',
	'BA' => '070',
	'BW' => '072',
	'BV' => '074',
	'BR' => '076',
	'IO' => '086',
	'BN' => '096',
	'BG' => '100',
	'BF' => '854',
	'BI' => '108',
	'KH' => '116',
	'CM' => '120',
	'CA' => '124',
	'CV' => '132',
	'KY' => '136',
	'CF' => '140',
	'TD' => '148',
	'CL' => '152',
	'CN' => '156',
	'CX' => '162',
	'CC' => '166',
	'CO' => '170',
	'KM' => '174',
	'CD' => '180',
	'CG' => '178',
	'CK' => '184',
	'CR' => '188',
	'CI' => '384',
	'HR' => '191',
	'CU' => '192',
	'CY' => '196',
	'CZ' => '203',
	'DK' => '208',
	'DJ' => '262',
	'DM' => '212',
	'DO' => '214',
	'TL' => '626',
	'EC' => '218',
	'EG' => '818',
	'SV' => '222',
	'GQ' => '226',
	'ER' => '232',
	'EE' => '233',
	'ET' => '231',
	'FK' => '238',
	'FO' => '234',
	'FJ' => '242',
	'FI' => '246',
	'FR' => '250',
	'FX' => '249',
	'GF' => '254',
	'PF' => '258',
	'TF' => '260',
	'GA' => '266',
	'GM' => '270',
	'GE' => '268',
	'DE' => '276',
	'GH' => '288',
	'GI' => '292',
	'GR' => '300',
	'GL' => '304',
	'GD' => '308',
	'GP' => '312',
	'GU' => '316',
	'GT' => '320',
	'GN' => '324',
	'GW' => '624',
	'GY' => '328',
	'HT' => '332',
	'HM' => '334',
	'HN' => '340',
	'HK' => '344',
	'HU' => '348',
	'IS' => '352',
	'IN' => '356',
	'ID' => '360',
	'IR' => '364',
	'IQ' => '368',
	'IE' => '372',
	'IL' => '376',
	'IT' => '380',
	'JM' => '388',
	'JP' => '392',
	'JO' => '400',
	'KZ' => '398',
	'KE' => '404',
	'KI' => '296',
	'KP' => '408',
	'KR' => '410',
	'KW' => '414',
	'KG' => '417',
	'LA' => '418',
	'LV' => '428',
	'LB' => '422',
	'LS' => '426',
	'LR' => '430',
	'LY' => '434',
	'LI' => '438',
	'LT' => '440',
	'LU' => '442',
	'MO' => '446',
	'MK' => '807',
	'MG' => '450',
	'MW' => '454',
	'MY' => '458',
	'MV' => '462',
	'ML' => '466',
	'MT' => '470',
	'MH' => '584',
	'MQ' => '474',
	'MR' => '478',
	'MU' => '480',
	'YT' => '175',
	'MX' => '484',
	'FM' => '583',
	'MD' => '498',
	'MC' => '492',
	'MN' => '496',
	'MS' => '500',
	'MA' => '504',
	'MZ' => '508',
	'MM' => '104',
	'NA' => '516',
	'NR' => '520',
	'NP' => '524',
	'NL' => '528',
	'AN' => '530',
	'NC' => '540',
	'NZ' => '554',
	'NI' => '558',
	'NE' => '562',
	'NG' => '566',
	'NU' => '570',
	'NF' => '574',
	'MP' => '580',
	'NO' => '578',
	'OM' => '512',
	'PK' => '586',
	'PW' => '585',
	'PS' => '275',
	'PA' => '591',
	'PG' => '598',
	'PY' => '600',
	'PE' => '604',
	'PH' => '608',
	'PN' => '612',
	'PL' => '616',
	'PT' => '620',
	'PR' => '630',
	'QA' => '634',
	'RE' => '638',
	'RO' => '642',
	'RU' => '643',
	'RW' => '646',
	'KN' => '659',
	'LC' => '662',
	'VC' => '670',
	'WS' => '882',
	'SM' => '674',
	'ST' => '678',
	'SA' => '682',
	'SN' => '686',
	'SC' => '690',
	'SL' => '694',
	'SG' => '702',
	'SK' => '703',
	'SI' => '705',
	'SB' => '090',
	'SO' => '706',
	'ZA' => '710',
	'GS' => '239',
	'ES' => '724',
	'LK' => '144',
	'SH' => '654',
	'PM' => '666',
	'SD' => '736',
	'SR' => '740',
	'SJ' => '744',
	'SZ' => '748',
	'SE' => '752',
	'CH' => '756',
	'SY' => '760',
	'TW' => '158',
	'TJ' => '762',
	'TZ' => '834',
	'TH' => '764',
	'TG' => '768',
	'TK' => '772',
	'TO' => '776',
	'TT' => '780',
	'TN' => '788',
	'TR' => '792',
	'TM' => '795',
	'TC' => '796',
	'TV' => '798',
	'UG' => '800',
	'UA' => '804',
	'AE' => '784',
	'GB' => '826',
	'US' => '840',
	'UM' => '581',
	'UY' => '858',
	'UZ' => '860',
	'VU' => '548',
	'VA' => '336',
	'VE' => '862',
	'VN' => '704',
	'VG' => '092',
	'VI' => '850',
	'WF' => '876',
	'EH' => '732',
	'YE' => '887',
	'YU' => '891',
	'ZM' => '894',
	'ZW' => '716',
};

1;
