package HSPC::PluginToolkit::Translit;

use strict;
use SWsoft::protect;

use HSPC::Util::Translit;
use HSPC::Util::Translit::TranslitUTF;

use Encode qw(encode decode _utf8_on FB_DEFAULT FB_CROAK);

use Exporter;
use vars qw(@ISA @EXPORT);

use Params::Validate qw(:all);


## Input parameters
## > source string
## > translitiration table
## > return original value if error while translitiration (i.e. while encode)
sub translit {
	my $source = shift;
	my $ttable_name = shift;
	my $ret_orig_val = shift || 0;

	if(ref($source) eq 'HASH'){
		my $translit_result = _translit_data($source,$ttable_name,$ret_orig_val);
		%$source = (%$source,%$translit_result);
	} else {
		my $translit_result = _translit_value($source,$ttable_name,$ret_orig_val);
		$source = $translit_result;
	}
	return $source;
}

sub _translit_value {
	my $source = shift;
	my $ttable_name = shift;
	my $ret_orig_val = shift || 0;

	my $encode_check = Encode::FB_DEFAULT;
	$encode_check = Encode::FB_CROAK if ($ret_orig_val);

	my $esource;
	if(($ttable_name eq &TTN_SPAIN_BASE) ||
		($ttable_name eq &TTN_GERMAN_BASE) ||
		($ttable_name eq &TTN_ISO_8859_1)){
		_utf8_on($source);	
		eval {
			$esource = encode('iso-8859-1',$source, $encode_check);
		}
	}
	elsif(($ttable_name eq &TTN_GOST_16876_71) ||
		($ttable_name eq &TTN_GOST_7_79_2000) ||
		($ttable_name eq &TTN_ISO_R9_1968) ||
		($ttable_name eq &TTN_ISO_R9_1968_WOA) ||
		($ttable_name eq &TTN_CUSTOM_CYRLAT)){
		_utf8_on($source);
		eval {
			$esource = encode('koi8-r',$source, $encode_check);
		}
	}

	if (!$@) {
		$source = HSPC::Util::Translit::translit($esource,$ttable_name);
	}
	return $source;
}

sub _translit_data {
	my $source = shift;
	my $ttable_name = shift;
	my $ret_orig_val = shift || 0;

	my $result = {};

	sub __translit_data {
		my $source = shift;
		my $ttable_name = shift;
		my $result = shift;

		foreach my $key (keys(%$source)){
			my $value = $source->{$key};
			if(ref($value) eq 'HASH'){
				my $h = {};
				__translit_data($value,$ttable_name,$h);
				if(keys(%$h)){
					$result->{$key} = $h;
				}
			} else {
				$result->{$key} = _translit_value($value,$ttable_name,$ret_orig_val);
			}
		}
	}

	__translit_data($source,$ttable_name,$result);

	return $result;
}

## Description: transliterates source data from UTF-8 into ASCII.
## Currently it transliterates Russian letters and european letters, which present in ISO-8859-1
## Optionally this sub may delete some specified or not transliterated characters
## Parameters:
##	*	data (HASHREF or SCALAR) - source data in UTF-8 encoding.
##			If it is HASHREF, then transliterates values of hash recursively
##	*	delete_nonascii_characters (0|1) - delete non-ASCII characters after transliteration
##			(we do not have transliterations for all UTF-8 characters,
##			 and this option allows simply to delete non-ASCII-letters, which were not transliterated)
##	*	characters_to_delete (ARRAYREF) - arrayref of characters, which will be deleted during transliteration
## Returns:
##		Transliterated data (HASH or SCALAR, as it was passed into input data)

sub translit_utf {
	my %h = validate(@_,{
		data => {type => HASHREF|SCALAR|UNDEF},
		delete_nonascii_characters => {type => SCALAR, optional => 1},
		characters_to_delete => {type => ARRAYREF, optional => 1},
	});
	
	return undef unless defined($h{data});
	
	my $delete_nonascii_characters = $h{delete_nonascii_characters};
	my $characters_to_delete = $h{characters_to_delete};

	sub __translit_data_utf2ascii {
		my %h = (
			data => undef,
			@_
		);
		my $result = {};
		foreach my $key (keys(%{$h{data}})){
			my $value = $h{data}->{$key};
			if(ref($value) eq 'HASH'){
				my $h = __translit_data_utf2ascii(data => $value);
				if(keys(%$h)){
					$result->{$key} = $h;
				}
			} else {
				$result->{$key} = HSPC::Util::Translit::TranslitUTF::translit_utf(
					source_string => $value,
					delete_nonascii_characters => $delete_nonascii_characters,
					characters_to_delete => $characters_to_delete
				);
			}
		}
		return $result;
	}
	
	if (ref($h{data}) eq 'HASH') {
		return __translit_data_utf2ascii(data => $h{data});
	} else {
		return HSPC::Util::Translit::TranslitUTF::translit_utf(
			source_string => $h{data}, 
			delete_nonascii_characters => $delete_nonascii_characters,
			characters_to_delete => $characters_to_delete,
		);
	}
}

@ISA    = qw(Exporter);
@EXPORT = qw(
	TTN_LIST

	TTN_GOST_16876_71
	TTN_GOST_7_79_2000
	TTN_ISO_R9_1968
	TTN_ISO_R9_1968_WOA
	TTN_CUSTOM_CYRLAT
	TTN_SPAIN_BASE
	TTN_GERMAN_BASE
	TTN_ISO_8859_1

	translit
	translit_utf
);

1;
