package HSPC::ProblemReporter;

use strict;
use warnings;
use locale;

use SWsoft::protect;

use base qw(Log::Log4perl::Appender::File);
use POSIX qw(strftime);

use IO::File ();
use File::Basename qw(dirname basename);

use constant RELEASE_FILE => "/etc/hspc-release";
use constant ERROR_PREP => "p";
use constant FATAL_PREP => "rep";

use constant LOG_CATEGORY_DELIMITER => ';=;;';

## This is the heart and soul of HSPc problem report generation engine.
## It is implemented as conventional appender for Log::Log4perl
## on top of Log::Log4perl::Appender::File, so it can be easily
## turned on, off or otherwise configured in HSPC::Logger's configuration
## file. 

sub new {
	my ($class, %opts) = @_;

	my $dir = $opts{repdir} || dirname($opts{filename}) || "/var/log/hspc";

	$opts{filename} = &_get_internal_name($dir);
	$opts{repdir} = $dir;

	my $self = $class->SUPER::new(%opts);
	return $self;
}

## The cleanup procedure should be processes before handling new request
sub cleanup {
	my $self = shift;
	$self->file_close();
	my $fh = new IO::File($self->filename, 'w') or return;
	undef $fh; ## Just truncate a file
	$self->file_open();
}

sub file_switch {
	my $self = shift;
	$self->SUPER::file_switch(
		_get_internal_name(dirname $self->filename) );
}

## Generate problem report
## => error_message - the error message that is to be written in 
## 		report's header section.
## => report_name - name of the file, where report must be saved
sub _generate_problemreport {
	my $self = shift;
	my $error_message = shift;
	my $report_name = shift;
	local $Log::Log4perl::caller_depth += 1;

	my $task_id = Log::Log4perl::MDC->get("current_task") || "default";
	Log::Log4perl::MDC->put("problem_$task_id", basename($report_name));
	my $in  = new IO::File($self->filename, 'r') or return;

	my ($current_section, %section);
	while(my $line = $in->getline) {
		my $delim = LOG_CATEGORY_DELIMITER;
		if($line =~ /^(.*?)$delim(.*)/o) {
			push @{$section{$1}}, $2;
			$current_section = $1;
		} elsif($current_section) {
			push @{$section{$current_section}}, $line;
		}
	}

	my $out = new IO::File($report_name, 'w', 0640) or return;

	print $out $self->_get_report_header($error_message, $report_name);
	print $out $self->_get_headline("ENVIRONMENT");
	print $out $self->_get_env;
	print $out $self->_get_headline("ENVIRONMENT", 1);
	print $out $self->_get_headline("CALL STACK");
	print $out $self->_get_stack_trace();
	print $out $self->_get_headline("CALL STACK", 1);
	for(sort keys %section) {
		print $out $self->_get_headline(uc $_);
		print $out join "\n", @{$section{$_}};
		print $out "\n";
		print $out $self->_get_headline(uc $_, 1);
	}

	if($self->{extended} and $self->{extended} ne 'no') {
		print $out $self->_get_headline("FULL STACK TRACE");
		print $out $self->_get_stack_trace('full');
		print $out $self->_get_headline("FULL STACK TRACE", 1);
	}
}

## Do normal logging and then generate a problem report
## is message was submitted at ERROR or FATAL loglevel.
sub log {
	my ($self, %p) = @_;

	unless(%p) {
		warn "NO sane parameters passed to HSPC::ProblemReporter::log!\n";
		return;
	}
	return unless %p; ## Don't you love it when there is at least $p{message} to log?

	$self->file_open() unless $self->{fh};
	local $SIG{__DIE__}; ## If we die here, we better die for real

	$p{message} = $p{log4p_category} . LOG_CATEGORY_DELIMITER . $p{message};
	if($p{log4p_level} eq 'ERROR' or $p{log4p_level} eq 'FATAL') {
		## generate problem report if error occurred
		my $nname = $self->_get_report_name($p{log4p_level});
		## Do normal job
		Log::Log4perl::MDC->put("last_report", '(See ' . $nname . ' for details)');
		Log::Log4perl::MDC->put('last_report_id', basename($nname));
		$self->SUPER::log(%p);
	
		$self->_generate_problemreport($p{message}, $nname);
		return;
	}
	Log::Log4perl::MDC->put('last_report', "");
	$self->SUPER::log(%p);
}

sub get_log_piece {
	my $self = shift;
	my $category = shift;

	my $res = "";
	open(REPORT, '<', $self->filename) or return $res;
	my $current_section;
	while(<REPORT>) {
		my $delim = LOG_CATEGORY_DELIMITER;
		if(/^(.*?)$delim(.*)/o) {
			$res .= $2 . "\n"
				if $1 eq $category;
			$current_section = $1;
		} elsif($current_section and $current_section eq $category) {
			$res .= $_;
		}
	}
	close REPORT;

	$res;
}

## Get full stack trace
## This method returns complete stack trace with all arguments
sub _get_stack_trace {
	my ($self, $full) = @_;

	my $res = "";
	my $i = $Log::Log4perl::caller_depth + 4;
	package DB;
	require HSPC::XRays;
	my @fields =
		qw(pack file line method hasargs wantar evaltext is_require);
	while(my @caller = caller $i++) {
		my %context; @context{@fields} = @caller;

		if($context{method} eq "(eval)") {
			$res .= sprintf 'eval %s at %s:%d',
				$context{evaltext}
					? '"' . ($full
						? $context{evaltext}
						: '...') . '"'
					: '{...}',
				 $context{file}, $context{line};
		} else {
			$res .= sprintf '%s(%s)', $context{method},
				$context{hasargs}
					? $full
						? HSPC::XRays::dump_args(
							\@DB::args)
						: '...'
					: '';
		}
		$res .= "\n";
	}
	$res;
}

## Get separator line
## => name -- separator name
## => closed -- open or close the section. Optional. False by default.
sub _get_headline {
	my $self = shift;
	my $name = shift;
	my $closed = shift;

	$name = "/". (uc $name)
		if $closed;

	return "--------------------- $name ---------------------"
		. ($closed ? "\n\n" : "\n");
}

sub _get_env {
	my $self = shift;

	my $env_txt = '';
	for (grep {$_ and $ENV{$_} and !ref($ENV{$_})} sort keys %ENV) {
		$env_txt .= sprintf '%-25s = %s%s', $_, $ENV{$_}, "\n";
	}

	return $env_txt;
}

## Generate header for problemreport
## For internal use only
sub _get_report_header {
	my $self = shift;
	my $err_txt = shift;
	my $report_id = shift;
	my $header = "";

	$err_txt =~ s/^.*?://;

	my $release;
	if(open RL, '<', RELEASE_FILE) {
		local $/;
		$release = <RL>;
		chomp $release;
	} else {
		$release = 'Unknown build';
	}

	my ($pack, $file, $line) = caller $Log::Log4perl::caller_depth + 4;
	$header .= $self->_get_headline("HEADER");
	$header .= "Build-ID: $release";
	$header .= "Report-ID: ". basename($report_id) . "\n"
		if $report_id;
	$header .= "Generated: ". strftime('%Y-%m-%d %H:%M:%S', localtime). "\n";
	$header .= "Error-TEXT: $err_txt";
	$header .= "Error-LOCATION: \t$file:".($line? $line : "??"). "\n"
		if $file;
	$header .= $self->_get_headline("HEADER", 1);
	
	return $header;
}

## For internal use only -- generate new 
## report** name
sub _get_internal_name {
	my $dir = shift;
	my $nname = ($dir ? $dir : "") . "/.report" . $$ . ".log";

	return $nname;
}

## For internal use only
## => level -- loglevel as string
sub _get_report_name {
	my $self = shift;
	my $lev = shift;

	my $prep = $lev eq 'ERROR' ? ERROR_PREP : FATAL_PREP;

	my $name = my $basename = dirname($self->filename) .
		"/". $prep . strftime('%Y-%m-%d_%H-%M-%S', localtime);
	for (my $i = 1; -e $name; ++$i) {
		$name = "$basename.$i";
	}

	return $name;
}

## Delete unnecessary files
sub DESTROY {
	my $self = shift;
	unlink $self->filename
		if -e $self->filename;
}


1;
