package HSPC::Util;
use strict;

use vars qw(@ISA @EXPORT_OK);

use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(
	&sw_load_file
	&sw_build_version
	&sw_purify_data
	&sw_find_custom_file
	&sw_compare_versions
	&sw_in_array
	&daemonize
);

use SWsoft::protect;	# Mark this module for encryption upon build

use HSPC::Logger qw(sw_die);
use Encode;


# > file_name
sub sw_load_file {
	my $fn = shift;
	$fn =~ s/[\0]//g; ## remove char with ASCII code equal 0
	$fn =~ s/[\s\t\&\;\`\'\\\"\|\*\?\~\<\>\^\(\)\[\]\{\}\$\n\r]//g; ## remove special chars
	$fn =~ s/\.\.//g; ## remove .. from path

	local *F;
	unless ( -e $fn && -s $fn && -r $fn ) {
		return undef;
	}
	unless (sysopen F, $fn, "O_RDONLY") { return undef; }
	local $/;
	<F>;
}


## => source: 'disk'/'db', 'db' by default
sub sw_build_version {
	my %h = @_;

	if($h{source} eq 'disk'){
		my $version_raw = sw_load_file('/etc/hspc-release');
		my @x = split(/\n/, $version_raw); ##second string
		my $v =$x[0];
		return $v;
	}else{
		my $data = HSPC::WebDB::select_hash('SELECT * FROM hspc_version ORDER BY id desc LIMIT 1');
		return wantarray
			? @$data{qw(version built installed)}
			: $data->{version};
	}
}

### This fuction clears data structure from 'bad' symbols
## => HASHREF, output to be 'purified'
sub sw_purify_data
{
	my $h = shift;

	unless (ref($h)){
		## Moment of purity!
		$h = substr($h, 0);
		Encode::_utf8_off($h);
	}

	if (ref($h) eq "ARRAY"){
		foreach (@{$h}){
			$_ = sw_purify_data($_);
		}
	}

	if (ref($h) eq "HASH"){
		foreach (keys %{$h}){
			my $val = sw_purify_data($h->{$_});
			delete $h->{$_}; ## removes broken key
			$h->{sw_purify_data($_)} = $val;
		}
	}
	return $h;
}

## => base_path, by default '/var/opt/hspc-root/custom/'
## => subdir - subdirectory of the base_path
## => file - file to look for
## => provider_id [optional], by default $ENV{system_obj}->provider_id
## => langdir_uc default 1
## < path to a customized file or path to original file if no customization
sub sw_find_custom_file {
	my (%h) = (langdir_uc=>1,@_);

	## base directory
	require HSPC::Config;
	my $base_path = $h{base_path} ||
		$HSPC::Config::CONFIG{HSPCROOT_ROOT}.'/custom/';
	$base_path .= '/' if substr($base_path,-1) ne '/';

	## append subdirectory if needed
	$base_path .= $h{subdir} if $h{subdir};
	$base_path .= '/' if substr($base_path,-1) ne '/';

	## file relative path
	my $file = $h{file};

	## provider custom dirs
	my @prov_dirs = ('default/','');
	my $provider_id = $h{provider_id};
	$provider_id ||= $ENV{system_obj}->provider_id() if $ENV{system_obj};
	unshift @prov_dirs, "$provider_id/" if $provider_id;

	## language custom dirs
	require HSPC::Localization;
	my $def_lang_id = 'en';
	my @lang_dirs = ("$def_lang_id/",'');
	my $lang_id = HSPC::Localization::current_lang();
	unshift @lang_dirs, "$lang_id/" if $lang_id ne $def_lang_id;

	## go through provider custom dirs
	foreach my $prov_dir (@prov_dirs) {

		## go through language custom dirs
		foreach my $lang_dir (@lang_dirs) {
			$lang_dir = uc($lang_dir) if ( $h{langdir_uc} );
			## check if file exists
			if(-r $base_path.$prov_dir.$lang_dir.$file) {
				return $h{is_relative_path} ? $prov_dir.$lang_dir.$file :
					$base_path.$prov_dir.$lang_dir.$file;
			}
		}
	}
	return undef;
}


## function to compare two version strings
## returns:
## -1   if a < b
##  1   if a > b
##  0   if a == b
sub sw_compare_versions {
	my ($ver_a,$ver_b) = @_;

	## split versions into major parts (by dashes)
	my @maj_a = split(/\-/,$ver_a);
	my @maj_b = split(/\-/,$ver_b);

	for(my $i=0; $i <= $#maj_a || $i <= $#maj_b; $i++) {
		##
		if($#maj_a < $i) {
			## a is smaller
			return -1;
		}
		elsif($#maj_b < $i) {
			## b is smaller
			return 1;
		}
		else {
			## split major parts into minor parts (by non-numeric symbol)
			my @min_a = split(/\D+/,$maj_a[$i]);
			my @min_b = split(/\D+/,$maj_b[$i]);

			for(my $j=0; $j <= $#min_a || $j <= $#min_b; $j++) {
				if($#min_a < $j) {
					## a is smaller
					return -1;
				}
				elsif($#min_b < $j) {
					## b is smaller
					return 1;
				}
				else {
					## compare
					if($min_a[$j] < $min_b[$j]) {
						return -1;
					} elsif($min_a[$j] > $min_b[$j]) {
						return 1;
					}
				}
			}
		}
	}
	return 0; ## equal
}

## Return 1 if first argument presents in second argument (array's ref), 0 otherwise
sub sw_in_array {
	my ($val, $arr) = @_;
	die("Not an array in second arg") unless(ref($arr) eq 'ARRAY');
	return scalar(grep {$val eq $_} @$arr);
}

## function, which forks apache to execute long task.
## Note: Do not forget to exit child process!
sub daemonize {
	defined (my $pid = fork) or &sw_die("Cannot fork: $!");
	unless ($pid) {
		## child
		## start new session
		use POSIX 'setsid';
		setsid or &sw_die("Can't start a new session: $!");

		## goto / directory
		chdir '/' or &sw_die("Can't chdir to /: $!");

		## close all handles, TODO: make it nice :)
		$ENV{system_obj}->force_close_all_db_connections()
			if $ENV{system_obj} && ref($ENV{system_obj});

		## reopen STD streams
		open STDIN,  '/dev/null';
		open STDOUT, '>/dev/null';
		open STDERR, '>/dev/null';
	}

	$pid;
}

1;

__END__
