package HSPC::Util::String;

use vars qw(@ISA @EXPORT_OK);
use strict;
use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(
sw_trim sw_escape_string sw_escape_for_code sw_encode_base64
sw_encode_base64_safe sw_decode_base64 is_valid_utf8_string
);

# trims spaces in the begining and the end of the given string
#
sub sw_trim{
		my @out = @_;
		for (@out) {
				$_ =~ s/^\s+//o;
				$_ =~ s/\s+$//o;
		}
		return wantarray ? @out : $out[0];
}


sub sw_escape_string {
	my $str = shift;
	$str =~ s/&/&amp;/g;
	$str =~ s/</&lt;/g;
	$str =~ s/>/&gt;/g;
	$str =~ s/\"/&quot;/g;
	return $str;
}


## escape string for fode
## =>> string
## =>  quote - " or ' default "
## < escaped string
sub sw_escape_for_code {
	my %h = (
		string => undef,
		quote  => '"',
		@_
	);
	my $string = $h{string};
	my $quote  = $h{quote};
	## \ -> \\
	$string =~ s/\\/\\\\/g;
	if ( $quote eq "'" ) {
		## ' -> \'
		$string =~ s/'/\\'/g;
		return $string;
	}

	## " -> \"
	$string =~ s/"/\\"/g;
	## $ -> \$
	$string =~ s/\$/\\\$/g;
	## @ -> \@
	$string =~ s/\@/\\\@/g;
	## % -> \%
	$string =~ s/\%/\\\%/g;

	return $string;
}##/sw_escape_for_code


#################################################################################
## Usage: sw_encode_base64 ($str[,$eol])
## Description: Base64 encoding routine
## Parameters:
##		$str - string to encode
##		$eol - end_of_line symbol. Default is empty (i.e. do not use it)
## Return value:
##	encoded string, braked by $eol symbol (for MIME::Base64 compatibility)
sub sw_encode_base64 {
	my ($str, $eol) = @_;
	use integer;
	return '' unless length($str);
	$eol = '' unless defined($eol);
	my $res = pack("u", $str);
	# Remove first character of each line, remove newlines
	$res =~ s|^.||mg;
	$res =~ s|\n||g;
	$res =~ tr|` -_|AA-Za-z0-9+/|;##`
	# fix padding at the end
	my $padding = (3 - length($str) % 3) % 3;
	$res =~ s/.{$padding}$/'=' x $padding/e
		if $padding;
	if (length $eol) {
		$res =~ s/(.{1,76})/$1$eol/g;
	}
	return $res;
}

#################################################################################
## Usage: sw_encode_base64_safe ($str)
## Description: safe Base64 encoding routine.
##		Replaces '/' and '+' symbols with '_' and '-' accordingly.
##		(RFC 3548. section "File and URL safe alphabet")
## Parameters:
##		$str - string to encode
## Return value:
##		encoded string

sub sw_encode_base64_safe {
	my $str = shift;
	return '' unless length($str);
	my $res = sw_encode_base64($str);
	## performing SAFENESS
	$res =~ tr/\//\_/;
	$res =~ tr/\+/\-/;
	return $res;
}

#################################################################################
## Usage: sw_decode_base64($str)
## Description: Base64 decoding routine.
## Parameters:
##		$str - string to decode (encoded by Base64 or safe Base64)
## Return value:
##		decoded string

sub sw_decode_base64 {
	my $str = shift;
	use integer;
	return '' unless length($str);
	## removing SAFENESS if it is
	if ($str =~ /[_\-]/){
		$str =~ tr/\_/\//;
		$str =~ tr/\-/\+/;
	}
	$str =~ tr|A-Za-z0-9+=/||cd;            # remove non-base64 chars
	if (length($str) % 4) {
		die("Length of base64 data not a multiple of 4: $str");
	}
	$str =~ s|=+$||;                        # remove padding
	$str =~ tr|A-Za-z0-9+/| -_|;            # convert to uuencoded format
	return '' unless length $str;
	my ($i, $l, $uustr);
	$l = length($str) - 60;
	for ($i = 0; $i <= $l; $i += 60) {
		$uustr .= "M".substr($str, $i, 60);
	}
	$str = substr($str, $i);
	# any leftover chars
	if ($str ne '') {
		$uustr .= chr(32 + length($str)*3/4).$str;
	}
	return unpack ("u", $uustr);
}


sub is_valid_utf8_string {
	my $string = shift;
	my $valid_utf8_regexp = <<'.' ;
  [\x{00}-\x{7f}]
		| [\x{c2}-\x{df}][\x{80}-\x{bf}]
		|         \x{e0} [\x{a0}-\x{bf}][\x{80}-\x{bf}]
		| [\x{e1}-\x{ec}][\x{80}-\x{bf}][\x{80}-\x{bf}]
		|         \x{ed} [\x{80}-\x{9f}][\x{80}-\x{bf}]
		| [\x{ee}-\x{ef}][\x{80}-\x{bf}][\x{80}-\x{bf}]
		|         \x{f0} [\x{90}-\x{bf}][\x{80}-\x{bf}]
		| [\x{f1}-\x{f3}][\x{80}-\x{bf}][\x{80}-\x{bf}][\x{80}-\x{bf}]
		|         \x{f4} [\x{80}-\x{8f}][\x{80}-\x{bf}][\x{80}-\x{bf}]
.
	$string =~ s/$valid_utf8_regexp//ogx;
	return 0 if length($string);
	return 1;
}##/is_valid_utf8_string


1;
