package HSPC::Util::Translit::TranslitUTF;

## Description: this package contains tables for 
## transliteration from UTF-8 into ASCII.

use strict;
use SWsoft::protect;
use Encode qw(_utf8_off);

use Exporter;
use vars qw(@ISA @EXPORT);
@ISA    = qw(Exporter);
@EXPORT = qw(
	translit_utf
);

use constant RULES => [
	## Russian symbols
	['\xd0\xb0','a'],
	['\xd0\xb1','b'],
	['\xd0\xb2','v'],
	['\xd0\xb3','g'],
	['\xd0\xb4','d'],
	['\xd0\xb5','e'],
	['\xd1\x91','e'],
	['\xd0\xb6','zh'],
	['\xd0\xb7','z'],
	['\xd0\xb8','i'],
	['\xd0\xb9','j'],
	['\xd0\xba','k'],
	['\xd0\xbb','l'],
	['\xd0\xbc','m'],
	['\xd0\xbd','n'],
	['\xd0\xbe','o'],
	['\xd0\xbf','p'],
	['\xd1\x80','r'],
	['\xd1\x81','s'],
	['\xd1\x82','t'],
	['\xd1\x83','u'],
	['\xd1\x84','f'],
	['\xd1\x85','kh'],
	['\xd1\x86','ts'],
	['\xd1\x87','ch'],
	['\xd1\x88','sh'],
	['\xd1\x89','shch'],
	['\xd1\x8a',''],
	['\xd1\x8b','y'],
	['\xd1\x8c',''],
	['\xd1\x8d','eh'],
	['\xd1\x8e','yu'],
	['\xd1\x8f','ya'],
	['\xd0\x90','A'],
	['\xd0\x91','B'],
	['\xd0\x92','V'],
	['\xd0\x93','G'],
	['\xd0\x94','D'],
	['\xd0\x95','E'],
	['\xd0\x81','E'],
	['\xd0\x96','ZH'],
	['\xd0\x97','Z'],
	['\xd0\x98','I'],
	['\xd0\x99','J'],
	['\xd0\x9a','K'],
	['\xd0\x9b','L'],
	['\xd0\x9c','M'],
	['\xd0\x9d','N'],
	['\xd0\x9e','O'],
	['\xd0\x9f','P'],
	['\xd0\xa0','R'],
	['\xd0\xa1','S'],
	['\xd0\xa2','T'],
	['\xd0\xa3','U'],
	['\xd0\xa4','F'],
	['\xd0\xa5','KH'],
	['\xd0\xa6','TS'],
	['\xd0\xa7','CH'],
	['\xd0\xa8','SH'],
	['\xd0\xa9','SHCH'],
	['\xd0\xaa',''],
	['\xd0\xab','Y'],
	['\xd0\xac',''],
	['\xd0\xad','EH'],
	['\xd0\xae','YU'],
	['\xd0\xaf','YA'],
	
	['\xc2\xaa','a'],
	['\xc2\xb7','.'],
	['\xc2\xba','o'],
	
	## ISO-8859-1 letters (transliteration is taken from 'transtab' by Markus Kuhn)
	## They cover special Spanish and German letters and letters of many other european languages
	## NOTE: In 3.2.2 German symbols with umlauts were transliterated in other way:
	## ['\xc3\x84','Ae'],['\xc3\x96','Oe'],['\xc3\x9c','Ue'],['\xc3\xa4','ae'],['\xc3\xb6','oe'],['\xc3\xbc','ue'],
	## Here they are transliterated in common way, i.e. without e
	['\xc3\x80','A'],
	['\xc3\x81','A'],
	['\xc3\x82','A'],
	['\xc3\x83','A'],
	['\xc3\x84','A'],
	['\xc3\x85','A'],
	['\xc3\x86','AE'],
	['\xc3\x87','C'],
	['\xc3\x88','E'],
	['\xc3\x89','E'],
	['\xc3\x8a','E'],
	['\xc3\x8b','E'],
	['\xc3\x8c','I'],
	['\xc3\x8d','I'],
	['\xc3\x8e','I'],
	['\xc3\x8f','I'],
	['\xc3\x90','D'],
	['\xc3\x91','N'],
	['\xc3\x92','O'],
	['\xc3\x93','O'],
	['\xc3\x94','O'],
	['\xc3\x95','O'],
	['\xc3\x96','O'],
	['\xc3\x98','O'],
	['\xc3\x99','U'],
	['\xc3\x9a','U'],
	['\xc3\x9b','U'],
	['\xc3\x9c','U'],
	['\xc3\x9d','Y'],
	['\xc3\x9e','Th'],
	['\xc3\x9f','ss'],
	['\xc3\xa0','a'],
	['\xc3\xa1','a'],
	['\xc3\xa2','a'],
	['\xc3\xa3','a'],
	['\xc3\xa4','a'],
	['\xc3\xa5','a'],
	['\xc3\xa6','ae'],
	['\xc3\xa7','c'],
	['\xc3\xa8','e'],
	['\xc3\xa9','e'],
	['\xc3\xaa','e'],
	['\xc3\xab','e'],
	['\xc3\xac','i'],
	['\xc3\xad','i'],
	['\xc3\xae','i'],
	['\xc3\xaf','i'],
	['\xc3\xb0','d'],
	['\xc3\xb1','n'],
	['\xc3\xb2','o'],
	['\xc3\xb3','o'],
	['\xc3\xb4','o'],
	['\xc3\xb5','o'],
	['\xc3\xb6','o'],
	['\xc3\xb8','o'],
	['\xc3\xb9','u'],
	['\xc3\xba','u'],
	['\xc3\xbb','u'],
	['\xc3\xbc','u'],
	['\xc3\xbd','y'],
	['\xc3\xbe','th'],
	['\xc3\xbf','y'],
	## Turkish symbols
	['\xc3\x87','C'],
	['\xc3\x96','O'],
	['\xc3\x9c','U'],
	['\xc3\xa7','c'],
	['\xc3\xb6','o'],
	['\xc3\xbc','u'],
	['\xc4\x9e','G'],
	['\xc4\x9f','g'],
	['\xc4\xb1','i'],
	['\xc4\xb0','I'],
	['\xc5\x9e','S'],
	['\xc5\x9f','s'],
	## Greek symbols
	['\xce\xb1','a'],
	['\xce\xac','a'],
	['\xce\x91','A'],
	['\xce\x86','A'],
	['\xce\xb2','b'],
	['\xce\x92','B'],
	['\xce\xb3','g'],
	['\xce\x93','G'],
	['\xce\xb4','d'],
	['\xce\x94','D'],
	['\xce\xb5','e'],
	['\xce\xad','e'],
	['\xce\x95','E'],
	['\xce\x88','E'],
	['\xce\xb6','z'],
	['\xce\x96','Z'],
	['\xce\xb7','i'],
	['\xce\xae','i'],
	['\xce\x97','I'],
	['\xce\x89','I'],
	['\xce\xb8','th'],
	['\xce\x98','Th'],
	['\xce\xb9','i'],
	['\xce\xaf','i'],
	['\xcf\x8a','i'],
	['\xce\x90','i'],
	['\xce\x99','I'],
	['\xce\x8a','I'],
	['\xce\xaa','I'],
	['\xce\xba','k'],
	['\xce\x9a','K'],
	['\xce\xbb','l'],
	['\xce\x9b','L'],
	['\xce\xbc','m'],
	['\xce\x9c','M'],
	['\xce\xbd','n'],
	['\xce\x9d','N'],
	['\xce\xbe','ks'],
	['\xce\x9e','Ks'],
	['\xce\xbf','o'],
	['\xcf\x8c','o'],
	['\xce\x9f','O'],
	['\xce\x8c','O'],
	['\xcf\x80','p'],
	['\xce\xa0','P'],
	['\xcf\x81','r'],
	['\xce\xa1','R'],
	['\xcf\x83','s'],
	['\xcf\x82','s'],
	['\xce\xa3','S'],
	['\xcf\x84','t'],
	['\xce\xa4','T'],
	['\xcf\x85','y'],
	['\xcf\x8d','y'],
	['\xcf\x8b','y'],
	['\xce\xb0','y'],
	['\xce\xa5','Y'],
	['\xce\x8e','Y'],
	['\xce\xab','Y'],
	['\xcf\x86','f'],
	['\xce\xa6','F'],
	['\xcf\x87','h'],
	['\xce\xa7','H'],
	['\xcf\x88','ps'],
	['\xce\xa8','Ps'],
	['\xcf\x89','o'],
	['\xcf\x8e','o'],
	['\xce\xa9','O'],
	['\xce\x8f','O'],
	## Lithuanian symbols
	['\x01\x04','A'],
	['\x01\x05','a'],
	['\x01\x0c','C'],
	['\x01\x0d','c'],
	['\x01\x16','E'],
	['\x01\x17','e'],
	['\x01\x18','E'],
	['\x01\x19','e'],
	['\x01\x2e','I'],
	['\x01\x2f','i'],
	['\x01\x72','U'],
	['\x01\x73','u'],
	['\x01\x6a','U'],
	['\x01\x6b','u'],
	['\x01\x60','S'],
	['\x01\x61','s'],
	['\x01\x7d','Z'],
	['\x01\x7e','z'],
	['\xc4\x84','A'],
	['\xc4\x85','a'],
	['\xc4\x8c','C'],
	['\xc4\x8d','c'],
	['\xc4\x96','E'],
	['\xc4\x97','e'],
	['\xc4\x98','E'],
	['\xc4\x99','e'],
	['\xc4\xae','I'],
	['\xc4\xaf','i'],
	['\xc5\xb2','U'],
	['\xc5\xb3','u'],
	['\xc5\xaa','U'],
	['\xc5\xab','u'],
	['\xc5\xa0','S'],
	['\xc5\xa1','s'],
	['\xc5\xbd','Z'],
	['\xc5\xbe','z'],
];

## Description: transliterates source text from UTF-8 into ASCII.
## Currently it transliterates Russian letters and european letters, which present in ISO-8859-1
## Optionally this sub may delete some specified or not transliterated characters
## Parameters:
##		source_string (SCALAR) - source text in UTF-8 encoding
##		delete_nonascii_characters (0|1) - delete non-ASCII characters after transliteration
##			(we do not have transliterations for all UTF-8 characters,
##			 and this option allows simply to delete non-ASCII-letters, which were not transliterated)
##		characters_to_delete (ARRAYREF) - arrayref of characters, which will be deleted during transliteration
## Returns:
##		Transliterated string
sub translit_utf {
	my %h = (
		source_string => undef,
		delete_nonascii_characters => 0,
		characters_to_delete => undef,
		@_
	);
	my $str = $h{source_string};
	return undef unless(defined($str));
	_utf8_off($str);
	
	foreach my $rule (@{&RULES}) {
		my $c_utf = $rule->[0];
		my $c_ascii = $rule->[1];
		$str =~ s/$c_utf/$c_ascii/g;
	}
	if ($h{delete_nonascii_characters}) {
		## This will delete non transliterated UTF-8 symbols
		$str =~ s/[\x80-\xff]//g;
	}
	if (defined($h{characters_to_delete}) && ref($h{characters_to_delete}) eq 'ARRAY') {
		foreach my $c (@{$h{characters_to_delete}}) {
			$str =~ s/$c//g;
		}
	}
	return $str;
}

1;
