package HSPC::XRays;

use SWsoft::protect;

use strict;
no strict 'refs';
no warnings 'redefine'; ## Just a paranoia

use IO::File;
use Fcntl ':flock';

use Storable();
use Scalar::Util();
use Filter::Util::Call;

use constant NONE	=> 0x0000;
use constant DEBUG	=> 0x0001;
use constant CODE	=> 0x0002;
use constant KEEP	=> 0x0004;
use constant SPLIT	=> 0x0008;
use constant NOFILE	=> 0x0010;

use constant MIN	=> 0x0020;
use constant ARGS	=> 0x0040;
use constant RET	=> 0x0080;
use constant TIME	=> 0x0100;
use constant MEM	=> 0x0200;

use constant PARSE	=> 0x1000;
use constant STOP	=> 0x8000;

use constant LOGDIR	=> '/var/log/hspc/';
use constant LIBDIR	=> '/var/lib/hspc/';
use constant FLAGFILE	=> '/var/run/hspc.xrays.flag';


our $d;
sub dump_args($) {
	require Data::Dumper;
	$d = new Data::Dumper([]) unless $d;
	$d->Useperl(0)->Terse(1)->Useqq(0)->Indent(0)->Deparse(0);
	return join(', ',
		map { s/\n/\\n/go; s/\r/\\r/go; s/\t/\\t/go; $_ }
		map { $d->Values([$_]); ref =~ /::/ ? $_ : $d->Dump() }
		@{$_[0]});
}

our %fds;
sub store_entry(@) {
	my ($trace, $in, $entry, $lev, $args, $st, $sm) = @_;

	unless ($fds{$$}) {
		my $fname = LOGDIR . ".$$.trace";
		$fds{$$} = new IO::File(
			$trace & NOFILE ? ('>&STDERR') : ($fname, 'a')
		) or die "Unable to create '$fname': $!\n";
	}

	$fds{$$}->print("$in|$lev|$entry");
	$fds{$$}->print('|');
	$fds{$$}->printf('%.7f', Time::HiRes::gettimeofday() - $st)
		if defined $st;
	$fds{$$}->print('|');
	$fds{$$}->print(GTop->new()->proc_mem($$)->size() - $sm)
		if defined $sm;
	$fds{$$}->print('|');
	$fds{$$}->print(dump_args $args)
		if ($in && $trace & ARGS) || (!$in && $trace & RET);
	$fds{$$}->print("\n");
	$fds{$$}->flush;
}

our ($trace, %pkgs, %fpkg, $running, $request, $level);
sub trace_pkg($$) {
	my ($trace, $pkg) = @_;
	return 0
		if $pkgs{$pkg}{excl} || $trace < MIN || !($trace & 0x0FFF);

	my $pkgi = exists $pkgs{$pkg} ? $pkgs{$pkg}{subs} : { };
	foreach my $sub (keys %$pkgi) {
		next if $pkgi->{$sub};

		my $symbol = "${pkg}::$sub";
		warn "REDEFINING $symbol\n" if $trace & DEBUG;

		## Redefine a subroutine storing original prototype
		my $proto = prototype $symbol;
		$pkgi->{$sub} = *$symbol{CODE} || next;
		*$symbol = Scalar::Util::set_prototype(sub {
			## Remember start values for deltas
			my ($st, $sm);
			$st = Time::HiRes::gettimeofday()
				if $trace & TIME;
			$sm = GTop->new()->proc_mem($$)->size()
				if $trace & MEM;

			## Do not forget to proxy $AUTOLOAD
			my $mark = "${pkg}::$sub";
			$mark .= '[' . (${"${pkg}::$sub"} = ${"$sub"}) . ']'
				if $sub eq 'AUTOLOAD';
			## Log entrance
			store_entry($trace, 1, $mark, ++$level, \@_);
			## Request the subroutine
			my @result = wantarray ? $pkgi->{$sub}->(@_)
				: scalar $pkgi->{$sub}->(@_);
			## Log exit
			store_entry($trace, 0, $mark, $level--,
				\@result, $st, $sm);

			return wantarray ? @result : $result[0];
		}, $proto);
	}
	$pkgs{$pkg}{runs} = %$pkgi + 0;
}

sub stop_pkg($) {
	my $pkg = shift;
	my $pkgi = $pkgs{$pkg}{subs} || { };
	foreach my $sub (keys %$pkgi) {
		next unless $pkgi->{$sub};
		*{"${pkg}::$sub"} = $pkgi->{$sub};
		undef $pkgi->{$sub};
	}
}

sub update_pkg_info($$) {
	my ($pkg, $file) = @_;

	my $ts = (stat $file)[9];
	$pkgs{$pkg}{ts} = $ts;

	my $inf = { ts => $ts, subs => [ keys %{ $pkgs{$pkg}{subs} } ] };
	my $out = new IO::File(LIBDIR . $pkg, 'w', 0666) or return 1;
	$out->print(Storable::freeze $inf);
}

sub restore_pkg_info($$) {
	my ($pkg, $file) = @_;

	my $out = new IO::File(LIBDIR . $pkg, 'r') or return;
	local $/ = undef;
	my $inf = Storable::thaw($out->getline);

	$pkgs{$pkg} = {
		ts	=> $inf->{ts},
		subs	=> { map { $_ => undef } @{ $inf->{subs} } },
	};

	$fpkg{$file}{$pkg} = 1;
	(stat $file)[9] == $pkgs{$pkg}{ts};
}

sub trace($) {
	$trace = shift;

	$level = 0;
	if ($trace & STOP) {
		## Stop the trace fully
		stop_pkg($_) for keys %pkgs;
		return $running = 0;
	}

	if (!$running && !($trace & PARSE) && $trace >= MIN) {
		## Start the trace fully
		$running += trace_pkg($trace, $_) for keys %pkgs;
		return $running;
	}

	require Time::HiRes if $trace & TIME;
	require GTop if $trace & MEM;

	$running = 0;
	my $old = "";
	*{"CORE::GLOBAL::require"} = sub {
		my $result;
		my $filename = shift;
		(my $pkg = $filename) =~ s#/#::#g; $pkg =~ s/\.pm.?$//;

		my ($cpkg, $cfile, $cline) = caller;
		die "Compilation failed in require at $cfile line $cline.\n"
			if exists $INC{$filename}
				&& !($result = $INC{$filename});
		## TODO: FIXME: KORUM: Dirty hack here - version check
		return 1 if $filename =~ /^v?[0-9.]+$/;

		my $process =
			!index($cpkg, 'HSPC') && index($cpkg, __PACKAGE__);
		delete($pkgs{$pkg}) ## Support for the Apache::Reload
			unless index($cpkg, 'Apache::Reload') ||
				index($pkg, 'HSPC');

		$process &&= $trace && !exists $pkgs{$cpkg} &&
			!restore_pkg_info($cpkg, $cfile);
		if ($process) {
			## Parse the code to collect functions list
			my ($curr, $s) = $cpkg;
			warn "FILTERING $cpkg\n" if $trace & DEBUG;
			filter_add(sub {
				return $s unless ($s = filter_read()) > 0;
				$curr = $1 if /^\s*package\s+([^\s;]+)/;
				unless ($pkgs{$curr}) {
					$fpkg{$cfile}{$curr} = 1;
					$pkgs{$curr} = {subs => { } };
				}
				my ($sub) = /(?:^|\s)sub\s+(\w+)/;
				$pkgs{$curr}{subs}{$sub} = undef if $sub;
				warn "CODE: [$curr] " . ($_ = $old = $_)
					if ($trace & CODE) && $old ne $_;
				$_ = $_; $s;
			}) unless $fpkg{$cfile};
			$fpkg{$cfile} = { $curr => 1 };
			$pkgs{$curr} = {
				excl	=> $pkg eq 'overload',
				subs	=> { },
			};
		}

		## Hide the package from prossible error message.
		local $SIG{__DIE__} = sub {
			my @err = @_;
			for (@err) {
				next if index($_, __FILE__) < 0;
				my $pos = rindex($_, 'at ');
				my $nwl = substr($_, -1, 1) eq "\n"
					? "\n" : '';
				$_ = substr($_, 0, $pos + 3) .
					"$cfile line $cline$nwl";
			}
			die @err;
		};
		## Process the 'require' itself.
		$result = CORE::require $filename
			unless defined $result;

		if (!index($pkg, 'HSPC')) {
			my $file = $INC{$filename};
			for (keys %{ $fpkg{$file} }) {
				update_pkg_info($_, $file)
					unless $pkgs{$_}{ts};
				trace_pkg($trace, $_) unless $pkgs{$_}{runs};
				$running += $pkgs{$_}{runs};
			}
		}
		$result;
	};
}

sub handler {
	my $r = shift;

	if ($r->current_callback eq 'PerlInitHandler') {
		$trace = NONE || undef(%fpkg) if $trace & PARSE;
		## Enable a trace if file exists and stop if not
		if (-e FLAGFILE) {
			trace(flag());
			++$request;
		} elsif ($running) {
			trace(STOP);
		}
	} elsif ($trace >= MIN) {
		## PerlCleanupHandler or PerlChildExitHandler
		$fds{$$}->print('-' x 30 . " $$:$request " . '-' x 30 . "\n")
			if $fds{$$} && $trace & SPLIT;

		## Support for the TProxy project
		my ($sid, $in, $out, $line);
		$sid = $r->headers_in->{TPROXYSID}
			if $r->current_callback ne 'PerlChildExitHandler' &&
				$r->headers_in;
		$out = new IO::File(LOGDIR . ".$sid.trace", 'a') if $sid;

		undef $fds{$$};
		my $fname = LOGDIR . ".$$.trace";
		if ($out) {
			flock($out, LOCK_EX);

			$in = new IO::File($fname, 'r');
			$out->print($line) while $in && ($line = $in->getline);
			$out->flush;

			flock($out, LOCK_UN);
			$out = $in = undef;
		}
		## Post-processing trace log
		unlink $fname unless $trace & KEEP;
		$trace = NONE if $trace & STOP;
	}

	require Apache::Constants;
	&Apache::Constants::OK;
}

sub flag(;$) {
	my $new = shift;

	my $f = new IO::File(FLAGFILE, $new ? 'w' : 'r') or
		die 'Unable to ' . ($new ? 'create' : 'access') .
			" a flag file: $!";

	if ($new && $new < STOP) {
		my @wr = (($new & 0xFF00) >> 8, $new & 0x00FF);
		die "Unable to write to a flag file: $!\n"
			unless $f->write(pack('C2', @wr), 2) > 0;
	} elsif ($new & STOP) {
		unlink FLAGFILE or
			warn "Unable to remove flagfile: $!\n";
	}
	return $new if $new;

	my $res;
	die "Unable to read a flag file: $!\n"
		unless $f->read($res, 2) > 0;
	my @rd = unpack('C2', $res);
	$res = ($rd[0] << 8) + $rd[1];

	undef $f; ## Be a paranoind here
	$res;
}

## Parse the code on startup
BEGIN {
	## Firstly, do not forget to hide self from call stack
	*CORE::GLOBAL::caller = sub {
		my $l = (shift || 0) + 1;
		return caller($l) unless ${__PACKAGE__ . '::running'};

		my $i = 0;
		index((caller $i++)[3], __PACKAGE__) && --$l while $l;
		my @res = caller($i - 1);

		## A special hack to prevent 'isolation corrupted' error
		$res[0] = substr($res[3], 0, rindex($res[3], '::'))
			if $res[0] eq __PACKAGE__;
		wantarray ? @res : $res[0];
	};

	## Make sure our caching directory exists
	mkdir(LIBDIR, 0777) unless -d LIBDIR;

	## Start the code parsing here
	trace(PARSE);
	$request = 0;
}


42;

__END__

=head1 NAME

HSPC::XRays - XRays diagnostic machine for HSPcomplete's every sickness.

=head1 SYNOPSYS

To enable tracing its needed to complete following simple steps:

=over 4

=item Modify HSPCD config file

Place following code BEFORE any of HSPcomplete modules usage.

	## Switch tracing 'on'
	PerlModule		HSPC::XRays
	PerlInitHandler		HSPC::XRays
	PerlCleanupHandler	HSPC::XRays
	PerlChildExitHandler	HSPC::XRays

=item Flagging the C<HSPC::XRays> module to be switched on.

	HSPC::XRays::flag(<FLAGS>);

=back

In case of tracing command-line script it is need to place following code on
top of your script:

	use HSPC::XRays;
	trace(<FLAGS>);


=head1 DESCRIPTION

This module will help to diagnose over 90% of HSPcomplete's sickness appears.
The module will trace execution of the HSPcomplete code for whole request
processing time and store the result for the special file. In case of problem
report will be generated the execution trace can be collected and included for
the problem report.
The tracing can be enabled or disabled in run-time, e.g. no need to restart
HSPCD service to proceed this operation. Unfortunaly tracing HSPcomplete
execution may noticeably decrease its performance.

=head1 FLAGS

=head2 Debug-oriented

Those flags are forces the package to print additional information on
process's STDERR

=over 4

=item DEBUG

Show all operations are performed by the module

=item CODE

Print a code which parsing by the package

=back

=head2 Tracing flags

=over 4

=item MIN

Proceed only minimal tracing. I.e. only methods calls logging.

=item ARGS

Dump arguments passed for the subroutine.

=item RET

Dump result returning after the method has been processed.

=item TIME

Store time delta in seconds. Requires C<Time::HiRes> package installed.

=item MEM

Store process's memory delta in bytes. Requires C<GTop> package installed.

=back

=head2 Global configuration (internal) flags

=over 4

=item PARSE

Parse the code for prepare subroutines collection

=item STOP

Stop the tracing for the current process

=back

=head1 COPYRIGHT

Copyright 2007 SWsoft.

=head1 AUTHOR

Konstantin Kardamanov <korum /at/ swsoft.com>
